/* $Header: /timshel2g/src/master/nrn/src/nmodl/nparout.c,v 1.1.1.1 1994/10/12 17:21:36 hines Exp $ */

/* The strategy is to use as much of parout and hparout method as possible.
The bulk of the variables is in a p-array. The variables that don't belong
in this p-array are indicated by a flag with sym->subtype & NRNNOTP.
All other variables have values in that single array but not all those values
are available from HOC.

Variables accessible to NEURON are variables that appear within
GLOBAL, SECTION, and RANGE statements.
SECTION variables are not currently implemented.

Variables that do not appear in the p-array are:
 1)externally declared variables such as celsius, t.
 2)parameters and assigned not declared in the NEURON{RANGE list}
   that are global with respect to sections.
 3) variables static to this model, ie. v
 4) read only variables like "diam"
States always are in the p-array.

USEION variables in the p-array have connections to other places and
depending on the context may get their value from somewhere else, or
add their value to somewhere else, or place their value somewhere else. 
The cases are:
	NONSPECIFIC and USEION WRITE  i... value added to proper ion current
		and total current.
	USEION READ  entry value assigned to local copy.
	USEION WRITE e.. ..o ..i exit value of local copy assigned to pointer..
		 It is an error for an ionic current or ionic variable
		to be a STATE. Use another variable as the state, make the
		ionic variable an ASSIGNED and just assign it at the
		proper place.
	  Alternatively, if they are STATE's then they should not be READ
	  since their value comes from the p-array itself.

POINTER variables are like USEION variables. Unfortunately, it is up to
the hoc user to make sure they point to the proper place with a connect
statement. At this time we only check for a null pointer. The pointers
are kept in the ppvar array.

each model creates a setdata_suffix(x) (or setdata_suffix(i)) function
which sets up _p and _ppvar for use by functions in the model called
directly by hoc.
*/	

/* FUNCTIONS are made external so they are callable from other models */
#define GLOBFUNCT 1

#include "modl.h"
#include "ytab.h"

#if VECTORIZE
int vectorize = 1;
/*
the idea is to put all variables into a vector of vectors so there
there is no static data. Every function has an implicit argument, _ix
which tells which set of data _p[_ix][...] to use. There are going to have
to be limits on the kinds of scopmath functions called since many of them
need static data. We can have special versions of the most useful of these.
ie sparse.c.
*/

#endif
#define NRNEXTRN	01	/* t, dt, celsius, etc. */
#define NRNCURIN	02	/* input value used */
#define NRNCUROUT	04	/* added to output value */
#define NRNRANGE	010	
#define NRNPRANGEIN	020
#define NRNPRANGEOUT	040
#define NRNGLOBAL	0100	/* same for all sections, defined here */
#define NRNSTATIC	0200	/* v */
#define NRNNOTP		0400	/* doesn't belong in p array */
#define NRNIONFLAG	01000	/* temporary flag to allow READ and WRITE
				   without declaring twice */
#define NRNSECTION	02000
#define NRNPOINTER	04000

#define IONEREV 0	/* Parameter */
#define IONIN	1
#define IONOUT	2
#define IONCUR	3	/* assigned */
#define IONDCUR 4

extern Symbol *indepsym;
extern Symbol *scop_indep;
extern List   *indeplist;
extern Symbol *stepsym;
extern char   *reprime();
extern List	*symlist[];
List *syminorder;
List *plotlist;
List *defs_list;

/* NEURON block information */
List *currents;
List *useion;
static List *rangeparm;
static List *rangedep;
static List *rangestate;
static List *nrnpointers;
static char suffix[50];
static char *mechname;
int point_process; /* 1 if a point process model */
static int diamdec = 0;	/*1 if diam is declared*/

#define SYMITER(arg) ITERATE(q, syminorder){ \
		s = SYM(q); if (s->type == arg)

#define SYMLISTITER for (i = 'A'; i <= 'z'; i++)\
	ITERATE(q, symlist[i])
	
#define IFTYPE(arg)	if ((s->subtype & arg)\
			 && ( (s->usage & EXPLICIT_DECL) != automatic) )

static defs_h();
static int iontype();
static nrndeclare();
static del_range();
static declare_p();
static int iondef();

/* varcount holds the index into the .var file and is saved in s->used
   parraycount holds the index into the p array and is saved in s->varnum
   pvarcount indexes pointers to variables such as ena
*/
static varcount, parraycount;

nrninit() {
	currents = newlist();
	rangeparm = newlist();
	rangedep = newlist();
	rangestate = newlist();
	useion = newlist();
	nrnpointers = newlist();
}

parout() {
	int i, ioncount, pointercount;
	Item *q, *q1;
	Symbol *s, *sion;
	double d1, d2;
	extern char *modprefix;
	char *modbase;

	defs_list = newlist();	/* relates hoc names to c-variables */

	Lappendstr(defs_list, "\
\n#include \"nmodl_redef.h\"\
\n#include \"section.h\"\
\n#include \"nmodl_redef.h\"\n\
\n#if METHOD3\nextern int _method3;\n#endif\n\
");
	Lappendstr(defs_list, "extern double *getarg();\n");
#if VECTORIZE
    if (vectorize) {
	Sprintf(buf, "static double **_p; static Datum **_ppvar;\n");
    }else
#endif
    {
	Sprintf(buf, "static double *_p; static Datum *_ppvar;\n");
    }
	Lappendstr(defs_list, buf);

	for (modbase = modprefix + strlen(modprefix); modbase != modprefix;
	    modbase--) {
		if (*modbase == '\\' || *modbase == '/') {
			modbase++;
			break;
		}
	}
	if (!mechname) {
		sprintf(suffix,"_%s", modbase);
		mechname = modbase;
	} else if (strcmp(mechname, "nothing") == 0) {
		suffix[0] = '\0';
		mechname = modbase;
	}else{
		sprintf(suffix, "_%s", mechname);
	}
	nrndeclare();
	varcount = parraycount = 0;
	declare_p();
	ioncount = iondef(&pointercount); /* first is _nd_area if point process */
	Lappendstr(defs_list, "static int hoc_nrnpointerindex = ");
	if (pointercount) {
		q = nrnpointers->next;
		Sprintf(buf, "%d;\n", SYM(q)->used);
	}else{
		Sprintf(buf, "-1;\n");
	}
	Lappendstr(defs_list, buf);
	/*above modified to also count and define pointers*/
	
#if 0
	Lappendstr(defs_list, "/* static variables special to NEURON */\n");
	SYMLISTITER {
		if (SYM(q)->nrntype & NRNSTATIC) {
			Sprintf(buf, "static double %s;\n", SYM(q)->name);
			Lappendstr(defs_list, buf);
		}
	}
#endif
	Lappendstr(defs_list, "/* external NEURON variables */\n");
	SYMLISTITER {
		s = SYM(q);
		if (s->nrntype & NRNEXTRN) {
			if (s->subtype & ARRAY) {
				Sprintf(buf, "extern double* %s;\n", s->name);
			}else{
				Sprintf(buf, "extern double %s;\n", s->name);
			}
			Lappendstr(defs_list, buf);
		}
	}
	
	Lappendstr(defs_list, "/* declaration of user functions */\n");
	SYMLISTITER {
		s = SYM(q);
		if (s->subtype & (FUNCT | PROCED) && s->name[0] != '_') {
Sprintf(buf, "static int _hoc_%s();\n", s->name);
			Lappendstr(defs_list, buf);
		}
	}

	/**** create special point process functions */
	if (point_process) {
		Lappendstr(defs_list, "static int _pointtype;\n");
		Lappendstr(defs_list, "static _hoc_create_pnt() {\n");
		Lappendstr(defs_list, "create_point_process(_pointtype); ret(1.);\n}\n");
		Lappendstr(defs_list, "static _hoc_loc_pnt() {\n");
		Lappendstr(defs_list, "loc_point_process(_pointtype); ret(1.);\n}\n");
		Lappendstr(defs_list, "static _hoc_get_loc_pnt() {\n");
		Lappendstr(defs_list, "double get_loc_point_process(); ret (get_loc_point_process(_pointtype));\n}\n");
	}
	/* function to set up _p and _ppvar */
	Lappendstr(defs_list, "static _hoc_setdata() {\n Prop *_prop, *hoc_getdata_pnt(), *hoc_getdata_range();\n");
	if (point_process) {
		Lappendstr(defs_list, "_prop = hoc_getdata_pnt(_pointtype);\n");
	}else{
		Sprintf(buf, "_prop = hoc_getdata_range(\"%s\");\n",mechname);
		Lappendstr(defs_list, buf);
	}
#if VECTORIZE
    if (vectorize) {
	Lappendstr(defs_list, "_p = &_prop->param; _ppvar = &_prop->dparam;\n");
    }else
#endif
    {
	Lappendstr(defs_list, "_p = _prop->param; _ppvar = _prop->dparam;\n");
    }
	Lappendstr(defs_list, "ret(1.);\n}\n");
	
	/* functions */
	Lappendstr(defs_list, "/* connect user functions to hoc names */\n");
	Lappendstr(defs_list,"static IntFunc hoc_intfunc[] = {\n");
	Sprintf(buf, "\"setdata_%s\", _hoc_setdata,\n", mechname);
	Lappendstr(defs_list, buf);
	if (point_process) {
		Sprintf(buf, "\"create_%s\", _hoc_create_pnt,\n", mechname);
		Lappendstr(defs_list, buf);
		Sprintf(buf, "\"loc_%s\", _hoc_loc_pnt,\n", mechname);
		Lappendstr(defs_list, buf);
		Sprintf(buf, "\"get_loc_%s\", _hoc_get_loc_pnt,\n", mechname);
		Lappendstr(defs_list, buf);
	}
 	SYMLISTITER {
		s = SYM(q);
		if ((s->subtype & (FUNCT | PROCED)) && s->name[0] != '_') {
Sprintf(buf, "\"%s%s\", _hoc_%s,\n", s->name, suffix, s->name);
			Lappendstr(defs_list, buf);
		}
	}
	Lappendstr(defs_list, "0, 0\n};\n");

#if GLOBFUNCT
	/* FUNCTION's are now global so callable from other models */
	/* change name to namesuffix. This propagates everywhere except
		to hoc_name*/
	/* but don't do it if suffix is empty */
	if (suffix[0]) SYMLISTITER {
		s = SYM(q);
		if ((s->subtype & FUNCT)) {
			Sprintf(buf, "#define %s %s%s\n", s->name, s->name, suffix);
			q1 = Lappendstr(defs_list, buf);
			q1->itemtype = VERBATIM;
		}
	}
	SYMLISTITER {
		s = SYM(q);
		if ((s->subtype & FUNCT)) {
			Sprintf(buf, "extern double %s();\n", s->name);
			Lappendstr(defs_list, buf);
		}
	}
#endif

	/* double scalars declared internally */
	Lappendstr(defs_list, "/* declare global and static user variables */\n");
 	SYMLISTITER { /* globals are now global with respect to C as well as hoc */
		s = SYM(q);
		if (s->nrntype & (NRNGLOBAL)) {
			if (suffix[0]) {
				Sprintf(buf, "#define %s %s%s\n", s->name, s->name, suffix);
				q1 = Lappendstr(defs_list, buf);
				q1->itemtype = VERBATIM;
			}
			decode_ustr(s, &d1, &d2, buf);
			if (s->subtype & ARRAY) {
				Sprintf(buf, "double %s[%d];\n", s->name, s->araydim);
			}else{
				Sprintf(buf, "double %s = %g;\n", s->name, d1);
			}
			Lappendstr(defs_list, buf);
		}
	}
 	SYMLISTITER {
		s = SYM(q);
		if (s->nrntype & (NRNSTATIC)) {
#if VECTORIZE && 0
    if (vectorize) {
diag("No statics allowed when VECTORIZING:", s->name);
    }
#endif
			decode_ustr(s, &d1, &d2, buf);
			if (s->subtype & ARRAY) {
				Sprintf(buf, "static double %s[%d];\n", s->name, s->araydim);
			}else{
				Sprintf(buf, "static double %s = %g;\n", s->name, d1);
			}
			Lappendstr(defs_list, buf);
		}
	}
	Lappendstr(defs_list, "/* connect global user variables to hoc */\n");
	Lappendstr(defs_list,"static DoubScal hoc_scdoub[] = {\n");
 	ITERATE(q, syminorder) {
		s = SYM(q);
		if (s->nrntype & NRNGLOBAL && !(s->subtype & ARRAY)) {
			Sprintf(buf, "\"%s%s\", &%s,\n", s->name, suffix, s->name);
			Lappendstr(defs_list, buf);
		}
	}
	Lappendstr(defs_list, "0,0\n};\n");
	
	/* double vectors */
	Lappendstr(defs_list,"static DoubVec hoc_vdoub[] = {\n");
 	ITERATE(q, syminorder) {
		s = SYM(q);
		if (s->nrntype & NRNGLOBAL && (s->subtype & ARRAY)) {
			Sprintf(buf, "\"%s%s\", %s, %d,\n", s->name, suffix, s->name, s->araydim);
			Lappendstr(defs_list, buf);
		}
	}
	Lappendstr(defs_list, "0,0,0\n};\n");
	Lappendstr(defs_list, "static double _sav_indep;\n");

	/******** what normally goes into cabvars.h structures */
	
	/*declaration of the range variables names to HOC */
	Lappendstr(defs_list, "\
static nrn_alloc(), nrn_init(); static double nrn_cur(); static int nrn_state();\n\
");
	Lappendstr(defs_list,
"/* connect range variables in _p that hoc is supposed to know about */\n");
	Lappendstr(defs_list, "\
static char *_mechanism[] = {\n\
");
	Sprintf(buf, "\"%s\",\n", mechname);
	Lappendstr(defs_list, buf);
	ITERATE(q, rangeparm) {
		s = SYM(q);
		if (s->subtype & ARRAY) {
			Sprintf(buf, "\"%s%s[%d]\",\n", s->name, suffix, s->araydim);
		}else{
			Sprintf(buf, "\"%s%s\",\n", s->name, suffix);
		}
		Lappendstr(defs_list, buf);
	}
	Lappendstr(defs_list, "0,\n");
	ITERATE(q, rangedep) {
		s = SYM(q);
		if (s->subtype & ARRAY) {
			Sprintf(buf, "\"%s%s[%d]\",\n", s->name, suffix, s->araydim);
		}else{
			Sprintf(buf, "\"%s%s\",\n", s->name, suffix);
		}
		Lappendstr(defs_list, buf);
	}
	Lappendstr(defs_list, "0,\n");
	ITERATE(q, rangestate) {
		s = SYM(q);
		if (s->subtype & ARRAY) {
			Sprintf(buf, "\"%s%s[%d]\",\n", s->name, suffix, s->araydim);
		}else{
			Sprintf(buf, "\"%s%s\",\n", s->name, suffix);
		}
		Lappendstr(defs_list, buf);
	}
	Lappendstr(defs_list, "0,\n");

	/* pointer variable names */
	ITERATE(q, nrnpointers) {
		s = SYM(q);
		if (s->subtype & ARRAY) {
			Sprintf(buf, "\"%s%s[%d]\",\n", s->name, suffix, s->araydim);
		}else{
			Sprintf(buf, "\"%s%s\",\n", s->name, suffix);
		}
		Lappendstr(defs_list, buf);
	}
	
	Lappendstr(defs_list, "0};\n");

	/*********Creation of the allocation function*/

#if VECTORIZE
    if (vectorize) {
	Lappendstr(defs_list, "\n\
static nrn_alloc(_prop)\n\
	Prop *_prop;\n\
{\n\
	Prop *prop_ion, *need_memb();\n\
	double *_p[1]; Datum *_ppvar[1]; int _ix = 0;\n\
");
Sprintf(buf, "	_p[0] = (double *)ecalloc(%d, sizeof(double));\n", parraycount);
	Lappendstr(defs_list, buf);
	Lappendstr(defs_list, "\t_prop->param = _p[0];\n");
	Lappendstr(defs_list, "	/*initialize range parameters*/\n");
	ITERATE(q, rangeparm) {
		s = SYM(q);
		decode_ustr(s, &d1, &d2, buf);
		Sprintf(buf, "	%s = %g;\n", s->name, d1);
		Lappendstr(defs_list, buf);
	}
	/* count the number of pointers needed */
	if (ioncount || diamdec || pointercount) {
Sprintf(buf, "	_ppvar[0] = (Datum *)ecalloc(%d, sizeof(Datum));\n", ioncount+diamdec + pointercount);
		Lappendstr(defs_list, buf);
		Lappendstr(defs_list, "\t_prop->dparam = _ppvar[0];\n");
		Lappendstr(defs_list,"\t/*connect ionic variables to this model*/\n");
	}
	if (diamdec) {
		Sprintf(buf, "prop_ion = need_memb(\"morphology\");\n");
		Lappendstr(defs_list, buf);
	  	Sprintf(buf,
	  	 "\t_ppvar[0][%d].pval = &prop_ion->param[0]; /* diam */\n",
	  	 ioncount),
	  	Lappendstr(defs_list, buf);
	}

	if (point_process) {
		ioncount = 1;
	}else{
		ioncount = 0;
	}
	ITERATE(q, useion) {
		sion = SYM(q);
		Sprintf(buf, "prop_ion = need_memb(\"%s_ion\");\n",sion->name);
		Lappendstr(defs_list, buf);
		q=q->next;
		ITERATE(q1, LST(q)) {
			SYM(q1)->nrntype |= NRNIONFLAG;
		  	Sprintf(buf,
		  	 "\t_ppvar[0][%d].pval = &prop_ion->param[%d]; /* %s */\n",
		  	 ioncount++, iontype(SYM(q1)->name, sion->name),
		  	 SYM(q1)->name);
		  	Lappendstr(defs_list, buf);
		}
		q=q->next;
		ITERATE(q1, LST(q)) {
			int itype = iontype(SYM(q1)->name, sion->name);
			
			if (SYM(q1)->nrntype & NRNIONFLAG) {
				SYM(q1)->nrntype &= ~NRNIONFLAG;
			}else{
			  	Sprintf(buf,
			  	 "\t_ppvar[0][%d].pval = &prop_ion->param[%d]; /* %s */\n",
			  	 ioncount++, itype, SYM(q1)->name);
			  	Lappendstr(defs_list, buf);
			}
			if (itype == IONCUR) {
				Sprintf(buf,
"\t_ppvar[0][%d].pval = &prop_ion->param[%d]; /* _ion_di%sdv */\n",
				 ioncount++, IONDCUR, sion->name);
			  	Lappendstr(defs_list, buf);
			}
		}
	}
    }else
#endif
    {
	Lappendstr(defs_list, "\n\
static nrn_alloc(_prop)\n\
	Prop *_prop;\n\
{\n\
	Prop *prop_ion, *need_memb();\n\
	double *_p; Datum *_ppvar;\n\
");
Sprintf(buf, "	_p = (double *)ecalloc(%d, sizeof(double));\n", parraycount);
	Lappendstr(defs_list, buf);
	Lappendstr(defs_list, "\t_prop->param = _p;\n");
	Lappendstr(defs_list, "	/*initialize range parameters*/\n");
	ITERATE(q, rangeparm) {
		s = SYM(q);
		decode_ustr(s, &d1, &d2, buf);
		Sprintf(buf, "	%s = %g;\n", s->name, d1);
		Lappendstr(defs_list, buf);
	}
	/* count the number of pointers needed */
	if (ioncount || diamdec || pointercount) {
Sprintf(buf, "	_ppvar = (Datum *)ecalloc(%d, sizeof(Datum));\n", ioncount+diamdec + pointercount);
		Lappendstr(defs_list, buf);
		Lappendstr(defs_list, "\t_prop->dparam = _ppvar;\n");
		Lappendstr(defs_list,"\t/*connect ionic variables to this model*/\n");
	}
	if (diamdec) {
		Sprintf(buf, "prop_ion = need_memb(\"morphology\");\n");
		Lappendstr(defs_list, buf);
	  	Sprintf(buf,
	  	 "\t_ppvar[%d].pval = &prop_ion->param[0]; /* diam */\n",
	  	 ioncount),
	  	Lappendstr(defs_list, buf);
	}

	if (point_process) {
		ioncount = 1;
	}else{
		ioncount = 0;
	}
	ITERATE(q, useion) {
		sion = SYM(q);
		Sprintf(buf, "prop_ion = need_memb(\"%s_ion\");\n",sion->name);
		Lappendstr(defs_list, buf);
		q=q->next;
		ITERATE(q1, LST(q)) {
			SYM(q1)->nrntype |= NRNIONFLAG;
		  	Sprintf(buf,
		  	 "\t_ppvar[%d].pval = &prop_ion->param[%d]; /* %s */\n",
		  	 ioncount++, iontype(SYM(q1)->name, sion->name),
		  	 SYM(q1)->name);
		  	Lappendstr(defs_list, buf);
		}
		q=q->next;
		ITERATE(q1, LST(q)) {
			int itype = iontype(SYM(q1)->name, sion->name);
			
			if (SYM(q1)->nrntype & NRNIONFLAG) {
				SYM(q1)->nrntype &= ~NRNIONFLAG;
			}else{
			  	Sprintf(buf,
			  	 "\t_ppvar[%d].pval = &prop_ion->param[%d]; /* %s */\n",
			  	 ioncount++, itype, SYM(q1)->name);
			  	Lappendstr(defs_list, buf);
			}
			if (itype == IONCUR) {
				Sprintf(buf,
"\t_ppvar[%d].pval = &prop_ion->param[%d]; /* _ion_di%sdv */\n",
				 ioncount++, IONDCUR, sion->name);
			  	Lappendstr(defs_list, buf);
			}
		}
	}
    } /* !vectorize */

	Lappendstr(defs_list, "\n}\n");

	/**** create function that registers the above info to NEURON and HOC*/
	Lappendstr(defs_list, "static _initlists();\n");
#if VECTORIZE
	Sprintf(buf, "_%s_reg() {\n	int _vectorized = %d;\n	_initlists();\n", modbase, vectorize);
#else
	Sprintf(buf, "_%s_reg() {\n	_initlists();\n", modbase);
#endif
	Lappendstr(defs_list, buf);
	ITERATE(q, useion) {
		Sprintf(buf, "\tion_reg(\"%s\");\n", SYM(q)->name);
		Lappendstr(defs_list, buf);
		q = q->next->next;
	}
#if VECTORIZE
	if (point_process) {
		Lappendstr(defs_list, "\
	_pointtype = point_register_mech(_mechanism, nrn_alloc, nrn_cur, nrn_state, nrn_init, hoc_nrnpointerindex, _vectorized);\n");
	}else{
		Lappendstr(defs_list, "\
	register_mech(_mechanism, nrn_alloc, nrn_cur, nrn_state, nrn_init, hoc_nrnpointerindex, _vectorized);\n");
	}
#else
	if (point_process) {
		Lappendstr(defs_list, "\
	_pointtype = point_register_mech(_mechanism, nrn_alloc, nrn_cur, nrn_state, nrn_init, hoc_nrnpointerindex);\n");
	}else{
		Lappendstr(defs_list, "\
	register_mech(_mechanism, nrn_alloc, nrn_cur, nrn_state, nrn_init, hoc_nrnpointerindex);\n");
	}
#endif
	Lappendstr(defs_list, "\
	hoc_register_var(hoc_scdoub, hoc_vdoub, hoc_intfunc);\n");
	Lappendstr(defs_list, "}\n");
}

decode_ustr(sym, pg1, pg2, s)	/* decode sym->u.str */
	Symbol *sym;
	char *s;
	double *pg1, *pg2;
{
	int i, n;
	char *cp, *cp1;
	
	switch (sym->subtype & (INDEP | DEP | STAT | PARM)) {

	case INDEP:	/* but doesnt get all info */
	case DEP:
	case STAT:
		assert(sym && sym->u.str);
		if (sym->subtype & ARRAY) { /* see parsact.c */
			i = sscanf(sym->u.str, "[%*d]\n%lf%*c%lf", pg1, pg2);
		}else{			
			i = sscanf(sym->u.str, "%lf%*c%lf", pg1, pg2);
		}
		assert(i == 2);
		for (n=0, cp = sym->u.str; n < 2;) {
			if (*cp++ == '\n') {
				n++;
			}
		}
		for (cp1 = s; *cp != '\n';) {
			*cp1++ = *cp++;
		}
		*cp1 = '\0';
		break;

	case PARM:
		assert(sym && sym->u.str);
		i = sscanf(sym->u.str, "%lf\n%s", pg1, s);
		if (i == 1) {
			s[0] = '\0';
			i = 2;
		}
		assert(i == 2);
		break;
	default:
		/*SUPPRESS 622*/
		assert(0);
	}
}

static
var_count(s)
	Symbol *s;
{
	defs_h(s);
		s->used = varcount++;
		s->varnum = parraycount;
		if (s->subtype & ARRAY) {
			parraycount += s->araydim;
		}else{
			parraycount++;
		}
}

static
defs_h(s)
	Symbol *s;
{
	Item *q;
	
#if VECTORIZE
    if (vectorize) {
	if (s->subtype & ARRAY) {
		Sprintf(buf, "#define %s (_p[_ix] + %d)\n", s->name, parraycount);
		q = lappendstr(defs_list, buf);
	} else {
		Sprintf(buf, "#define %s _p[_ix][%d]\n", s->name, parraycount);
		q = lappendstr(defs_list, buf);
	}
    }else
#endif
    {
	if (s->subtype & ARRAY) {
		Sprintf(buf, "#define %s (_p + %d)\n", s->name, parraycount);
		q = lappendstr(defs_list, buf);
	} else {
		Sprintf(buf, "#define %s _p[%d]\n", s->name, parraycount);
		q = lappendstr(defs_list, buf);
	}
    } /* !vectorize */
	q->itemtype = VERBATIM;
}


nrn_list(q1, q2)
	Item *q1, *q2;
{
	List **plist;
	Item *q;
		
	switch (SYM(q1)->type) {
	case RANGE:
		plist = (List **)0;
		for (q = q1->next; q != q2->next; q = q->next) {
			SYM(q)->nrntype |= NRNRANGE;
		}
		break;
	case SUFFIX:
		plist = (List **)0;
		mechname = SYM(q2)->name;
		if (strcmp(SYM(q1)->name, "POINT_PROCESS") == 0) {
			point_process = 1;
		}
		break;
	case NONSPECIFIC:
		plist = &currents;
		for (q = q1->next; q != q2->next; q = q->next) {
			SYM(q)->nrntype |= NRNRANGE;
		}
		break;
	case SECTION:
		diag("NEURON SECTION variables not implemented", (char *)0);
		break;
	case GLOBAL:
#if VECTORIZE
		vectorize = 0;
#endif
		for (q = q1->next; q != q2->next; q = q->next) {
			SYM(q)->nrntype |= NRNGLOBAL | NRNNOTP;
		}
		plist = (List **)0;
		break;
	case EXTERNAL:
#if VECTORIZE
		vectorize = 0;
#endif
		for (q = q1->next; q != q2->next; q = q->next) {
			SYM(q)->nrntype |= NRNEXTRN | NRNNOTP;
		}
		plist = (List **)0;
		break;
	case POINTER:
		plist = &nrnpointers;
		for (q = q1->next; q != q2->next; q = q->next) {
			SYM(q)->nrntype |= NRNNOTP | NRNPOINTER;
		}
		break;
	}
	if (plist) {
		if (!*plist) {
			*plist = newlist();
		}
		assert (q1 != q2);
		movelist(q1->next, q2, *plist);
	}
}

nrn_use(q1, q2, q3)
	Item *q1, *q2, *q3;
{
	int used;
	Item *q, *qr, *qw;
	List *readlist, *writelist;
	Symbol *ion;
	
	ion = SYM(q1);
	/* is it already used */
	used = 0;
	ITERATE(q, useion) {
		if (SYM(q) == SYM(q1)) {
			used = 1;
		}
		q = q->next->next;
	}
	if (used) { /* READ gets promoted to WRITE */
		diag("mergeing of neuron models not supported yet", (char *)0);
	}else{ /* create all the ionic variables */
		Lappendsym(useion, ion);
		readlist = newlist();
		writelist = newlist();
		qr = lappendsym(useion, SYM0);
		qw = lappendsym(useion, SYM0);
		LST(qr) = readlist;
		LST(qw) = writelist;
		if (q2) { Item *qt = q2->next;
			move(q1->next->next, q2, readlist);
			if (q3) {
				move(qt->next, q3, writelist);
			}
		}else if (q3) {
			move(q1->next->next, q3, writelist);
		}
		ITERATE(q, readlist) {
			if (iontype(SYM(q)->name, ion->name) == IONCUR) {
				SYM(q)->nrntype |= NRNCURIN;
			}else{
				SYM(q)->nrntype |= NRNPRANGEIN;
			}
		}
		ITERATE(q, writelist) {
			if (iontype(SYM(q)->name, ion->name) == IONCUR) {
				if (!currents) {
					currents = newlist();
				}
				Lappendsym(currents, SYM(q));
				SYM(q)->nrntype |= NRNCUROUT;
			}else{
				SYM(q)->nrntype |= NRNPRANGEOUT;
			}
		}
	}
}

static
int iontype(s1, s2)	/* returns index of variable in ion mechanism */
	char *s1, *s2;
{
	Sprintf(buf, "i%s", s2);
	if (strcmp(buf, s1) == 0) {
		return IONCUR;
	}
	Sprintf(buf, "e%s", s2);
	if (strcmp(buf, s1) == 0) {
		return IONEREV;
	}
	Sprintf(buf, "%si", s2);
	if (strcmp(buf, s1) == 0) {
		return IONIN;
	}
	Sprintf(buf, "%so", s2);
	if (strcmp(buf, s1) == 0) {
		return IONOUT;
	}
	Sprintf(buf, "%s is not a valid ionic variable for %s", s1, s2);
	diag(buf, (char *)0);
	return -1;
}

static Symbol *
ifnew_install(name)
	char *name;
{
	Symbol *s;
	
	if ((s = lookup(name)) == SYM0) {
		s = install(name, NAME);
		parminstall(s, "0", "");
	}
	return s;
}

static nrndeclare() {
	Symbol *s;
	Item *q;
	
	s=lookup("diam"); if (s) {
		s->nrntype |= NRNNOTP|NRNPRANGEIN; diamdec=1;
	}
#if VECTORIZE
    if (vectorize) {
	s = ifnew_install("v");
	s->nrntype = NRNNOTP; /* this is a lie, it goes in at end specially */
    }else
#endif
    {
	s = ifnew_install("v");
	s->nrntype |= NRNSTATIC | NRNNOTP;
    }
	s = ifnew_install("t");
	s->nrntype |= NRNEXTRN | NRNNOTP;
	s = ifnew_install("dt");
	s->nrntype |= NRNEXTRN | NRNNOTP;
	s = ifnew_install("delta_t");
	s->nrntype |= NRNNOTP;
	Lappendstr(defs_list, "\n#define delta_t dt\n");

	s=lookup("usetable"); if (s) { s->nrntype |= NRNGLOBAL | NRNNOTP;}
	s=lookup("celsius");if(s){s->nrntype |= NRNEXTRN | NRNNOTP;}
	s=lookup("celcius"); if (s) diag("celcius should be spelled celsius",
					(char *)0);
	
 	ITERATE(q, syminorder) {
		s = SYM(q);
		if (s->type == NAME || s->type == PRIME) {
			if (s->subtype & PARM && s->nrntype & NRNRANGE) {
				Lappendsym(rangeparm, s);
			} else if (s->subtype & STAT) {
				s->nrntype |= NRNRANGE;
				Lappendsym(rangestate, s);
			} else if (s->subtype & DEP && s->nrntype & NRNRANGE) {
				Lappendsym(rangedep, s);
			}
			if (s != indepsym && !s->nrntype) {
				if (s->subtype & PARM) {
					if (s->usage & EXPLICIT_DECL) {
						s->nrntype |= NRNGLOBAL;
						s->nrntype |= NRNNOTP;
					}else{
						s->nrntype |= NRNSTATIC;
						s->nrntype |= NRNNOTP;
					}
				}
			}
		}
	}
	/* some ionic variables don't need duplicates known to hoc */
	del_range(rangeparm);
	del_range(rangestate);
	del_range(rangedep);
}

static del_range(range)
	List *range;
{
	Item *q, *q1;
	Symbol *s;
	
	for (q = ((Item *)range)->next; q != (Item *)range; q = q1) {
		q1 = q->next;
		s = SYM(q);
		if (s->nrntype & (NRNPRANGEIN | NRNPRANGEOUT)) {
			delete(q);
		}
	}
}
	

static declare_p() {
	Item *q;
	
	ITERATE(q, syminorder) {
		SYM(q)->used = -1;
	}
	ITERATE(q, rangeparm) {
		var_count(SYM(q));
	}
	ITERATE(q, rangedep) {
		var_count(SYM(q));
	}
	ITERATE(q, rangestate) {
		var_count(SYM(q));
	}
	ITERATE(q, syminorder) {
		if (!(SYM(q)->nrntype & NRNNOTP) && SYM(q)->used < 0) {
			var_count(SYM(q));
		}
	}
#if VECTORIZE
	if (vectorize) {
		Symbol* s = ifnew_install("v");
		var_count(s);
	}
#endif

}

List *
set_ion_variables(block)
	int block;	/* 0 means equation block */
{
	/*ARGSUSED*/
	Item *q, *q1;
	static List *l;

	l = newlist();
	ITERATE(q, useion) {
		q = q->next;
		q = q->next;
		ITERATE(q1, LST(q)) {
			if (SYM(q1)->nrntype & NRNCUROUT) {
				if ( block == 0) {
Sprintf(buf, " _ion_%s += %s", SYM(q1)->name, SYM(q1)->name);
					Lappendstr(l, buf);
					if (point_process) {
						Sprintf(buf, "* 1.e2/ (_nd_area);\n");
					}else{
						Sprintf(buf, ";\n");
					}
				}else{
					Sprintf(buf, "");
				}
			}else{
Sprintf(buf, " _ion_%s = %s;\n", SYM(q1)->name, SYM(q1)->name);
			}
			Lappendstr(l, buf);
		}
	}
	return l;
}

List *
get_ion_variables(block)
	int block;	/* 0 means equation block */
{
	/*ARGSUSED*/
	Item *q, *q1;
	static List *l;

	l = newlist();
	ITERATE(q, useion) {
		q = q->next;
		ITERATE(q1, LST(q)) {
Sprintf(buf, " %s = _ion_%s;\n", SYM(q1)->name, SYM(q1)->name);
			Lappendstr(l, buf);
if (point_process && (SYM(q1)->nrntype & NRNCURIN)) {
Fprintf(stderr, "WARNING: Dimensions may be wrong for READ %s with POINT_PROCESS\n", SYM(q1)->name);
}
		}
		q = q->next;
		ITERATE(q1, LST(q)) {
Sprintf(buf, " %s = _ion_%s;\n", SYM(q1)->name, SYM(q1)->name);
			if (SYM(q1)->subtype & STAT) {
				Lappendstr(l, buf);
if (SYM(q1)->nrntype & NRNCUROUT) {
Fprintf(stderr, "WARNING: WRITE %s with it a STATE may not be translated correctly\n", SYM(q1)->name);
}
			}
		}
	}
	return l;
}

static int
iondef(p_pointercount) int *p_pointercount; {
	int ioncount;
	Item *q, *q1, *q2;
	Symbol *sion;

	ioncount = 0;
#if VECTORIZE
    if (vectorize) {
	if (point_process) {
		ioncount = 1;
		q = lappendstr(defs_list, "#define _nd_area  *_ppvar[_ix][0].pval\n");
		q->itemtype = VERBATIM;
	}
	ITERATE(q, useion) {
		sion = SYM(q);
		q=q->next;
		ITERATE(q1, LST(q)) {
			SYM(q1)->nrntype |= NRNIONFLAG;
			Sprintf(buf, "#define _ion_%s	*_ppvar[_ix][%d].pval\n",
				SYM(q1)->name, ioncount);
			q2 = lappendstr(defs_list, buf);
			q2->itemtype = VERBATIM;
			ioncount++;
		}
		q=q->next;
		ITERATE(q1, LST(q)) {
			if (SYM(q1)->nrntype & NRNIONFLAG) {
				SYM(q1)->nrntype &= ~NRNIONFLAG;
			}else{
				Sprintf(buf, "#define _ion_%s	*_ppvar[_ix][%d].pval\n",
					SYM(q1)->name, ioncount);
				q2 = lappendstr(defs_list, buf);
				q2->itemtype = VERBATIM;
				ioncount++;
			}
			if (iontype(SYM(q1)->name, sion->name) == IONCUR) {
Sprintf(buf, "#define _ion_di%sdv\t*_ppvar[_ix][%d].pval\n", sion->name, ioncount);
				q2 = lappendstr(defs_list, buf);
				q2->itemtype = VERBATIM;
				ioncount++;
			}
					 
		}
	}
	*p_pointercount = 0;
	ITERATE(q, nrnpointers) {
		sion = SYM(q);
		Sprintf(buf, "#define %s	*_ppvar[_ix][%d].pval\n",
			sion->name, ioncount + *p_pointercount);
		sion->used = ioncount + *p_pointercount;
		(*p_pointercount)++;
		q2 = lappendstr(defs_list, buf);
		q2->itemtype = VERBATIM;
	}

	if (diamdec) { /* must be last */
		Sprintf(buf, "#define diam	*_ppvar[_ix][%d].pval\n", ioncount);
		q2 = lappendstr(defs_list, buf);
		q2->itemtype = VERBATIM;
	} /* notice that ioncount is not incremented */
    }else
#endif
    {
	if (point_process) {
		ioncount = 1;
		q = lappendstr(defs_list, "#define _nd_area  *_ppvar[0].pval\n");
		q->itemtype = VERBATIM;
	}
	ITERATE(q, useion) {
		sion = SYM(q);
		q=q->next;
		ITERATE(q1, LST(q)) {
			SYM(q1)->nrntype |= NRNIONFLAG;
			Sprintf(buf, "#define _ion_%s	*_ppvar[%d].pval\n",
				SYM(q1)->name, ioncount);
			q2 = lappendstr(defs_list, buf);
			q2->itemtype = VERBATIM;
			ioncount++;
		}
		q=q->next;
		ITERATE(q1, LST(q)) {
			if (SYM(q1)->nrntype & NRNIONFLAG) {
				SYM(q1)->nrntype &= ~NRNIONFLAG;
			}else{
				Sprintf(buf, "#define _ion_%s	*_ppvar[%d].pval\n",
					SYM(q1)->name, ioncount);
				q2 = lappendstr(defs_list, buf);
				q2->itemtype = VERBATIM;
				ioncount++;
			}
			if (iontype(SYM(q1)->name, sion->name) == IONCUR) {
Sprintf(buf, "#define _ion_di%sdv\t*_ppvar[%d].pval\n", sion->name, ioncount);
				q2 = lappendstr(defs_list, buf);
				q2->itemtype = VERBATIM;
				ioncount++;
			}
					 
		}
	}
	*p_pointercount = 0;
	ITERATE(q, nrnpointers) {
		sion = SYM(q);
		Sprintf(buf, "#define %s	*_ppvar[%d].pval\n",
			sion->name, ioncount + *p_pointercount);
		sion->used = ioncount + *p_pointercount;
		(*p_pointercount)++;
		q2 = lappendstr(defs_list, buf);
		q2->itemtype = VERBATIM;
	}

	if (diamdec) { /* must be last */
		Sprintf(buf, "#define diam	*_ppvar[%d].pval\n", ioncount);
		q2 = lappendstr(defs_list, buf);
		q2->itemtype = VERBATIM;
	} /* notice that ioncount is not incremented */
    } /* !vectorize */
	return ioncount;
}

List *
begin_dion_stmt()
{
	Item *q, *q1, *qbrak;
	static List *l;
	char *strion;

	l = newlist();
	qbrak = lappendstr(l, "\t{");
	ITERATE(q, useion) {
		strion = SYM(q)->name;
		q = q->next;
		q = q->next;
		ITERATE(q1, LST(q)) {
			if (SYM(q1)->nrntype & NRNCUROUT) {
				Sprintf(buf, " _di%s = %s;\n",
				strion,  SYM(q1)->name);
				Lappendstr(l, buf);
				Sprintf(buf, "static double _di%s;\n", strion);
				Insertstr(qbrak->next, buf);
			}
		}
	}
	return l;
}

List *
end_dion_stmt(strdel)
	char *strdel;
{
	Item *q, *q1;
	static List *l;
	char *strion;

	l = newlist();
	ITERATE(q, useion) {
		strion = SYM(q)->name;
		q = q->next;
		q = q->next;
		ITERATE(q1, LST(q)) {
			if (SYM(q1)->nrntype & NRNCUROUT) {
Sprintf(buf, " _ion_di%sdv += (_di%s - %s)/%s",
				strion, strion, SYM(q1)->name, strdel);
				Lappendstr(l, buf);
				if (point_process) {
					Lappendstr(l, "* 1.e2/ (_nd_area);\n");
				}else{
					Lappendstr(l, ";\n");
				}
			}
		}
	}
	Lappendstr(l, "\t}\n");
	return l;
}

