%{
/* $Header: /timshel2g/src/master/nrn/src/nmodl/parse1.y,v 1.1.1.1 1994/10/12 17:21:39 hines Exp $ */
/*
$Log: parse1.y,v $
 * Revision 1.1.1.1  1994/10/12  17:21:39  hines
 * NEURON 3.0 distribution
 *
 * Revision 9.192  1994/09/26  18:20:19  hines
 * USEION ... VALENCE real
 * required when a model uses an ion other than na, k, ca
 * USEION ... WRITE cai, cao
 * promotes cai, cao to STATE
 * USEION ... READ cai, cao
 * promotes cai, cao to CONST
 *
 * Revision 9.189  1994/08/23  15:45:09  hines
 * In SOLVE procedure the VERBATIM return 0; is no longer
 * requires since all procedures have this by default.
 * In nocmodl, INDEPENDENT no longer required since if doesn't exist then
 * t (ms) automatically declared.
 *
 * Revision 9.188  1994/08/18  12:12:43  hines
 * properly put pragmas in with if... else stmts.
 * Dont use pragmas in nonvectorized part of kinetic block
 *
 * Revision 9.185  1994/07/30  17:14:21  hines
 * standard check for bison and changes to kinetic vectorization surrounded
 * by proper conditional preprocessor directives
 *
 * Revision 9.184  1994/07/26  14:04:22  hines
 * avoid objectcenter warning about retrieving int from pointer when
 * procedure is solved. The pointer in question is null and the int
 * is never used.
 *
 * Revision 9.182  1994/07/21  13:11:16  hines
 * allow vectorization of kinetic blocks on cray when using sparse method.
 * not tested yet
 *
 * Revision 9.180  1994/05/18  18:11:56  hines
 * INCLUDE "file"
 * tries originalpath/file ./file MODL_INCLUDEpaths/file
 *
 * Revision 9.174  1993/11/04  15:53:20  hines
 * port to solaris2 (no more warnings)
 *
 * Revision 9.171  1993/07/08  14:46:07  hines
 * Alternative to NONSPECIFIC_CURRENT is ELECTRODE_CURRENT
 * at this time is identical to old way. Introduced because membrane
 * mechanisms for electrode currents don't work right because the
 * current is assumed to pass from the extracellular space to the inside
 * and thus current injection does not change vext (and it is opposite in
 * sign to what one would expect). This change is not complete here. Only
 * the scaffolding is now in place and it has been verified to be identical
 * to the old way. The change is mostly to non-vectorized models and
 * now
 * void nrn_cur(Node*, double*, Datum*) so that the model gets v directly
 * from the node and changes d and rhs from the node.
 *
 * Revision 9.166  93/03/21  17:47:55  hines
 * for some reason vectorized models with functions with no args
 * were begin translated at f(ix_,)
 * 
 * Revision 9.161  93/02/15  08:57:29  hines
 * linux
 * 
 * Revision 9.159  93/02/11  16:55:46  hines
 * minor mods for NeXT
 * 
 * Revision 9.158  93/02/02  10:32:48  hines
 * create static func before usage
 * 
 * Revision 9.142  92/08/05  16:23:14  hines
 * can vectorize hh. need work on tables though.
 * 
 * Revision 9.138  92/06/01  12:04:14  hines
 * NEURON { EXTERNAL name, name, ...}
 * translates to extern double name;
 * GLOBAL changed to translate to
 * #define name name_suffix
 * extern double name
 * 
 * Revision 9.134  92/03/31  06:50:55  hines
 * CONSTANT could not be a negative number. Fixed
 * 
 * Revision 9.129  92/02/15  21:12:54  hines
 * if bison used compile with -DBISON=1
 * 
 * Revision 9.128  92/02/05  14:47:53  hines
 * saber warning free
 * FUNCTION made global for use by other models. #define GLOBFUNC 1
 * 
 * Revision 9.126  92/02/05  08:34:07  hines
 * NEURON {POINTER ...} allows a model to connect to external variables
 * #includes in nparout moved to beginning of defslist so crucial fields in
 * section structures not defined into nonsense.
 * 
 * Revision 9.120  91/09/16  15:46:13  hines
 * reorganization of how NEURON variables are declared.
 * RANGE: definitely range variables, can be param, assigned, or state
 * GLOBAL: definitely global variables, can be param or assigned
 * SECTION: not implemented
 * By default params are global variables.
 * By default states are range
 * By default  assigned are in p array but not accessible from NEURON
 * NON-SPECIFIC current not accessible unless specified in RANGE.
 * 
 * Revision 9.119  91/09/11  11:51:24  hines
 * extraneous rule in conststmt: removed
 * 
 * Revision 9.106  91/06/29  08:44:52  hines
 * more general units factor and slightly different syntax for conversion
 * 
 * Revision 9.100  91/05/06  13:27:59  hines
 * malloc not properly declared when compiling with turboc
 * 
 * Revision 9.98  91/03/18  09:11:41  hines
 * FORALL i {} loops over all elements of vectors in statements. vector
 * length must be the same.
 * 
 * Revision 9.94  91/01/24  13:59:18  hines
 * translation error when last token of LOCAL statement was the first token
 * after the LOCAL statement. Fixed by changing symbols at the parser instead
 * of the lexical analyser.
 * 
 * Revision 9.86  90/12/12  18:18:11  hines
 * FROM index : index is an integer and cast to double wherever it
 * appears in expressons. It is not cast to int whenever it appears
 * in intexpr
 * 
 * Revision 9.85  90/12/12  17:24:01  hines
 * nmodl: FROM i translates i as double  fh.mod works properly
 * 
 * Revision 9.82  90/12/12  09:01:15  hines
 * LOCAL arrays allowed
 * 
 * Revision 9.79  90/12/11  10:55:27  hines
 * TABLE works with hocmodl
 * 
 * Revision 9.78  90/12/10  16:56:46  hines
 * TABLE allowed in FUNCTION and PROCEDURE
 * 
 * Revision 9.76  90/12/07  09:27:21  hines
 * new list structure that uses unions instead of void *element
 * 
 * Revision 9.64  90/11/26  08:16:11  hines
 * external functions of type EXTDEF3 get two extra arguments.
 * The first arg is a pointer to _reset which should be set by the
 * function upon the occurence of a discontinuity.
 * The second arg is a pointer to a double which may be used by the
 * function to store information to help determine if a discontinuity
 * has occurred.  init.c gives a list of these functions.
 * 
 * Revision 9.63  90/11/23  15:07:19  hines
 * factor calculation using units.c of model
 * 
 * Revision 9.58  90/11/20  17:24:12  hines
 * CONSTANT changed to PARAMETER
 * CONSTANT now refers to variables that don't get put in .var file
 * 
 * Revision 9.57  90/11/20  16:04:21  hines
 * EQUATION changed to BREAKPOINT
 * STEADYSTATE keyword added, EQUATION keyword still exists
 * 
 * Revision 9.56  90/11/15  16:15:39  hines
 * unit parentheses for numbers and functions were not being removed.
 * 
 * Revision 9.55  90/11/15  16:04:54  hines
 * units definition syntax is (unit) (unit)
 * 
 * Revision 9.54  90/11/13  15:48:38  hines
 * allow function definitions and function definition arguments to
 * have units specifications. Also numbers can have units specifications.
 * 
 * Revision 9.52  90/11/10  15:44:58  hines
 * nmodl: uses new NEURON { USEION ... format
 * passive.c works
 * 
 * Revision 9.50  90/11/02  08:31:50  hines
 * Allow UNITS block and make a  units defintion scale factor a static
 * variable so that it doesn't have to appear in the var file. eg.
 * UNITS { FARADAY = 96520 (coul) }
 * 
 * Revision 9.48  90/11/02  06:59:32  hines
 * new way of specifying NEURON info. Keywords are
 * NEURON SUFFIX CURRENT NEED GET SET
 * 
 * Revision 9.29  90/09/10  14:09:16  hines
 * certain functions specified in extdef2.h can have arguments which are
 * pointers to functions and pointers to vectors.
 * 
 * Revision 9.25  90/08/30  11:27:46  hines
 * units computations removed
 * 
 * Revision 9.17  90/08/07  08:43:56  hines
 * functions and procedures interfaced to NMODL and HMODL
 * 
 * Revision 9.10  90/07/27  13:58:08  hines
 * nmodl handles declarations about right for first pass at this.
 * 
 * Revision 9.5  90/07/18  07:59:44  hines
 * define for arrays now (p + n) instead of &p[n]. This allows the c file
 * to have arrays that look like a[i] instead of *(a + i).
 * 
 * Revision 9.4  90/06/04  08:22:15  mlh
 * extend exponents from both expressions dimensionless to
 * allowing exponent to be positive integer.
 * 
 * Revision 9.3  90/06/01  11:02:00  mlh
 * checks for conformability of units but does not check for
 * correct conversion factors.
 * in x^y, both x and y must be dimensionless
 * 
 * Revision 9.2  90/05/30  17:01:07  mlh
 * not working yet, just adding files administratively
 * 
 * Revision 8.4  90/01/18  11:46:29  mlh
 * SOLVEFOR statement added
 * syntax is 
 *   blocktype blockname [SOLVEFOR name, name, ...] {statment}
 * where blocktype is KINETIC, NONLINEAR, or LINEAR
 * Only the states in the SOLVEFOR statement are solved
 * in that block. If the statement is not present then
 * all states are solved that appear in that block.
 * 
 * Revision 8.3  90/01/16  11:06:12  mlh
 * error checking and cleanup after error and call to abort_run()
 * 
 * Revision 8.2  89/11/01  10:18:31  mlh
 * Warning given if state is assigned a value in a DERIVATIVE block
 * 
 * Revision 8.1  89/10/25  13:48:12  mlh
 * PUTQ and GETQ implemented
 * 
 * Revision 8.0  89/09/22  17:26:51  nfh
 * Freezing
 * 
 * Revision 7.3  89/09/07  07:38:27  mlh
 * was failing to prevent multiple declarations involving the SWEEP variable
 * 
 * Revision 7.2  89/09/05  08:44:57  mlh
 * COMPARTMENT index no longer gives undefined variable warning
 * 
 * Revision 7.1  89/09/05  08:09:31  mlh
 * COMPARTMENT keyword for KINETIC block
 * COMPARTMENT size {scalar state list}
 * COMPARTMENT index, size(index) {vector state list}
 * 
 * Revision 7.0  89/08/30  13:32:23  nfh
 * Rev 7 is now Experimental; Rev 6 is Testing
 * 
 * Revision 6.0  89/08/14  16:27:07  nfh
 * Rev 6.0 is latest of 4.x; now the Experimental version
 * 
 * Revision 4.2  89/08/11  09:56:44  mlh
 * simultaneous nonlinear equations allowed in DERIVATIVE block
 * 
 * Revision 4.1  89/07/31  07:55:20  mlh
 * Slight problem with error messages. Now missing partial equation in PARTIAL
 * block says DEL2 instead of DEL
 * 
 * Revision 4.0  89/07/24  17:03:37  nfh
 * Freezing rev 3.  Rev 4 is now Experimental
 * 
 * Revision 3.2  89/07/18  11:55:13  mlh
 * first_time removed and MODEL_LEVEL used for declaration precedence
 * 
 * Revision 1.3  89/07/18  09:50:50  mlh
 * variable declaration precedence in submodels handled with
 * MODEL_LEVEL level prefix generated by merge. Lowest level takes
 * precedence.
 * 
 * Revision 1.2  89/07/07  08:52:14  mlh
 * START field in INDEPENDENT block
 * SWEEP keyword replaces SCOP in INDEPENDENT block
 * FIRST and LAST keywords in PARTIAL block for boundaries rplace index
 * y'0 etc. allowed for constants
 * 
 * Revision 1.1  89/07/06  14:45:58  mlh
 * Initial revision
 * 
*/
#include "modl.h"

#if __TURBOC__ || NeXT
extern void *malloc(), *realloc();
#else
extern char *malloc(), *realloc();
#endif

#if defined(__STDC__)
#define sdebug(arg1,arg2) {}
#define qdebug(arg1,arg2) {}
#else
#define sdebug(arg1,arg2) {Fprintf(stderr,"arg1:%s\n", arg2); Fflush(stderr);}
#define qdebug(arg1,arg2) {Item *q; Fprintf(stderr,"arg1:");Fflush(stderr);\
	for (q=arg2; q->type != 0; q=q->next){\
		if (q->type == SYMBOL)\
			Fprintf(stderr,"%s\n", SYM(q)->name);\
		else if (q->type == STRING)\
			Fprintf(stderr,"%s\n", STR(q);\
		else\
			Fprintf(stderr,"Illegal Item type\n");\
		Fflush(stderr);}\
		Fprintf(stderr,"\n");Fflush(stderr);}
#endif

#define ldebug(arg1, arg2) qdebug(arg1, arg2->next)

#if BISON
#define myerr(arg) static int ierr=0;\
if (!(ierr++))yyerror(arg); --yyssp; --yyvsp; YYERROR
#else
#define myerr(arg) static int ierr=0;\
if (!(ierr++))yyerror(arg); --yyps; --yypv; YYERROR
#endif

extern char *modelline;
extern List *firstlist; /* NAME symbols in order that they appear in file */
extern int lexcontext; /* ':' can return 3 different tokens */
extern List *solveforlist; /* List of symbols that are actually to be solved
				for in a block. See in_solvefor() */
static int stateblock; /* 0 if dependent, 1 if state */
static int blocktype;
static int inequation; /* inside an equation?*/
static int nstate;	/* number of states seen in an expression */
static int leftside;	/* inside left hand side of equation? */
static int pstate;	/* number of state in a primary expression */
static int tstate;	/* number of states in a term */
static Item *lastok;	/* last token accepted by expr */
static int sensused;	/* a SENS statement occurred in this block */
static Symbol *matchindex; /* local symbol for implied MATCH loop */
static int model_level = 0; /* the model level prepended to declaration
				blocks by merge */
static int scopindep = 0;/* SCoP independent explicitly declared if 1 */
static int extdef2 = 0; /* flag that says we are in an EXTDEF2 function */
static List *table_list = LIST0; /* table information for TABLE statement */
static int forallindex = 0;	/* 0 not in FORALL, -1 just starting, 
					>0 index of arrays used (must all
					be the same */
static Item* astmt_end_;	/* see kinetic.c vectorizing */
%}

%union {
	Item	*qp;
	char	*str;
	List	*lp;
	int	i;
}

%token	<qp>	VERBATIM COMMENT MODEL CONSTANT INDEPENDENT DEPENDENT STATE
%token	<qp>	INITIAL1 DERIVATIVE SOLVE USING WITH STEPPED DISCRETE
%token	<qp>	FROM FORALL1 TO BY WHILE IF ELSE START1 STEP SENS SOLVEFOR
%token	<qp>	PROCEDURE PARTIAL DEL DEL2 DEFINE1 IFERROR PARAMETER
%token	<qp>	DERFUNC EQUATION TERMINAL LINEAR NONLINEAR FUNCTION1 LOCAL
%token	<qp>	METHOD LIN1 NONLIN1 PUTQ GETQ TABLE DEPEND BREAKPOINT
%token	<qp>	INCLUDE1
%token	<qp>	'{' '}' '(' ')' '[' ']' '@' '+' '*' '-' '/' '=' '^' ':' ','
%token	<qp>	'~'
%token	<qp>	OR AND GT LT LE EQ NE NOT
%token	<qp>	NAME PRIME REAL INTEGER DEFINEDVAR
%type	<qp>	Name NUMBER real intexpr integer
%token	<qp>	STRING PLOT VS LAG RESET MATCH MODEL_LEVEL SWEEP FIRST LAST
%type	<str>	line model units optindex unit
%type	<qp>	name number nonlineqn primary term linexpr numlist
%type	<qp>	expr ostmt astmt stmtlist locallist locallist1
%type	<qp>	varname exprlist define1 queuestmt
%type	<qp>	asgn fromstmt whilestmt ifstmt solveblk funccall ifsolerr
%type	<qp>	opinc opstart senslist sens lagstmt forallstmt
%type	<qp>	parmasgn stepped indepdef depdef
%type	<qp>	matchblk matchlist match matchname pareqn firstlast

%token	<qp>	KINETIC CONSERVE REACTION REACT1 COMPARTMENT UNITS
%token	<qp>	UNITSON UNITSOFF
%type	<qp>	reaction conserve react compart namelist unitblk
%type	<qp>	solvefor solvefor1 uniton

%type	<lp>	tablst tablst1 dependlst arglist arglist1
%type	<i>	locoptarray
/* interface to NEURON */
%token	<qp>	NEURON SUFFIX NONSPECIFIC READ WRITE USEION VALENCE
%token	<qp>	GLOBAL SECTION RANGE POINTER EXTERNAL
%token	<qp>	ELECTRODE_CURRENT
%type	<qp>	neuronblk nrnuse nrnlist valence

/* precedence in expressions--- low to high */
%left   OR
%left   AND
%left   GT GE LT LE EQ NE
%left   '+' '-' /* left associative, same precedence */
%left   '*' '/' '%'/* left assoc., higher precedence */
%left   UNARYMINUS NOT
%right  '^'     /* exponentiation */

%%
top: 	all {/*ldebug(top, intoken)*/;}
	|error {diag("Illegal block", (char *)0);}
	;
all:	/*nothing*/
	| all model
	| all locallist
		/* move the declarations into firstlist */
		{replacstr($2, "static double");
		movelist($2, lastok->next, firstlist);}
	| all define1
	| all declare
	| all MODEL_LEVEL INTEGER {model_level = atoi(STR($3));}
		declare {model_level = 0;}
	| all 
		{if (sensused)
			diag("sensitivity analysis not implemented for",
				" this block type");
		}
	    proc
	| all VERBATIM 
		/* read everything and move as is to end of procfunc */
		{inblock(SYM($2)->name); replacstr($2, "\n/*VERBATIM*/\n");
		movelist($2,intoken->prev, procfunc);}
	| all COMMENT
		/* read everything and delete */
		{inblock(SYM($2)->name); deltokens($2, intoken->prev);}
	| all uniton
	| all INCLUDE1 STRING
		{include_file($3);}
	;
model: MODEL line 
		{if (modelline == NULL) modelline = $2;
			deltokens($1, intoken->prev);}
	;
line:	{$$ = inputline();}
	;
define1: DEFINE1 NAME INTEGER
		/* all subsequent occurences of NAME will be replaced
			by integer during parseing.  See 'integer:' */
	{ Symbol *sp = SYM($2);
	 if (sp->subtype)
		diag(sp->name, " used before DEFINEed");
	 sp->u.str = STR($3);
	 sp->type = DEFINEDVAR;
	 deltokens($1, $3);}
	| DEFINE1 error {myerr("syntax: DEFINE name integer");}
	;
Name:	NAME
		{ Symbol *checklocal();
		  SYM($1) = checklocal(SYM($1));  /* it was a bug
			when this was done to the lookahead token in lex */
		}
	;
declare: parmblk | indepblk | depblk | stateblk | stepblk
	| plotdecl | neuronblk | unitblk | constblk
	;
parmblk: PARAMETER '{' parmbody '}' {deltokens($1, $4);}
	;
parmbody: /*nothing*/
	| parmbody parmasgn
		{ explicit_decl(model_level, $2);}
/*	| parmbody stepped */ /*now has its own block*/
	;
parmasgn: NAME '=' number units
		/* install in syminorder and put info in Symbol->u.str
			Note that usage is EXPLICIT_DECL */
		{parminstall(SYM($1), STR($3), $4);}
	| NAME units
		{parminstall(SYM($1), "0", $2);}
	| error {diag("name = number", (char *)0);}
	;
units:	/*nothing*/ {$$ = stralloc("", (char *)0);}
	| unit
	;
unit:	'(' {$<str>$ = inputtopar();} /*string*/ ')'
		/*does not include parentheses*/
		 {$$ = $<str>2; delete($1); delete($3);}
	;
uniton: UNITSON {replacstr($1, "");}
	| UNITSOFF {replacstr($1, "");}
	;
stepblk: STEPPED '{' stepbdy '}' {deltokens($1, $4);}
	;
stepbdy: /*nothing*/
	| stepbdy stepped
		{ explicit_decl(model_level, $2); }
	;
stepped: NAME '=' numlist units
		/* install */
		{steppedinstall(SYM($1), $3, lastok, $4);}
	;
numlist: number ',' number
	| numlist ',' number
	;
name:	Name
	| PRIME
	;
number: NUMBER	{lastok = $1;}
	| '-' NUMBER
		/* replace the string with -string and discard '-'*/
                { Sprintf(buf, "-%s", STR($2));
		 STR($2) = stralloc(buf, STR($2)); $$ = $2;
		delete($1); lastok = $2;
		}
 	;
NUMBER: integer | REAL
	;
integer: INTEGER
	| DEFINEDVAR {replacstr($1, SYM($1)->u.str);}
	;
real:	REAL {lastok = $1;}
	| integer	/* add a .0 to the string */
		{Sprintf(buf, "%s.0", STR($1));
		STR($1) = stralloc(buf, STR($1));
		lastok = $1;
		}
	;
indepblk: INDEPENDENT '{' indepbody '}' {deltokens($1, $4);}
	;
indepbody: /*nothing*/
	| indepbody indepdef
		{ explicit_decl(model_level, $2); }
	| indepbody SWEEP {scopindep = 1;} indepdef
		{ explicit_decl(model_level, $4);
		  scopindep = 0;
		}
	;
indepdef: NAME FROM number TO number withby integer opstart units
		/*indepinstall()*/
		{indepinstall(SYM($1), STR($3),
		 STR($5), STR($7), $8, $9, scopindep);
		}
	| error {diag("name FROM number TO number WITH integer\n", (char *)0);}
	;
withby:	WITH
	;
depblk:	DEPENDENT {stateblock = 0;} '{' depbody '}' {deltokens($1, $5);}
	;
depbody: /*nothing*/
	| depbody depdef
		{ explicit_decl(model_level, $2);}
	;
depdef: name opstart units
		/*depinstall()*/
		{depinstall(stateblock, SYM($1), 0, "0", "1", $3, $2, 1);
		 }
	| name '[' integer ']' opstart units
		{int i = atoi(STR($3));
		 if (i < 1) diag("Array index must be > 0", (char *)0);
		 depinstall(stateblock, SYM($1), i,
			 "0", "1", $6, $5, 1);
		}
	| name FROM number TO number opstart units
		{depinstall(stateblock, SYM($1), 0, STR($3),
		 STR($5), $7, $6, 1);
		}
	| name '[' integer ']' FROM number TO number opstart units
		{int i = atoi(STR($3));
		 if (i < 1) diag("Array index must be > 0", (char *)0);
		 depinstall(stateblock, SYM($1), i,
		 STR($6), STR($8), $10, $9, 1);
		}
	| error {
diag("name FROM number TO number START number\n",
"FROM...TO and START are optional, name can be name[integer]\n");}
	;
opstart: /*nothing*/ {$$ = ITEM0;}
	| START1 number {$$ = $2;}
	;
stateblk: STATE  {stateblock = 1;} '{' depbody '}' {deltokens($1, $5);}
	;
plotdecl: PLOT pvlist VS name optindex
		/* construct plotlist. Used in parout.c */
		{ Item *q;
		 q = linsertsym(plotlist, SYM($4)); Insertstr(q->next, $5); }
		
	| PLOT error { diag("PLOT namelist VS name", (char *)0);}
	;
pvlist:	name optindex
		{ if (plotlist->next == plotlist) {
			Lappendsym(plotlist, SYM($1));
			Lappendstr(plotlist, $2);
		  }else{
			diag("Only one PLOT declaration allowed", (char *)0);
		  }
		}
	| pvlist ',' name optindex
		{ Lappendsym(plotlist, SYM($3)); Lappendstr(plotlist, $4);}
	;
optindex: /*nothing*/
		{ $$ = "-1";}
	| '[' INTEGER ']'
		{ $$ = STR($2);}
	;
proc:	 {blocktype = INITIAL1;} initblk
		/*blocktype set prior to parsing*/
	| {lexcontext = NONLINEAR; blocktype = DERIVATIVE;} derivblk
	| {blocktype = BREAKPOINT;} brkptblk
	| {lexcontext = blocktype = LINEAR;} linblk
	| {lexcontext = blocktype = NONLINEAR;} nonlinblk
	| {blocktype = FUNCTION1;} funcblk
	| {blocktype = PROCEDURE;} procedblk
	| {blocktype = TERMINAL;} terminalblk
	| {blocktype = DISCRETE;} discretblk
	| {lexcontext = blocktype = PARTIAL;} partialblk
	| {lexcontext = blocktype = KINETIC;ostmt_start();} kineticblk {see_ostmt();}
	;
initblk: INITIAL1 stmtlist '}'
		{movelist($2, $3, initfunc);}
	;
stmtlist: '{' {pushlocal();} stmtlist1 {poplocal();}
	| '{' locallist stmtlist1
		{poplocal();}
	;
locallist: LOCAL locallist1
		{ replacstr($1, "double");
		  Insertstr(lastok->next, ";\n");
		}
	| LOCAL error {myerr("Illegal LOCAL declaration");}
	;
locallist1: NAME locoptarray
		/* locals are placed in a stack of symbol lists and given
			the prefix _l */
		{pushlocal();
		 SYM($1) = copylocal(SYM($1));
		 if ($2) {
			SYM($1)->araydim = $2;
			SYM($1)->subtype |= ARRAY;
		 }else{
			lastok = $1;
		 }
		}
	| locallist1 ',' NAME locoptarray
		{SYM($3) = copylocal(SYM($3));
		 if ($4) {
			SYM($3)->araydim = $4;
			SYM($3)->subtype |= ARRAY;
		 }else{
			lastok = $3;
		 }
		}
	;
locoptarray: /*nothing*/
		{$$ = 0;}
	| '[' integer ']'
		{$$ = atoi(STR($2)); lastok = $3;}
	;
stmtlist1: /*nothing*/
	| stmtlist1 {if (blocktype == KINETIC) see_ostmt();} ostmt
		{if (blocktype == KINETIC) see_ostmt();}
	| stmtlist1 astmt {if (blocktype == KINETIC) { see_astmt($2, astmt_end_); }}
	;
ostmt:	fromstmt
	| forallstmt
	| whilestmt
	| ifstmt
	| stmtlist '}'
	| solveblk
	| VERBATIM 
		{inblock(SYM($1)->name);
		replacstr($1, "\n/*VERBATIM*/\n");}
		
	| COMMENT
		{inblock(SYM($1)->name); deltokens($1, intoken->prev);}
	| sens
	| compart	{check_block(KINETIC, blocktype, "COMPARTMENT");}
	| conserve	{check_block(KINETIC, blocktype, "CONSERVE");}
	| lagstmt
	| queuestmt
	| RESET
		{ replacstr($1, " _reset = 1;\n"); }
	| matchblk
	| pareqn	/* 2nd order partial equation and boundary conditions*/
	| tablestmt
		{if (blocktype !=FUNCTION1 && blocktype != PROCEDURE) {
			diag("TABLE valid only for FUNCTION or PROCEDURE", (char *)0);
		}}
	| uniton
	| error
		{myerr("Illegal statement");}
	;	
astmt:	asgn
		/* ';' is added when relevant */
		{astmt_end_ = insertsym(lastok->next, semi);}
	| {inequation = 1;} reaction {
		$$ = $2; inequation = 0;
		astmt_end_ = insertstr(lastok->next->next->next, "");}
	| funccall
		{astmt_end_ = insertsym(lastok->next, semi);}
	;

asgn:	varname '=' expr
		/* depending on blocktype, varname may get marked as used. */
		{ if (blocktype == DERIVATIVE && SYM($1)->type == PRIME) {
			/* put Dvar in a derivative used list */
			deriv_used(SYM($1));
			}
		  if (blocktype == DERIVATIVE && (SYM($1)->subtype & STAT)) {
			Fprintf(stderr,
"WARNING: %s (a STATE) is assigned a value\
 in a DERIVATIVE block.\n Multistep integrators (such as Runge) may not\
 work correctly.\n", SYM($1)->name);
		  }
		  if (blocktype == DISCRETE && SYM($1)->type == NAME
			&& (SYM($1)->subtype & STAT)) {
			SYM($1)->used++;
			}
		}
	| nonlineqn expr '=' expr
		/* put info in equation list */
		{ inequation = 0;
		Insertstr($2, " -(");
		replacstr($3, ") + ");
		if (nstate == 0)
{yyerror("previous equation contains no state variables"); YYERROR;}
		 eqnqueue($1);
		}
	| lineqn leftlinexpr '=' linexpr
		{ inequation = 0;
		delete($3);
		if (nstate == 0)
{yyerror("previous equation contains no state variables"); YYERROR;}
		}
	;
varname: name
		/* much marking depending on context */
		{lastok = $1;
		if (!extdef2){SYM($1)->usage |= DEP;}
		if (SYM($1)->subtype & ARRAY && !extdef2)
			{myerr("variable needs an index");}
		if (inequation && (SYM($1)->subtype & STAT) && in_solvefor(SYM($1))) {
			SYM($1)->used++;
			nstate++; pstate++; tstate++;
		}
		if (SYM($1)->subtype & INTGER) {
			lastok = insertstr($1->next, ")");
			$1 = insertstr($1, "((double)");
		}
		}
	| name '[' intexpr ']'
		{lastok = $4;
		SYM($1)->usage |= DEP;
		if ((SYM($1)->subtype & ARRAY) == 0)
			{myerr("variable is not an array");}
		if (inequation && (SYM($1)->subtype & STAT) && in_solvefor(SYM($1))) {
			SYM($1)->used++;
			nstate++; pstate++; tstate++;
		}
		  if (forallindex) {
			if (forallindex == -1) {
				forallindex = SYM($1)->araydim;
			}
			if (forallindex != SYM($1)->araydim) {
Sprintf(buf, "%s dimension not same as other dimensions used in FORALL statement",
SYM($1)->name);
				diag(buf, (char *)0);
			}
		  }
		}
	| NAME '@' integer
		{lastok = $3;
		SYM($1)->usage |= DEP; disc_var_seen($1, $2, $3, 0);}
	| NAME '@' integer '[' intexpr ']' 
		{lastok = $6;
		SYM($1)->usage |= DEP; disc_var_seen($1, $2, $3, ARRAY);}
	;
intexpr: Name
		{lastok = $1;
		 SYM($1)->usage |= DEP;
		 if (!(SYM($1)->subtype & INTGER)) {
		 	lastok = insertstr($1->next, ")");
			$1 = insertstr($1, "((int)");
		 }
		}
	| integer		{ lastok = $1;}
	| '(' intexpr ')'	{ lastok = $3;}
	| intexpr '+' intexpr
	| intexpr '-' intexpr
	| intexpr '*' intexpr
	| intexpr '/' intexpr
	| error {myerr("Illegal integer expression");}
	;
expr:	varname
	| real units
	| funccall
	| '(' expr ')'	{lastok = $3;}
	| expr '+' expr
	| expr '-' expr
	| expr '*' expr
	| expr '/' expr
	| expr '^' expr	/* converted to pow() */
		{ $$ = insertstr($1, "pow("); replacstr($2, ",");
			lastok = insertstr(lastok->next, ")"); }
	| expr OR expr {replacstr($2, " ||");}
	| expr AND expr {replacstr($2, " &&");}
	| expr GT expr
	| expr LT expr
	| expr GE expr
	| expr LE expr
	| expr EQ expr {replacstr($2, " ==");}
	| expr NE expr {replacstr($2, " !=");}
	| NOT expr	 {replacstr($1, " !");}
	| '-' expr %prec UNARYMINUS
	| error {myerr("Illegal expression");}
	;
nonlineqn: NONLIN1 {inequation = 1; nstate = 0;}
	;
lineqn: LIN1
		/* initialize a bunch of equation info */
		{inequation = 1; nstate = 0;
		pstate = 0; tstate = 0; init_lineq($1); leftside = -1;}
	;
leftlinexpr: linexpr {leftside = 1;}
	;
linexpr: primary
		/* put terms in a list */
		{linterm($1, lastok, pstate, leftside); pstate = 0;}
	| '-' primary
		{delete($1);
		linterm($2, lastok, pstate, -leftside); pstate = 0;}
	| linexpr '+' primary
		{delete($2);
		linterm($3, lastok, pstate, leftside); pstate = 0;}
	| linexpr '-' primary
		{delete($2);
		linterm($3, lastok, pstate, -leftside); pstate = 0;}
	;
primary: term
		{ if (tstate == 1) {
			lin_state_term($1, lastok);
		  }
		  tstate = 0;
		}
	| primary '*' term
		{ if (tstate == 1) {
			lin_state_term($3, lastok);
		  }
		  tstate = 0;
		}
	| primary '/' term
		{ if (tstate) {
			diag("state ocurs in denominator", (char *)0);
		  }
		}
	;
term:	varname
	| real
	| funccall {if (tstate) diag("states not permitted in function calls",
			(char *)0);}
	| '(' expr ')' { lastok = $3;
			if (tstate) diag("states not permitted between ",
				"parentheses");}
	| error
{diag("Some operators are not permitted in linear\n",
"expressions unless the terms containing them are\nenclosed in parentheses");}
	;
funccall: NAME '('
		{ if (SYM($1)->subtype & EXTDEF2) { extdef2 = 1;}}
	   exprlist ')'
		{lastok = $5; SYM($1)->usage |= FUNCT;
		 if (SYM($1)->subtype & EXTDEF2) { extdef2 = 0;}
		 if (SYM($1)->subtype & EXTDEF3) { add_reset_args($2);}
#if VECTORIZE
		 vectorize_use_func($1,$2,$4,$5);
#endif
		}
	;
exprlist: /*nothing*/{$$ = ITEM0;}
	| expr
	| STRING
	| exprlist ',' expr
	| exprlist ',' STRING
	;
fromstmt: FROM NAME {pushlocal(); SYM($2) = copylocal(SYM($2));
			SYM($2)->subtype |= INTGER;}
	'=' intexpr TO intexpr opinc stmtlist '}'
		/* translate the whole thing using _lNAME as an integer */
		{ replacstr($1, "{int ");
		poplocal();
		Insertstr($4, ";for (");
		Insertstr($4, SYM($2)->name);
		Insertstr($6, ";");
		Insertstr($6, SYM($2)->name);
		replacstr($6, "<=");
		if ($8) {
			Insertstr($8, ";");
			Insertstr($8, SYM($2)->name);
			replacstr($8, "+=");
		} else {
			Insertstr($9, ";");
			Insertstr($9, SYM($2)->name);
			Insertstr($9, "++");
		}
		Insertstr($9, ")");
		Insertstr($10, "}");
		}
	|FROM error {
myerr("FROM intvar = intexpr TO intexpr BY intexpr { statements }");}
	;
opinc: /*nothing*/ {$$ = ITEM0;}
	| BY intexpr
	;
forallstmt: FORALL1 NAME {pushlocal(); SYM($2) = copylocal(SYM($2));
			  SYM($2)->subtype |= INTGER;
			  if (forallindex) {
diag("Nested FORALL statements not allowed", (char *)0);
			  }
			  forallindex = -1;
			}
	stmtlist '}'
		/* translate the whole thing using _lNAME as an integer */
		{ replacstr($1, "{int ");
		poplocal();
		if (forallindex == -1) {
diag("FORALL range is undefined since no arrays used", " within the statement");
		}
		Sprintf(buf, "; for (%s=0; %s<%d; %s++)", SYM($2)->name,
			SYM($2)->name, forallindex, SYM($2)->name);
		Insertstr($4, buf);
		Insertstr($5, "}");
		}
	|FORALL1 error {
myerr("FORALL intvar { statements }");}
	;
whilestmt: WHILE '(' expr ')' stmtlist '}'
	;
ifstmt:	IF '(' expr ')' stmtlist '}' optelseif optelse
	;
optelseif: /*nothing*/
	| optelseif ELSE IF '(' expr ')' stmtlist '}'
		{
#if VECTORIZE
			vectorize_if_else_stmt(blocktype);
#endif
		}
	;
optelse: /*nothing*/
	| ELSE stmtlist '}'
		{
#if VECTORIZE
			vectorize_if_else_stmt(blocktype);
#endif
		}
	;
derivblk: DERIVATIVE NAME stmtlist '}'
		/* process using massagederiv() */
		{massagederiv($1, $2, $3, $4, sensused); sensused = 0;}
	;
linblk: LINEAR NAME solvefor {init_linblk($2);} stmtlist '}'
		/*massage_linblk()*/
		{massage_linblk($1, $2, $5, $6, sensused);
		lexcontext = 0; sensused = 0;
		}
	;
nonlinblk: NONLINEAR NAME solvefor stmtlist '}'
		/*massagenonlin()*/
		{massagenonlin($1, $2, $4, $5, sensused);
		lexcontext = 0; sensused = 0;
		}
	;
discretblk: DISCRETE NAME stmtlist '}'
		 /*massagediscblk()*/
		{massagediscblk($1, $2, $3, $4);}
	;
partialblk: PARTIAL NAME stmtlist '}'
		/*massagepartial()*/
		{massagepartial($1, $2, $3, $4);
		lexcontext = 0;
		}
	| PARTIAL error {
diag("within the PARTIAL block must occur an equation with the syntax ---\n",
"~ V' = F*DEL2(V) + G\n"); }
	;
pareqn: '~' PRIME '=' NAME '*' DEL2 '(' NAME ')' '+' NAME
		{partial_eqn($2, $4, $8, $11);}
	| '~' DEL NAME '[' firstlast ']' '=' expr
		{partial_bndry(0, $3, $5, $8, lastok);}
	| '~' NAME '[' firstlast ']' '=' expr
		{partial_bndry(2, $2, $4, $7, lastok);}
	;
firstlast: FIRST | LAST
	;
funcblk: FUNCTION1 NAME '(' arglist ')' units
		{IGNORE(copylocal(SYM($2)));}
	stmtlist '}'
		/* boilerplate added to form double function(){...}
		   Note all arguments have prefix _l */
		{ Symbol *s = SYM($2);
		table_massage(table_list, $1, $2, $4); freelist(&table_list);
#define GLOBFUNCT 1
#if GLOBFUNCT && NMODL
		replacstr($1, "\ndouble");
#else
		replacstr($1, "\nstatic double");
#endif
		defarg($3, $5);
		Sprintf(buf, "double _l%s;\n", s->name);
		Insertstr($8->next, buf);
		Sprintf(buf, "\nreturn _l%s;\n", s->name);
		Insertstr($9, buf);
		movelist($1, $9, procfunc);
		if (SYM($2)->subtype & FUNCT) {
			diag(SYM($2)->name, " declared as FUNCTION twice");
		}
		SYM($2)->subtype |= FUNCT;
		SYM($2)->usage |= FUNCT;
#if HMODL || NMODL
		hocfunc(s, $3, $5);
#endif
		poplocal(); freelist(&$4);}
	;
arglist: /*nothing*/ {pushlocal(); $$ = LIST0;}
	| {pushlocal();} arglist1 {$$ = $2;}
	;
arglist1: name units
		{SYM($1) = copylocal(SYM($1));
		 $$ = newlist(); Lappendsym($$, SYM($1));
		}
	| arglist1 ',' name units
		{SYM($3) = copylocal(SYM($3)); Lappendsym($$, SYM($3));}
	;
procedblk: PROCEDURE NAME '(' arglist ')' units stmtlist '}'
		{Symbol *s = SYM($2);
		s->u.i = 0; 	/* avoid objectcenter warning if solved */
		table_massage(table_list, $1, $2, $4); freelist(&table_list);
		replacstr($1, "\nstatic int "); defarg($3, $5);
		Insertstr($8, " return 0;");
		movelist($1, $8, procfunc);
		if (SYM($2)->subtype & PROCED) {
			diag(SYM($2)->name, " declared as PROCEDURE twice");
		}
		SYM($2)->subtype |= PROCED;
		SYM($2)->usage |= FUNCT;
#if HMODL || NMODL
		hocfunc(s, $3, $5);
#endif
		poplocal(); freelist(&$4);}
	;
solveblk: SOLVE NAME ifsolerr
		/* no processing, merely save in solve list */
		{ solvequeue($2, ITEM0, blocktype, $3); }
	| SOLVE NAME USING METHOD ifsolerr
		{ solvequeue($2, $4, blocktype, $5); }
	| SOLVE error { myerr("Illegal SOLVE statement");}
	;
ifsolerr: /*nothing*/
		{ $$ = ITEM0; }
	| IFERROR stmtlist '}'
		{ $$ = $3; }
	;
solvefor: /*nothing*/
		{ if (solveforlist) {
			freelist(&solveforlist);
		  }
		}
	| solvefor1
	;
solvefor1: SOLVEFOR NAME
		{ solveforlist = newlist(); Lappendsym(solveforlist, SYM($2));
		  delete($1); delete($2);
		}
	| solvefor1 ',' NAME
		{ Lappendsym(solveforlist, SYM($3)); delete($2); delete($3);}
	| SOLVEFOR error { myerr("Syntax: SOLVEFOR name, name, ...");}
	;
brkptblk: BREAKPOINT stmtlist '}'
		/* move it all to modelfunc */
		{movelist($2, $3, modelfunc);}
	;
terminalblk: TERMINAL stmtlist '}'
		{movelist($2, $3, termfunc);}
	;
sens:	SENS senslist
		/* sensused flag is true */
		{ sensused = 1;
		  delete($1);
		}
	|SENS error {myerr("syntax is SENS var1, var2, var3, etc");}
	;
senslist: varname
		/* put in list. Used when enclosing block is massaged */
		{ sensparm($1); delete($1);}
	| senslist ',' varname
		{ sensparm($3); deltokens($2, $3);}
	;

conserve: CONSERVE react '=' expr
		{/* react originally designed for reactions and is unchanged*/
		massageconserve($1, $3, lastok);}
	| CONSERVE error {myerr("Illegal CONSERVE syntax");}
	;
compart: COMPARTMENT NAME ','
	 {pushlocal(); SYM($2) = copylocal(SYM($2));}
	 expr '{' namelist '}'
		{massagecompart($5, $6, $8, SYM($2)); poplocal();}
	| COMPARTMENT expr '{' namelist '}'
		{massagecompart($2, $3, $5, SYM0);}
	;
namelist: NAME
	| namelist NAME
	;
kineticblk: KINETIC NAME solvefor stmtlist '}'
		/* put all reactions in a list for processing when relevant
		   SOLVE is processed */
		{massagekinetic($1, $2, $4, $5, sensused);
		lexcontext = 0; sensused = 0;
		}
	;
reaction: REACTION react REACT1 {leftreact();} react '(' expr ',' expr ')'
		{massagereaction($1,$3,$6,$8,$10);}
	| REACTION react LT LT  '(' expr ')'
		{flux($1, $3, $7);}
	| REACTION react '-' GT '(' expr ')'
		{flux($1, $3, $7);}
	| REACTION error {myerr("Illegal reaction syntax");}
	;
react:	varname {reactname($1, lastok, ITEM0);}
	|integer varname	{reactname($2, lastok, $1);}
	|react '+' varname	{reactname($3, lastok, ITEM0);}
	|react '+' integer varname {reactname($4, lastok, $3);}
	;
lagstmt: LAG name BY NAME
		{lag_stmt($1, blocktype);}
	| LAG error {myerr("Lag syntax is: LAG name BY const");}
	;
queuestmt: PUTQ name {queue_stmt($1, $2);}
	| GETQ name {queue_stmt($1, $2);}
	;
matchblk: MATCH {checkmatch(blocktype);} '{' matchlist '}'
		{deltokens($1, $5);}
	;
matchlist: match
	| matchlist match
	;
match:	name
		{matchinitial($1);}
	| matchname '(' expr ')' '=' expr
		{ matchbound($1, $2, $4, $6, lastok, matchindex);
		  if (matchindex) {
			poplocal();
			matchindex = SYM0;
		  }
		}
	| error
		{myerr("MATCH syntax is state0 or state(expr)=expr or\
state[i](expr(i)) = expr(i)");}
	;
matchname: name
		{matchindex = SYM0;}
	| name '[' NAME ']'
		{ pushlocal();
		  matchindex = copylocal(SYM($3));
		}
	;
unitblk: UNITS '{' unitbody '}'
		{ deltokens($1,$4);}
	;
unitbody: /*nothing*/
		{modl_units();}
	| unitbody unitdef
	| unitbody factordef
	;
unitdef: unit '=' unit
		{install_units($1, $3);}
	| unit error {myerr("Unit definition syntax: (units) = (units)");}
	;
factordef: NAME '=' real unit
		{ SYM($1)->subtype |= CONST;
		  Sprintf(buf, "static double %s = %s;\n", SYM($1)->name,
			STR($3));
		  Lappendstr(firstlist, buf);
		}
	| NAME '=' unit unit
		{Item *q; double d, unit_mag();
		    Unit_push($3);
			Unit_push($4); unit_div();
		    SYM($1)->subtype |= CONST;
		    Sprintf(buf, "static double %s = %g;\n", SYM($1)->name,
			unit_mag());
		    Lappendstr(firstlist, buf);
		    unit_pop();
		}
	| NAME '=' unit '-' GT unit 
		{ double unit_mag();
		    Unit_push($3); Unit_push($6); unit_div();
		    SYM($1)->subtype |= CONST;
		    Sprintf(buf, "static double %s = %g;\n", SYM($1)->name,
			unit_mag());
		    Lappendstr(firstlist, buf);
		    unit_pop();
		}
	| error {myerr("Unit factor syntax: examples:\n\
foot2inch = (foot) -> (inch)\n\
F = 96520 (coulomb)\n\
R = (k-mole) (joule/degC)");
		}
	;
constblk: CONSTANT '{' conststmt '}'
	;
conststmt: /*nothing*/
	| conststmt NAME '=' number units
		{ SYM($2)->subtype |= CONST;
		  Sprintf(buf, "static double %s = %s;\n", SYM($2)->name,
			STR($4));
		  Lappendstr(firstlist, buf);
		}
	;
tablestmt: TABLE tablst dependlst FROM expr TO expr WITH INTEGER
		{ Item *q;
		  if (table_list) {
			diag("only one TABLE per function", (char *)0);
		  }
		  table_list = newlist();
		  Lappendlst(table_list, $2);
		  q = lappendlst(table_list, newlist());
		  movelist($4->next, $6->prev, LST(q));
		  q = lappendlst(table_list, newlist());
		  movelist($6->next, $8->prev, LST(q));
		  Lappendstr(table_list, STR($9));
		  Lappendlst(table_list, $3);
		  deltokens($1, $9);
		}
	| TABLE error { myerr("syntax: TABLE [list] [DEPEND list] FROM expr TO expr WITH integer");}
	;
tablst: /*Nothing*/
		{$$ = LIST0;}
	| tablst1
	;
tablst1: Name
		{$$ = newlist(); Lappendsym($$, SYM($1));}
	| tablst1 ',' Name
		{ Lappendsym($1, SYM($3));}
	;
dependlst: /*NOTHING*/
		{$$ = LIST0;}
	| DEPEND tablst1
		{$$ = $2;}
	;
neuronblk: NEURON '{' nrnstmt '}'
		{ deltokens($1,$4);}
	;
nrnstmt: /*nothing*/
	| nrnstmt SUFFIX NAME
		{ nrn_list($2, $3);}
	| nrnstmt nrnuse
	| nrnstmt NONSPECIFIC nrnlist
		{ nrn_list($2,$3);}
	| nrnstmt ELECTRODE_CURRENT nrnlist
		{ nrn_list($2,$3);}
	| nrnstmt SECTION nrnlist
		{ nrn_list($2, $3);}
	| nrnstmt RANGE nrnlist
		{ nrn_list($2, $3);}
	| nrnstmt GLOBAL nrnlist
		{ nrn_list($2, $3);}
	| nrnstmt POINTER nrnlist
		{ nrn_list($2, $3);}
	| nrnstmt EXTERNAL nrnlist
		{ nrn_list($2, $3);}
	;
nrnuse: USEION NAME READ nrnlist valence
		{nrn_use($2, $4, ITEM0, $5);}
	|USEION NAME WRITE nrnlist valence
		{nrn_use($2, ITEM0, $4, $5);}
	|USEION NAME READ nrnlist WRITE nrnlist valence
		{nrn_use($2, $4, $6, $7);}
	| error
		{myerr("syntax is: USEION ion READ list WRITE list");}
	;
nrnlist: NAME
	| nrnlist ',' NAME
		{ delete($2); $$ = $3;}
	| error
		{myerr("syntax is: keyword name , name, ..., name");}
	;
valence: /*nothing*/
		{$$ = ITEM0;}
	| VALENCE real
		{$$ = $2;}
	;
%%
yyerror(s)	/* called for yacc syntax error */
	char *s;
{
	Fprintf(stderr, "%s:\n ", s);
}


#if !NMODL
nrn_list(q1, q2)
	Item *q1, *q2;
{
	/*ARGSUSED*/
}
nrn_use(q1, q2, q3)
	Item *q1, *q2, *q3;
{
	/*ARGSUSED*/
}
#endif
