/* $Header: /timshel2g/src/master/nrn/src/nmodl/solve.c,v 1.1.1.1 1994/10/12 17:21:37 hines Exp $ */
/*
$Log: solve.c,v $
 * Revision 1.1.1.1  1994/10/12  17:21:37  hines
 * NEURON 3.0 distribution
 *
 * Revision 9.182  1994/07/21  13:11:16  hines
 * allow vectorization of kinetic blocks on cray when using sparse method.
 * not tested yet
 *
 * Revision 9.178  1994/05/02  12:48:35  hines
 * on abort with NOCMODL call nrn_complain(_p) to tell location where
 * failure occurred (only SOLVE statement) and print line and .mod file
 *
 * Revision 9.159  1993/02/11  16:55:47  hines
 * minor mods for NeXT
 *
 * Revision 9.157  93/02/01  15:18:17  hines
 * static functions should be declared before use.
 * inline is keyword for some compilers.
 * 
 * Revision 9.144  92/08/06  09:59:16  hines
 * put cray pragmas at appropriate places
 * get rid of initmodel references to saveing t and incrementing ninit
 * 
 * Revision 9.142  92/08/05  16:23:16  hines
 * can vectorize hh. need work on tables though.
 * 
 * Revision 9.140  92/07/27  11:34:13  hines
 * some bugs fixed. no vectorizing except for SOLVE procedure (and that not done yet)
 * 
 * Revision 9.128  92/02/05  14:47:55  hines
 * saber warning free
 * FUNCTION made global for use by other models. #define GLOBFUNC 1
 * 
 * Revision 9.99  91/04/17  16:15:14  hines
 * DISCRETE block integrates by delta_indepvar instead of 1.0.
 * Also delta_indepvar not changed automatically and  model() may return
 * at indepvar at value different from request if break not an integral
 * value of delta_indepvar.
 * 
 * Revision 9.95  91/01/29  09:15:12  hines
 * nmodl doesn't create modl_set_dt because multiple mechanisms would make
 * multiple declarations.
 * 
 * Revision 9.87  90/12/13  11:24:48  hines
 * calculation of _nstep cast to (int)
 * 
 * Revision 9.76  90/12/07  09:27:28  hines
 * new list structure that uses unions instead of void *element
 * 
 * Revision 9.71  90/11/30  08:22:21  hines
 * modl_set_dt should only be created for time dependent models
 * 
 * Revision 9.68  90/11/28  09:11:37  hines
 * location where we are checking for STADYSTATE may not be a SYMBOL
 * 
 * Revision 9.62  90/11/23  10:32:41  hines
 * steadystate in BREAKPOINT block had failed to allocate delta_t
 * 
 * Revision 9.61  90/11/23  10:01:48  hines
 * STEADYSTATE of kinetic and derivative blocks
 * 
 * Revision 9.58  90/11/20  17:24:19  hines
 * CONSTANT changed to PARAMETER
 * CONSTANT now refers to variables that don't get put in .var file
 * 
 * Revision 9.57  90/11/20  16:04:24  hines
 * EQUATION changed to BREAKPOINT
 * STEADYSTATE keyword added, EQUATION keyword still exists
 * 
 * Revision 9.47  90/11/01  15:20:30  hines
 * SOLVE procedure allowed. The procedure needs to return an error code
 * with a VERBATIM statement
 * 
 * Revision 9.46  90/10/30  14:29:20  hines
 * calc_dt is gone. Not needed because due to scopfit, dt needs to be
 * calculated at every break point.
 * _reset no longer used to say that dt has changed. The integrator will
 * have to check that itself.
 * 
 * Revision 9.41  90/10/30  08:37:11  hines
 * saber warning free except for ytab.c and lex.c
 * 
 * Revision 9.31  90/10/08  11:34:04  hines
 * simsys prototype
 * 
 * Revision 9.28  90/09/09  17:28:27  hines
 * merge the summer's work at nbsr into modl
 * 
 * Revision 9.13  90/07/30  14:37:19  hines
 * NMODL looks pretty good. Ready to start testing. Have not yet tried
 * to compile the .c file.
 *
 * Revision 8.10  90/07/30  14:54:40  jmk
 * Corrected calculation of delta-indepvariable to work correctly with
 * varying breakpoint interval (e.g. SCoPfit).
 * 
 * Revision 9.11  90/07/30  11:51:16  hines
 * NMODL getting better, almost done
 * 
 * Revision 8.9  90/04/09  08:41:39  mlh
 * implicit method for derivative blocks (allows mixed equations also).
 * The solve statement must precede the derivative block
 * 
 * Revision 8.8  90/01/23  16:10:35  mlh
 * delta_t is calculated for the first two calls after
 * a call to initmodel().  This is because the breakpoint
 * interval can change aftera the first call to model().
 * The implementation is accomplished by _calc_dt=2
 * in initmodel() and _calc_dt-- when _calc_dt is nonzero
 * in model()
 * 
 * Revision 8.7  90/01/19  07:37:39  mlh
 * _modl_cleanup() called by abort_run()
 * 
 * Revision 8.6  90/01/16  11:06:21  mlh
 * error checking and cleanup after error and call to abort_run()
 * 
 * Revision 8.5  89/11/21  08:17:41  mlh
 * More reliable method for ensuring that solve for loop ends up exactly
 * at the final time value.
 * 
 * Revision 8.4  89/11/21  07:37:08  mlh
 * _calc_dt used in scopcore and therefore must be declared even for
 * time independent models.
 * 
 * Revision 8.3  89/11/14  16:37:56  mlh
 * _reset set to true whenever initmodel is called and whenever _calc_dt
 * is true
 * 
 * Revision 8.2  89/11/13  15:16:49  mlh
 * _called_init changed to _calc_dt and made global so
 * that scop can cause dt to be recomputed on an
 * extend run command
 * 
 * Revision 8.1  89/10/25  13:42:55  mlh
 * Bug with discrete models in that delta_indep is used
 * Fixed by generating the proper line only when type is DERF
 * 
 * Revision 8.0  89/09/22  17:27:00  nfh
 * Freezing
 * 
 * Revision 7.2  89/09/07  07:45:36  mlh
 * was failing to initialize time after match
 * many bugs in handling exact for loop of t to the break point
 * 
 * Revision 7.1  89/09/05  08:04:47  mlh
 * Ensure that there are an integer number of steps / break
 * 
 * Revision 7.0  89/08/30  13:32:37  nfh
 * Rev 7 is now Experimental; Rev 6 is Testing
 * 
 * Revision 6.0  89/08/14  16:27:17  nfh
 * Rev 6.0 is latest of 4.x; now the Experimental version
 * 
 * Revision 4.0  89/07/24  17:03:47  nfh
 * Freezing rev 3.  Rev 4 is now Experimental
 * 
 * Revision 3.3  89/07/21  09:26:41  mlh
 * Discrete equitions evaluated at time given by independent variable
 * in the sense that the state on the left hand side refers to state(t)
 * and explicit dependence on t works naturally.
 * 
 * Revision 1.4  89/07/21  09:16:14  mlh
 * Discrete equations evaluated at time given by ndependent variable
 * in the sense that state on the left hand side refers to state(t)
 * and explicit dependence on t works naturally.  This is done by
 * incrementing t before calling discrete block and updating the
 * history at the beginning instead of the end.
 * 
 * Revision 1.3  89/07/18  11:22:36  mlh
 * eliminate first_time, etc.
 * 
 * Revision 1.2  89/07/07  08:55:05  mlh
 * START field in INDEPENDENT block
 * SWEEP keyword replaces SCOP in INDEPENDENT block
 * FIRST and LAST keywords in PARTIAL block for boundaries rplace index
 * y'0 etc. allowed for constants
 * 
 * Revision 1.1  89/07/06  14:51:04  mlh
 * Initial revision
 * 
*/

#include "modl.h"
#include "ytab.h"
#include "symbol.h"

/* make it an error if 2 solve statements are called on a single call to
model() */
extern List *indeplist;
Symbol *deltaindep = SYM0;
extern Symbol *indepsym;
extern char* current_line();
#if NMODL
extern List *nrnstate;
#if VECTORIZE
extern int vectorize;
extern char* cray_pragma();
#endif
#endif
static whileloop();
static check_ss_consist();

/* Need list of solve statements. We impress the
general list structure to handle it.  The element is a pointer to an
item which is the first item in the statement sequence in another list.
*/
static List *solvq;	/* list of the solve statement locations */
int numlist = 0;	/* number of slist's */

solvequeue(q1, q2, blocktype, qerr) /*solve NAME=q1 [using METHOD=q2]*/
				/* q2 = 0 means method wasn't there */
				/* qerr in ITEM0 or else the closing
				   brace of an IFERROR stmt */
	Item *q1, *q2, *qerr;
	int blocktype;
{
	/* the solvq list is organized in groups of an item element
	followed by the method symbol( null if default to be used) */
	/* The itemtype field of the first is used to carry the blocktype*/
	/* SOLVE and METHOD method are deleted */
	/* The list now consists of triples in which the third element
	   is a list containing the complete IFERROR statement.
	*/
	Item *lq, *qtemp;
	List *errstmt;

	if (!solvq) {
		solvq = newlist();
	}
#if NMODL
	/* if the blocktype is equation then move the solve statement to
		the nrnstate function.  Everything else stays in the
		model function to be used as the nrncurrent function */
	if (blocktype == BREAKPOINT) {
		if (!nrnstate) {
			nrnstate = newlist();
		}
		Lappendstr(nrnstate, "{");
		if (qerr) {
			movelist(q1->prev, qerr, nrnstate);
		}else if (q2) {
			movelist(q1->prev, q2, nrnstate);
		}else{
			movelist(q1->prev, q1, nrnstate);
		}
		Lappendstr(nrnstate, "}");
	}
#endif
	lq = lappendsym(solvq, SYM0);
	ITM(lq) = q1;
	lq->itemtype = blocktype;
	/* handle STEADYSTATE option */
	if (q1->next->itemtype == SYMBOL &&
	    strcmp("STEADYSTATE", SYM(q1->next)->name) == 0) {
		lq->itemtype = -blocktype; /* gets put back below */
	}
	if (q2) {
		qtemp = q2->next;	/* The IFERROR location */
		Lappendsym(solvq, SYM(q2));
		if (strcmp(SYM(q2)->name, "derivimplicit") == 0) {
			add_deriv_imp_list(SYM(q1)->name);
		}
		delete(q2->prev);
		delete(q2);
	}else{
		qtemp = q1->next;
		Lappendsym(solvq, SYM0);
	}
	delete(q1->prev);

	/* handle the error statement */
	/* put one in if it isn't already there */
	if (qerr == ITEM0) {
#if NOCMODL
sprintf(buf, "if(error){fprintf(stderr,\"%s\\n\"); nrn_complain(_p); abort_run(error);}\n",
current_line());
		qtemp = qerr = insertstr(qtemp, buf);
#else
		qtemp = qerr = insertstr(qtemp,
"if(error){abort_run(error);}\n");
#endif
	}else{
		replacstr(qtemp, "if (error)");
	}
	errstmt = newlist();
	lq = lappendsym(solvq, SYM0);
	LST(lq) = errstmt;
	movelist(qtemp, qerr, errstmt);		
}
			
/* go through the solvq list and construct the proper while loop and calls*/
solvhandler()
{
	Item *lq, *qsol, *follow;
	List *errstmt;
	Symbol *fun, *method;
	int numeqn, listnum, btype, steadystate;
	
	if (!solvq)
		solvq = newlist();
	init_disc_vars(); /*why not here?*/
	ITERATE(lq, solvq) { /* remember it goes by 3's */
		steadystate=0;
		btype = lq->itemtype;
		if (btype < 0) {
			btype = lq->itemtype = -btype;
			steadystate = 1;
			check_ss_consist(lq);
		}
		qsol = ITM(lq);
		lq = lq->next;
		method = SYM(lq);
		lq = lq->next;
		errstmt = LST(lq);
		/* err stmt handling assumes qsol->next is where it goes. */
		fun = SYM(qsol);
		numeqn = fun->used;
		listnum =fun->u.i;
		follow = qsol->next; /* where p[0] gets updated */
		/* Check consistency of method and block type */
		if (method && !(method->subtype & fun->subtype)) {
			Sprintf("Method %s can't be used with Block %s",
				method->name, fun->name);
			diag(buf, (char *)0);
		}
		
		switch (fun->subtype) {

		case DERF:
#if VECTORIZE
			vectorize = 0;
#endif
			if (method == SYM0) {
				method = lookup("adrunge");
			}
			if (btype == BREAKPOINT && !steadystate) {
				/* derivatives recalculated after while loop */
				Sprintf(buf, " %s();\n", fun->name);
				Insertstr(follow, buf);
				/* envelope calls go after the while loop */
				sens_nonlin_out(follow, fun);
			}
			if (btype == BREAKPOINT) whileloop(qsol, (long)DERF, steadystate);
			solv_diffeq(qsol, fun,  method, numeqn, listnum, steadystate, btype);
			break;
		case KINF:
			if (method == SYM0) {
				method = lookup("_advance");
			}
			if (btype == BREAKPOINT && (method->subtype & DERF)) {
#if VECTORIZE
				vectorize = 0;
#endif
				/* derivatives recalculated after while loop */
				Sprintf(buf, " %s();\n", fun->name);
				Insertstr(follow, buf);
				/* envelope calls go after the while loop */
				sens_nonlin_out(follow, fun);
			}
			if (btype == BREAKPOINT) whileloop(qsol, (long)DERF, steadystate);
			solv_diffeq(qsol, fun,  method, numeqn, listnum, steadystate, btype);
			break;
		case NLINF:
#if VECTORIZE
			vectorize = 0;
#endif
			if (method == SYM0) {
				method = lookup("newton");
			}
			solv_nonlin(qsol, fun, method, numeqn, listnum);
			break;
		case LINF:
#if VECTORIZE
			vectorize = 0;
#endif
			if (method == SYM0) {
				method = lookup("simeq");
			}
			solv_lineq(qsol, fun, method, numeqn, listnum);
			break;
		case DISCF:
#if VECTORIZE
			vectorize = 0;
#endif
			if (btype == BREAKPOINT) whileloop(qsol, (long)DISCRETE, 0);
			Sprintf(buf, "0; %s += delta_%s; %s();\n",
				indepsym->name, indepsym->name, fun->name);
			replacstr(qsol, buf);
			break;
#if 1	/* really wanted? */
		case PROCED:
			if (btype == BREAKPOINT) whileloop(qsol, (long)DERF, 0);
			Sprintf(buf, " %s();\n", fun->name);
			replacstr(qsol, buf);
#if VECTORIZE
	Sprintf(buf, "%s for (_ix = 0; _ix < _count; ++_ix) { %s(_ix); }\n",
		cray_pragma(), fun->name);
	vectorize_substitute(qsol, buf);
#endif
			break;
#endif
		case PARF:
#if VECTORIZE
			vectorize = 0;
#endif
			if (btype == BREAKPOINT) whileloop(qsol, (long)DERF, 0);
			solv_partial(qsol, fun);
			break;
		default:
			diag("Illegal or unimplemented SOLVE type: ", fun->name);
			break;
		}
	/* add the error check */
	Insertstr(qsol, "error =");
	move(errstmt->next, errstmt->prev, qsol->next);
#if VECTORIZE
	if (errstmt->next == errstmt->prev) {
		vectorize_substitute(qsol->next, "");
		vectorize_substitute(qsol->prev, "");
	}else{
		vectorize = 0;
	}
#endif
	freelist(&errstmt);
	/* under all circumstances, on return from model,
	 p[0] = current indepvar */
	/* obviously ok if indepvar is original; if it has been changed
	away from time
	then _sav_indep will be reset to starting value of original when
	initmodel is called on every call to model */
#if NMODL
#else
		if (btype == BREAKPOINT) {
#if SIMSYS
			Sprintf(buf, "_sav_indep = %s;\n", indepsym->name);
#else
			Sprintf(buf, "_sav_indep = _p[_indepindex];\n");
#endif
			Insertstr(follow, buf);
		}
#endif
	}
}

save_dt(q)	/* save and restore the value of indepvar */
	Item *q;
{
	/*ARGSUSED*/
#if 0	/* integrators no longer increment time */
	static int first=1;

	if (first) {
		first = 0;
		Linsertstr(procfunc, "double _savlocal;\n");
	}
	Sprintf(buf, "_savlocal = %s;\n", indepsym->name);
	Insertstr(q, buf);
	Sprintf(buf, "%s = _savlocal;\n", indepsym->name);
	Insertstr(q->next, buf);
#endif
}

char *saveindep = "";

static
whileloop(qsol, type, ss)
	Item *qsol;
	long type;
	int ss;
{
/* no solve statement except this is allowed to
change the indepvar. Time passed to integrators
must therefore be saved and restored. */
/* Note that when the user changes the independent variable within Scop,
the while loop still uses the original independent variable. In this case
1) _break must be set to the entry value of the original independent variable
2) initmodel() must be called after each entry to model()
3) in initmodel() we are very careful not to destroy the entry value of
   time-- on exit it has its original value.  Note that _sav_indep gets
   any value of time that is set within initmodel().
*/
/* executing more that one for loop in a single call to model() is an error
which is trapped in scop */
	static int called = 0, firstderf = 1;
	char *cp;

	switch (type) {
	case DERF:
	case DISCRETE:
		if (firstderf) { /* time dependent process so create a deltastep */
			double d[3];
#ifndef NeXT			
			double atof();
#endif
			int i;
			Item *q;
			char sval[30];
			Sprintf(buf, "delta_%s", indepsym->name);
			for (i=0, q=indeplist->next; i<3; i++, q=q->next) {
				d[i] = atof(STR(q));
			}
			if (type == DERF) {
				Sprintf(sval, "%g", (d[1]-d[0])/d[2]);
			}else if (type == DISCRETE) {
				Sprintf(sval, "1.0");
			}
			deltaindep = ifnew_parminstall(buf, sval, "");
			firstderf = 0;
#if NMODL
#else
Sprintf(buf, "_modl_set_dt(_dt) double _dt; { %s = _dt;}\n",
		deltaindep->name);
			Lappendstr(procfunc, buf);
#endif
		}
		if (type == DERF) {
			cp = deltaindep->name;
		}else if (type == DISCRETE) {
			cp = "0.0";
		}
		if (ss) {
			return;
		}
		break;
	default:
		/*SUPPRESS 622*/
		assert(0);
	}
#if NMODL
	if (strcmp(indepsym->name, "t") != 0) {
		diag("The independent variable name must be `t'", (char *)0);
	}
#else
	Sprintf(buf, "_save = _break = %s; %s = _sav_indep;\n",
		indepsym->name, indepsym->name);
	Insertstr(qsol, buf);
#if !SIMSYS
	Sprintf(buf, "if (_p + _indepindex != &%s) {initmodel(_pp); %s = _sav_indep;}\n",
		indepsym->name, indepsym->name);
	Insertstr(qsol, buf);
#endif
#endif

	if (called) {
Fprintf(stderr, "Warning: More than one integrating SOLVE statement in an \
BREAKPOINT block.\nThe simulation will be incorrect if more than one is used \
at a time.\n");
	}
#if NMODL
	Insertstr(qsol, "{\n");
#else
	Sprintf(buf, "if (%s < _break) {\n",indepsym->name);
	Insertstr(qsol, buf);

/* ensure that there are an integer number of steps / break */
if (type == DERF) {
	Sprintf(buf, " { int _nstep; double _dt, _y;\n\
	_y = _break - %s; _dt = %s;\n", indepsym->name, deltaindep->name);
 	Insertstr(qsol, buf);
 	Insertstr(qsol, "_nstep = (int)(_y/_dt + .9);\n if (_nstep==0) _nstep = 1;\n");
 	Sprintf(buf, "%s = _y/((double)_nstep);\n", deltaindep->name);
 	Insertstr(qsol, buf);
	Sprintf(buf, "\n  }\n");
	Insertstr(qsol, buf);
}
 
	if (type == DERF) {
		Sprintf(buf, "_break -= .5* %s;\n", deltaindep->name);
		Insertstr(qsol, buf);
	}
#endif
	Sprintf(buf, "for (; %s < _break; %s += %s) {\n",
		indepsym->name,
		indepsym->name, cp);
	Insertstr(qsol, buf);
	/* close the while loop; note that integrators have been called */
	if (type == DERF) {
		Sprintf(buf, "\n}}\n %s = _save;\n", indepsym->name);
	}else if (type == DISCRETE) {
		Sprintf(buf, "\n}}\n");
	}		
	Insertstr(qsol->next,buf);
	if (!called) {
		/* fix up initmodel as per 3) above.
		In cout.c _save is declared */
		Sprintf(buf, "%s0", indepsym->name);
		IGNORE(ifnew_parminstall(buf, STR(indeplist->prev->prev),
			STR(indeplist->prev)));
		Sprintf(buf, " _save = %s;\n %s = %s0;\n",
			indepsym->name, indepsym->name, indepsym->name);
		saveindep = stralloc(buf, (char *)0);
		/* Assert no more additions to initfunc involving
		the value of time */
		Sprintf(buf, " _sav_indep = %s; %s = _save;\n",
			indepsym->name, indepsym->name);
		Lappendstr(initfunc, buf);
#if VECTORIZE
		vectorize_substitute(initfunc->prev, "");
#endif
	}
	called++;
}

/* steady state consistency means that KINETIC blocks whenever solved must
invoke same integrator (default is advance) and DERIVATIVE blocks whenever
solved must invoke the derivimplicit method.
*/
static
check_ss_consist(qchk)
	Item *qchk;
{
	Item *q;
	Symbol *fun, *method;
	
	ITERATE(q, solvq) {
		fun = SYM(ITM(q));
		if (fun == SYM(qchk)) {
			method= SYM(q->next);
			switch (q->itemtype) {
			case DERF:			
				if (!method || 
				   strcmp("derivimplicit", method->name)!=0) {
diag("STEADYSTATE requires all SOLVE's of this DERIVATIVE block to use the\n\
`derivimplicit' method:", fun->name);
				}
				break;
			case KINF:
				if (SYM(qchk->next) != method
				   || (method &&
				       strcmp("sparse", method->name) != 0)){
diag("STEADYSTATE requires all SOLVE's of this KINETIC block to use the\n\
same method (`advance' or `sparse'). :", fun->name);
				}
				break;
			default:
diag("STEADYSTATE only valid for SOLVEing a KINETIC or DERIVATIVE block:", fun->name);
				break;
			}
		}
		q = q->next->next;
	}
}
