#if defined(__TURBOC__) || NeXT 
extern void *malloc(), *calloc(), *realloc();
#else
extern char *malloc(), *calloc(), *realloc();
#endif


# line 2 "parse1.y"
/* $Header: /timshel2g/src/master/nrn/src/nmodl/ytab.c,v 1.2 1994/10/12 20:55:38 hines Exp $ */
/*
$Log: ytab.c,v $
 * Revision 1.2  1994/10/12  20:55:38  hines
 * first pass at neuron under cvs control
 *
 * Revision 1.1.1.1  1994/10/12  17:21:39  hines
 * NEURON 3.0 distribution
 *
 * Revision 9.192  1994/09/26  18:20:19  hines
 * USEION ... VALENCE real
 * required when a model uses an ion other than na, k, ca
 * USEION ... WRITE cai, cao
 * promotes cai, cao to STATE
 * USEION ... READ cai, cao
 * promotes cai, cao to CONST
 *
 * Revision 9.189  1994/08/23  15:45:09  hines
 * In SOLVE procedure the VERBATIM return 0; is no longer
 * requires since all procedures have this by default.
 * In nocmodl, INDEPENDENT no longer required since if doesn't exist then
 * t (ms) automatically declared.
 *
 * Revision 9.188  1994/08/18  12:12:43  hines
 * properly put pragmas in with if... else stmts.
 * Dont use pragmas in nonvectorized part of kinetic block
 *
 * Revision 9.185  1994/07/30  17:14:21  hines
 * standard check for bison and changes to kinetic vectorization surrounded
 * by proper conditional preprocessor directives
 *
 * Revision 9.184  1994/07/26  14:04:22  hines
 * avoid objectcenter warning about retrieving int from pointer when
 * procedure is solved. The pointer in question is null and the int
 * is never used.
 *
 * Revision 9.182  1994/07/21  13:11:16  hines
 * allow vectorization of kinetic blocks on cray when using sparse method.
 * not tested yet
 *
 * Revision 9.180  1994/05/18  18:11:56  hines
 * INCLUDE "file"
 * tries originalpath/file ./file MODL_INCLUDEpaths/file
 *
 * Revision 9.174  1993/11/04  15:53:20  hines
 * port to solaris2 (no more warnings)
 *
 * Revision 9.171  1993/07/08  14:46:07  hines
 * Alternative to NONSPECIFIC_CURRENT is ELECTRODE_CURRENT
 * at this time is identical to old way. Introduced because membrane
 * mechanisms for electrode currents don't work right because the
 * current is assumed to pass from the extracellular space to the inside
 * and thus current injection does not change vext (and it is opposite in
 * sign to what one would expect). This change is not complete here. Only
 * the scaffolding is now in place and it has been verified to be identical
 * to the old way. The change is mostly to non-vectorized models and
 * now
 * void nrn_cur(Node*, double*, Datum*) so that the model gets v directly
 * from the node and changes d and rhs from the node.
 *
 * Revision 9.166  93/03/21  17:47:55  hines
 * for some reason vectorized models with functions with no args
 * were begin translated at f(ix_,)
 * 
 * Revision 9.161  93/02/15  08:57:29  hines
 * linux
 * 
 * Revision 9.159  93/02/11  16:55:46  hines
 * minor mods for NeXT
 * 
 * Revision 9.158  93/02/02  10:32:48  hines
 * create static func before usage
 * 
 * Revision 9.142  92/08/05  16:23:14  hines
 * can vectorize hh. need work on tables though.
 * 
 * Revision 9.138  92/06/01  12:04:14  hines
 * NEURON { EXTERNAL name, name, ...}
 * translates to extern double name;
 * GLOBAL changed to translate to
 * #define name name_suffix
 * extern double name
 * 
 * Revision 9.134  92/03/31  06:50:55  hines
 * CONSTANT could not be a negative number. Fixed
 * 
 * Revision 9.129  92/02/15  21:12:54  hines
 * if bison used compile with -DBISON=1
 * 
 * Revision 9.128  92/02/05  14:47:53  hines
 * saber warning free
 * FUNCTION made global for use by other models. #define GLOBFUNC 1
 * 
 * Revision 9.126  92/02/05  08:34:07  hines
 * NEURON {POINTER ...} allows a model to connect to external variables
 * #includes in nparout moved to beginning of defslist so crucial fields in
 * section structures not defined into nonsense.
 * 
 * Revision 9.120  91/09/16  15:46:13  hines
 * reorganization of how NEURON variables are declared.
 * RANGE: definitely range variables, can be param, assigned, or state
 * GLOBAL: definitely global variables, can be param or assigned
 * SECTION: not implemented
 * By default params are global variables.
 * By default states are range
 * By default  assigned are in p array but not accessible from NEURON
 * NON-SPECIFIC current not accessible unless specified in RANGE.
 * 
 * Revision 9.119  91/09/11  11:51:24  hines
 * extraneous rule in conststmt: removed
 * 
 * Revision 9.106  91/06/29  08:44:52  hines
 * more general units factor and slightly different syntax for conversion
 * 
 * Revision 9.100  91/05/06  13:27:59  hines
 * malloc not properly declared when compiling with turboc
 * 
 * Revision 9.98  91/03/18  09:11:41  hines
 * FORALL i {} loops over all elements of vectors in statements. vector
 * length must be the same.
 * 
 * Revision 9.94  91/01/24  13:59:18  hines
 * translation error when last token of LOCAL statement was the first token
 * after the LOCAL statement. Fixed by changing symbols at the parser instead
 * of the lexical analyser.
 * 
 * Revision 9.86  90/12/12  18:18:11  hines
 * FROM index : index is an integer and cast to double wherever it
 * appears in expressons. It is not cast to int whenever it appears
 * in intexpr
 * 
 * Revision 9.85  90/12/12  17:24:01  hines
 * nmodl: FROM i translates i as double  fh.mod works properly
 * 
 * Revision 9.82  90/12/12  09:01:15  hines
 * LOCAL arrays allowed
 * 
 * Revision 9.79  90/12/11  10:55:27  hines
 * TABLE works with hocmodl
 * 
 * Revision 9.78  90/12/10  16:56:46  hines
 * TABLE allowed in FUNCTION and PROCEDURE
 * 
 * Revision 9.76  90/12/07  09:27:21  hines
 * new list structure that uses unions instead of void *element
 * 
 * Revision 9.64  90/11/26  08:16:11  hines
 * external functions of type EXTDEF3 get two extra arguments.
 * The first arg is a pointer to _reset which should be set by the
 * function upon the occurence of a discontinuity.
 * The second arg is a pointer to a double which may be used by the
 * function to store information to help determine if a discontinuity
 * has occurred.  init.c gives a list of these functions.
 * 
 * Revision 9.63  90/11/23  15:07:19  hines
 * factor calculation using units.c of model
 * 
 * Revision 9.58  90/11/20  17:24:12  hines
 * CONSTANT changed to PARAMETER
 * CONSTANT now refers to variables that don't get put in .var file
 * 
 * Revision 9.57  90/11/20  16:04:21  hines
 * EQUATION changed to BREAKPOINT
 * STEADYSTATE keyword added, EQUATION keyword still exists
 * 
 * Revision 9.56  90/11/15  16:15:39  hines
 * unit parentheses for numbers and functions were not being removed.
 * 
 * Revision 9.55  90/11/15  16:04:54  hines
 * units definition syntax is (unit) (unit)
 * 
 * Revision 9.54  90/11/13  15:48:38  hines
 * allow function definitions and function definition arguments to
 * have units specifications. Also numbers can have units specifications.
 * 
 * Revision 9.52  90/11/10  15:44:58  hines
 * nmodl: uses new NEURON { USEION ... format
 * passive.c works
 * 
 * Revision 9.50  90/11/02  08:31:50  hines
 * Allow UNITS block and make a  units defintion scale factor a static
 * variable so that it doesn't have to appear in the var file. eg.
 * UNITS { FARADAY = 96520 (coul) }
 * 
 * Revision 9.48  90/11/02  06:59:32  hines
 * new way of specifying NEURON info. Keywords are
 * NEURON SUFFIX CURRENT NEED GET SET
 * 
 * Revision 9.29  90/09/10  14:09:16  hines
 * certain functions specified in extdef2.h can have arguments which are
 * pointers to functions and pointers to vectors.
 * 
 * Revision 9.25  90/08/30  11:27:46  hines
 * units computations removed
 * 
 * Revision 9.17  90/08/07  08:43:56  hines
 * functions and procedures interfaced to NMODL and HMODL
 * 
 * Revision 9.10  90/07/27  13:58:08  hines
 * nmodl handles declarations about right for first pass at this.
 * 
 * Revision 9.5  90/07/18  07:59:44  hines
 * define for arrays now (p + n) instead of &p[n]. This allows the c file
 * to have arrays that look like a[i] instead of *(a + i).
 * 
 * Revision 9.4  90/06/04  08:22:15  mlh
 * extend exponents from both expressions dimensionless to
 * allowing exponent to be positive integer.
 * 
 * Revision 9.3  90/06/01  11:02:00  mlh
 * checks for conformability of units but does not check for
 * correct conversion factors.
 * in x^y, both x and y must be dimensionless
 * 
 * Revision 9.2  90/05/30  17:01:07  mlh
 * not working yet, just adding files administratively
 * 
 * Revision 8.4  90/01/18  11:46:29  mlh
 * SOLVEFOR statement added
 * syntax is 
 *   blocktype blockname [SOLVEFOR name, name, ...] {statment}
 * where blocktype is KINETIC, NONLINEAR, or LINEAR
 * Only the states in the SOLVEFOR statement are solved
 * in that block. If the statement is not present then
 * all states are solved that appear in that block.
 * 
 * Revision 8.3  90/01/16  11:06:12  mlh
 * error checking and cleanup after error and call to abort_run()
 * 
 * Revision 8.2  89/11/01  10:18:31  mlh
 * Warning given if state is assigned a value in a DERIVATIVE block
 * 
 * Revision 8.1  89/10/25  13:48:12  mlh
 * PUTQ and GETQ implemented
 * 
 * Revision 8.0  89/09/22  17:26:51  nfh
 * Freezing
 * 
 * Revision 7.3  89/09/07  07:38:27  mlh
 * was failing to prevent multiple declarations involving the SWEEP variable
 * 
 * Revision 7.2  89/09/05  08:44:57  mlh
 * COMPARTMENT index no longer gives undefined variable warning
 * 
 * Revision 7.1  89/09/05  08:09:31  mlh
 * COMPARTMENT keyword for KINETIC block
 * COMPARTMENT size {scalar state list}
 * COMPARTMENT index, size(index) {vector state list}
 * 
 * Revision 7.0  89/08/30  13:32:23  nfh
 * Rev 7 is now Experimental; Rev 6 is Testing
 * 
 * Revision 6.0  89/08/14  16:27:07  nfh
 * Rev 6.0 is latest of 4.x; now the Experimental version
 * 
 * Revision 4.2  89/08/11  09:56:44  mlh
 * simultaneous nonlinear equations allowed in DERIVATIVE block
 * 
 * Revision 4.1  89/07/31  07:55:20  mlh
 * Slight problem with error messages. Now missing partial equation in PARTIAL
 * block says DEL2 instead of DEL
 * 
 * Revision 4.0  89/07/24  17:03:37  nfh
 * Freezing rev 3.  Rev 4 is now Experimental
 * 
 * Revision 3.2  89/07/18  11:55:13  mlh
 * first_time removed and MODEL_LEVEL used for declaration precedence
 * 
 * Revision 1.3  89/07/18  09:50:50  mlh
 * variable declaration precedence in submodels handled with
 * MODEL_LEVEL level prefix generated by merge. Lowest level takes
 * precedence.
 * 
 * Revision 1.2  89/07/07  08:52:14  mlh
 * START field in INDEPENDENT block
 * SWEEP keyword replaces SCOP in INDEPENDENT block
 * FIRST and LAST keywords in PARTIAL block for boundaries rplace index
 * y'0 etc. allowed for constants
 * 
 * Revision 1.1  89/07/06  14:45:58  mlh
 * Initial revision
 * 
*/
#include "modl.h"

#if __TURBOC__ || NeXT
extern void *malloc(), *realloc();
#else
extern char *malloc(), *realloc();
#endif

#if defined(__STDC__)
#define sdebug(arg1,arg2) {}
#define qdebug(arg1,arg2) {}
#else
#define sdebug(arg1,arg2) {Fprintf(stderr,"arg1:%s\n", arg2); Fflush(stderr);}
#define qdebug(arg1,arg2) {Item *q; Fprintf(stderr,"arg1:");Fflush(stderr);\
	for (q=arg2; q->type != 0; q=q->next){\
		if (q->type == SYMBOL)\
			Fprintf(stderr,"%s\n", SYM(q)->name);\
		else if (q->type == STRING)\
			Fprintf(stderr,"%s\n", STR(q);\
		else\
			Fprintf(stderr,"Illegal Item type\n");\
		Fflush(stderr);}\
		Fprintf(stderr,"\n");Fflush(stderr);}
#endif

#define ldebug(arg1, arg2) qdebug(arg1, arg2->next)

#if BISON
#define myerr(arg) static int ierr=0;\
if (!(ierr++))yyerror(arg); --yyssp; --yyvsp; YYERROR
#else
#define myerr(arg) static int ierr=0;\
if (!(ierr++))yyerror(arg); --yyps; --yypv; YYERROR
#endif

extern char *modelline;
extern List *firstlist; /* NAME symbols in order that they appear in file */
extern int lexcontext; /* ':' can return 3 different tokens */
extern List *solveforlist; /* List of symbols that are actually to be solved
				for in a block. See in_solvefor() */
static int stateblock; /* 0 if dependent, 1 if state */
static int blocktype;
static int inequation; /* inside an equation?*/
static int nstate;	/* number of states seen in an expression */
static int leftside;	/* inside left hand side of equation? */
static int pstate;	/* number of state in a primary expression */
static int tstate;	/* number of states in a term */
static Item *lastok;	/* last token accepted by expr */
static int sensused;	/* a SENS statement occurred in this block */
static Symbol *matchindex; /* local symbol for implied MATCH loop */
static int model_level = 0; /* the model level prepended to declaration
				blocks by merge */
static int scopindep = 0;/* SCoP independent explicitly declared if 1 */
static int extdef2 = 0; /* flag that says we are in an EXTDEF2 function */
static List *table_list = LIST0; /* table information for TABLE statement */
static int forallindex = 0;	/* 0 not in FORALL, -1 just starting, 
					>0 index of arrays used (must all
					be the same */
static Item* astmt_end_;	/* see kinetic.c vectorizing */

# line 343 "parse1.y"
typedef union  {
	Item	*qp;
	char	*str;
	List	*lp;
	int	i;
} YYSTYPE;
# define VERBATIM 257
# define COMMENT 258
# define MODEL 259
# define CONSTANT 260
# define INDEPENDENT 261
# define DEPENDENT 262
# define STATE 263
# define INITIAL1 264
# define DERIVATIVE 265
# define SOLVE 266
# define USING 267
# define WITH 268
# define STEPPED 269
# define DISCRETE 270
# define FROM 271
# define FORALL1 272
# define TO 273
# define BY 274
# define WHILE 275
# define IF 276
# define ELSE 277
# define START1 278
# define STEP 279
# define SENS 280
# define SOLVEFOR 281
# define PROCEDURE 282
# define PARTIAL 283
# define DEL 284
# define DEL2 285
# define DEFINE1 286
# define IFERROR 287
# define PARAMETER 288
# define DERFUNC 289
# define EQUATION 290
# define TERMINAL 291
# define LINEAR 292
# define NONLINEAR 293
# define FUNCTION1 294
# define LOCAL 295
# define METHOD 296
# define LIN1 297
# define NONLIN1 298
# define PUTQ 299
# define GETQ 300
# define TABLE 301
# define DEPEND 302
# define BREAKPOINT 303
# define INCLUDE1 304
# define OR 305
# define AND 306
# define GT 307
# define LT 308
# define LE 309
# define EQ 310
# define NE 311
# define NOT 312
# define NAME 313
# define PRIME 314
# define REAL 315
# define INTEGER 316
# define DEFINEDVAR 317
# define STRING 318
# define PLOT 319
# define VS 320
# define LAG 321
# define RESET 322
# define MATCH 323
# define MODEL_LEVEL 324
# define SWEEP 325
# define FIRST 326
# define LAST 327
# define KINETIC 328
# define CONSERVE 329
# define REACTION 330
# define REACT1 331
# define COMPARTMENT 332
# define UNITS 333
# define UNITSON 334
# define UNITSOFF 335
# define NEURON 336
# define SUFFIX 337
# define NONSPECIFIC 338
# define READ 339
# define WRITE 340
# define USEION 341
# define VALENCE 342
# define GLOBAL 343
# define SECTION 344
# define RANGE 345
# define POINTER 346
# define EXTERNAL 347
# define ELECTRODE_CURRENT 348
# define GE 349
# define UNARYMINUS 350
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 1275 "parse1.y"

yyerror(s)	/* called for yacc syntax error */
	char *s;
{
	Fprintf(stderr, "%s:\n ", s);
}


#if !NMODL
nrn_list(q1, q2)
	Item *q1, *q2;
{
	/*ARGSUSED*/
}
nrn_use(q1, q2, q3)
	Item *q1, *q2, *q3;
{
	/*ARGSUSED*/
}
#endif
int yyexca[] ={
-1, 0,
	0, 3,
	257, 3,
	258, 3,
	259, 3,
	260, 3,
	261, 3,
	262, 3,
	263, 3,
	264, 3,
	265, 3,
	269, 3,
	270, 3,
	282, 3,
	283, 3,
	286, 3,
	288, 3,
	291, 3,
	292, 3,
	293, 3,
	294, 3,
	295, 3,
	303, 3,
	304, 3,
	319, 3,
	324, 3,
	328, 3,
	333, 3,
	334, 3,
	335, 3,
	336, 3,
	-2, 0,
-1, 1,
	0, -1,
	-2, 0,
-1, 2,
	0, 1,
	-2, 10,
-1, 9,
	265, 87,
	270, 101,
	282, 97,
	283, 103,
	291, 99,
	292, 91,
	293, 93,
	294, 95,
	303, 89,
	328, 105,
	-2, 85,
-1, 175,
	41, 229,
	-2, 230,
-1, 176,
	41, 229,
	-2, 230,
-1, 213,
	125, 109,
	330, 141,
	-2, 118,
-1, 214,
	125, 110,
	330, 141,
	-2, 118,
-1, 320,
	271, 296,
	302, 296,
	-2, 0,
-1, 420,
	41, 196,
	44, 196,
	-2, 0,
-1, 516,
	40, 279,
	-2, 276,
	};
# define YYNPROD 322
# define YYLAST 1109
int yyact[]={

   324,   111,   247,   508,   227,   521,   515,   389,   441,   355,
   375,    65,   333,   234,   241,   242,   322,   336,   185,   198,
   259,   184,    93,   448,   334,   460,   132,   421,   397,   395,
    50,   396,   611,   398,   522,   523,   240,   144,   620,   397,
   395,    97,   396,   327,   398,   385,   240,   552,   372,    66,
   104,   126,   127,    37,    64,   246,   126,   127,   231,   126,
   127,   126,   127,   128,    10,    11,    14,    36,    29,    30,
    31,   518,   372,    66,   386,   384,    32,   392,   599,   114,
   399,   192,   229,    67,    66,   549,   163,   376,   164,   119,
   188,   399,   139,    16,   380,    28,   619,   370,   601,   125,
   145,   509,    15,    63,    36,    29,    30,    31,   572,    67,
   200,    13,   519,    32,   513,   169,   397,   395,   388,   396,
   528,   398,   178,   179,   142,   143,    33,   160,    67,    66,
   368,     8,    28,   453,   372,    66,   364,   126,   127,   280,
    35,    26,    27,    34,   372,    66,   362,   126,   127,   261,
   260,    67,    66,   326,   372,    66,   136,   136,   190,   190,
    67,    66,   219,    33,   263,   258,    66,   199,   399,   201,
   202,   203,   204,   205,   206,    67,   220,    35,   518,   207,
    34,   197,   226,   124,   123,   120,   230,   367,   122,    56,
   230,    53,   233,   363,   131,   118,   230,   117,   239,   116,
   115,   113,   529,   361,   328,   359,    77,   456,   504,   330,
   424,    15,   191,   135,   135,   230,   325,   249,   262,   218,
    83,   191,    81,    79,   442,    87,   134,   243,   443,   245,
   574,   156,   230,    91,    85,    67,    66,   237,   274,   230,
   230,   275,   276,   267,   443,   121,    55,   173,    54,   270,
   598,   130,   292,   450,   540,   352,   273,   254,   254,   524,
   277,   281,   282,   283,   284,   432,   550,   187,   272,    67,
    66,   286,   237,   550,   235,   230,   165,   350,    67,    66,
   141,   230,   230,   353,   354,    89,   430,   346,   459,   348,
   349,    75,   358,   402,   403,   405,   406,   407,    73,     3,
   208,   339,   400,   401,   402,   403,   405,   406,   407,   112,
   228,   458,   146,   148,   374,   213,   155,   378,   152,   150,
   151,   153,   154,   149,   393,   378,   103,   622,   615,   409,
   410,   411,   390,   391,   357,   404,   379,   381,   382,   613,
    12,   418,   356,   570,   357,   419,   404,   423,   408,   133,
   221,   428,   231,   126,   127,   230,   422,   433,   569,   547,
   417,   246,   126,   127,   244,   431,   439,   440,   427,   543,
   538,   617,   397,   395,   494,   396,   434,   398,   436,   339,
   401,   402,   403,   405,   406,   407,   360,   345,   269,   397,
   395,   604,   396,   264,   398,   461,   462,   463,   464,   465,
   466,   467,   468,   469,   470,   471,   472,   473,   474,   328,
   225,   338,   489,   487,   330,   488,   224,   490,   215,   177,
   331,   484,   170,   404,   399,   476,   425,   423,   492,   105,
   255,   255,   495,   479,   480,   230,   422,   497,   477,   478,
   501,   399,   166,   112,   499,   505,   451,   498,   510,   493,
   101,   512,   491,   496,   236,   100,   339,   339,   339,   339,
   339,   237,   607,    95,   525,    70,    69,    68,   390,   526,
   397,   395,   516,   396,   337,   398,   329,   258,    66,   246,
   126,   127,   565,    67,   214,   423,   126,   127,    61,   338,
   423,   423,   423,   423,   422,   209,   171,    58,   163,   422,
   422,   422,   422,   423,   544,   545,    57,   137,   548,   539,
   530,   435,   422,   546,   328,   533,   534,   535,   536,   330,
   541,   553,   399,   399,   585,   559,   557,   223,   542,   371,
   378,   377,   575,   564,   558,   516,   232,     5,   560,   377,
   271,   397,   395,   230,   396,   567,   398,   351,   196,   593,
   397,   395,   337,   396,   182,   398,   573,   554,   520,   328,
   482,   566,   579,   580,   330,   340,   338,   338,   338,   338,
   338,   423,   194,   195,   454,   584,   344,   589,   590,   591,
   422,   581,   397,   157,   106,   340,    96,   398,   181,   602,
   335,   305,   342,   399,     7,   449,   582,   605,   600,   423,
   608,   609,   399,   594,   343,   612,   163,   588,   422,   138,
   592,   397,   395,   174,   396,   618,   398,   586,   397,   395,
   180,   396,   621,   398,   606,   331,   576,   183,   342,   337,
   337,   337,   337,   337,   399,   400,   401,   402,   403,   405,
   406,   407,   397,   395,   596,   396,   500,   398,   452,   412,
   168,   323,   400,   401,   402,   403,   405,   406,   407,   489,
   487,   506,   488,   399,   490,   511,   110,   503,   397,   395,
   399,   396,   448,   398,   502,   397,   395,   212,   396,   404,
   398,   329,   258,    66,   246,   126,   127,   481,   397,   395,
   447,   396,   210,   398,   399,   193,   404,   457,   475,   397,
   395,   577,   396,   444,   398,   397,   395,   343,   396,   347,
   398,   445,   489,   487,   162,   488,   413,   490,   414,   279,
   399,   397,   395,   571,   396,   531,   398,   399,   532,   551,
   331,   342,   240,   400,   401,   402,   403,   405,   406,   407,
   399,   217,   489,    94,   377,   397,   395,   490,   396,   616,
   398,   399,   537,   489,   487,   415,   488,   399,   490,   578,
   416,   163,   448,   563,   394,   556,   285,   603,   614,   397,
   395,   161,   396,   399,   398,   331,   610,   404,   163,   489,
   487,   341,   488,   268,   490,   265,   329,   258,    66,   246,
   126,   127,   485,   489,   487,   163,   488,   399,   490,   278,
   248,   341,   446,   587,   400,   401,   402,   403,   405,   406,
   407,   562,   561,   400,   401,   402,   403,   405,   406,   407,
   555,   399,   366,   365,   176,   175,   107,   109,   159,   158,
   486,   329,   373,    66,   246,   126,   127,   108,   258,    66,
   246,   126,   127,   383,   527,   507,   222,   426,   404,   216,
   597,   583,   438,   437,   420,   332,   257,   404,   258,    66,
   246,   126,   127,   253,   304,   250,   167,    92,    49,    90,
    48,    88,    47,    86,   400,   401,   402,   403,   405,   406,
   407,   400,   401,   402,   403,   405,   406,   407,    46,    84,
   568,    45,    82,    44,    80,    43,   112,    78,    42,   319,
    76,    41,    74,    40,    72,   400,   401,   402,   403,   405,
   406,   407,    39,    62,    60,    59,   429,   186,   404,    99,
   102,   211,    98,    25,    22,   404,    21,    20,    19,    18,
    17,   400,   401,   402,   403,   405,   406,   407,   400,   401,
   402,   403,   405,   406,   407,    38,     9,    71,     2,   404,
     1,   400,   401,   402,   403,   405,   406,   407,   147,    23,
   266,   455,   400,   401,   402,   403,   405,   406,   407,   387,
   172,    24,   297,   298,   321,   404,   303,   517,   514,   302,
   189,   140,   404,   129,   400,   401,   402,   403,   405,   406,
   407,   289,   299,   296,   369,   404,   595,   293,   291,   290,
   288,   252,   300,     6,   483,    52,   404,   251,   400,   401,
   402,   403,   405,   406,   407,   287,   238,   256,     4,    51,
     0,     0,     0,     0,     0,     0,     0,     0,   404,   306,
   294,   295,   400,   401,   402,   403,   405,   406,   407,   311,
     0,     0,     0,     0,   307,   308,     0,     0,   309,   310,
     0,     0,   404,   312,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,   316,   317,   320,     0,   404,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,   315,   301,   318,     0,     0,     0,
     0,     0,   314,     0,     0,   313,     0,    26,    27 };
int yypact[]={

    43, -1000,  -193, -1000, -1000, -1000, -1000, -1000,  -263, -1000,
 -1000, -1000, -1000,  -288, -1000,   -65,   -67, -1000, -1000, -1000,
 -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,   383,   374,
 -1000, -1000,   365,  -153,   344,   343,   342, -1000, -1000,    34,
    26,   -97,   -69,   -71,   -74,   -48,   -66,    15,   -50,  -306,
 -1000, -1000,   699, -1000,   495,  -275, -1000, -1000, -1000,   332,
   327, -1000,     6, -1000,   493, -1000, -1000, -1000, -1000, -1000,
 -1000,  -156, -1000,   320, -1000,  -112, -1000,   320, -1000,  -113,
 -1000,  -114, -1000,  -116, -1000,  -118, -1000,   320, -1000,  -128,
 -1000,   -68, -1000,  -129,  -130, -1000,  -255, -1000,   -62,   -99,
 -1000, -1000,   -33,  -230,  -230, -1000,  -279,   -25,   458,   -37,
 -1000,   317,   -84,   320,   297,   -34,   -34,   785,   784,   294,
   320,   320, -1000,   -34,   495,   461, -1000, -1000, -1000, -1000,
   566, -1000, -1000, -1000, -1000,    -4, -1000,   -35,   -44, -1000,
 -1000,   634,   493,   493,   455, -1000,  -132, -1000,  -146,  -146,
  -146,  -146,  -146,  -146,  -146,  -134, -1000, -1000, -1000, -1000,
   239,   631, -1000, -1000, -1000,   616, -1000, -1000, -1000,   293,
 -1000, -1000,   697,   -94,   320, -1000, -1000, -1000,   291,   285,
   320, -1000, -1000,    37, -1000, -1000,  -100,    37, -1000, -1000,
   183, -1000, -1000,    37, -1000, -1000, -1000, -1000,   688, -1000,
 -1000,   688,   688,   688,   688,   688,   688,  -325,   738, -1000,
    46,   759,    37,  -148,  -148, -1000,   320,  -149, -1000, -1000,
   268,   744,  -230,   742, -1000, -1000,   263,   738, -1000,  -257,
 -1000, -1000, -1000,    -5,   738,  -255,    37,    37,   755,   675,
  -174,  -146,  -146, -1000,   738,   721, -1000, -1000, -1000,   738,
   773, -1000, -1000,  -314, -1000,   590,   369,   545,   564,   485,
 -1000, -1000,   262, -1000, -1000,   738,   665,   738,   738, -1000,
 -1000, -1000,    37, -1000,   454,   -18, -1000, -1000,    37,    37,
 -1000,     2,    -8, -1000, -1000,  -102, -1000, -1000, -1000, -1000,
 -1000, -1000,   261, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
 -1000, -1000, -1000, -1000, -1000, -1000, -1000,  -110,  -120,   783,
   782,  -126,  -159,   519,  -169,  -162,  -230,  -230, -1000,  -239,
  -138, -1000,  -179,   369,   703, -1000,   738, -1000,   369,   369,
   369, -1000,   588,   673,   713,   525, -1000, -1000, -1000, -1000,
   369, -1000,  -255, -1000,   170, -1000, -1000,  -230, -1000,   320,
    18,    -6,    37, -1000, -1000, -1000,  -146,  -260, -1000,   738,
 -1000, -1000, -1000, -1000, -1000,   369,   369,   -43, -1000,   659,
 -1000, -1000,   528,   667,   679,   629, -1000, -1000,  -241,   -21,
 -1000, -1000, -1000,   323,   587,  -180,   483,   -95, -1000,   653,
 -1000,   -20, -1000,   727,   369,   369,   369,   369,   369,   369,
   369,   369,   369,   369,   369,   369,   369,   369, -1000,   657,
   429,   429,   545,   525,   525,   525,   525,   713,   646,   469,
   474,   737, -1000, -1000,   170, -1000,   320,   738,   249,  -255,
 -1000,   738,    37,   -41,    -8, -1000, -1000,   585,   320,   633,
   626, -1000,   -88,   320,  -241, -1000,  -212,   369,  -265, -1000,
  -199,  -185,  -201,   467,  -292,   -12,  -204,  -204, -1000,  -188,
  -105,   727,   540,   540,   429,   429,   429,    74,   -14,   663,
   663,   663,   663,   663,   663, -1000,   673,   713,   713, -1000,
 -1000, -1000,   170,   684,   727, -1000, -1000,   170,   170,   170,
   170,   711,   245, -1000, -1000,   -41, -1000,   -19,   738, -1000,
   170,   244,   320,   320,   -59,   234, -1000,   369,   -40, -1000,
   727, -1000,  -241, -1000,   -78, -1000,   466,   780, -1000,   723,
  -292,   441, -1000, -1000,   369,   653, -1000,  -265,   772,   771,
   670, -1000,   164,   700,   700, -1000, -1000, -1000, -1000,   738,
    37, -1000,   617, -1000,   233,   218, -1000, -1000,   600, -1000,
 -1000, -1000, -1000, -1000,  -205,   369,   -55,   439,   565,   428,
   719,   369,   369, -1000,   727, -1000, -1000,   -41,   170, -1000,
 -1000,  -212,   431,   576,   763,   546,   369,   369,   369,   569,
   508,   738,   370,   -27,   -47, -1000,   537,  -215,   369,   727,
   499,   347, -1000, -1000, -1000,   320,   170, -1000,   186, -1000,
   369,   735,   727,  -284,   369,   214,   751,   728,   203,   727,
   706, -1000,   330, -1000,   369, -1000,  -217, -1000,    -3, -1000,
   320,   202, -1000 };
int yypgo[]={

     0,    11,   310,   153,    27,     2,  1019,  1018,    21,   429,
    18,    20,     4,  1017,    24,    17,    12,  1016,     0,  1015,
  1007,     1,   537,  1005,   216,  1004,  1003,  1002,  1001,  1000,
   999,   998,   997,    43,     8,   996,    13,   994,   993,   992,
   991,   983,   981,   349,   980,   979,   978,     6,   977,   976,
     5,   974,   973,    10,   972,     3,   971,   496,   970,   340,
   969,     7,   961,   350,   960,   463,   959,   958,    19,     9,
   950,   948,   594,   947,   946,   945,   930,   929,   928,   927,
   926,   924,   923,   922,   921,   920,   919,   917,   916,   915,
   507,   914,   913,   912,   904,   903,   902,   901,   900,   898,
   897,   895,   894,   893,   892,   891,   889,   888,   873,   872,
   871,   870,   869,   868,   867,   866,   315,   865,   864,   863,
   856,   855,   854,   853,   852,   851,   850,   849,   847,   846,
   845,   844,   843,   837,   829,   828,   827,   826 };
int yyr1[]={

     0,    70,    70,    71,    71,    71,    71,    71,    73,    71,
    74,    71,    71,    71,    71,    71,     7,     6,    26,    26,
     1,    72,    72,    72,    72,    72,    72,    72,    72,    72,
    76,    83,    83,    41,    41,    41,     8,     8,    84,    10,
    59,    59,    80,    85,    85,    42,    17,    17,    11,    11,
    12,    12,     2,     2,     5,     5,     3,     3,    77,    86,
    86,    87,    86,    43,    43,    88,    89,    78,    90,    90,
    44,    44,    44,    44,    44,    36,    36,    91,    79,    81,
    81,    92,    92,     9,     9,    93,    75,    95,    75,    97,
    75,    99,    75,   101,    75,   103,    75,   105,    75,   107,
    75,   109,    75,   111,    75,   113,    75,    94,   115,    21,
    21,    22,    22,    23,    23,    65,    65,   116,   117,   116,
   116,    19,    19,    19,    19,    19,    19,    19,    19,    19,
    19,    19,    19,    19,    19,    19,    19,    19,    19,    19,
    20,   119,    20,    20,    28,    28,    28,    24,    24,    24,
    24,     4,     4,     4,     4,     4,     4,     4,     4,    18,
    18,    18,    18,    18,    18,    18,    18,    18,    18,    18,
    18,    18,    18,    18,    18,    18,    18,    18,    18,    13,
   120,   121,    16,    16,    16,    16,    14,    14,    14,    15,
    15,    15,    15,    15,   122,    33,    25,    25,    25,    25,
    25,   123,    29,    29,    35,    35,   124,    40,    40,    30,
    31,   125,   125,   126,   126,    96,   127,   100,   102,   110,
   112,   112,    49,    49,    49,    50,    50,   128,   104,    63,
   129,    63,    64,    64,   106,    32,    32,    32,    34,    34,
    57,    57,    58,    58,    58,    98,   108,    38,    38,    37,
    37,    52,    52,   130,    54,    54,    55,    55,   114,   131,
    51,    51,    51,    51,    53,    53,    53,    53,    39,    39,
    27,    27,   132,    45,    46,    46,    47,    47,    47,    48,
    48,    56,   133,   133,   133,   134,   134,   135,   135,   135,
   135,    82,   136,   136,   118,   118,    60,    60,    61,    61,
    62,    62,    66,   137,   137,   137,   137,   137,   137,   137,
   137,   137,   137,    67,    67,    67,    67,    68,    68,    68,
    69,    69 };
int yyr2[]={

     0,     3,     3,     0,     4,     5,     4,     4,     1,    11,
     1,     6,     5,     5,     4,     7,     5,     1,     7,     5,
     3,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     9,     0,     5,     9,     5,     3,     1,     2,     1,     7,
     3,     3,     9,     0,     5,     9,     6,     6,     2,     2,
     3,     5,     2,     2,     2,     3,     3,     3,     9,     0,
     5,     1,     9,    19,     3,     2,     1,    11,     0,     5,
     7,    13,    15,    21,     3,     1,     5,     1,    11,    11,
     5,     5,     9,     1,     7,     1,     4,     1,     4,     1,
     4,     1,     4,     1,     4,     1,     4,     1,     4,     1,
     4,     1,     4,     1,     4,     1,     5,     7,     1,     7,
     7,     5,     5,     5,     9,     1,     7,     0,     1,     7,
     5,     2,     2,     2,     2,     4,     2,     3,     3,     2,
     3,     3,     2,     2,     3,     2,     2,     3,     2,     3,
     3,     1,     5,     3,     7,     9,     9,     3,     9,     7,
    13,     3,     3,     7,     6,     6,     6,     6,     3,     2,
     4,     2,     7,     6,     6,     6,     6,     7,     7,     7,
     6,     6,     6,     6,     7,     7,     5,     4,     3,     3,
     3,     3,     3,     5,     7,     7,     3,     7,     7,     2,
     2,     3,     7,     3,     1,    11,     1,     2,     2,     6,
     6,     1,    21,     5,     1,     4,     1,    11,     5,    12,
    16,     0,    17,     0,     7,     9,     1,    13,    11,     9,
     9,     5,    23,    17,    15,     2,     2,     1,    19,     1,
     1,     5,     5,     9,    17,     7,    11,     5,     1,     7,
     1,     2,     5,     7,     5,     7,     7,     5,     5,     3,
     7,     9,     5,     1,    17,    11,     2,     4,    11,     1,
    21,    15,    15,     5,     3,     5,     7,     9,     9,     5,
     5,     5,     1,    11,     2,     4,     3,    13,     3,     3,
     9,     9,     1,     4,     4,     7,     5,     9,     9,    13,
     3,     8,     0,    11,    19,     5,     1,     2,     3,     7,
     1,     5,     9,     0,     7,     4,     7,     7,     7,     7,
     7,     7,     7,    11,    11,    15,     3,     2,     7,     3,
     1,     5 };
int yychk[]={

 -1000,   -70,   -71,   256,    -7,   -22,   -26,   -72,   324,   -74,
   257,   258,   -59,   304,   259,   295,   286,   -76,   -77,   -78,
   -79,   -80,   -81,   -66,   -56,   -82,   334,   335,   288,   261,
   262,   263,   269,   319,   336,   333,   260,   316,   -75,   -93,
   -95,   -97,   -99,  -101,  -103,  -105,  -107,  -109,  -111,  -113,
   318,    -6,   -23,   256,   313,   313,   256,   123,   123,   -89,
   -91,   123,   -92,   256,   -11,    -1,   314,   313,   123,   123,
   123,   -73,   -94,   264,   -96,   265,   -98,   303,  -100,   292,
  -102,   293,  -104,   294,  -106,   282,  -108,   291,  -110,   270,
  -112,   283,  -114,   328,    44,   -65,    91,   316,   -83,   -86,
   123,   123,   -85,   320,    44,    -9,    91,  -137,  -133,  -136,
   -72,   -21,   123,   313,   -21,   313,   313,   313,   313,   -21,
   313,   313,   256,   313,   313,    -5,   316,   317,   125,   -41,
   313,   256,   125,   -43,   325,   313,   256,   -90,   -90,   125,
   -42,   313,   -11,   -11,   316,   125,   337,   -67,   338,   348,
   344,   345,   343,   346,   347,   341,   256,   125,  -134,  -135,
   -10,   313,   256,    40,   125,   313,   125,  -115,   -22,   -21,
   125,   -57,   -58,   281,   -57,    40,    40,   125,   -21,   -21,
   -57,   -65,    93,    61,    -8,   -10,   -87,   271,   125,   -44,
   -11,   256,   125,    61,    -9,    -9,    93,   313,   -68,   313,
   256,   -68,   -68,   -68,   -68,   -68,   -68,   313,    61,   256,
    61,   -84,    61,  -116,  -116,   125,  -127,    44,   313,   256,
   -21,   -63,  -129,   -63,   125,   125,   -21,   -12,    -2,    45,
    -5,   315,   -43,   -12,   -36,    91,   271,   278,   -17,   -12,
    44,   339,   340,   -10,    -3,   -10,   315,    -5,    41,   -12,
  -117,   -20,   -28,  -119,   -33,   -24,   -13,  -120,   313,   -11,
   298,   297,   -21,   313,   125,    41,   -64,   -11,    41,   125,
    -8,    -2,   273,    -8,    -5,   -12,   -12,    -8,    44,    44,
   313,   -68,   -68,   -10,   -10,    45,    -8,   -19,   -29,   -40,
   -30,   -31,   -21,   -32,   257,   258,   -38,   -54,   -52,   -39,
   -27,   322,   -45,   -49,  -118,   -59,   256,   271,   272,   275,
   276,   266,   280,   332,   329,   321,   299,   300,   323,   126,
   301,   -51,   330,    61,   -18,   -24,    -3,   -33,    40,   312,
    45,   256,  -121,   -16,   -14,    45,   -15,   -24,    -3,   -33,
    40,   256,    64,    40,    91,   125,    -8,    44,    -8,    -8,
   -12,    93,   273,   -12,   -12,   -69,   340,   342,   -69,   307,
   125,   313,   256,   313,   256,    40,    40,   313,   256,   -37,
   256,   -24,   313,   313,   -18,   -53,   256,   -24,    -5,   -11,
   256,   -11,   -11,  -132,   314,   284,   313,   -60,   256,   -61,
    -1,   -53,   256,   -18,    61,    43,    45,    42,    47,    94,
   305,   306,   307,   308,   349,   309,   310,   311,    -8,   -18,
   -18,   -18,    61,    43,    45,    42,    47,   -14,   -18,    -5,
  -122,    -4,    -1,    -5,    40,   256,  -128,   -11,   -21,   -88,
   268,   -36,   271,   -12,   -68,    -3,   -10,  -123,  -124,   -18,
   -18,   -34,   267,   287,    44,    44,   123,    61,    43,   -24,
   274,   123,    61,   313,    91,   -62,   302,    44,   331,   308,
    45,   -18,   -18,   -18,   -18,   -18,   -18,   -18,   -18,   -18,
   -18,   -18,   -18,   -18,   -18,    41,   -16,   -14,   -14,   -15,
   -15,    41,    91,   -25,   -18,   318,    93,    43,    45,    42,
    47,    -4,   -21,    -8,   125,    -5,    -8,   -12,   -36,   -69,
    61,   -21,    41,    41,   296,   -21,   -24,  -130,   -55,   313,
   -18,   -24,    -5,   313,   -46,   -47,   -11,   -48,   256,   313,
    91,   -50,   326,   327,   271,   -61,    -1,  -131,   308,   307,
    -4,    41,    44,    -4,    -4,    -4,    -4,    41,   125,   -36,
   273,    -8,    -4,   125,   -21,   -21,   -34,   125,   -18,   125,
   313,   -24,   125,   -47,    91,    40,    42,   -50,    93,   -18,
   -53,    40,    40,    93,   -18,   318,    -8,   -12,   273,   125,
   125,   123,   313,   -18,   285,    93,    61,   273,    40,   -18,
   -18,   -36,    -4,  -125,   -55,    93,    41,    40,    61,   -18,
   -18,   -18,    41,    41,    -8,   -35,   274,  -126,   277,   125,
    61,   313,   -18,   268,    44,   -21,    -4,   276,   -21,   -18,
    41,   316,   -18,   125,    40,   125,    43,    41,   -18,   313,
    41,   -21,   125 };
int yydef[]={

    -2,    -2,    -2,     2,     4,     5,     6,     7,     0,    -2,
    12,    13,    14,     0,    17,     0,     0,    21,    22,    23,
    24,    25,    26,    27,    28,    29,    40,    41,     0,     0,
    66,    77,     0,     0,     0,     0,     0,     8,    11,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    15,    16,   111,   112,   115,     0,    19,    31,    59,     0,
     0,    43,     0,    80,    83,    48,    49,    20,   303,   282,
   292,     0,    86,     0,    88,     0,    90,     0,    92,     0,
    94,     0,    96,     0,    98,     0,   100,     0,   102,     0,
   104,     0,   106,     0,     0,   113,     0,    18,     0,     0,
    68,    68,     0,     0,     0,    81,     0,     0,     0,     0,
     9,     0,   108,     0,     0,   240,   240,     0,     0,     0,
     0,     0,   221,   240,   115,     0,    54,    55,    30,    32,
    36,    35,    58,    60,    61,     0,    64,     0,     0,    42,
    44,     0,    83,    83,     0,   302,     0,   305,     0,     0,
     0,     0,     0,     0,     0,     0,   316,   281,   283,   284,
     0,     0,   290,    38,   291,     0,   107,   117,   117,     0,
   245,   216,   241,     0,     0,    -2,    -2,   246,     0,     0,
     0,   114,   116,     0,    34,    37,     0,     0,    67,    69,
    75,    74,    78,     0,    79,    82,    84,   304,   306,   317,
   319,   307,   308,   309,   310,   311,   312,     0,     0,   286,
     0,     0,     0,    -2,    -2,   215,     0,     0,   242,   244,
     0,     0,     0,     0,   219,   220,     0,    36,    50,     0,
    52,    53,    62,     0,    36,     0,     0,     0,    36,     0,
     0,     0,     0,   285,     0,     0,    56,    57,    39,    36,
     0,   120,   140,     0,   143,     0,     0,     0,    20,   147,
   179,   180,     0,   243,   218,    36,   231,    36,    36,   258,
    33,    51,     0,    70,     0,     0,    76,    45,     0,     0,
   318,   320,   320,   287,   288,     0,   293,   119,   121,   122,
   123,   124,     0,   126,   127,   128,   129,   130,   131,   132,
   133,   134,   135,   136,   137,   138,   139,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   272,     0,
    -2,   142,     0,     0,     0,   159,    36,   161,     0,     0,
     0,   178,     0,   181,   182,     0,   186,   189,   190,   191,
     0,   193,     0,   194,     0,   217,   227,     0,   232,     0,
     0,    75,     0,    47,    46,   313,     0,     0,   314,     0,
   125,   201,   203,   206,   208,     0,     0,   238,   237,   247,
   248,   249,    20,    20,     0,     0,   252,   264,     0,     0,
   269,   270,   271,     0,     0,     0,     0,   300,   295,   297,
   298,     0,   263,   144,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,   160,     0,
   176,   177,     0,     0,     0,     0,     0,   183,     0,   149,
    -2,     0,   151,   152,     0,   158,     0,    36,     0,     0,
    65,    36,     0,    75,   320,   321,   289,     0,     0,     0,
     0,   235,     0,     0,     0,   253,     0,     0,     0,   265,
     0,     0,     0,     0,     0,     0,     0,     0,   259,     0,
     0,   145,   163,   164,   165,   166,   167,   168,   169,   170,
   171,   172,   173,   174,   175,   162,   146,   184,   185,   187,
   188,   192,     0,     0,   197,   198,   148,     0,     0,     0,
     0,     0,     0,   233,   234,    75,    71,     0,    36,   315,
     0,     0,     0,     0,   238,     0,   250,     0,     0,   256,
   251,   266,     0,   268,     0,   274,    -2,     0,   278,     0,
     0,     0,   225,   226,     0,   301,   299,     0,     0,     0,
     0,   195,     0,   154,   155,   156,   157,   153,   228,    36,
     0,    72,     0,   207,     0,     0,   236,   239,     0,   255,
   257,   267,   273,   275,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   150,   199,   200,    63,    75,     0,   209,
   211,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    36,   204,   213,     0,   280,     0,     0,     0,   224,
     0,     0,   261,   262,    73,     0,     0,   210,     0,   254,
     0,     0,   223,     0,     0,     0,   205,     0,     0,   277,
     0,   294,     0,   202,     0,   214,     0,   260,     0,   222,
     0,     0,   212 };
typedef struct { char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

yytoktype yytoks[] =
{
	"VERBATIM",	257,
	"COMMENT",	258,
	"MODEL",	259,
	"CONSTANT",	260,
	"INDEPENDENT",	261,
	"DEPENDENT",	262,
	"STATE",	263,
	"INITIAL1",	264,
	"DERIVATIVE",	265,
	"SOLVE",	266,
	"USING",	267,
	"WITH",	268,
	"STEPPED",	269,
	"DISCRETE",	270,
	"FROM",	271,
	"FORALL1",	272,
	"TO",	273,
	"BY",	274,
	"WHILE",	275,
	"IF",	276,
	"ELSE",	277,
	"START1",	278,
	"STEP",	279,
	"SENS",	280,
	"SOLVEFOR",	281,
	"PROCEDURE",	282,
	"PARTIAL",	283,
	"DEL",	284,
	"DEL2",	285,
	"DEFINE1",	286,
	"IFERROR",	287,
	"PARAMETER",	288,
	"DERFUNC",	289,
	"EQUATION",	290,
	"TERMINAL",	291,
	"LINEAR",	292,
	"NONLINEAR",	293,
	"FUNCTION1",	294,
	"LOCAL",	295,
	"METHOD",	296,
	"LIN1",	297,
	"NONLIN1",	298,
	"PUTQ",	299,
	"GETQ",	300,
	"TABLE",	301,
	"DEPEND",	302,
	"BREAKPOINT",	303,
	"INCLUDE1",	304,
	"{",	123,
	"}",	125,
	"(",	40,
	")",	41,
	"[",	91,
	"]",	93,
	"@",	64,
	"+",	43,
	"*",	42,
	"-",	45,
	"/",	47,
	"=",	61,
	"^",	94,
	":",	58,
	",",	44,
	"~",	126,
	"OR",	305,
	"AND",	306,
	"GT",	307,
	"LT",	308,
	"LE",	309,
	"EQ",	310,
	"NE",	311,
	"NOT",	312,
	"NAME",	313,
	"PRIME",	314,
	"REAL",	315,
	"INTEGER",	316,
	"DEFINEDVAR",	317,
	"STRING",	318,
	"PLOT",	319,
	"VS",	320,
	"LAG",	321,
	"RESET",	322,
	"MATCH",	323,
	"MODEL_LEVEL",	324,
	"SWEEP",	325,
	"FIRST",	326,
	"LAST",	327,
	"KINETIC",	328,
	"CONSERVE",	329,
	"REACTION",	330,
	"REACT1",	331,
	"COMPARTMENT",	332,
	"UNITS",	333,
	"UNITSON",	334,
	"UNITSOFF",	335,
	"NEURON",	336,
	"SUFFIX",	337,
	"NONSPECIFIC",	338,
	"READ",	339,
	"WRITE",	340,
	"USEION",	341,
	"VALENCE",	342,
	"GLOBAL",	343,
	"SECTION",	344,
	"RANGE",	345,
	"POINTER",	346,
	"EXTERNAL",	347,
	"ELECTRODE_CURRENT",	348,
	"GE",	349,
	"%",	37,
	"UNARYMINUS",	350,
	"-unknown-",	-1	/* ends search */
};

char * yyreds[] =
{
	"-no such reduction-",
	"top : all",
	"top : error",
	"all : /* empty */",
	"all : all model",
	"all : all locallist",
	"all : all define1",
	"all : all declare",
	"all : all MODEL_LEVEL INTEGER",
	"all : all MODEL_LEVEL INTEGER declare",
	"all : all",
	"all : all proc",
	"all : all VERBATIM",
	"all : all COMMENT",
	"all : all uniton",
	"all : all INCLUDE1 STRING",
	"model : MODEL line",
	"line : /* empty */",
	"define1 : DEFINE1 NAME INTEGER",
	"define1 : DEFINE1 error",
	"Name : NAME",
	"declare : parmblk",
	"declare : indepblk",
	"declare : depblk",
	"declare : stateblk",
	"declare : stepblk",
	"declare : plotdecl",
	"declare : neuronblk",
	"declare : unitblk",
	"declare : constblk",
	"parmblk : PARAMETER '{' parmbody '}'",
	"parmbody : /* empty */",
	"parmbody : parmbody parmasgn",
	"parmasgn : NAME '=' number units",
	"parmasgn : NAME units",
	"parmasgn : error",
	"units : /* empty */",
	"units : unit",
	"unit : '('",
	"unit : '(' ')'",
	"uniton : UNITSON",
	"uniton : UNITSOFF",
	"stepblk : STEPPED '{' stepbdy '}'",
	"stepbdy : /* empty */",
	"stepbdy : stepbdy stepped",
	"stepped : NAME '=' numlist units",
	"numlist : number ',' number",
	"numlist : numlist ',' number",
	"name : Name",
	"name : PRIME",
	"number : NUMBER",
	"number : '-' NUMBER",
	"NUMBER : integer",
	"NUMBER : REAL",
	"integer : INTEGER",
	"integer : DEFINEDVAR",
	"real : REAL",
	"real : integer",
	"indepblk : INDEPENDENT '{' indepbody '}'",
	"indepbody : /* empty */",
	"indepbody : indepbody indepdef",
	"indepbody : indepbody SWEEP",
	"indepbody : indepbody SWEEP indepdef",
	"indepdef : NAME FROM number TO number withby integer opstart units",
	"indepdef : error",
	"withby : WITH",
	"depblk : DEPENDENT",
	"depblk : DEPENDENT '{' depbody '}'",
	"depbody : /* empty */",
	"depbody : depbody depdef",
	"depdef : name opstart units",
	"depdef : name '[' integer ']' opstart units",
	"depdef : name FROM number TO number opstart units",
	"depdef : name '[' integer ']' FROM number TO number opstart units",
	"depdef : error",
	"opstart : /* empty */",
	"opstart : START1 number",
	"stateblk : STATE",
	"stateblk : STATE '{' depbody '}'",
	"plotdecl : PLOT pvlist VS name optindex",
	"plotdecl : PLOT error",
	"pvlist : name optindex",
	"pvlist : pvlist ',' name optindex",
	"optindex : /* empty */",
	"optindex : '[' INTEGER ']'",
	"proc : /* empty */",
	"proc : initblk",
	"proc : /* empty */",
	"proc : derivblk",
	"proc : /* empty */",
	"proc : brkptblk",
	"proc : /* empty */",
	"proc : linblk",
	"proc : /* empty */",
	"proc : nonlinblk",
	"proc : /* empty */",
	"proc : funcblk",
	"proc : /* empty */",
	"proc : procedblk",
	"proc : /* empty */",
	"proc : terminalblk",
	"proc : /* empty */",
	"proc : discretblk",
	"proc : /* empty */",
	"proc : partialblk",
	"proc : /* empty */",
	"proc : kineticblk",
	"initblk : INITIAL1 stmtlist '}'",
	"stmtlist : '{'",
	"stmtlist : '{' stmtlist1",
	"stmtlist : '{' locallist stmtlist1",
	"locallist : LOCAL locallist1",
	"locallist : LOCAL error",
	"locallist1 : NAME locoptarray",
	"locallist1 : locallist1 ',' NAME locoptarray",
	"locoptarray : /* empty */",
	"locoptarray : '[' integer ']'",
	"stmtlist1 : /* empty */",
	"stmtlist1 : stmtlist1",
	"stmtlist1 : stmtlist1 ostmt",
	"stmtlist1 : stmtlist1 astmt",
	"ostmt : fromstmt",
	"ostmt : forallstmt",
	"ostmt : whilestmt",
	"ostmt : ifstmt",
	"ostmt : stmtlist '}'",
	"ostmt : solveblk",
	"ostmt : VERBATIM",
	"ostmt : COMMENT",
	"ostmt : sens",
	"ostmt : compart",
	"ostmt : conserve",
	"ostmt : lagstmt",
	"ostmt : queuestmt",
	"ostmt : RESET",
	"ostmt : matchblk",
	"ostmt : pareqn",
	"ostmt : tablestmt",
	"ostmt : uniton",
	"ostmt : error",
	"astmt : asgn",
	"astmt : /* empty */",
	"astmt : reaction",
	"astmt : funccall",
	"asgn : varname '=' expr",
	"asgn : nonlineqn expr '=' expr",
	"asgn : lineqn leftlinexpr '=' linexpr",
	"varname : name",
	"varname : name '[' intexpr ']'",
	"varname : NAME '@' integer",
	"varname : NAME '@' integer '[' intexpr ']'",
	"intexpr : Name",
	"intexpr : integer",
	"intexpr : '(' intexpr ')'",
	"intexpr : intexpr '+' intexpr",
	"intexpr : intexpr '-' intexpr",
	"intexpr : intexpr '*' intexpr",
	"intexpr : intexpr '/' intexpr",
	"intexpr : error",
	"expr : varname",
	"expr : real units",
	"expr : funccall",
	"expr : '(' expr ')'",
	"expr : expr '+' expr",
	"expr : expr '-' expr",
	"expr : expr '*' expr",
	"expr : expr '/' expr",
	"expr : expr '^' expr",
	"expr : expr OR expr",
	"expr : expr AND expr",
	"expr : expr GT expr",
	"expr : expr LT expr",
	"expr : expr GE expr",
	"expr : expr LE expr",
	"expr : expr EQ expr",
	"expr : expr NE expr",
	"expr : NOT expr",
	"expr : '-' expr",
	"expr : error",
	"nonlineqn : NONLIN1",
	"lineqn : LIN1",
	"leftlinexpr : linexpr",
	"linexpr : primary",
	"linexpr : '-' primary",
	"linexpr : linexpr '+' primary",
	"linexpr : linexpr '-' primary",
	"primary : term",
	"primary : primary '*' term",
	"primary : primary '/' term",
	"term : varname",
	"term : real",
	"term : funccall",
	"term : '(' expr ')'",
	"term : error",
	"funccall : NAME '('",
	"funccall : NAME '(' exprlist ')'",
	"exprlist : /* empty */",
	"exprlist : expr",
	"exprlist : STRING",
	"exprlist : exprlist ',' expr",
	"exprlist : exprlist ',' STRING",
	"fromstmt : FROM NAME",
	"fromstmt : FROM NAME '=' intexpr TO intexpr opinc stmtlist '}'",
	"fromstmt : FROM error",
	"opinc : /* empty */",
	"opinc : BY intexpr",
	"forallstmt : FORALL1 NAME",
	"forallstmt : FORALL1 NAME stmtlist '}'",
	"forallstmt : FORALL1 error",
	"whilestmt : WHILE '(' expr ')' stmtlist '}'",
	"ifstmt : IF '(' expr ')' stmtlist '}' optelseif optelse",
	"optelseif : /* empty */",
	"optelseif : optelseif ELSE IF '(' expr ')' stmtlist '}'",
	"optelse : /* empty */",
	"optelse : ELSE stmtlist '}'",
	"derivblk : DERIVATIVE NAME stmtlist '}'",
	"linblk : LINEAR NAME solvefor",
	"linblk : LINEAR NAME solvefor stmtlist '}'",
	"nonlinblk : NONLINEAR NAME solvefor stmtlist '}'",
	"discretblk : DISCRETE NAME stmtlist '}'",
	"partialblk : PARTIAL NAME stmtlist '}'",
	"partialblk : PARTIAL error",
	"pareqn : '~' PRIME '=' NAME '*' DEL2 '(' NAME ')' '+' NAME",
	"pareqn : '~' DEL NAME '[' firstlast ']' '=' expr",
	"pareqn : '~' NAME '[' firstlast ']' '=' expr",
	"firstlast : FIRST",
	"firstlast : LAST",
	"funcblk : FUNCTION1 NAME '(' arglist ')' units",
	"funcblk : FUNCTION1 NAME '(' arglist ')' units stmtlist '}'",
	"arglist : /* empty */",
	"arglist : /* empty */",
	"arglist : arglist1",
	"arglist1 : name units",
	"arglist1 : arglist1 ',' name units",
	"procedblk : PROCEDURE NAME '(' arglist ')' units stmtlist '}'",
	"solveblk : SOLVE NAME ifsolerr",
	"solveblk : SOLVE NAME USING METHOD ifsolerr",
	"solveblk : SOLVE error",
	"ifsolerr : /* empty */",
	"ifsolerr : IFERROR stmtlist '}'",
	"solvefor : /* empty */",
	"solvefor : solvefor1",
	"solvefor1 : SOLVEFOR NAME",
	"solvefor1 : solvefor1 ',' NAME",
	"solvefor1 : SOLVEFOR error",
	"brkptblk : BREAKPOINT stmtlist '}'",
	"terminalblk : TERMINAL stmtlist '}'",
	"sens : SENS senslist",
	"sens : SENS error",
	"senslist : varname",
	"senslist : senslist ',' varname",
	"conserve : CONSERVE react '=' expr",
	"conserve : CONSERVE error",
	"compart : COMPARTMENT NAME ','",
	"compart : COMPARTMENT NAME ',' expr '{' namelist '}'",
	"compart : COMPARTMENT expr '{' namelist '}'",
	"namelist : NAME",
	"namelist : namelist NAME",
	"kineticblk : KINETIC NAME solvefor stmtlist '}'",
	"reaction : REACTION react REACT1",
	"reaction : REACTION react REACT1 react '(' expr ',' expr ')'",
	"reaction : REACTION react LT LT '(' expr ')'",
	"reaction : REACTION react '-' GT '(' expr ')'",
	"reaction : REACTION error",
	"react : varname",
	"react : integer varname",
	"react : react '+' varname",
	"react : react '+' integer varname",
	"lagstmt : LAG name BY NAME",
	"lagstmt : LAG error",
	"queuestmt : PUTQ name",
	"queuestmt : GETQ name",
	"matchblk : MATCH",
	"matchblk : MATCH '{' matchlist '}'",
	"matchlist : match",
	"matchlist : matchlist match",
	"match : name",
	"match : matchname '(' expr ')' '=' expr",
	"match : error",
	"matchname : name",
	"matchname : name '[' NAME ']'",
	"unitblk : UNITS '{' unitbody '}'",
	"unitbody : /* empty */",
	"unitbody : unitbody unitdef",
	"unitbody : unitbody factordef",
	"unitdef : unit '=' unit",
	"unitdef : unit error",
	"factordef : NAME '=' real unit",
	"factordef : NAME '=' unit unit",
	"factordef : NAME '=' unit '-' GT unit",
	"factordef : error",
	"constblk : CONSTANT '{' conststmt '}'",
	"conststmt : /* empty */",
	"conststmt : conststmt NAME '=' number units",
	"tablestmt : TABLE tablst dependlst FROM expr TO expr WITH INTEGER",
	"tablestmt : TABLE error",
	"tablst : /* empty */",
	"tablst : tablst1",
	"tablst1 : Name",
	"tablst1 : tablst1 ',' Name",
	"dependlst : /* empty */",
	"dependlst : DEPEND tablst1",
	"neuronblk : NEURON '{' nrnstmt '}'",
	"nrnstmt : /* empty */",
	"nrnstmt : nrnstmt SUFFIX NAME",
	"nrnstmt : nrnstmt nrnuse",
	"nrnstmt : nrnstmt NONSPECIFIC nrnlist",
	"nrnstmt : nrnstmt ELECTRODE_CURRENT nrnlist",
	"nrnstmt : nrnstmt SECTION nrnlist",
	"nrnstmt : nrnstmt RANGE nrnlist",
	"nrnstmt : nrnstmt GLOBAL nrnlist",
	"nrnstmt : nrnstmt POINTER nrnlist",
	"nrnstmt : nrnstmt EXTERNAL nrnlist",
	"nrnuse : USEION NAME READ nrnlist valence",
	"nrnuse : USEION NAME WRITE nrnlist valence",
	"nrnuse : USEION NAME READ nrnlist WRITE nrnlist valence",
	"nrnuse : error",
	"nrnlist : NAME",
	"nrnlist : nrnlist ',' NAME",
	"nrnlist : error",
	"valence : /* empty */",
	"valence : VALENCE real",
};
#endif /* YYDEBUG */
#line 1 "/usr/lib/yaccpar"
/*	@(#)yaccpar 1.10 89/04/04 SMI; from S5R3 1.10	*/

/*
** Skeleton parser driver for yacc output
*/

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	{  return(0); }
#define YYABORT		{  return(1); }
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-1000)

/*
** static variables used by the parser
*/
static YYSTYPE *yyv;			/* value stack */
static int *yys;			/* state stack */

static YYSTYPE *yypv;			/* top of value stack */
static int *yyps;			/* top of state stack */

static int yystate;			/* current state */
static int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */

int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */


/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
int
yyparse()
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */
	unsigned yymaxdepth = YYMAXDEPTH;

	/*
	** Initialize externals - yyparse may be called more than once
	*/
if (!yyv) {

	yyv = (YYSTYPE*)malloc(yymaxdepth*sizeof(YYSTYPE));
	yys = (int*)malloc(yymaxdepth*sizeof(int));
}

	if (!yyv || !yys)
	{
		yyerror( "out of memory" );
		return(1);
	}
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

	goto yystack;
	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			(void)printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				(void)printf( "end-of-file\n" );
			else if ( yychar < 0 )
				(void)printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				(void)printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ yymaxdepth ] )	/* room on stack? */
		{
			/*
			** reallocate and recover.  Note that pointers
			** have to be reset, or bad things will happen
			*/
			int yyps_index = (yy_ps - yys);
			int yypv_index = (yy_pv - yyv);
			int yypvt_index = (yypvt - yyv);
			yymaxdepth += YYMAXDEPTH;
			yyv = (YYSTYPE*)realloc((char*)yyv,
				yymaxdepth * sizeof(YYSTYPE));
			yys = (int*)realloc((char*)yys,
				yymaxdepth * sizeof(int));
			if (!yyv || !yys)
			{
				yyerror( "yacc stack overflow" );
				return(1);
			}
			yy_ps = yys + yyps_index;
			yy_pv = yyv + yypv_index;
			yypvt = yyv + yypvt_index;
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			(void)printf( "Received token " );
			if ( yychar == 0 )
				(void)printf( "end-of-file\n" );
			else if ( yychar < 0 )
				(void)printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				(void)printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				(void)printf( "Received token " );
				if ( yychar == 0 )
					(void)printf( "end-of-file\n" );
				else if ( yychar < 0 )
					(void)printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					(void)printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( "syntax error" );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						(void)printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					(void)printf( "Error recovery discards " );
					if ( yychar == 0 )
						(void)printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						(void)printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						(void)printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			(void)printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 1:
# line 395 "parse1.y"
{/*ldebug(top, intoken)*/;} break;
case 2:
# line 396 "parse1.y"
{diag("Illegal block", (char *)0);} break;
case 5:
# line 402 "parse1.y"
{replacstr(yypvt[-0].qp, "static double");
		movelist(yypvt[-0].qp, lastok->next, firstlist);} break;
case 8:
# line 406 "parse1.y"
{model_level = atoi(STR(yypvt[-0].qp));} break;
case 9:
# line 407 "parse1.y"
{model_level = 0;} break;
case 10:
# line 409 "parse1.y"
{if (sensused)
			diag("sensitivity analysis not implemented for",
				" this block type");
		} break;
case 12:
# line 416 "parse1.y"
{inblock(SYM(yypvt[-0].qp)->name); replacstr(yypvt[-0].qp, "\n/*VERBATIM*/\n");
		movelist(yypvt[-0].qp,intoken->prev, procfunc);} break;
case 13:
# line 420 "parse1.y"
{inblock(SYM(yypvt[-0].qp)->name); deltokens(yypvt[-0].qp, intoken->prev);} break;
case 15:
# line 423 "parse1.y"
{include_file(yypvt[-0].qp);} break;
case 16:
# line 426 "parse1.y"
{if (modelline == NULL) modelline = yypvt[-0].str;
			deltokens(yypvt[-1].qp, intoken->prev);} break;
case 17:
# line 429 "parse1.y"
{yyval.str = inputline();} break;
case 18:
# line 434 "parse1.y"
{ Symbol *sp = SYM(yypvt[-1].qp);
	 if (sp->subtype)
		diag(sp->name, " used before DEFINEed");
	 sp->u.str = STR(yypvt[-0].qp);
	 sp->type = DEFINEDVAR;
	 deltokens(yypvt[-2].qp, yypvt[-0].qp);} break;
case 19:
# line 440 "parse1.y"
{myerr("syntax: DEFINE name integer");} break;
case 20:
# line 443 "parse1.y"
{ Symbol *checklocal();
		  SYM(yypvt[-0].qp) = checklocal(SYM(yypvt[-0].qp));  /* it was a bug
			when this was done to the lookahead token in lex */
		} break;
case 30:
# line 451 "parse1.y"
{deltokens(yypvt[-3].qp, yypvt[-0].qp);} break;
case 32:
# line 455 "parse1.y"
{ explicit_decl(model_level, yypvt[-0].qp);} break;
case 33:
# line 461 "parse1.y"
{parminstall(SYM(yypvt[-3].qp), STR(yypvt[-1].qp), yypvt[-0].str);} break;
case 34:
# line 463 "parse1.y"
{parminstall(SYM(yypvt[-1].qp), "0", yypvt[-0].str);} break;
case 35:
# line 464 "parse1.y"
{diag("name = number", (char *)0);} break;
case 36:
# line 466 "parse1.y"
{yyval.str = stralloc("", (char *)0);} break;
case 38:
# line 469 "parse1.y"
{yyval.str = inputtopar();} break;
case 39:
# line 471 "parse1.y"
{yyval.str = yypvt[-1].str; delete(yypvt[-2].qp); delete(yypvt[-0].qp);} break;
case 40:
# line 473 "parse1.y"
{replacstr(yypvt[-0].qp, "");} break;
case 41:
# line 474 "parse1.y"
{replacstr(yypvt[-0].qp, "");} break;
case 42:
# line 476 "parse1.y"
{deltokens(yypvt[-3].qp, yypvt[-0].qp);} break;
case 44:
# line 480 "parse1.y"
{ explicit_decl(model_level, yypvt[-0].qp); } break;
case 45:
# line 484 "parse1.y"
{steppedinstall(SYM(yypvt[-3].qp), yypvt[-1].qp, lastok, yypvt[-0].str);} break;
case 50:
# line 492 "parse1.y"
{lastok = yypvt[-0].qp;} break;
case 51:
# line 495 "parse1.y"
{ Sprintf(buf, "-%s", STR(yypvt[-0].qp));
		 STR(yypvt[-0].qp) = stralloc(buf, STR(yypvt[-0].qp)); yyval.qp = yypvt[-0].qp;
		delete(yypvt[-1].qp); lastok = yypvt[-0].qp;
		} break;
case 55:
# line 503 "parse1.y"
{replacstr(yypvt[-0].qp, SYM(yypvt[-0].qp)->u.str);} break;
case 56:
# line 505 "parse1.y"
{lastok = yypvt[-0].qp;} break;
case 57:
# line 507 "parse1.y"
{Sprintf(buf, "%s.0", STR(yypvt[-0].qp));
		STR(yypvt[-0].qp) = stralloc(buf, STR(yypvt[-0].qp));
		lastok = yypvt[-0].qp;
		} break;
case 58:
# line 512 "parse1.y"
{deltokens(yypvt[-3].qp, yypvt[-0].qp);} break;
case 60:
# line 516 "parse1.y"
{ explicit_decl(model_level, yypvt[-0].qp); } break;
case 61:
# line 517 "parse1.y"
{scopindep = 1;} break;
case 62:
# line 518 "parse1.y"
{ explicit_decl(model_level, yypvt[-0].qp);
		  scopindep = 0;
		} break;
case 63:
# line 524 "parse1.y"
{indepinstall(SYM(yypvt[-8].qp), STR(yypvt[-6].qp),
		 STR(yypvt[-4].qp), STR(yypvt[-2].qp), yypvt[-1].qp, yypvt[-0].str, scopindep);
		} break;
case 64:
# line 527 "parse1.y"
{diag("name FROM number TO number WITH integer\n", (char *)0);} break;
case 66:
# line 531 "parse1.y"
{stateblock = 0;} break;
case 67:
# line 531 "parse1.y"
{deltokens(yypvt[-4].qp, yypvt[-0].qp);} break;
case 69:
# line 535 "parse1.y"
{ explicit_decl(model_level, yypvt[-0].qp);} break;
case 70:
# line 539 "parse1.y"
{depinstall(stateblock, SYM(yypvt[-2].qp), 0, "0", "1", yypvt[-0].str, yypvt[-1].qp, 1);
		 } break;
case 71:
# line 542 "parse1.y"
{int i = atoi(STR(yypvt[-3].qp));
		 if (i < 1) diag("Array index must be > 0", (char *)0);
		 depinstall(stateblock, SYM(yypvt[-5].qp), i,
			 "0", "1", yypvt[-0].str, yypvt[-1].qp, 1);
		} break;
case 72:
# line 548 "parse1.y"
{depinstall(stateblock, SYM(yypvt[-6].qp), 0, STR(yypvt[-4].qp),
		 STR(yypvt[-2].qp), yypvt[-0].str, yypvt[-1].qp, 1);
		} break;
case 73:
# line 552 "parse1.y"
{int i = atoi(STR(yypvt[-7].qp));
		 if (i < 1) diag("Array index must be > 0", (char *)0);
		 depinstall(stateblock, SYM(yypvt[-9].qp), i,
		 STR(yypvt[-4].qp), STR(yypvt[-2].qp), yypvt[-0].str, yypvt[-1].qp, 1);
		} break;
case 74:
# line 557 "parse1.y"
{
diag("name FROM number TO number START number\n",
"FROM...TO and START are optional, name can be name[integer]\n");} break;
case 75:
# line 561 "parse1.y"
{yyval.qp = ITEM0;} break;
case 76:
# line 562 "parse1.y"
{yyval.qp = yypvt[-0].qp;} break;
case 77:
# line 564 "parse1.y"
{stateblock = 1;} break;
case 78:
# line 564 "parse1.y"
{deltokens(yypvt[-4].qp, yypvt[-0].qp);} break;
case 79:
# line 568 "parse1.y"
{ Item *q;
		 q = linsertsym(plotlist, SYM(yypvt[-1].qp)); Insertstr(q->next, yypvt[-0].str); } break;
case 80:
# line 571 "parse1.y"
{ diag("PLOT namelist VS name", (char *)0);} break;
case 81:
# line 574 "parse1.y"
{ if (plotlist->next == plotlist) {
			Lappendsym(plotlist, SYM(yypvt[-1].qp));
			Lappendstr(plotlist, yypvt[-0].str);
		  }else{
			diag("Only one PLOT declaration allowed", (char *)0);
		  }
		} break;
case 82:
# line 582 "parse1.y"
{ Lappendsym(plotlist, SYM(yypvt[-1].qp)); Lappendstr(plotlist, yypvt[-0].str);} break;
case 83:
# line 585 "parse1.y"
{ yyval.str = "-1";} break;
case 84:
# line 587 "parse1.y"
{ yyval.str = STR(yypvt[-1].qp);} break;
case 85:
# line 589 "parse1.y"
{blocktype = INITIAL1;} break;
case 87:
# line 591 "parse1.y"
{lexcontext = NONLINEAR; blocktype = DERIVATIVE;} break;
case 89:
# line 592 "parse1.y"
{blocktype = BREAKPOINT;} break;
case 91:
# line 593 "parse1.y"
{lexcontext = blocktype = LINEAR;} break;
case 93:
# line 594 "parse1.y"
{lexcontext = blocktype = NONLINEAR;} break;
case 95:
# line 595 "parse1.y"
{blocktype = FUNCTION1;} break;
case 97:
# line 596 "parse1.y"
{blocktype = PROCEDURE;} break;
case 99:
# line 597 "parse1.y"
{blocktype = TERMINAL;} break;
case 101:
# line 598 "parse1.y"
{blocktype = DISCRETE;} break;
case 103:
# line 599 "parse1.y"
{lexcontext = blocktype = PARTIAL;} break;
case 105:
# line 600 "parse1.y"
{lexcontext = blocktype = KINETIC;ostmt_start();} break;
case 106:
# line 600 "parse1.y"
{see_ostmt();} break;
case 107:
# line 603 "parse1.y"
{movelist(yypvt[-1].qp, yypvt[-0].qp, initfunc);} break;
case 108:
# line 605 "parse1.y"
{pushlocal();} break;
case 109:
# line 605 "parse1.y"
{poplocal();} break;
case 110:
# line 607 "parse1.y"
{poplocal();} break;
case 111:
# line 610 "parse1.y"
{ replacstr(yypvt[-1].qp, "double");
		  Insertstr(lastok->next, ";\n");
		} break;
case 112:
# line 613 "parse1.y"
{myerr("Illegal LOCAL declaration");} break;
case 113:
# line 618 "parse1.y"
{pushlocal();
		 SYM(yypvt[-1].qp) = copylocal(SYM(yypvt[-1].qp));
		 if (yypvt[-0].i) {
			SYM(yypvt[-1].qp)->araydim = yypvt[-0].i;
			SYM(yypvt[-1].qp)->subtype |= ARRAY;
		 }else{
			lastok = yypvt[-1].qp;
		 }
		} break;
case 114:
# line 628 "parse1.y"
{SYM(yypvt[-1].qp) = copylocal(SYM(yypvt[-1].qp));
		 if (yypvt[-0].i) {
			SYM(yypvt[-1].qp)->araydim = yypvt[-0].i;
			SYM(yypvt[-1].qp)->subtype |= ARRAY;
		 }else{
			lastok = yypvt[-1].qp;
		 }
		} break;
case 115:
# line 638 "parse1.y"
{yyval.i = 0;} break;
case 116:
# line 640 "parse1.y"
{yyval.i = atoi(STR(yypvt[-1].qp)); lastok = yypvt[-0].qp;} break;
case 118:
# line 643 "parse1.y"
{if (blocktype == KINETIC) see_ostmt();} break;
case 119:
# line 644 "parse1.y"
{if (blocktype == KINETIC) see_ostmt();} break;
case 120:
# line 645 "parse1.y"
{if (blocktype == KINETIC) { see_astmt(yypvt[-0].qp, astmt_end_); }} break;
case 127:
# line 654 "parse1.y"
{inblock(SYM(yypvt[-0].qp)->name);
		replacstr(yypvt[-0].qp, "\n/*VERBATIM*/\n");} break;
case 128:
# line 658 "parse1.y"
{inblock(SYM(yypvt[-0].qp)->name); deltokens(yypvt[-0].qp, intoken->prev);} break;
case 130:
# line 660 "parse1.y"
{check_block(KINETIC, blocktype, "COMPARTMENT");} break;
case 131:
# line 661 "parse1.y"
{check_block(KINETIC, blocktype, "CONSERVE");} break;
case 134:
# line 665 "parse1.y"
{ replacstr(yypvt[-0].qp, " _reset = 1;\n"); } break;
case 137:
# line 669 "parse1.y"
{if (blocktype !=FUNCTION1 && blocktype != PROCEDURE) {
			diag("TABLE valid only for FUNCTION or PROCEDURE", (char *)0);
		}} break;
case 139:
# line 674 "parse1.y"
{myerr("Illegal statement");} break;
case 140:
# line 678 "parse1.y"
{astmt_end_ = insertsym(lastok->next, semi);} break;
case 141:
# line 679 "parse1.y"
{inequation = 1;} break;
case 142:
# line 679 "parse1.y"
{
		yyval.qp = yypvt[-0].qp; inequation = 0;
		astmt_end_ = insertstr(lastok->next->next->next, "");} break;
case 143:
# line 683 "parse1.y"
{astmt_end_ = insertsym(lastok->next, semi);} break;
case 144:
# line 688 "parse1.y"
{ if (blocktype == DERIVATIVE && SYM(yypvt[-2].qp)->type == PRIME) {
			/* put Dvar in a derivative used list */
			deriv_used(SYM(yypvt[-2].qp));
			}
		  if (blocktype == DERIVATIVE && (SYM(yypvt[-2].qp)->subtype & STAT)) {
			Fprintf(stderr,
"WARNING: %s (a STATE) is assigned a value\
 in a DERIVATIVE block.\n Multistep integrators (such as Runge) may not\
 work correctly.\n", SYM(yypvt[-2].qp)->name);
		  }
		  if (blocktype == DISCRETE && SYM(yypvt[-2].qp)->type == NAME
			&& (SYM(yypvt[-2].qp)->subtype & STAT)) {
			SYM(yypvt[-2].qp)->used++;
			}
		} break;
case 145:
# line 705 "parse1.y"
{ inequation = 0;
		Insertstr(yypvt[-2].qp, " -(");
		replacstr(yypvt[-1].qp, ") + ");
		if (nstate == 0)
{yyerror("previous equation contains no state variables"); YYERROR;}
		 eqnqueue(yypvt[-3].qp);
		} break;
case 146:
# line 713 "parse1.y"
{ inequation = 0;
		delete(yypvt[-1].qp);
		if (nstate == 0)
{yyerror("previous equation contains no state variables"); YYERROR;}
		} break;
case 147:
# line 721 "parse1.y"
{lastok = yypvt[-0].qp;
		if (!extdef2){SYM(yypvt[-0].qp)->usage |= DEP;}
		if (SYM(yypvt[-0].qp)->subtype & ARRAY && !extdef2)
			{myerr("variable needs an index");}
		if (inequation && (SYM(yypvt[-0].qp)->subtype & STAT) && in_solvefor(SYM(yypvt[-0].qp))) {
			SYM(yypvt[-0].qp)->used++;
			nstate++; pstate++; tstate++;
		}
		if (SYM(yypvt[-0].qp)->subtype & INTGER) {
			lastok = insertstr(yypvt[-0].qp->next, ")");
			yypvt[-0].qp = insertstr(yypvt[-0].qp, "((double)");
		}
		} break;
case 148:
# line 735 "parse1.y"
{lastok = yypvt[-0].qp;
		SYM(yypvt[-3].qp)->usage |= DEP;
		if ((SYM(yypvt[-3].qp)->subtype & ARRAY) == 0)
			{myerr("variable is not an array");}
		if (inequation && (SYM(yypvt[-3].qp)->subtype & STAT) && in_solvefor(SYM(yypvt[-3].qp))) {
			SYM(yypvt[-3].qp)->used++;
			nstate++; pstate++; tstate++;
		}
		  if (forallindex) {
			if (forallindex == -1) {
				forallindex = SYM(yypvt[-3].qp)->araydim;
			}
			if (forallindex != SYM(yypvt[-3].qp)->araydim) {
Sprintf(buf, "%s dimension not same as other dimensions used in FORALL statement",
SYM(yypvt[-3].qp)->name);
				diag(buf, (char *)0);
			}
		  }
		} break;
case 149:
# line 755 "parse1.y"
{lastok = yypvt[-0].qp;
		SYM(yypvt[-2].qp)->usage |= DEP; disc_var_seen(yypvt[-2].qp, yypvt[-1].qp, yypvt[-0].qp, 0);} break;
case 150:
# line 758 "parse1.y"
{lastok = yypvt[-0].qp;
		SYM(yypvt[-5].qp)->usage |= DEP; disc_var_seen(yypvt[-5].qp, yypvt[-4].qp, yypvt[-3].qp, ARRAY);} break;
case 151:
# line 762 "parse1.y"
{lastok = yypvt[-0].qp;
		 SYM(yypvt[-0].qp)->usage |= DEP;
		 if (!(SYM(yypvt[-0].qp)->subtype & INTGER)) {
		 	lastok = insertstr(yypvt[-0].qp->next, ")");
			yypvt[-0].qp = insertstr(yypvt[-0].qp, "((int)");
		 }
		} break;
case 152:
# line 769 "parse1.y"
{ lastok = yypvt[-0].qp;} break;
case 153:
# line 770 "parse1.y"
{ lastok = yypvt[-0].qp;} break;
case 158:
# line 775 "parse1.y"
{myerr("Illegal integer expression");} break;
case 162:
# line 780 "parse1.y"
{lastok = yypvt[-0].qp;} break;
case 167:
# line 786 "parse1.y"
{ yyval.qp = insertstr(yypvt[-2].qp, "pow("); replacstr(yypvt[-1].qp, ",");
			lastok = insertstr(lastok->next, ")"); } break;
case 168:
# line 788 "parse1.y"
{replacstr(yypvt[-1].qp, " ||");} break;
case 169:
# line 789 "parse1.y"
{replacstr(yypvt[-1].qp, " &&");} break;
case 174:
# line 794 "parse1.y"
{replacstr(yypvt[-1].qp, " ==");} break;
case 175:
# line 795 "parse1.y"
{replacstr(yypvt[-1].qp, " !=");} break;
case 176:
# line 796 "parse1.y"
{replacstr(yypvt[-1].qp, " !");} break;
case 178:
# line 798 "parse1.y"
{myerr("Illegal expression");} break;
case 179:
# line 800 "parse1.y"
{inequation = 1; nstate = 0;} break;
case 180:
# line 804 "parse1.y"
{inequation = 1; nstate = 0;
		pstate = 0; tstate = 0; init_lineq(yypvt[-0].qp); leftside = -1;} break;
case 181:
# line 807 "parse1.y"
{leftside = 1;} break;
case 182:
# line 811 "parse1.y"
{linterm(yypvt[-0].qp, lastok, pstate, leftside); pstate = 0;} break;
case 183:
# line 813 "parse1.y"
{delete(yypvt[-1].qp);
		linterm(yypvt[-0].qp, lastok, pstate, -leftside); pstate = 0;} break;
case 184:
# line 816 "parse1.y"
{delete(yypvt[-1].qp);
		linterm(yypvt[-0].qp, lastok, pstate, leftside); pstate = 0;} break;
case 185:
# line 819 "parse1.y"
{delete(yypvt[-1].qp);
		linterm(yypvt[-0].qp, lastok, pstate, -leftside); pstate = 0;} break;
case 186:
# line 823 "parse1.y"
{ if (tstate == 1) {
			lin_state_term(yypvt[-0].qp, lastok);
		  }
		  tstate = 0;
		} break;
case 187:
# line 829 "parse1.y"
{ if (tstate == 1) {
			lin_state_term(yypvt[-0].qp, lastok);
		  }
		  tstate = 0;
		} break;
case 188:
# line 835 "parse1.y"
{ if (tstate) {
			diag("state ocurs in denominator", (char *)0);
		  }
		} break;
case 191:
# line 842 "parse1.y"
{if (tstate) diag("states not permitted in function calls",
			(char *)0);} break;
case 192:
# line 844 "parse1.y"
{ lastok = yypvt[-0].qp;
			if (tstate) diag("states not permitted between ",
				"parentheses");} break;
case 193:
# line 848 "parse1.y"
{diag("Some operators are not permitted in linear\n",
"expressions unless the terms containing them are\nenclosed in parentheses");} break;
case 194:
# line 852 "parse1.y"
{ if (SYM(yypvt[-1].qp)->subtype & EXTDEF2) { extdef2 = 1;}} break;
case 195:
# line 854 "parse1.y"
{lastok = yypvt[-0].qp; SYM(yypvt[-4].qp)->usage |= FUNCT;
		 if (SYM(yypvt[-4].qp)->subtype & EXTDEF2) { extdef2 = 0;}
		 if (SYM(yypvt[-4].qp)->subtype & EXTDEF3) { add_reset_args(yypvt[-3].qp);}
#if VECTORIZE
		 vectorize_use_func(yypvt[-4].qp,yypvt[-3].qp,yypvt[-1].qp,yypvt[-0].qp);
#endif
		} break;
case 196:
# line 862 "parse1.y"
{yyval.qp = ITEM0;} break;
case 201:
# line 868 "parse1.y"
{pushlocal(); SYM(yypvt[-0].qp) = copylocal(SYM(yypvt[-0].qp));
			SYM(yypvt[-0].qp)->subtype |= INTGER;} break;
case 202:
# line 872 "parse1.y"
{ replacstr(yypvt[-9].qp, "{int ");
		poplocal();
		Insertstr(yypvt[-6].qp, ";for (");
		Insertstr(yypvt[-6].qp, SYM(yypvt[-8].qp)->name);
		Insertstr(yypvt[-4].qp, ";");
		Insertstr(yypvt[-4].qp, SYM(yypvt[-8].qp)->name);
		replacstr(yypvt[-4].qp, "<=");
		if (yypvt[-2].qp) {
			Insertstr(yypvt[-2].qp, ";");
			Insertstr(yypvt[-2].qp, SYM(yypvt[-8].qp)->name);
			replacstr(yypvt[-2].qp, "+=");
		} else {
			Insertstr(yypvt[-1].qp, ";");
			Insertstr(yypvt[-1].qp, SYM(yypvt[-8].qp)->name);
			Insertstr(yypvt[-1].qp, "++");
		}
		Insertstr(yypvt[-1].qp, ")");
		Insertstr(yypvt[-0].qp, "}");
		} break;
case 203:
# line 891 "parse1.y"
{
myerr("FROM intvar = intexpr TO intexpr BY intexpr { statements }");} break;
case 204:
# line 894 "parse1.y"
{yyval.qp = ITEM0;} break;
case 206:
# line 897 "parse1.y"
{pushlocal(); SYM(yypvt[-0].qp) = copylocal(SYM(yypvt[-0].qp));
			  SYM(yypvt[-0].qp)->subtype |= INTGER;
			  if (forallindex) {
diag("Nested FORALL statements not allowed", (char *)0);
			  }
			  forallindex = -1;
			} break;
case 207:
# line 906 "parse1.y"
{ replacstr(yypvt[-4].qp, "{int ");
		poplocal();
		if (forallindex == -1) {
diag("FORALL range is undefined since no arrays used", " within the statement");
		}
		Sprintf(buf, "; for (%s=0; %s<%d; %s++)", SYM(yypvt[-3].qp)->name,
			SYM(yypvt[-3].qp)->name, forallindex, SYM(yypvt[-3].qp)->name);
		Insertstr(yypvt[-1].qp, buf);
		Insertstr(yypvt[-0].qp, "}");
		} break;
case 208:
# line 916 "parse1.y"
{
myerr("FORALL intvar { statements }");} break;
case 212:
# line 925 "parse1.y"
{
#if VECTORIZE
			vectorize_if_else_stmt(blocktype);
#endif
		} break;
case 214:
# line 933 "parse1.y"
{
#if VECTORIZE
			vectorize_if_else_stmt(blocktype);
#endif
		} break;
case 215:
# line 941 "parse1.y"
{massagederiv(yypvt[-3].qp, yypvt[-2].qp, yypvt[-1].qp, yypvt[-0].qp, sensused); sensused = 0;} break;
case 216:
# line 943 "parse1.y"
{init_linblk(yypvt[-1].qp);} break;
case 217:
# line 945 "parse1.y"
{massage_linblk(yypvt[-5].qp, yypvt[-4].qp, yypvt[-1].qp, yypvt[-0].qp, sensused);
		lexcontext = 0; sensused = 0;
		} break;
case 218:
# line 951 "parse1.y"
{massagenonlin(yypvt[-4].qp, yypvt[-3].qp, yypvt[-1].qp, yypvt[-0].qp, sensused);
		lexcontext = 0; sensused = 0;
		} break;
case 219:
# line 957 "parse1.y"
{massagediscblk(yypvt[-3].qp, yypvt[-2].qp, yypvt[-1].qp, yypvt[-0].qp);} break;
case 220:
# line 961 "parse1.y"
{massagepartial(yypvt[-3].qp, yypvt[-2].qp, yypvt[-1].qp, yypvt[-0].qp);
		lexcontext = 0;
		} break;
case 221:
# line 964 "parse1.y"
{
diag("within the PARTIAL block must occur an equation with the syntax ---\n",
"~ V' = F*DEL2(V) + G\n"); } break;
case 222:
# line 969 "parse1.y"
{partial_eqn(yypvt[-9].qp, yypvt[-7].qp, yypvt[-3].qp, yypvt[-0].qp);} break;
case 223:
# line 971 "parse1.y"
{partial_bndry(0, yypvt[-5].qp, yypvt[-3].qp, yypvt[-0].qp, lastok);} break;
case 224:
# line 973 "parse1.y"
{partial_bndry(2, yypvt[-5].qp, yypvt[-3].qp, yypvt[-0].qp, lastok);} break;
case 227:
# line 978 "parse1.y"
{IGNORE(copylocal(SYM(yypvt[-4].qp)));} break;
case 228:
# line 982 "parse1.y"
{ Symbol *s = SYM(yypvt[-7].qp);
		table_massage(table_list, yypvt[-8].qp, yypvt[-7].qp, yypvt[-5].lp); freelist(&table_list);
#define GLOBFUNCT 1
#if GLOBFUNCT && NMODL
		replacstr(yypvt[-8].qp, "\ndouble");
#else
		replacstr(yypvt[-8].qp, "\nstatic double");
#endif
		defarg(yypvt[-6].qp, yypvt[-4].qp);
		Sprintf(buf, "double _l%s;\n", s->name);
		Insertstr(yypvt[-1].qp->next, buf);
		Sprintf(buf, "\nreturn _l%s;\n", s->name);
		Insertstr(yypvt[-0].qp, buf);
		movelist(yypvt[-8].qp, yypvt[-0].qp, procfunc);
		if (SYM(yypvt[-7].qp)->subtype & FUNCT) {
			diag(SYM(yypvt[-7].qp)->name, " declared as FUNCTION twice");
		}
		SYM(yypvt[-7].qp)->subtype |= FUNCT;
		SYM(yypvt[-7].qp)->usage |= FUNCT;
#if HMODL || NMODL
		hocfunc(s, yypvt[-6].qp, yypvt[-4].qp);
#endif
		poplocal(); freelist(&yypvt[-5].lp);} break;
case 229:
# line 1006 "parse1.y"
{pushlocal(); yyval.lp = LIST0;} break;
case 230:
# line 1007 "parse1.y"
{pushlocal();} break;
case 231:
# line 1007 "parse1.y"
{yyval.lp = yypvt[-0].lp;} break;
case 232:
# line 1010 "parse1.y"
{SYM(yypvt[-1].qp) = copylocal(SYM(yypvt[-1].qp));
		 yyval.lp = newlist(); Lappendsym(yyval.lp, SYM(yypvt[-1].qp));
		} break;
case 233:
# line 1014 "parse1.y"
{SYM(yypvt[-1].qp) = copylocal(SYM(yypvt[-1].qp)); Lappendsym(yyval.lp, SYM(yypvt[-1].qp));} break;
case 234:
# line 1017 "parse1.y"
{Symbol *s = SYM(yypvt[-6].qp);
		s->u.i = 0; 	/* avoid objectcenter warning if solved */
		table_massage(table_list, yypvt[-7].qp, yypvt[-6].qp, yypvt[-4].lp); freelist(&table_list);
		replacstr(yypvt[-7].qp, "\nstatic int "); defarg(yypvt[-5].qp, yypvt[-3].qp);
		Insertstr(yypvt[-0].qp, " return 0;");
		movelist(yypvt[-7].qp, yypvt[-0].qp, procfunc);
		if (SYM(yypvt[-6].qp)->subtype & PROCED) {
			diag(SYM(yypvt[-6].qp)->name, " declared as PROCEDURE twice");
		}
		SYM(yypvt[-6].qp)->subtype |= PROCED;
		SYM(yypvt[-6].qp)->usage |= FUNCT;
#if HMODL || NMODL
		hocfunc(s, yypvt[-5].qp, yypvt[-3].qp);
#endif
		poplocal(); freelist(&yypvt[-4].lp);} break;
case 235:
# line 1035 "parse1.y"
{ solvequeue(yypvt[-1].qp, ITEM0, blocktype, yypvt[-0].qp); } break;
case 236:
# line 1037 "parse1.y"
{ solvequeue(yypvt[-3].qp, yypvt[-1].qp, blocktype, yypvt[-0].qp); } break;
case 237:
# line 1038 "parse1.y"
{ myerr("Illegal SOLVE statement");} break;
case 238:
# line 1041 "parse1.y"
{ yyval.qp = ITEM0; } break;
case 239:
# line 1043 "parse1.y"
{ yyval.qp = yypvt[-0].qp; } break;
case 240:
# line 1046 "parse1.y"
{ if (solveforlist) {
			freelist(&solveforlist);
		  }
		} break;
case 242:
# line 1053 "parse1.y"
{ solveforlist = newlist(); Lappendsym(solveforlist, SYM(yypvt[-0].qp));
		  delete(yypvt[-1].qp); delete(yypvt[-0].qp);
		} break;
case 243:
# line 1057 "parse1.y"
{ Lappendsym(solveforlist, SYM(yypvt[-0].qp)); delete(yypvt[-1].qp); delete(yypvt[-0].qp);} break;
case 244:
# line 1058 "parse1.y"
{ myerr("Syntax: SOLVEFOR name, name, ...");} break;
case 245:
# line 1062 "parse1.y"
{movelist(yypvt[-1].qp, yypvt[-0].qp, modelfunc);} break;
case 246:
# line 1065 "parse1.y"
{movelist(yypvt[-1].qp, yypvt[-0].qp, termfunc);} break;
case 247:
# line 1069 "parse1.y"
{ sensused = 1;
		  delete(yypvt[-1].qp);
		} break;
case 248:
# line 1072 "parse1.y"
{myerr("syntax is SENS var1, var2, var3, etc");} break;
case 249:
# line 1076 "parse1.y"
{ sensparm(yypvt[-0].qp); delete(yypvt[-0].qp);} break;
case 250:
# line 1078 "parse1.y"
{ sensparm(yypvt[-0].qp); deltokens(yypvt[-1].qp, yypvt[-0].qp);} break;
case 251:
# line 1082 "parse1.y"
{/* react originally designed for reactions and is unchanged*/
		massageconserve(yypvt[-3].qp, yypvt[-1].qp, lastok);} break;
case 252:
# line 1084 "parse1.y"
{myerr("Illegal CONSERVE syntax");} break;
case 253:
# line 1087 "parse1.y"
{pushlocal(); SYM(yypvt[-1].qp) = copylocal(SYM(yypvt[-1].qp));} break;
case 254:
# line 1089 "parse1.y"
{massagecompart(yypvt[-3].qp, yypvt[-2].qp, yypvt[-0].qp, SYM(yypvt[-6].qp)); poplocal();} break;
case 255:
# line 1091 "parse1.y"
{massagecompart(yypvt[-3].qp, yypvt[-2].qp, yypvt[-0].qp, SYM0);} break;
case 258:
# line 1099 "parse1.y"
{massagekinetic(yypvt[-4].qp, yypvt[-3].qp, yypvt[-1].qp, yypvt[-0].qp, sensused);
		lexcontext = 0; sensused = 0;
		} break;
case 259:
# line 1103 "parse1.y"
{leftreact();} break;
case 260:
# line 1104 "parse1.y"
{massagereaction(yypvt[-9].qp,yypvt[-7].qp,yypvt[-4].qp,yypvt[-2].qp,yypvt[-0].qp);} break;
case 261:
# line 1106 "parse1.y"
{flux(yypvt[-6].qp, yypvt[-4].qp, yypvt[-0].qp);} break;
case 262:
# line 1108 "parse1.y"
{flux(yypvt[-6].qp, yypvt[-4].qp, yypvt[-0].qp);} break;
case 263:
# line 1109 "parse1.y"
{myerr("Illegal reaction syntax");} break;
case 264:
# line 1111 "parse1.y"
{reactname(yypvt[-0].qp, lastok, ITEM0);} break;
case 265:
# line 1112 "parse1.y"
{reactname(yypvt[-0].qp, lastok, yypvt[-1].qp);} break;
case 266:
# line 1113 "parse1.y"
{reactname(yypvt[-0].qp, lastok, ITEM0);} break;
case 267:
# line 1114 "parse1.y"
{reactname(yypvt[-0].qp, lastok, yypvt[-1].qp);} break;
case 268:
# line 1117 "parse1.y"
{lag_stmt(yypvt[-3].qp, blocktype);} break;
case 269:
# line 1118 "parse1.y"
{myerr("Lag syntax is: LAG name BY const");} break;
case 270:
# line 1120 "parse1.y"
{queue_stmt(yypvt[-1].qp, yypvt[-0].qp);} break;
case 271:
# line 1121 "parse1.y"
{queue_stmt(yypvt[-1].qp, yypvt[-0].qp);} break;
case 272:
# line 1123 "parse1.y"
{checkmatch(blocktype);} break;
case 273:
# line 1124 "parse1.y"
{deltokens(yypvt[-4].qp, yypvt[-0].qp);} break;
case 276:
# line 1130 "parse1.y"
{matchinitial(yypvt[-0].qp);} break;
case 277:
# line 1132 "parse1.y"
{ matchbound(yypvt[-5].qp, yypvt[-4].qp, yypvt[-2].qp, yypvt[-0].qp, lastok, matchindex);
		  if (matchindex) {
			poplocal();
			matchindex = SYM0;
		  }
		} break;
case 278:
# line 1139 "parse1.y"
{myerr("MATCH syntax is state0 or state(expr)=expr or\
state[i](expr(i)) = expr(i)");} break;
case 279:
# line 1143 "parse1.y"
{matchindex = SYM0;} break;
case 280:
# line 1145 "parse1.y"
{ pushlocal();
		  matchindex = copylocal(SYM(yypvt[-1].qp));
		} break;
case 281:
# line 1150 "parse1.y"
{ deltokens(yypvt[-3].qp,yypvt[-0].qp);} break;
case 282:
# line 1153 "parse1.y"
{modl_units();} break;
case 285:
# line 1158 "parse1.y"
{install_units(yypvt[-2].str, yypvt[-0].str);} break;
case 286:
# line 1159 "parse1.y"
{myerr("Unit definition syntax: (units) = (units)");} break;
case 287:
# line 1162 "parse1.y"
{ SYM(yypvt[-3].qp)->subtype |= CONST;
		  Sprintf(buf, "static double %s = %s;\n", SYM(yypvt[-3].qp)->name,
			STR(yypvt[-1].qp));
		  Lappendstr(firstlist, buf);
		} break;
case 288:
# line 1168 "parse1.y"
{Item *q; double d, unit_mag();
		    Unit_push(yypvt[-1].str);
			Unit_push(yypvt[-0].str); unit_div();
		    SYM(yypvt[-3].qp)->subtype |= CONST;
		    Sprintf(buf, "static double %s = %g;\n", SYM(yypvt[-3].qp)->name,
			unit_mag());
		    Lappendstr(firstlist, buf);
		    unit_pop();
		} break;
case 289:
# line 1178 "parse1.y"
{ double unit_mag();
		    Unit_push(yypvt[-3].str); Unit_push(yypvt[-0].str); unit_div();
		    SYM(yypvt[-5].qp)->subtype |= CONST;
		    Sprintf(buf, "static double %s = %g;\n", SYM(yypvt[-5].qp)->name,
			unit_mag());
		    Lappendstr(firstlist, buf);
		    unit_pop();
		} break;
case 290:
# line 1186 "parse1.y"
{myerr("Unit factor syntax: examples:\n\
foot2inch = (foot) -> (inch)\n\
F = 96520 (coulomb)\n\
R = (k-mole) (joule/degC)");
		} break;
case 293:
# line 1196 "parse1.y"
{ SYM(yypvt[-3].qp)->subtype |= CONST;
		  Sprintf(buf, "static double %s = %s;\n", SYM(yypvt[-3].qp)->name,
			STR(yypvt[-1].qp));
		  Lappendstr(firstlist, buf);
		} break;
case 294:
# line 1203 "parse1.y"
{ Item *q;
		  if (table_list) {
			diag("only one TABLE per function", (char *)0);
		  }
		  table_list = newlist();
		  Lappendlst(table_list, yypvt[-7].lp);
		  q = lappendlst(table_list, newlist());
		  movelist(yypvt[-5].qp->next, yypvt[-3].qp->prev, LST(q));
		  q = lappendlst(table_list, newlist());
		  movelist(yypvt[-3].qp->next, yypvt[-1].qp->prev, LST(q));
		  Lappendstr(table_list, STR(yypvt[-0].qp));
		  Lappendlst(table_list, yypvt[-6].lp);
		  deltokens(yypvt[-8].qp, yypvt[-0].qp);
		} break;
case 295:
# line 1217 "parse1.y"
{ myerr("syntax: TABLE [list] [DEPEND list] FROM expr TO expr WITH integer");} break;
case 296:
# line 1220 "parse1.y"
{yyval.lp = LIST0;} break;
case 298:
# line 1224 "parse1.y"
{yyval.lp = newlist(); Lappendsym(yyval.lp, SYM(yypvt[-0].qp));} break;
case 299:
# line 1226 "parse1.y"
{ Lappendsym(yypvt[-2].lp, SYM(yypvt[-0].qp));} break;
case 300:
# line 1229 "parse1.y"
{yyval.lp = LIST0;} break;
case 301:
# line 1231 "parse1.y"
{yyval.lp = yypvt[-0].lp;} break;
case 302:
# line 1234 "parse1.y"
{ deltokens(yypvt[-3].qp,yypvt[-0].qp);} break;
case 304:
# line 1238 "parse1.y"
{ nrn_list(yypvt[-1].qp, yypvt[-0].qp);} break;
case 306:
# line 1241 "parse1.y"
{ nrn_list(yypvt[-1].qp,yypvt[-0].qp);} break;
case 307:
# line 1243 "parse1.y"
{ nrn_list(yypvt[-1].qp,yypvt[-0].qp);} break;
case 308:
# line 1245 "parse1.y"
{ nrn_list(yypvt[-1].qp, yypvt[-0].qp);} break;
case 309:
# line 1247 "parse1.y"
{ nrn_list(yypvt[-1].qp, yypvt[-0].qp);} break;
case 310:
# line 1249 "parse1.y"
{ nrn_list(yypvt[-1].qp, yypvt[-0].qp);} break;
case 311:
# line 1251 "parse1.y"
{ nrn_list(yypvt[-1].qp, yypvt[-0].qp);} break;
case 312:
# line 1253 "parse1.y"
{ nrn_list(yypvt[-1].qp, yypvt[-0].qp);} break;
case 313:
# line 1256 "parse1.y"
{nrn_use(yypvt[-3].qp, yypvt[-1].qp, ITEM0, yypvt[-0].qp);} break;
case 314:
# line 1258 "parse1.y"
{nrn_use(yypvt[-3].qp, ITEM0, yypvt[-1].qp, yypvt[-0].qp);} break;
case 315:
# line 1260 "parse1.y"
{nrn_use(yypvt[-5].qp, yypvt[-3].qp, yypvt[-1].qp, yypvt[-0].qp);} break;
case 316:
# line 1262 "parse1.y"
{myerr("syntax is: USEION ion READ list WRITE list");} break;
case 318:
# line 1266 "parse1.y"
{ delete(yypvt[-1].qp); yyval.qp = yypvt[-0].qp;} break;
case 319:
# line 1268 "parse1.y"
{myerr("syntax is: keyword name , name, ..., name");} break;
case 320:
# line 1271 "parse1.y"
{yyval.qp = ITEM0;} break;
case 321:
# line 1273 "parse1.y"
{yyval.qp = yypvt[-0].qp;} break;
	}
	goto yystack;		/* reset registers in driver code */
}
