/* $Header: /timshel2g/src/master/nrn/src/nrnoc/cabcode.c,v 1.3 1994/11/08 14:16:40 hines Exp $ */
/*
$Log: cabcode.c,v $
 * Revision 1.3  1994/11/08  14:16:40  hines
 * certain kinds of section loops caused nontermination in nrn_parent_info
 *
 * Revision 1.2  1994/10/25  17:24:08  hines
 * use right function
 *
 * Revision 1.1.1.1  1994/10/12  17:22:32  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.74  1994/07/21  14:22:21  hines
 * fixed an error in forall inside object resulting from recent change
 * to object names.
 *
 * Revision 4.73  1994/07/21  11:54:06  hines
 * nrn_get_mechtype(mechname) and nrn_instance_count(mechtype)
 * introduced to provide info required for vectorizing kinetic blocks
 *
 * Revision 4.72  1994/07/12  13:49:42  hines
 * use hoc_object_name instead of building own format for it
 *
 * Revision 4.64  1994/06/29  13:24:34  hines
 * under unusual circumstances sections could be unreffed and never freed. fixed
 * benign uninitialized memory read of param_size when prop is freed before
 * memory is allocated for param. fixed
 *
 * Revision 4.63  1994/06/27  11:36:40  hines
 * small changes to allow code coverage tests for cabcode.c
 * dimensioned sections were not freed properly.
 * fix v(0:1) = 1:2 for both orientations
 *
 * Revision 4.62  1994/06/21  12:39:28  hines
 * allow recursive calls to interpreter even in case of hoc_execerror
 *
 * Revision 4.56  1994/05/02  12:56:19  hines
 * don't print error message for mechanism not inserted if hoe_execerror_messages is false
 * nrn_complain(double* _p) pirnts location for this param on the standard error
 * device
 *
 * Revision 4.51  1994/03/07  21:53:20  hines
 * new_section(s)() takes Object* arg instead of Object** arg now
 * this was done so that fix for
 * ob = new Obj(ob) could be more easily handled in oc/SRC/hoc_oop.c
 *
 * Revision 4.50  1993/12/15  15:04:19  hines
 * SectionRef class holds pointer to section.
 * special .sec method handles section stack properly
 *
 * Revision 4.46  1993/08/20  13:00:12  hines
 * give message about a.a - a.a which is parsed as a.a {-a.a}
 * fix some problems regarding define_shape
 * allow change of L and diam when 3dpoints exist (pt3dconst(0) is default)
 *
 * Revision 4.40  93/07/16  08:35:41  hines
 * push_section("sectionname")
 * 
 * Revision 4.35  93/06/01  08:59:32  hines
 * can see if a mechanism is an ion with nrn_is_ion(type)
 * can't uninsert ions
 * 
 * Revision 4.33  93/05/31  15:33:04  hines
 * for (ch=sec->child; ch; ch=ch->sibling) is loop over all sections connect
 * directly to sec in order of increasing distance from connection end of sec.
 * there is no longer a Node->child pointer.
 * 
 * Revision 4.31  93/05/25  14:03:42  hines
 * insert and uninsert can be done in c.
 * 
 * Revision 4.28  93/04/29  16:35:58  hines
 * sibling pointers now reffed. Fix several problems with disconnecting
 * and reconnecting sections as well as handle point processes moving with
 * their proper section.
 * Still not perfect though.
 * 
 * Revision 4.27  93/04/21  13:23:49  hines
 * watch out for nil parentnode when haven't done setup_topology yet.
 * 
 * Revision 4.26  93/04/21  10:33:16  hines
 * point processes located at ends of sections stay in those sections when
 * those sections are disconnected from their parent.
 * 
 * Revision 4.25  93/04/20  08:51:41  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.21  93/03/23  17:23:44  hines
 * allow deletion of all sections
 * fix bug having to do with deletion of multiple sections
 * 
 * Revision 4.12  93/02/02  10:23:33  hines
 * declare static functions before use
 * 
 * Revision 4.11  93/01/27  13:50:01  hines
 * extracellular works again
 * 
 * Revision 4.9  93/01/21  12:20:30  hines
 * watch out for freed sections in old style fstim, fsyn, and fclamp
 * correct memory leakage on access section
 * 
 * Revision 4.8  93/01/14  13:04:55  hines
 * changed meaning of OPSTR
 * 
 * Revision 4.7  93/01/14  10:38:14  hines
 * Steve Fisher's contribution to RCS parameter dumps.
 * 
 * Revision 4.6  93/01/12  09:02:55  hines
 * setting hoc strings now done with hoc_assign_str(char**, char*)
 * 
 * Revision 4.5  93/01/07  08:13:02  hines
 * psection prints Ra
 * warning if Ra set but not in every section
 * fix batch_run so not extra iteration on cray (tstop -=dt/4)
 * 
 * Revision 4.3  93/01/05  17:21:48  hines
 * can push and pop null sections.
 * void sec_access_pop()   Section* nrn_pop_sec()
 * 
 * Revision 3.47  92/12/30  12:38:30  hines
 * buf needed to be static in sec_and_position
 * 
 * Revision 3.46  92/12/22  16:54:53  hines
 * section_object_seen needs to be initialized after an error
 * 
 * Revision 3.44  92/12/17  08:26:58  hines
 * delete_section() removes currently accessed section from the list of
 * sections to be simulated and releases its property and nodes.
 * 
 * Revision 3.43  92/12/16  14:23:32  hines
 * Ra switched from global variable to section variable in which it can
 * take on different values in different sections. Similar to L.
 * No need to set diam_changed when Ra changed.
 * 
 * Revision 3.41  92/12/02  10:00:19  hines
 * error in secname when called from within an object
 * is_point_process(Object*)
 * 
 * Revision 3.40  92/11/27  08:21:05  hines
 * boolean nrn_exists(Symbol*, Node*)
 * sections contain a short volatile_mark used for associative searching.
 * 
 * Revision 3.39  92/11/16  11:21:27  hines
 * double nrn_diameter(Node*)
 * nrn_define_shape uses diam and L and parent position to define
 * two pt3d points so we can see shape in nrniv
 * 
 * Revision 3.38  92/11/13  07:36:10  hines
 * Imakefile didn't get oc..noiv.c for saber
 * section_pathname now gives error message when path not seen
 * 
 * Revision 3.37  92/11/09  17:15:02  hines
 * using topol_distance in nrniv SpacePlot
 * also added
 * double section_length(Section*) and
 * int arc0at0(Section*) which returns true if x=0 is connected to parent
 * 
 * Revision 3.36  92/10/30  12:56:33  hines
 * hoc_section_pathname(Section*)
 * 
 * Revision 3.35  92/10/27  12:09:47  hines
 * list.c list.h moved from nrnoc  to oc
 * 
 * Revision 3.34  92/10/27  09:22:47  hines
 * diam_changed can be set from hoc (to be used by xvalue of L an diam
 * 	that use pointers.)
 * steer for point processes always comes from steer_point_process and that
 * is how one can determine that an object is a point process.
 * need to have a better secname implementation that actually returns a
 * valid path
 * 
 * Revision 3.33  92/10/24  16:30:08  hines
 * freeing vars notifies interviews.
 * sec_and_position(sec, inode) returns string in form section(x)
 * some portability fixes from pittsburgh
 * 
 * Revision 3.31  92/10/21  14:57:06  hines
 * when section stack overflow we print the section stack
 * 
 * Revision 3.25  92/10/10  10:24:47  hines
 * ob.sec.range, ob.sec.range(x), ob.sec.property now allowed.
 * ob.sec.range(x) = expr means just change the value of node closest to x
 * ob.sec.range(x:y) = expr:expr syntax not implemented.
 * if range is a pointer variable it cannot be set with this syntax. use
 * ob.sec setpointer range, var
 * 
 * Revision 3.23  92/10/09  09:13:50  hines
 * clean up some warnings and bugs. sectionname was broken and
 * sometimes fstim and fsyn were used when they hadn't gotten a section
 * pointer.
 * 
 * Revision 3.19  92/10/07  10:45:44  hines
 * a simpler connect statement is now available in addition to the old
 * baroque syntax. it is
 * connect sec1(x), sec2(0 or 1)
 * 
 * 
 * Revision 3.16  92/09/30  16:51:45  hines
 * fix some problems haveing to do with reallocating segments and
 * strength of stimuli.
 * 
 * Revision 3.11  92/09/24  17:03:31  hines
 * METHOD3 option. use with spatial_method(i) with i=0-3
 * 0 is the default and is the normal neuron method with zero area nodes
 * at boundaries and branch points.
 * 1 is the standard method with half area nodes at boundaries
 * 2 is a modified second order method which weights by 2/12, 8/12, 2/12
 * 3 is a third order correct method which weights by 1/12 10/12 1/12
 * with the last three methods x = i/nnode with i=0 to nnode
 * methods 1 and 2 are at present quite inefficient since they use the
 * data structures of 3.
 * Works in steady state. not tested with dynamic simulations
 * At this time, discontinuities in parameters (except diameter) is
 * not handled well across nodes and all parameters are assumed to
 * be constant in interval between node and parent node.
 * Good deal of difficulty remains in managing values that change discontinuously
 * 
 * Revision 3.6  92/08/25  12:27:00  hines
 * hoc_araystr() moved to oc/code.c
 * 
 * Revision 3.4  92/08/18  08:56:39  hines
 * sections in different objects can have different array sizes and
 * be created in procedures within objects
 * 
 * Revision 3.3  92/08/18  07:32:29  hines
 * arrays in different objects can have different sizes
 * 
 * Revision 2.86  92/08/13  11:20:03  hines
 * default parameters collected into membdef.h
 * 
 * Revision 2.85  92/08/12  16:17:13  hines
 * uninsert mechanism_name
 * do not uninsert an ion unless no other mechanism in the section uses the
 * ion. Invalid memory references will result.
 * 
 * Revision 2.82  92/08/10  16:02:30  hines
 * section lists with seclistdef ,,,
 * append_seclist(seclist)
 * remove_seclist(seclist)
 * forsec seclist
 * ifsec seclist
 * sections are now reference counted.
 * 
 * Revision 2.81  92/08/10  10:28:19  hines
 * forall within an object is just the sections in the object.
 * 
 * Revision 2.80  92/08/08  12:56:55  hines
 * access object.section
 * 
 * Revision 2.79  92/08/08  12:39:07  hines
 * object section name of form template_index:section
 * 
 * Revision 2.78  92/08/08  11:57:25  hines
 * ob.section.var not implemented but hooks are there to do it later
 * 
 * Revision 2.77  92/08/08  09:22:59  hines
 * need to node_alloc the root when fewer nodes are needed as well
 * 
 * Revision 2.76  92/08/08  08:42:45  hines
 * section_unlink removes references to the section by other sections
 * 
 * Revision 2.75  92/08/07  16:11:49  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.73  92/08/05  10:28:24  hines
 * got rid of objectcenter messages
 * 
 * Revision 2.72  92/08/05  09:42:47  hines
 * push_section() and pop_section() names now same as in neuron
 * 
 * Revision 2.69  92/07/31  13:52:40  hines
 * forsec "string" and ifsec "string" as in neuron
 * 
 * Revision 2.67  92/07/31  13:02:09  hines
 * pushsec
 * 
 * Revision 2.54  92/02/26  10:31:50  hines
 * xview out. neuron examples verified (required fixes to the section
 * stack usage)
 * 
 * Revision 2.53  92/02/19  15:04:25  hines
 * preparing to allow sections in objects
 * syntax is ob.section stmt    section.var
 * needs major changes in treatment of section array
 * 
 * Revision 2.51  92/02/05  12:10:02  hines
 * setdata_mechname(x) and setdata_mechname(i) will specify the range variable
 * data for continuous mechanisms and point processes.
 * 
 * Revision 2.50  92/02/05  08:47:58  hines
 * Can connect nmodl POINTER variables to external variables using:
 * connect var_suffix.section(x), anyvar for range variables or
 * connect var_suffix[i], anyvar for point process variables
 * This can be dangerous if the user fails to make these variables point to
 * something.
 * 
 * Revision 2.49  92/01/30  11:08:26  hines
 * verified with respect to all neuron/examples. Under control of imake.
 * ObjectCenter warning free. Some changes to match neuron.
 * 
 * Revision 2.48  91/11/05  11:22:04  hines
 * all neuron/examples produce same results with nrnoc as neuron
 * 
 * Revision 2.45  91/09/23  09:17:07  hines
 * allow changing nseg without recreateing sections.
 * 
 * Revision 2.44  91/09/16  16:34:10  hines
 * sections now kept on separate stack.  getloc_... pushes the section
 * onto the stack.  Must do popsec() when done.
 * 
 * Revision 2.40  91/08/16  14:01:47  hines
 * get_loc_pointprocess(index, string) returns arc location and also
 * returns name of section in string variable. popsec() removed.
 * 
 * Revision 2.32  91/08/08  16:41:09  hines
 * saber lint free. can make saber_src_all which is all the src pertaining
 * to neuron
 * 
 * Revision 2.31  91/08/08  14:57:24  hines
 * 3-D information attached to sections for simulation of reconstructed
 * neurons.  This information is not discarded when nseg is changed.
 * 
 * Revision 2.30  91/08/05  17:29:00  hines
 * allow rangevar(x:y) with x>y or x<y
 * 
 * Revision 2.24  91/06/20  17:26:12  hines
 * diamlst() --- trapezoidal integration of diam, area, ri.
 * second order effect of taper on area computed correctly.
 * 
 * Revision 2.21  91/03/18  10:46:50  hines
 * NEMO ifdef
 * some builtins can take address of varname as argument. eg regraph(&v(.5))
 * 
 * Revision 2.19  91/03/03  16:34:01  hines
 * in section arrays with multiple dimensions not allocating enough memory
 * to keep indices.
 * fixed.
 * 
 * Revision 2.14  91/02/14  07:28:47  hines
 * vector range variables had their indices popped every time dprop was called.
 * So if more than one segment was accessed at a time, we would get a
 * stack underflow.
 * 
 * Revision 2.10  90/12/15  12:22:02  hines
 * distance() function added
 * 
 * Revision 2.8  90/12/14  13:45:25  hines
 * point process works but current is /cm2
 * 
 * Revision 2.7  90/12/14  09:47:58  hines
 * First pass at handling point processes.
 * 
 * Revision 2.6  90/12/11  15:40:56  hines
 * range variables can be vectors
 * syntax is  range[index].section[index](expr: expr)
 * 
 * Revision 1.87  90/10/29  12:28:03  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:24  hines
 * saber warnings gone
 * 
 * Revision 1.78  90/10/10  08:39:41  hines
 * fix some warnings given by saber-c. It saw possible error in node_index
 * 
 * Revision 1.77  90/10/09  08:01:04  hines
 * extracellular axial resistance now Mohms/cm  axial capacitance
 * not working
 * 
 * Revision 1.65  90/05/17  15:56:44  jamie
 * moving regexp.c to hoc project
 * changed *regexp* to *hoc_regexp*
 * 
 * Revision 1.62  90/05/08  09:15:37  mlh
 * eliminate a lint message
 * 
 * Revision 1.60  90/03/15  16:50:31  mlh
 * default length of 10 microns when section created.
 * 
 * Revision 1.52  90/01/24  07:03:37  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.48  90/01/16  07:04:06  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.42  89/10/13  09:00:10  mlh
 * free the clamp, synapse, and stim when new sections created
 * de-lint some minor things
 * 
 * Revision 1.37  89/09/20  11:59:37  mlh
 * ionic reversal potentials and currents now in Eion
 * other mechanisms, such as HH, use and modify Eion parameters.
 * a mechanism allocation routine can "need" another mechanism and
 * communicate via Datum.pval
 * 
 * Revision 1.34  89/09/19  13:59:26  mlh
 * node_index now checks that nnode not , 0
 * 
 * Revision 1.31  89/09/14  13:55:24  mlh
 * error in node_ptr that showed up on pc. Sometimes failed to set inode.
 * 
 * Revision 1.28  89/09/01  13:31:14  mlh
 * secname() now works with multiple dimensions
 * 
 * Revision 1.26  89/08/31  14:47:21  mlh
 * name of mechanism out of memb_func and back in mechanism[].
 * memb_func[].sym now points to mechanism symbol.
 * This allows us to retrieve property names.
 * 
 * Revision 1.25  89/08/31  10:28:17  mlh
 * regular expressions for issection()
 * 
 * Revision 1.23  89/08/29  15:09:15  mlh
 * bug in turboc with respect to incrementing section and node pointers.
 * explicitly iterate over indices
 * 
 * Revision 1.22  89/08/29  10:16:13  mlh
 * forall loop over sections, issection(), ismembrane(), sectionname()
 * 
 * Revision 1.19  89/08/13  08:16:36  mlh
 * addsection poped dimensions in wrong order
 * 
 * Revision 1.18  89/08/12  14:48:31  mlh
 * Lint free
 * 
 * Revision 1.17  89/08/12  12:37:41  mlh
 * range variable can be in an expression. Always returns the .5 value.
 * 
 * Revision 1.15  89/08/11  15:11:37  mlh
 * rall rall branching, several bugs fixed
 * 
 * Revision 1.13  89/08/10  11:06:28  mlh
 * var.sec syntax, no overloading of range, stack the open sections
 * 
 * Revision 1.10  89/08/09  10:48:33  mlh
 * fstim works, v accessible at ends, for (x) goes from 0<=x<=1
 * 
 * Revision 1.9  89/08/07  10:22:26  mlh
 * getting good v with single section. matrix solving for v (not deltav)
 * 
 * Revision 1.8  89/08/05  14:21:53  mlh
 * The root of the tree has a node of length 0.
 * We can define the voltage from  0<x<1 and get it right even
 * at 0 and 1
 * The last node of each section is has no length
 * good start on calculating matrix coefficients
 * 
 * Revision 1.7  89/08/04  14:45:26  mlh
 * 2nd order correct in dx handled pretty well with nnode-1 segments
 * and last node being at the end of the section with 0 length.
 * Problem with connecting to position 0 since first node
 * not at beginning of section.
 * 
 * Revision 1.6  89/08/02  16:10:46  mlh
 * range variables implemented in open mechanism context
 * 
 * Revision 1.5  89/07/28  11:28:25  mlh
 * topology is correct and error messages give almost complete section name
 * 
 * Revision 1.4  89/07/27  16:13:58  mlh
 * at point where topology() works
 * 
 * Revision 1.3  89/07/21  17:38:11  mlh
 * compiles
 * 
 * Revision 1.1  89/07/20  07:51:30  mlh
 * Initial revision
 * 
*/

#define HOC_L_LIST 1
#include <stdio.h>
#include "section.h"
#include "membfunc.h"
#include "ytab.h"
#include "membdef.h"

extern int hoc_execerror_messages;
#define symlist	hoc_symlist
extern void sec_free(), section_order();
extern Prop *prop_alloc();
extern char *secname();
extern Symlist *symlist;
extern Section* nrn_sec_pop();
extern Node	*node_ptr();
extern double	*dprop();
extern double	nrn_section_orientation(); /* x=0,1 end connected to parent */
extern double	nrn_connection_position(); /* x of parent for this section */
extern Prop	*nrn_mechanism(); /*returns prop given mech type, node */
extern Prop	*nrn_mechanism_check(); /*returns prop given mech type, section, and inode */
					/* error if mech not at this position */

int	tree_changed = 1;	/* installing section, changeing nseg
				and connecting sections set this flag.
				The flag is set to 0 when the topology
				is set up */
int	diam_changed = 1;	/* changing diameter, length set this flag
				The flag is set to 0 when node.a and node.b
				is set up */
/* switching from Ra being a global variable to it being a section variable
opens up the possibility of a great deal of confusion and inadvertant wrong
results. To help avoid this we print a warning message whenever the value
in one section is set but no others. But only the first time through treeset.
*/
#if RA_WARNING
int nrn_ra_set = 0;
#endif

extern hoc_List* rootsection_list;

#define NSECSTACK 20
static Section *secstack[NSECSTACK + 1];
static Section **secptr = secstack;

#define	OVERLOAD 0	/* Setting to 1 will introduce bugs */
#if OVERLOAD	/* will be needed only if overloading range variables */
static short	access_mechanism;
static Prop	**access_prop;	/* array of pointers to node properties for
					above access_mechanism */
#endif

extern char* hoc_object_name();
extern section_object_seen;
static int range_vec_indx();

nrn_initcode() {
	while(secptr > secstack) {
		nrn_popsec();
	}
	secptr = secstack;
	section_object_seen = 0;
}

oc_save_cabcode(a1, a2)
	Section** *a1;
	int	*a2;
{
	*a1 = secptr;
	*a2 = section_object_seen;
}

oc_restore_cabcode(a1, a2)
	Section** *a1;
	int	*a2;
{
	while(secptr > *a1) {
		nrn_popsec();
	}
	secptr = *a1;
	section_object_seen = *a2;
}

nrn_pushsec(sec)
	Section *sec;
{
	secptr++;
	if (secptr >= secstack + NSECSTACK) {
		int i = NSECSTACK;
		hoc_warning("section stack overflow", (char *)0);
		while (--secptr >= secstack) {
			fprintf(stderr, "%*s%s\n", i--, "",  secname(*secptr));
		}
		hoc_execerror("section stack overflow", (char *)0);
	}
	*secptr = sec;
	if (sec) {
#if 0
		printf("pushsec %s\n", sec->prop->dparam[0].sym->name);
#endif
		++sec->refcount;
	}
}

nrn_popsec() {
	if (secptr > secstack) {
		Section* sec = *secptr--;
		if (!sec) {
			return;
		}
#if 0
		if (sec->prop) {
			printf("popsec %s\n", sec->prop->dparam[0].sym->name);
		}else{
			printf("popsec with no properties\n");
		}
#endif
		if (--sec->refcount <= 0) {
#if 0
			printf("sec freed after pop\n");
#endif
			free((char*)sec);
		}
	}
}

sec_access_pop() {
	nrn_popsec();
}

#if 0
static
free_point_process() {
	free_all_point();
	free_clamp();
	free_stim();
	free_syn();
}
#endif

#if 0
clear_sectionlist()	/* merely change all SECTION to UNDEF */
{
printf("clear_sectionlist not fixed yet, doing nothing\n");
return;
	Symbol *s;
	
	free_point_process();
	if (symlist) for (s=symlist->first; s; s = s->next) {
		if (s->type == SECTION) {
			s->type = UNDEF;
no longer done this way see OPARINFO
			if (s->arayinfo) {
				free((char *)s->arayinfo);
				s->arayinfo = (Arrayinfo *)0;
			}
		}
	nrn_initcode();
	secstack[0] = (Section *)0;
#if OVERLOAD
	access_mechanism = 0;
#endif
	}
	if (section) {
		sec_free(section, nsection);
		section = (Section *)0;
		nsection = 0;
	}
}
#endif

add_section() /* symbol at pc+1, number of indices at pc+2 */
{
	Symbol *sym;
	int nsub, size;
	Item** pitm;
	extern Objectdata* hoc_top_level_data;
	extern Object* hoc_thisobject;
	
	sym = (pc++)->sym;
/*printf("declaring %s as section\n", sym->name);*/
	if (sym->type == SECTION) { int total, i;
		total = hoc_total_array(sym);
		for (i=0; i<total; ++i) {
			sec_free(*(OPSECITM(sym) + i));
		}
		free((char*)OPSECITM(sym));
		hoc_freearay(sym);
	}else{
		assert (sym->type == UNDEF);
		sym->type = SECTION;
		hoc_install_object_data_index(sym);
	}
	nsub = (pc++)->i;
	if (nsub) {
		size = hoc_arayinfo_install(sym, nsub);
	}else{
		size = 1;
	}
hoc_objectdata[sym->u.oboff].psecitm = pitm = (Item **)emalloc(size*sizeof(Item*));
	if (hoc_objectdata == hoc_top_level_data) {
		new_sections((Object*)0, sym, pitm, size);
	}else{
		new_sections(hoc_thisobject, sym, pitm, size);
	}
#if 0
printf("%s", s->name);
for (i=0; i<ndim; i++) {printf("[%d]",s->arayinfo->sub[i]);}
printf(" is a section name\n");
#endif
}

static Section* new_section(ob, sym, i)
	Object* ob;
	Symbol* sym;
	int i;
{
	Section* sec;
	Prop* prop;
	static Symbol* nseg;

	if (!nseg) {
		nseg = hoc_lookup("nseg");
	}
	sec = sec_alloc();
	section_ref(sec);
	prop = prop_alloc(&(sec->prop), CABLESECTION);
	prop->dparam[0].sym = sym;
	prop->dparam[5].i = i;
	if (ob) {
		prop->dparam[6].obj = ob;
	}else{
		prop->dparam[6].obj = (Object*)0;
	}
	nrn_pushsec(sec);
	cable_prop_assign(nseg, (double)DEF_nseg);
	nrn_popsec();
	tree_changed = 1;
/*printf("new_section %s\n", secname(sec));*/
	return sec;
}
	
new_sections(ob, sym, pitm, size)
	Object* ob;
	Symbol* sym;
	Item** pitm;
	int size;
{
	int i;
	for (i=0; i < size; ++i) {
		Section* sec = new_section(ob, sym, i);
		pitm[i] = lappendsec(section_list, sec);
	}
}

delete_section() {
	extern Objectdata* hoc_top_level_data;
	Section* sec;
	Object* ob;
	Item** pitm;
	Symbol* sym;
	int i;
	sec = chk_access();
	sym = sec->prop->dparam[0].sym;
	ob = sec->prop->dparam[6].obj;
	i = sec->prop->dparam[5].i;
	if (ob) {
		pitm = ob->u.dataspace[sym->u.oboff].psecitm + i;
	}else{
		pitm = hoc_top_level_data[sym->u.oboff].psecitm + i;
	}
	sec_free(*pitm);
	*pitm = 0;
	ret(0.);
}

/*
At this point, all the sections are cables and each section has the following
properties (except for section 0). Only one property with 8 Datums

section[i].prop->dparam[0].sym 	pointer to section symbol
			[1].val	position (0--1) of connection to parent
			[2].val	length of section in microns
			[3].val	first node at position 0 or 1
			[4].val rall branch
			[5].i	aray index
			[6].obj pointer to the object pointer
			[7].val Ra

The first element allows us to find the symbol when we know the section number.
If an array section the index can be computed with
i - (section[i].sym)->u.u_auto
*/

double section_length(sec)
	Section* sec;
{
	return sec->prop->dparam[2].val;
}
int arc0at0(sec)
	Section* sec;
{
	return ((sec->prop->dparam[3].val) ? 0 : 1);
}

double nrn_ra(sec)
	Section* sec;
{
	return sec->prop->dparam[7].val;
}

cab_alloc(p)
	Prop *p;
{
	Datum *pd;
#define CAB_SIZE 8
	pd = (Datum *)emalloc(CAB_SIZE*sizeof(Datum));
	pd[1].val = pd[3].val = 0.;
	pd[2].val = DEF_L;
	pd[4].val = DEF_rallbranch;
	pd[7].val = DEF_Ra;
	p->dparam = pd;
	p->param_size = CAB_SIZE; /* this one is special since it refers to dparam */
}

morph_alloc(p)
	Prop *p;
{
	
	double *pd;
	pd = (double *)emalloc(1*sizeof(double));
	pd[0] = DEF_diam; /* microns */
	diam_changed = 1;
	p->param = pd;
	p->param_size = 1;
}
	
double nrn_diameter(nd)
	Node* nd;
{
	Prop* p;
	p = nrn_mechanism(MORPHOLOGY, nd);
	return p->param[0];
}	

nrn_chk_section(s)
	Symbol *s;
{
	if (s->type != SECTION) {
		execerror("Not a SECTION name:", s->name);
	}
}

Section*
chk_access()
{
	Section* sec = *secptr;
	if (sec == (Section *)0) {
		execerror("Section access unspecified", (char *)0);
	}
	if (sec->prop) {
		return *secptr;
	}else{
		execerror("Accessing a deleted section", (char*)0);
	}
}

nrn_rootnode_free(nd, sec)
	Node* nd;
	Section* sec;
{
	Item* q;
	ITERATE(q, rootsection_list) {
		if (sec == (Section*)(VOIDITM(q)) ) {
			hoc_l_delete(q);
			section_unref(sec);
			node_destruct(nd, 1);
			break;
		}
	}
}

/*sibling and child pointers do not ref sections to avoid mutual references */
/* the sibling list is ordered according to increasing distance from parent */

nrn_remove_sibling_list(sec)
	Section* sec;
{
	Section* s;
	if (sec->parentsec) {
		if (sec->parentsec->child == sec) {
			sec->parentsec->child = sec->sibling;
			return;
		}
		for (s = sec->parentsec->child; s; s = s->sibling) {
			if (s->sibling == sec) {
				s->sibling = sec->sibling;
				return;
			}
		}
	}
}

nrn_add_sibling_list(sec)
	Section* sec;
{
	Section* s;
	double x;
	if (sec->parentsec) {
		x = nrn_connection_position(sec);
		if (!arc0at0(sec->parentsec)) {
			x = 1. - x;
		}
		s = sec->parentsec->child;
		if (!s || x <= nrn_connection_position(s)) {
			sec->sibling = s;
			sec->parentsec->child = sec;
			return;
		}
		for (; s->sibling; s = s->sibling) {
			if (x <= nrn_connection_position(s->sibling)) {
				sec->sibling = s->sibling;
				s->sibling = sec;
				return;
			}
		}
		s->sibling = sec;
		sec->sibling = 0;
	}
}

static
reverse_sibling_list(sec)
	Section* sec;
{
	int scnt;
	Section* ch;
	Section** pch;
/*printf("reversing children sibling list for %s\n", secname(sec));*/
	for (scnt=0, ch = sec->child; ch; ++scnt, ch=ch->sibling) {
		hoc_pushobj((Object**)ch);			
	}
	pch = &sec->child;
	while(scnt--) {
		Object** hoc_objpop();
		ch = (Section*)hoc_objpop();
		*pch = ch;
		pch = &ch->sibling;
		ch->parentnode = 0;
	}
	*pch = 0;
}

disconnect() {
	nrn_disconnect(chk_access());
	ret(0.);
}

nrn_disconnect(sec)
	Section* sec;
{
	Section* ch;
	Section* oldpsec = sec->parentsec;
	Node* oldpnode = sec->parentnode;
	if (!oldpsec) {
		return;
	}
	nrn_remove_sibling_list(sec);
	sec->parentsec = (Section*)0;
	sec->parentnode = (Node*)0;
	nrn_parent_info(sec);
	nrn_relocate_old_points(sec, oldpnode, sec, sec->parentnode);
	for (ch = sec->child; ch; ch = ch->sibling) if (nrn_at_beginning(ch)){
		ch->parentnode = sec->parentnode;
		nrn_relocate_old_points(ch, oldpnode, ch, ch->parentnode);
	}
	section_unref(oldpsec);
	tree_changed = 1;
}

static connectsec_impl(parent, sec) Section* parent, *sec;
{
	Section* ch;
	Section* oldpsec = sec->parentsec;
	Node* oldpnode = sec->parentnode;
	double d1, d2;
	Datum *pd;
	d2 = xpop();
	d1 = xpop();
	if (d1 != 0. && d1 != 1.) {
		hoc_execerror(secname(sec), " must connect at position 0 or 1");
	}
	if (d2 < 0. || d2 > 1.) {
		hoc_execerror(secname(sec), " must connect from 0<=x<=1 of parent");
	}
	if (sec->parentsec) {
		fprintf(stderr, "Notice: %s(%g)",
			secname(sec), nrn_section_orientation(sec)
		);
		fprintf(stderr, " had previously been connected to parent %s(%g)\n",
			secname(sec->parentsec), nrn_connection_position(sec)
		);
		nrn_remove_sibling_list(sec);
	}
	if (nrn_section_orientation(sec) != d1) {
		reverse_sibling_list(sec);
	}
	pd = sec->prop->dparam;
	pd[1].val = d2;
	pd[3].val = d1;
	section_ref(parent);
	sec->parentsec = parent;
	nrn_add_sibling_list(sec);
	sec->parentnode = (Node*)0;
	nrn_parent_info(sec);
	nrn_relocate_old_points(sec, oldpnode, sec, sec->parentnode);
	for (ch = sec->child; ch; ch = ch->sibling) if (nrn_at_beginning(ch)){
		ch->parentnode = sec->parentnode;
		nrn_relocate_old_points(ch, oldpnode, ch, ch->parentnode);
	}
	if (oldpsec) {
		section_unref(oldpsec);
	}else if (oldpnode) {
		nrn_rootnode_free(oldpnode, sec);
	}
	tree_changed = 1;
	diam_changed = 1;
}

simpleconnectsection() /* 2 expr on stack and two sections on section stack */
	/* for a prettier syntax: connect sec1(x), sec2(x) */
{
	Section* parent, *child;
	parent = nrn_sec_pop();
	child = nrn_sec_pop();
	connectsec_impl(parent, child);	
}

connectsection() /* 2 expr on stack and section symbol on section stack */
{
	Section* parent, *child;
	child = nrn_sec_pop();
	parent = chk_access();
	connectsec_impl(parent, child);
}

static Section *
Sec_access()	/* section symbol at pc */
{
	Item* itm;
	Symbol *s = (pc++)->sym;

	if (!s) {
		return chk_access();
	}
	nrn_chk_section(s);
	itm = OPSECITM(s)[range_vec_indx(s)];
	if (!itm) {
		hoc_execerror(s->name, ": section was deleted");
	}
	return itm->element.sec;
#if OVERLOAD
	access_mechanism = 0;
#endif
#if 0
printf("accessing %s with index %d\n", s->name, access_index);	
#endif
}

sec_access() { /* access section */
	Section* sec = chk_access();
	++sec->refcount;
	nrn_popsec();
	if (*secptr) {
		section_unref(*secptr);
	}
	*secptr = sec;
}

sec_access_object() { /* access section */
	Section* sec;
	if (! section_object_seen) {
		hoc_execerror("Access: Not a section", (char*)0);
	}
	sec = chk_access();
	++sec->refcount;
	nrn_popsec();
	if (*secptr) {
		section_unref(*secptr);
	}
	*secptr = sec;
	section_object_seen = 0;
}

sec_access_push()
{
	nrn_pushsec(Sec_access());
}

Section*
nrn_sec_pop()
{
	Section* sec = chk_access();
	nrn_popsec();
#if OVERLOAD
access_mechanism = 0;
#endif
	return sec;
}

/* in an object syntax a section may either be last or next to last
in either case it is pushed when it is seen in hoc_object_component
and section_object_seen is set.
If it was last then this is called with it set and we do nothing.
ie the section is on the stack till the end of the next statement.
If it was second to last then it hoc_object_component will set it
back to 0 and pop the section. Therefore we need to push a section
below to keep the stack ok when it is popped at the end of the next
statement.
*/

ob_sec_access_push(qsec)
	Item* qsec;
{
	if (!qsec) {
		hoc_execerror("section in the object was deleted", (char*)0);
	}
	nrn_pushsec(qsec->element.sec);
}

ob_sec_access() {
	if (!section_object_seen) {
		if (hoc_stack_type() != NUMBER) {
hoc_execerror("statement parsed as: sec stmt and sec is not a section\n",
"enclose in () if the statement should be parsed as an expression");
		}
		hoc_xpop();
		nrn_pushsec(*secptr);
	}
	section_object_seen = 0;
}

/* For now there is always one more node than segments and the last
node has no properties. This fact is spread everywhere in which
nnode is dealt with */

mech_access() {	/* section symbol at pc */
	Section* sec = chk_access();
	Symbol* s = (pc++)->sym;
	mech_insert1(sec, s);
}

mech_insert1(sec, s)
	Section* sec;
	Symbol* s;
{
	int n, i;
	Node *nd, *pnd;
	Prop *m;
	/* make sure that all nodes but last of the current section have this
	   membrane type */
	/* subsequent range variables refer to this mechanism */
	/* For now, we assume:
	   1) Parameters for different mechanisms do not have to
	      have distinct names. Thus access mech marks those
	      symbols which are allowed.
	      Another access section gets back to the geometry properties.
	   2) Membrane is installed in entire section. (except last node)
	*/
	/* Is the property already allocated*/
	n = sec->nnode - 1;
	nd = sec->node;
#if METHOD3
	/* For method3 the parent must also have the property. Also
	   the last node may have it because one of the connecting sections
	   have it. */
	if (_method3) {
		Section* psec = sec->parentsec;

		/* must be in parent */
		Node* pnd = psec->node + sec->parentnode;
		if (!nrn_mechanism(s->subtype, pnd)) {
			prop_alloc(&pnd->prop, s->subtype);
		}
		/* method3 has mechanisms in the "zero area" node. */
		n = sec->nnode;

		/* may already be in last node */
		if (nrn_mechanism(s->subtype, nd + n - 1)) {
			--n;
		}
	}
#endif
	m = nrn_mechanism(s->subtype, nd);
	if (!m) { /* all nodes get the property */
		for (i = n - 1; i >= 0; i--) {
			pnd = nd + i;
			IGNORE(prop_alloc(&pnd->prop, s->subtype));
		}
#if EXTRACELLULAR
		if (s->subtype == EXTRACELL) {
			prop_alloc(&nd[n].prop, s->subtype); /*even last node*/
			extcell_2d_alloc(sec);
		}
#endif
	}
}

mech_uninsert() {
	Section* sec = chk_access();
	Symbol* s = (pc++)->sym;	
	mech_uninsert1(sec, s);
}

mech_uninsert1(sec, s)
	Section* sec;
	Symbol* s;
{
	/* remove the mechanism from the currently accessed section */
	int n, i;
	Prop *m, *mnext;
	int type = s->subtype;

	if (type == MORPHOLOGY 
#if EXTRACELLULAR
	  || type == EXTRACELL
#endif
	) {
		hoc_warning("Can't uninsert mechanism", s->name);
		return;
	}
	if (nrn_is_ion(type)) {
		hoc_warning("Not allowed to uninsert ions at this time", s->name);
		return;
	}
#if METHOD3
	if (_method3) {
		hoc_execerror("Can't uninsert mechanisms when spatial method is non_zero", s->name);
	}
#endif
	n = sec->nnode;
	for (i=0; i < n; ++i) {
		mnext = sec->node[i].prop;
		if (mnext && mnext->type == type) {
			sec->node[i].prop = mnext->next;
			single_prop_free(mnext);
			continue;
		}
		for (m = mnext; m; m = mnext) {
			mnext = m->next;
			if (mnext && mnext->type == type) {
				m->next = mnext->next;
				single_prop_free(mnext);
				break;
			}
		}
	}
}

nrn_rangeconst(sec, s, d)
	Section* sec;
	Symbol* s;
	double d;
{
	short n, i;
	Node *nd;
	int indx;
	
#if METHOD3
	/* this is highly restrictive at this time. All parameters except
	morphology must be uniform across section connections because
	the density of channels is assumed to be uniform during matrix
	construction. ie the node does not know which side gets which value
	*/
	if (_method3) {
		n = sec->nnode;
	}else
#endif
	n = sec->nnode - 1;
	if (s->u.rng.type == VINDEX) {
		nd = node_ptr(sec, 0., (double *)0);
		nd->v = d;
		nd = node_ptr(sec, 1., (double *)0);
		nd->v = d;
		for (i=0; i<n; i++) {
			sec->node[i].v = d;
		}
	}else{
		indx = range_vec_indx(s);
		for (i=0; i<n; i++) {
			*(dprop(s, indx, sec, i)) = d;
		}
		if (s->u.rng.type == MORPHOLOGY) {
			nrn_diam_change(sec, d);
			diam_changed = 1;
		}
#if EXTRACELLULAR
		if (s->u.rng.type == EXTRACELL && s->u.rng.index == 0) {
			diam_changed = 1;
		}
#endif
#if METHOD3
		/* this is a hack and way too restrictive */
		if (_method3) { /*the parent node also gets the value */
			if (tree_changed) {
				setup_topology();
			}
			*(dprop(s, indx, sec->parentsec, sec->parentnode)) = d;
		}
#endif
	}
}

range_const()	/* rangevariable symbol at pc, value on stack */
{
	Section* sec;
	double d;
	Symbol *s = (pc++)->sym;
	
	d = xpop();
	sec = nrn_sec_pop();
	nrn_rangeconst(sec, s, d);
	hoc_pushx(d);
}

static int
range_vec_indx(s)
	Symbol *s;
{
	int indx;
	
	if (ISARRAY(s)) {
		indx = hoc_araypt(s, SYMBOL);
	}else{
		indx = 0;
	}
	return indx;
}


#if 0
/* obsolete */
static double *
pnrnpnt(s)
	Symbol *s;
{
	int vec_index, nrnpnt_index;
	Point_process *point;
	extern Point_process *point_process[];

	vec_index = nrnpnt_araypt(s, &nrnpnt_index);
	point = point_process[s->u.rng.type] + nrnpnt_index;
	if (!point->prop) {
hoc_execerror(s->name, ": loc_...(index,x) has not been executed.");
	}
	if (s->subtype != NRNPOINTER) {
		return point->prop->param + s->u.rng.index + vec_index;	
	}else{
double **p = &((point->prop->dparam)[s->u.rng.index + vec_index].pval);
		if (!(*p)) {
			hoc_execerror(s->name, "wasn't made to point to anything");
		}
		return *p;
	}
}

nrnpnt_push() {
	Symbol *s = (pc++)->sym;
	hoc_pushpx(pnrnpnt(s));
}
	

nrnpnt_asgn() { /* symbol at pc, value on stack, index on stack */
	double d, *p;
	Symbol *s = (pc++)->sym;

	d = xpop();
	p = pnrnpnt(s);
	*p = d;
	hoc_pushx(d);
}

nrnpnt_eval() { /* symbol at pc,  index on stack */
	double d;
	Symbol *s = (pc++)->sym;

	d = *pnrnpnt(s);
	hoc_pushx(d);
}

static Datum *
ppnrnpnt(s)
	Symbol *s;
{
	Datum *p;
	int vec_index, nrnpnt_index;
	Point_process *point;
	extern Point_process *point_process[];

	vec_index = nrnpnt_araypt(s, &nrnpnt_index);
	point = point_process[s->u.rng.type] + nrnpnt_index;
	if (!point->prop) {
hoc_execerror(s->name, ": loc_...(index,x) has not been executed.");
	}
	p = (point->prop->dparam) + s->u.rng.index + vec_index;
	return p;
}

/* above is obsolete */
#endif

Prop *
nrn_mechanism(type, nd) /* returns property for mechanism at the node */
	int type;
	Node* nd;
{
	Prop *m;
	for (m = nd->prop; m; m = m->next) {
		if (m->type == type) {
			break;
		}
	}
	return m;
}
Prop *
nrn_mechanism_check(type, sec, inode)
	int type, inode;
	Section* sec;
{
	Prop *m;
	m = nrn_mechanism(type, sec->node + inode);
	if (!m) {
		if (hoc_execerror_messages) {
		  Fprintf(stderr, "%s mechanism not inserted in section %s\n",
			memb_func[type].sym->name, secname(sec));
		}
		hoc_execerror("", (char *)0);
	}
	return m;
}
	
Prop *
hoc_getdata_range(mechname)
	char *mechname;
{
	Symbol *s;
	int inode;
	Section *sec;
	
	s = hoc_lookup(mechname);
	assert(s && s->type == MECHANISM);
	sec = chk_access();
	inode = node_index(sec, *getarg(1));
	return nrn_mechanism_check(s->subtype, sec, inode);
}

static Datum *
pdprop(s, indx, sec, inode) /* basically copied from dprop for use
				to connect a nmodl POINTER */
	Symbol *s;
	Section* sec;
	short inode;
	int indx;
{
	Prop *m;
	
	m = nrn_mechanism_check(s->u.rng.type, sec, inode);
	return 	m->dparam + s->u.rng.index + indx;
}

connectpointer() { /* pointer symbol at pc, target variable on stack, maybe
	range variable location on stack */
	Datum *dat;
	double *pd, *hoc_pxpop();
	double d;
	Symbol *s = (pc++)->sym;
	pd = hoc_pxpop();
	if (s->subtype != NRNPOINTER) {
		hoc_execerror(s->name, "not a model variable POINTER");
	}
#if 0
/* can't be since parser sees object syntax and generates different code. */
	if (s->type == NRNPNTVAR) {
		dat = ppnrnpnt(s);
	}else
#endif
	{
		short i;
		Section* sec;

		d = xpop();
		sec = nrn_sec_pop();
		i = node_index(sec, d);
		dat = pdprop(s, range_vec_indx(s), sec, i);
	}
	dat->pval = pd;
}

range_interpolate_single() /*symbol at pc, 2 values on stack*/
{
	double x, y;
	Symbol* s;
	Section* sec;
	double* pd, *nrn_rangepointer();
	
	s = (pc++)->sym;
	y = xpop();
	x = xpop();
	sec = nrn_sec_pop();

	if (s->u.rng.type == MORPHOLOGY) {
		if (!can_change_morph(sec)) {
			return;
		}
		diam_changed = 1;
	}
	
	pd = nrn_rangepointer(sec, s, x);
	*pd = y;
	
#if EXTRACELLULAR
	if (s->u.rng.type == EXTRACELL && s->u.rng.index == 0) {
		diam_changed = 1;
	}
#endif
}

range_interpolate() /*symbol at pc, 4 values on stack*/
{
	short i, i1, i2;
	Section* sec;
	double y1, y2, x1, x2, z, dz;
	Symbol *s = (pc++)->sym;
	int indx;
	Node *nd;

#if METHOD3
	if (_method3) {
		hoc_execerror("Range variable interpolation not implemented",
		"for new spatial discretization methods");
	}
#endif
	y2 = xpop(); y1 = xpop();
	x2 = xpop(); x1 = xpop();
	sec = nrn_sec_pop();
	i2 = node_index(sec, x2);
	i1 = node_index(sec, x1);
	if (nrn_section_orientation(sec)) {
		int it; double t;
		it = i1; i1 = i2; i2 = it;
		t = y1; y1 = y2; y2 = t;
		t = x1; x1 = x2; x2 = t;
	}
	
	if (i2 > i1) {
		dz = (y2 - y1)/((double)(i2 - i1));
		z = y1;
	} else {
		dz = 0.;
		z = (y2 + y1)/2.;
	}
	if (s->u.rng.type == VINDEX) {
		if (x1 == 0. || x1 == 1.) {
			nd = node_ptr(sec, x1, (double *)0);
			nd->v = y1;
		}
		if (x2 == 1. || x2 == 0.) {
			nd = node_ptr(sec, x2, (double *)0);
			nd->v = y2;
		}
		for (i=i1; i<=i2; i++) {
			sec->node[i].v = z;
			z += dz;
		}
		return;
	}
	if (s->u.rng.type == MORPHOLOGY) {
		if (!can_change_morph(sec)) {
			return;
		}
		diam_changed = 1;
	}
	indx = range_vec_indx(s);
	for (i=i1; i<=i2; i++) {
		*dprop(s, indx, sec, i) = z;
		z += dz;
	}
#if EXTRACELLULAR
	if (s->u.rng.type == EXTRACELL && s->u.rng.index == 0) {
		diam_changed = 1;
	}
#endif
}

int nrn_exists(s, node) 
	Symbol* s;
	Node* node;
{
	if (s->u.rng.type == VINDEX) {
		return 1;
	}else if (nrn_mechanism(s->u.rng.type, node)) {
		return 1;
	}else{
		return 0;
	}
}

double* nrn_rangepointer(sec, s, d)
	Section* sec;
	Symbol* s;
	double d;
{
	short i;
	Node *nd;

	if (s->u.rng.type == VINDEX) {
		nd = node_ptr(sec, d, (double *)0);
		return &nd->v;
	}
	i = node_index(sec, d);
	return dprop(s, range_vec_indx(s), sec, i);
}

rangevarevalpointer() /* symbol at pc, location on stack, return pointer on stack */
{
	short i;
	Section* sec;
	double d;
	Symbol *s = (pc++)->sym;
	Node *nd;

	d = xpop();
	sec = nrn_sec_pop();
	if (s->u.rng.type == VINDEX) {
		nd = node_ptr(sec, d, (double *)0);
		hoc_pushpx(&nd->v);
		return;
	}
	i = node_index(sec, d);
	hoc_pushpx(dprop(s, range_vec_indx(s), sec, i));
}

rangevareval() /* symbol at pc, location on stack, return value on stack */
{
	double *pd, *hoc_pxpop();

	rangevarevalpointer();
	pd = hoc_pxpop();
	hoc_pushx(*pd);
}

rangepoint() /* symbol at pc, return value on stack */
{
	hoc_pushx(.5);
	rangevareval();
}

#if OVERLOAD
int
chk_mechsym(s)
	Symbol *s;
{
	if (s->u.rng.type == VINDEX) {
		return VINDEX;
	}
	if (access_mechanism != s->u.rng.type) {
		execerror("Must open mechanism for" , s->name);
	}
	return s->u.rng.index;
}
#endif

int
node_index(sec, x) /* returns nearest index to x */
	Section *sec;
	double x;
{
	int i;
	double n;

	if (x < 0. || x > 1.) {
		hoc_execerror("range variable domain is 0<=x<=1", (char *)0);
	}
#if METHOD3
	if (_method3) {
		n = sec->nnode;
	}else
#endif
	n = sec->nnode - 1;
#if 0
	if (n < 0) {
		hoc_execerror(secname(sec), "has no segments");
	}
#else
	assert(n >= 0);
#endif
	i = (int)(n*x);
	if (i == n) {
		i = n - 1;
	}
	if (sec->prop->dparam[3].val) {
		i = n - i - 1;
	}
	return i;
}

/* return -1 if x at connection end, nnode-1 if at other end */
int
node_index_exact(sec, x)
	Section* sec;
	double x;
{
	if (x == 0.) {
		if (arc0at0(sec)) {
			return -1;
		}else{
			return sec->nnode - 1;
		}
	}else if ( x == 1.) {
		if (arc0at0(sec)) {
			return sec->nnode - 1;
		}else{
			return -1;
		}
	}else{
		return node_index(sec, x);
	}
}
/*USERPROPERTY subtype of VAR used for non range variables associated
with section property
may have special actions (eg. nseg)*/

double
cable_prop_eval(sym)
	Symbol *sym;
{
	Section* sec;
	sec = nrn_sec_pop();
	switch(sym->u.rng.type) {
	
	case 0: /* not in property list so must be nnode */
#if METHOD3
		if (_method3) {
			return (double) sec->nnode;
		}else
#endif
		return (double)	sec->nnode - 1;
	case CABLESECTION:
		return sec->prop->dparam[sym->u.rng.index].val;
	default:
		hoc_execerror(sym->name, " not a USERPROPERTY");
	}
	return 0.;
}
double*
cable_prop_eval_pointer(sym)
	Symbol *sym;
{
	Section* sec;
	sec = nrn_sec_pop();
	switch(sym->u.rng.type) {
	
	case CABLESECTION:
		return &sec->prop->dparam[sym->u.rng.index].val;
	default:
		hoc_execerror(sym->name, " not a USERPROPERTY that can be pointed to");
	}
	return (double *)0;
}

cable_prop_assign(sym, d)
	Symbol *sym;
	double d;
{
	int i;
	Section* sec;
	Node *nd;
	sec = nrn_sec_pop();
	switch(sym->u.rng.type) {
	
	case 0: /* not in property list so must be nnode */
		{ short n = (short) d;
		if (n < 1) {
			hoc_execerror(sym->name, " must be positive");
		}
#if METHOD3
		if (_method3) {
			--n;
		}
#else
		if (sec->nnode == n + 1) {
			return;
		}else
#endif
		{
			node_alloc(sec, n + 1);
			tree_changed = 1;			
			diam_changed = 1;
			nd = sec->node;
#if METHOD3
			if (_method3) {
				++n;
			}
#endif
			for (i=0; i<n; i++) {
				IGNORE(prop_alloc(&nd[i].prop, MORPHOLOGY));
				IGNORE(prop_alloc(&nd[i].prop, CAP));
			}
		}
		}
		break;
	case CABLESECTION:
		if (sym->u.rng.index == 2) {
			nrn_length_change(sec, d);
		}
		diam_changed = 1;
		sec->prop->dparam[sym->u.rng.index].val = d;
#if RA_WARNING
		if (sym->u.rng.index == 7) {
			++nrn_ra_set;
		}
#endif
		break;
	default:
		hoc_execerror(sym->name, " not a USERPROPERTY");
	}
}


double
nrn_connection_position(sec)
	Section* sec;
{
	return sec->prop->dparam[1].val;
}

double
nrn_section_orientation(sec)
	Section* sec;
{
	return sec->prop->dparam[3].val;
}

int nrn_at_beginning(sec)
	Section * sec;
{
	assert(sec->parentsec);
	return nrn_connection_position(sec) == nrn_section_orientation(sec->parentsec);
}

static nrn_rootnode_alloc(sec)
	Section* sec;
{
	Node* nd, *node_construct();
	nd = node_construct(1);
	lappendvoid(rootsection_list, (void*)sec);	
	section_ref(sec);			
	sec->parentnode = nd;
}

Section*
nrn_trueparent(sec)
	Section* sec;
{
	Section* psec;
	for (psec = sec->parentsec; psec; psec = psec->parentsec) {
		if (nrn_at_beginning(sec)) {
			sec = psec;
		}else{
			break;
		}
	}
	return psec;
}

nrn_parent_info(s)
	Section* s;
{
	/* determine the parentnode using only the authoritative
	info of parentsec
	and arc length connection position */
	/* side effects are that on exit, s will have the right parentnode
	   and the true parent (not connected to any section) will have the
	   right parentnode
	*/

	Section* sec, *psec, *true_parent;
	Node* pnode;
	double x;
	
	true_parent = (Section*)0;
	for (sec = s, psec = sec->parentsec; psec; sec = psec, psec = sec->parentsec) {
		if (psec == s) {
			fprintf(stderr, "%s connection to ", secname(s));
			fprintf(stderr, "%s will form a loop\n", secname(s->parentsec));
			nrn_disconnect(s);
			hoc_execerror(secname(s), "connection will form loop");
		}
		x = nrn_connection_position(sec);
		if (x != nrn_section_orientation(psec)) {
			true_parent = psec;
			if (x == 1. || x == 0.) {
				pnode = psec->node + psec->nnode - 1;
			}else{
				pnode = psec->node + node_index(psec, x);
			}
			break;
		}
	}
	if (true_parent == (Section*)0) {
		if (sec->parentnode) {
			/* non nil parent node in section without a parent is
				definitely valid
			*/
			pnode = sec->parentnode;
		}else{
			nrn_rootnode_alloc(sec);
			pnode = sec->parentnode;
		}
	}
	s->parentnode = pnode;
}

setup_topology()
{
	Item* qsec;
			
	/* use connection info in section property to connect nodes. */
	/* for the moment we assume uniform dx and range 0-1 */

	/* Sections may be connected to a parent at an orientation (usually
		0) at which the node belongs to some ancestor of the parent.
		All difficulties derive from that fact.
		ie. the parentnode may not belong to the parentsec. And
		parent nodes may be invalid. And if they are valid then
		we want to maintain the useful info such as point processes
		which are in them. ie, we only change the parentnode here
		if it is invalid.
		To help with this we maintain a rootsectionlist (hoc_List
		of sections)
	*/
	
	rootnodecount = 0;

	ForAllSections(sec)
#if 0
		if (sec->nnode < 1) { /* last node is not a segment */
			hoc_execerror(secname(sec),
				" has no segments");
		}
#else
		assert(sec->nnode > 0);
#endif
		nrn_parent_info(sec);
		if (!sec->parentsec) {
			++rootnodecount;
		}
	}

#if METHOD3
		if (_method3) {
			int i;
			Node* nd = root/*obsolete*/section->node;
			for (i=0; i<unconnected; i++) {
				IGNORE(prop_alloc(&nd[i].prop, MORPHOLOGY));
				IGNORE(prop_alloc(&nd[i].prop, CAP));
			}
		}
#endif
	section_order();
	tree_changed = 0;
}

char *
secname(sec) /* name of section (for use in error messages) */
	Section *sec;
{
	extern Objectdata* hoc_top_level_data;
	extern char* hoc_araystr();
	static char name[200];
	Symbol *s;
	int indx;
	Object* ob;

	if (sec && sec->prop) {
		s = sec->prop->dparam[0].sym;
		indx = sec->prop->dparam[5].i;
		ob = sec->prop->dparam[6].obj;
		if (ob) {
			Sprintf(name, "%s.%s%s",
				hoc_object_name(ob),
				s->name, hoc_araystr(s, indx, ob->u.dataspace));
		}else{
			Sprintf(name, "%s%s", s->name,
				hoc_araystr(s, indx, hoc_top_level_data));
		}
	}else{
		name[0] = '\0';
	}
	return name;
}

char* hoc_section_pathname(sec)
	Section* sec;
{
	extern char* hoc_araystr(), *hoc_object_pathname();
	Symbol* s;
	int indx;
	static char name[200];
	Object* ob;

	if (sec && sec->prop) {
		s = sec->prop->dparam[0].sym;
		indx = sec->prop->dparam[5].i;
		ob = sec->prop->dparam[6].obj;
		if (ob) {
			char* p = hoc_object_pathname(ob);
			if (p) {
				Sprintf(name, "%s.%s%s", p,
				s->name, hoc_araystr(s, indx, ob->u.dataspace));
			}else{
				hoc_execerror("Can't find a pathname for", secname(sec));
				return secname(sec);
			}
		}else{
			Sprintf(name, "%s%s", s->name, hoc_araystr(s, indx, hoc_objectdata));
		}
	}else{
		name[0] = '\0';
	}
	return name;
}

double
nrn_arc_position(sec, node)
	Section *sec;
	Node* node;
{
	int inode;
	double x;
	
	assert(sec);
	if (sec->parentnode == node) {
		x = 0.;
	}else if ((inode = node - sec->node) == sec->nnode-1) {
		x = 1.;
	}else{
		x = ((double)inode+.5)/((double)sec->nnode - 1.);
	}
	if (arc0at0(sec)) {
		return  x;
	}else{
		return 1. - x;
	}
}

#if 0
double nrn_arc_position(sec, i)
	Section* sec;
	int i;
{
	double x;
	int n;
	n = sec->nnode - 1;
	if (i == n) {
		x = 1.;
	}else{
#if METHOD3
		if (_method3) {
			x = (i+1)/((double)n);
		}else
#endif
		x = (i+.5)/((double)n);
	}
	if (sec->prop->dparam[3].val) {
		x = 1. - x;
	}
	return x;
}
#endif

char * sec_and_position(sec, nd)
	Section* sec;
	Node* nd;
{
	char* buf;
	static char buf1[200];
	double x;
	assert(sec);
	buf = secname(sec);
	x = nrn_arc_position(sec, nd);
	sprintf(buf1, "%s(%g)", buf, x);
	return buf1;
}

int
segment_limits(pdx)
	double *pdx;
{
	int n;
	Section* sec;
	double l;
	
	sec = chk_access();
#if METHOD3
	if (_method3) {
		n = sec->nnode;
	}else
#endif
	n = sec->nnode - 1;
/*	l = sec->prop->dparam[2].val;*/
	l = 1.;
	*pdx = l/((double)n);
	return sec->nnode;
}

#undef PI
#define PI 3.14159

Node*
node_exact(sec, x)
	Section *sec;
	double x;
{
	/* like node_index but give proper node when
		x is 0 or 1 as well as in between
	*/
	Node* node;
		
	assert(sec);
#if METHOD3
   if (_method3) {
	inode = (int)(x*sec->nnode + .5);	/* ranges from 0 to nnode */
		if (tree_changed) {
			setup_topology();
		}
		if(sec->prop->dparam[3].val) {
			inode = sec->nnode - inode;
		}
		--inode;
		if (inode < 0) {
			inode = sec->parentnode;
			sec = sec->parentsec;
			*psec = sec;
		}
   }else
#endif
   {
	if (x <= 0. || x >= 1.) {
		if (tree_changed) {
			setup_topology();
		}
		x = (x < 0.)?0.:x;
		x = (x > 1.)?1.:x;
		if(sec->prop->dparam[3].val) {
		 x = 1. - x;
		}
		if (x == 0.) {
			node = sec->parentnode;
		} else {
			node = sec->node + sec->nnode - 1;
		}
	} else {		
		node = sec->node + node_index(sec, x);
	}
   }
	return node;
}

Node *
node_ptr(sec, x, parea) /* returns pointer to proper node */
	Section *sec;	/* and the area of the node */
	double x, *parea;
{
	Node *nd;

	nd = node_exact(sec, x);
	if(parea) {
		if (diam_changed) {
			recalc_diam();
		}
		*parea = nd->area;
	}
	return nd;
}

int nrn_get_mechtype(mechname) char* mechname; {
	Symbol* s, *hoc_table_lookup();
	s = hoc_lookup(mechname);
	assert(s);
	if (s->type == TEMPLATE) {
		s = hoc_table_lookup(mechname, s->u.template->symtable);
		assert(s && s->type == MECHANISM);
	}
	return s->subtype;
}

#if VECTORIZE
int nrn_instance_count(mechtype) int mechtype; {
	extern int v_structure_change;
	if (v_structure_change) {
		v_setup_vectors();
	}
	return memb_list[mechtype].nodecount;
}
#endif

double *
dprop(s, indx, sec, inode) /* returns location of property symbol */
	Symbol *s;
	Section* sec;
	short inode;
	int indx;
{
	Prop *m;
	
	m = nrn_mechanism_check(s->u.rng.type, sec, inode);
#if EXTRACELLULAR
/* this does not handle vext(0) and vext(1) properly at this time */
	if (m->type == EXTRACELL && s->u.rng.index == 4) {
		return &sec->node[inode].extnode->v;
	}
#endif
	if (s->subtype != NRNPOINTER) {
		return &(m->param[s->u.rng.index]) + indx;
	}else{
double **p = &((m->dparam)[s->u.rng.index + indx].pval);
		if (!(*p)) {
			hoc_execerror(s->name, "wasn't made ot point to anything");
		}
		return *p;
	}
}

static char* objectname() {
	static char buf[100];
	extern Object* hoc_thisobject;
	if (hoc_thisobject) {
		Sprintf(buf, "%s", hoc_object_name(hoc_thisobject));
	}else{
		buf[0] = '\0';
	}
	return buf;
}

#define relative(pc)	(pc + (pc)->i)

forall_section() {
	/*statement pointed to by pc
	continuation pointed to by pc+1. template used is shortfor in code.c
	of hoc system.
	*/
	/* if inside object then forall refers only to sections in the object */
	
	Inst *savepc = pc;
	Item* qsec;
	extern int hoc_returning;
	char buf[200];
	char** s;
	char** hoc_strpop();

	s = hoc_strpop();
	buf[0] = '\0';
	if (s) {
		sprintf(buf, "%s.*%s.*", objectname(), *s);
	}else{
		char* o = objectname();
		if (o[0]) {
			sprintf(buf, "%s.*", o);
		}
	}
#if 0
	ForAllSections(sec)
#else
	/* do the iteration safely. a possible command is to delete the section*/
	for (qsec = section_list->next; qsec != section_list;) {
		Section* sec = SEC(qsec);
		qsec = qsec->next;
#endif
		if (buf[0]) {
			hoc_regexp_compile(buf);
			if (! hoc_regexp_search(secname(sec))) {
				continue;
			}
		}
		nrn_pushsec(sec);
		hoc_execute(relative(savepc));
		nrn_popsec();
		if (hoc_returning == 1 || hoc_returning == 4) {
			break;
		}else if (hoc_returning == 2) {
			hoc_returning = 0;
			break;
		}else{
			hoc_returning = 0;
		}
	}
	if (!hoc_returning)
		pc =relative(savepc+1);
}

hoc_ifsec() {
	Inst *savepc = pc;
	char buf[200];
	char** s;
	char** hoc_strpop();
	extern int hoc_returning;
	
	s = hoc_strpop();
	sprintf(buf, ".*%s.*", *s);
	hoc_regexp_compile(buf);
	if (hoc_regexp_search(secname(chk_access()))) {
		hoc_execute(relative(savepc));
	}
	if (!hoc_returning)
		pc = relative(savepc+1);
}

issection() { /* returns true if string is the access section */
	hoc_regexp_compile(gargstr(1));
	if (hoc_regexp_search(secname(chk_access()))) {
		ret(1.);
	}else{
		ret(0.);
	}
}

ismembrane() { /* return true if string is an inserted membrane in the
		access section */
	char *str;
	Prop *p;

	str = gargstr(1);
	for (p = chk_access()->node->prop; p; p = p->next) {
		if (strcmp(memb_func[p->type].sym->name, str) == 0) {
			ret(1.);
			return;
		}
	}
	ret(0.);
}


char* secaccessname() {
	return secname(chk_access());
}

sectionname() {
	char *buf, **cpp, **hoc_pgargstr();
	
	buf = secname(chk_access());
	cpp = hoc_pgargstr(1);
	hoc_assign_str(cpp, buf);
	ret(1.);
}

this_section() {
	/* return section number of currently accessed section at
		arc length postition x */
	
	Section* sec;
	sec = chk_access();
	ret((double)(long)(sec));
}
this_node() {
	/* return node number of currently accessed section at
		arc length postition x */
	
	Section* sec;
	Node* nd;
	sec = chk_access();
	nd = node_exact(sec, *getarg(1));
	ret((double)(long)nd);
}
parent_section() {
	/* return section number of currently accessed section at
		arc length postition x */
	
	Section* sec;
	sec = chk_access();
	ret((double)(long)(sec->parentsec));
}
parent_node() {
	Section* sec;
	hoc_execerror("parent_node() needs to be re-implemented", 0);
	sec = chk_access();
	ret((double)(long)(sec->parentnode));
}

pop_section() {
	nrn_popsec();
	ret(1.);
}

push_section() {
	Section* sec;
	if (hoc_is_str_arg(1)) {
		Item* qsec;
		char* s;
		sec = (Section*)0;
		s = gargstr(1);
		ForAllSections(sec1) /* I can't imagine a more inefficient way */
			if (strcmp(s, secname(sec1)) == 0) {
				sec = sec1;
				break;
			}
		}
		if (!sec) {
			hoc_execerror("push_section: arg not a sectionname:", s);
		}
	}else{
		sec = (Section*)(long)(*getarg(1));
	}
	if (!sec || !sec->prop || sec->prop->dparam[0].sym->type != SECTION) {
		hoc_execerror("Not a Section pointer", (char*)0);
	}
	nrn_pushsec(sec);
	ret(1.0);
}


#if FISHER
assign_hoc_str(str, val, global) 

    /* Assign hoc string in global dataspace if global == 1,
       else use current dataspace. */

    char *str, *val;
    int global;
{
    Symbol *sym, *hoc_table_lookup();
    char** pstr;
    extern Object* hoc_thisobject;
    extern Objectdata* hoc_top_level_data;
    Objectdata* obdsave = hoc_objectdata; /* save current dataspace */

    if (!global && hoc_thisobject) {
    	sym = hoc_table_lookup(str, hoc_thisobject->template->symtable);
    }else{
	hoc_objectdata = hoc_top_level_data;
	sym = hoc_lookup(str);
    }
    if (sym && sym->type == STRING) {
	/* point to string */
	pstr = OPSTR(sym);
	hoc_assign_str(pstr, val);	
    }
    hoc_objectdata = obdsave;
}
#endif

