/* $Header: /timshel2g/src/master/nrn/src/nrnoc/cabvars.h,v 1.1.1.1 1994/10/12 17:22:30 hines Exp $ */
/*
$Log: cabvars.h,v $
 * Revision 1.1.1.1  1994/10/12  17:22:30  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.71  1994/07/12  12:56:00  hines
 * SEClamp single electrode clamp is built-in
 *
 * Revision 4.42  1993/07/27  15:52:05  hines
 * Xmech dummy mechananism for installation of unknown mechanism for
 * teaching and examination purposes.
 *
 * Revision 4.24  93/04/14  08:05:35  hines
 * trivial typo
 * 
 * Revision 4.20  93/03/21  17:51:17  hines
 * new APCount() point process which is useful for determining occurence and
 * time of single action potential
 * 
 * node_data("filenam") prints minimal complete state (but not global variables)
 * for use with a minimal neuron system without the interpreter
 * 
 * Revision 3.43  92/12/16  14:24:28  hines
 * Ra switched from global variable to section variable in which it can
 * take on different values in different sections. Similar to L.
 * No need to set diam_changed when Ra changed.
 * 
 * Revision 2.71  92/08/05  09:26:50  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.65  92/04/13  16:23:15  hines
 * Old HH and passive built-in models discarded in favor of hh and pas
 * models from .mod files. The hh variables now have _hh suffixes
 * and the passive variables are g_pas and e_pas.
 * stim.mod, syn.mod, and vclmp.mod have also been added.
 * ftim, fsyn, and fclamp are still present but are considered obsolete
 * and will be eventually phased out.
 * 
 * Revision 2.60  92/03/20  09:38:27  hines
 * finitialize()
 * 
 * Revision 2.55  92/03/10  09:25:52  hines
 * nrniv: automatic menus - mainmenu(), good start.
 * 
 * Revision 1.89  90/11/12  07:36:38  hines
 * Big change in way ions are handled.  Each ion species has its own
 * mechanism which allocates current, reversal potential, inside concentration
 * and outside concentration.  Mechanisms which need an ion register their
 * need via ion_reg(name) during initialization.
 * 
 * Revision 1.87  90/10/29  12:31:19  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.75  90/08/06  17:22:50  hines
 * all intefacing of mechanisms with modl now comes from modl_reg();
 * 
 * Revision 1.73  90/07/31  17:06:49  hines
 * NMODL uses two include files, one for func reg names and other for
 * declarations.
 * New user models no longer need to modify cabvars.h
 * 
 * Revision 1.72  90/07/31  10:46:30  hines
 * detailed info removed from cabvars. Only thing necessary is to
 * declare a registration function.
 * 
 * Revision 1.48  90/01/16  07:05:15  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.45  89/10/30  09:55:08  mlh
 * range variables have a subtype of CONST, DEPENDENT, or STATE
 * 
 * Revision 1.37  89/09/20  12:02:15  mlh
 * ionic reversal potentials and currents now in Eion
 * other mechanisms, such as HH, use and modify Eion parameters.
 * a mechanism allocation routine can "need" another mechanism and
 * communicate via Datum.pval
 * 
 * Revision 1.26  89/08/31  14:49:03  mlh
 * name of mechanism out of memb_func and back in mechanism[].
 * memb_func[].sym now points to mechanism symbol.
 * This allows us to retrieve property names.
 * 
 * Revision 1.21  89/08/24  09:20:56  mlh
 * New channel additions localized to cabvars.h which in turn is
 * used only by init.c
 * 
 * Revision 1.16  89/08/12  09:50:29  mlh
 * HH added
 * 
 * Revision 1.15  89/08/11  15:12:00  mlh
 * rall rall branching, several bugs fixed
 * 
 * Revision 1.7  89/08/04  14:47:45  mlh
 * 2nd order correct in dx handled pretty well with nnode-1 segments
 * and last node being at the end of the section with 0 length.
 * Problem with connecting to position 0 since first node
 * not at beginning of section.
 * 
 * Revision 1.6  89/08/02  16:11:44  mlh
 * range variables implemented in open mechanism context
 * 
 * Revision 1.4  89/07/27  16:14:32  mlh
 * at point where topology() works
 * 
 * Revision 1.1  89/07/26  07:01:02  mlh
 * Initial revision
 * 
*/

#define XMECH 0

static struct {		/* USERPROPERTY */
	char	*name;
	short	type;
	short	index;
} usrprop[] = {
	"nseg", 0, 0,
	"L", 1, 2,
	"rallbranch", 1, 4,
	"Ra", 1, 7,
	0, 0, 0
};

extern  capac_reg(), passive0_reg(), _passive_reg(),
#if EXTRACELLULAR
	extracell_reg(),
#endif
	_stim_reg(),
	_syn_reg(),
	_svclmp_reg(),
	_vclmp_reg(),
	_apcount_reg(),
#if XMECH
	_xmech_reg(),
#endif
	_hh_reg();

static Pfri mechanism[] = { /* type will start at 3 */
	capac_reg,
	_passive_reg,
#if EXTRACELLULAR
	/* extracellular requires special handling and must be type 5 */
	extracell_reg,
#endif
	passive0_reg,	/* twice as fast as model description? */
	_stim_reg,
	_syn_reg,
	_svclmp_reg,
	_vclmp_reg,
	_apcount_reg,
	_hh_reg,
#if XMECH
	_xmech_reg,
#endif
	0
};

static char *morph_mech[] = { /* this is type 2 */
	"morphology", "diam", 0,0,0,
};

extern cab_alloc();
extern morph_alloc();

Memb_func memb_func[NMECHMAX] = { /* see membfunc.h */
	NULL_CUR, NULL_ALLOC, NULL_STATE, NULL_INITIALIZE, 0,	/*Unused*/
	NULL_CUR, cab_alloc, NULL_STATE, NULL_INITIALIZE, 0,	/*CABLESECTION*/
};

