/* $Header: /timshel2g/src/master/nrn/src/nrnoc/capac.c,v 1.1.1.1 1994/10/12 17:22:32 hines Exp $ */
/*
$Log: capac.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:32  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.77  1994/09/26  18:26:47  hines
 * ions more nearly do the "right thing"
 * presence of mechanism that WRITE's a concentration computes
 * the reversal potential every fadvance and finitialize, initializes
 * the concentrations to global variables, makes the concentrations have
 * the type STATE and the reversal potential the style DEP
 * presence of mech that READ's a concentration computes reversal
 * every finitialize, and treats concentration as CONST and reversal
 * potential style DEP.
 * If no mechanism reads or writes concentrations the reversal is
 * treated as CONST.
 *
 * Revision 4.65  1994/06/29  20:46:44  hines
 * allow easier coverage of treeset.c and benign umr of capac.c
 *
 * Revision 4.38  1993/07/13  11:48:10  hines
 * declare all current functions properly (now Pfri instead of Pfrd)
 *
 * Revision 3.33  92/10/24  16:30:11  hines
 * freeing vars notifies interviews.
 * sec_and_position(sec, inode) returns string in form section(x)
 * some portability fixes from pittsburgh
 * 
 * Revision 3.14  92/09/27  19:28:51  hines
 * secondorder works for _method3=1 and 2 but not 3
 * 
 * Revision 3.11  92/09/24  17:03:33  hines
 * METHOD3 option. use with spatial_method(i) with i=0-3
 * 0 is the default and is the normal neuron method with zero area nodes
 * at boundaries and branch points.
 * 1 is the standard method with half area nodes at boundaries
 * 2 is a modified second order method which weights by 2/12, 8/12, 2/12
 * 3 is a third order correct method which weights by 1/12 10/12 1/12
 * with the last three methods x = i/nnode with i=0 to nnode
 * methods 1 and 2 are at present quite inefficient since they use the
 * data structures of 3.
 * Works in steady state. not tested with dynamic simulations
 * At this time, discontinuities in parameters (except diameter) is
 * not handled well across nodes and all parameters are assumed to
 * be constant in interval between node and parent node.
 * Good deal of difficulty remains in managing values that change discontinuously
 * 
 * Revision 2.87  92/08/13  11:21:04  hines
 * didn't get checked in with last group
 * 
 * Revision 2.73  92/08/05  10:28:52  hines
 * got rid of objectcenter messages
 * 
 * Revision 2.71  92/08/05  09:26:52  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.60  92/03/20  09:38:37  hines
 * finitialize()
 * 
 * Revision 2.50  92/02/05  08:48:03  hines
 * Can connect nmodl POINTER variables to external variables using:
 * connect var_suffix.section(x), anyvar for range variables or
 * connect var_suffix[i], anyvar for point process variables
 * This can be dangerous if the user fails to make these variables point to
 * something.
 * 
 * Revision 1.91  90/11/29  17:19:41  hines
 * secondorder replaces implicit
 * secondorder=0 means implicit with no adjustment of current
 * secondorder=1 means crank-nicolson with no adjustment of current
 * secondorder=2 means crank-nicolson with adjustment of current to t-dt/2
 * 
 * Revision 1.87  90/10/29  12:31:21  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:31  hines
 * saber warnings gone
 * 
 * Revision 1.79  90/10/10  08:49:48  hines
 * Oops. fix of warnings didn't make it to the files
 * 
 * Revision 1.72  90/07/31  10:47:30  hines
 * detailed info removed from cabvars. Only thing necessary is to
 * declare a registration function.
 * 
 * Revision 1.52  90/01/24  07:04:34  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.44  89/10/30  07:43:21  mlh
 * crank-nicolson available with implicit=0
 * 
 * Revision 1.18  89/08/12  14:48:41  mlh
 * Lint free
 * 
 * Revision 1.16  89/08/12  09:50:41  mlh
 * inconsistent units now fixed
 * 
 * Revision 1.9  89/08/07  10:23:28  mlh
 * getting good v with single section. matrix solving for v (not deltav)
 * 
 * Revision 1.3  89/07/21  17:38:20  mlh
 * compiles
 * 
 * Revision 1.1  89/07/20  07:51:59  mlh
 * Initial revision
 * 
*/

#include	"section.h"
#include	"membdef.h"
extern double dt;
extern int secondorder;
static char *mechanism[] = { "capacitance", "cm",0,0,0 };
static cap_alloc(), cap_cur();
capac_reg() {
#if VECTORIZE
	register_mech(mechanism, cap_alloc, cap_cur, (Pfri)0, (Pfri)0, -1, 1);
#else
	register_mech(mechanism, cap_alloc, cap_cur, (Pfri)0, (Pfri)0, -1);
#endif
}

#if VECTORIZE

static cap_cur(count, vnode, vdata, vpdata, type )
/*ARGSUSED*/
	int count, type;
	Node *vnode[];
	double *vdata[];
	Datum *vpdata[];
{
#define cm  vdata[i][0]
#define cmdt  vdata[i][1]
#define vm  vnode[i]->v

	int i;

#if METHOD3
   if (_method3) {
	if (!secondorder) {
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < count; ++i) {
			vnode[i]->thisnode.Cdt = 1.e-3 * cm / dt;
			vnode[i]->thisnode.GC += vnode[i]->thisnode.Cdt;
			vnode[i]->thisnode.EC += vnode[i]->thisnode.Cdt * vm;
		}
	}else{
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < count; ++i) {
			vnode[i]->thisnode.Cdt = 2.e-3 * cm / dt;
			vnode[i]->thisnode.GC += vnode[i]->thisnode.Cdt;
			vnode[i]->thisnode.EC += vnode[i]->thisnode.Cdt * vm;
		}
	}
   }else
#endif
   {
	if (!secondorder) {
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < count; ++i) {
			cmdt = 1.e-3 * cm / dt;
			vnode[i]->d += cmdt;
			vnode[i]->rhs += cmdt * vm;
		}
	}else{
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < count; ++i) {
			cmdt = 2.e-3 * cm / dt;
			vnode[i]->d += cmdt;
			vnode[i]->rhs += cmdt * vm;
		}
	}
   }
}

/* the rest can be constructed automattically from the above info*/

static cap_alloc(p)
	Prop *p;
{
	double *pd;
#define	nparm 2
	pd = (double *)ecalloc(nparm, sizeof(double));
	pd[0] = DEF_cm;	/*default capacitance/cm^2*/
	p->param = pd;
	p->param_size = nparm;
}

#else /* ! Vectorize */

static double cap_cur(p, pd, v)
	Prop *p;
	double *pd;
	double v;
{
	double cm, cmdt;

	cm = p->param[0];
	if (!secondorder) {
		cmdt = 1.e-3*cm/dt;
	}else{
		cmdt = 2.e-3*cm/dt;
	}
			
	*pd += cmdt;
	return -cmdt*v;	/*assumes no iterations in that v is vold*/
}

/* the rest can be constructed automattically from the above info*/

static cap_alloc(p)
	Prop *p;
{
	double *pd;
#define	nparm 1
	pd = (double *)emalloc(nparm * sizeof(double));
	pd[0] = DEF_cm;	/*default capacitance/cm^2*/
	p->param = pd;
}
#endif
