/* $Header: /timshel2g/src/master/nrn/src/nrnoc/clamp.c,v 1.1.1.1 1994/10/12 17:22:32 hines Exp $ */
/*
$Log: clamp.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:32  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.55  1994/03/08  17:49:02  hines
 * change of clamp_resist takes immediate effect
 *
 * Revision 4.9  93/01/21  12:21:13  hines
 * watch out for freed sections in old style fstim, fsyn, and fclamp
 * correct memory leakage on access section
 * 
 * Revision 2.75  92/08/07  16:11:52  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 1.84  90/10/24  10:56:34  hines
 * saber warnings gone
 * 
 * Revision 1.52  90/01/24  07:04:39  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.42  89/10/13  09:01:09  mlh
 * free the clamp, synapse, and stim when new sections created
 * de-lint some minor things
 * 
 * Revision 1.41  89/10/12  08:14:23  mlh
 * did not handle case of maxlevel=0 correctly
 * 
 * Revision 1.40  89/09/28  08:32:20  mlh
 * clamp was not turning off after last pulse
 * 
 * Revision 1.39  89/09/25  16:06:47  mlh
 * print point current info with prstim()
 * 
 * Revision 1.30  89/09/06  10:25:47  mlh
 * newcable to neuron
 * 
 * Revision 1.18  89/08/12  14:48:44  mlh
 * Lint free
 * 
 * Revision 1.14  89/08/10  16:21:59  mlh
 * clamp works at least with one level
 * 
 * Revision 1.1  89/08/10  15:36:30  mlh
 * Initial revision
 * 
*/


/* modified from fstim.c */
/*
fclamp(maxlevel, loc)
 allocates space for maxlevel level changes all at location loc
 previously existing clamp is released.
 All clamp levels are initialized to clamp_resist series resistance.
 and 0 duration.  After the final level change the clamp is off.

fclamp(i, duration, vc)
 the maxlevel level changes concatenate. This function specifies
 the magnitude and duration of the ith level.

fclampv()
 the value of the clamp potential at the global time t
 
fclampi()
 the clamp current at the global time t

*/

#include "neuron.h"
#include "section.h"

static double loc;
static Node *pnd;
static Section* sec;
static double gtemp;
static maxlevel = 0;		/* size of clamp array */
static double *duration, *vc, *tswitch; /* maxlevel of them */
static int oldsw = 0;
static double clampval();

extern double *getarg(), chkarg();
extern char *secname();

print_clamp() {
	int i;
	
	if (maxlevel == 0) return;
	Printf("%s fclamp(%d, %g) /* Second arg is location */\n\
/* fclamp( #, duration(ms), magnitude(mV)) ; clamp_resist = %g */\n", secname(sec),
	maxlevel, loc, clamp_resist);
	for (i=0; i<maxlevel; i++) {
		Printf("   fclamp(%2d,%13g,%14g)\n", i, duration[i], vc[i]);
	}
}

fclampv() {

	if (maxlevel) {
		ret(clampval());
	}else{
		ret(0.);
	}
}

fclampi() {
	double v;
	
	if (maxlevel) {
		v = clampval();
		if (gtemp) {
			ret( -(pnd->v - v)/clamp_resist );
		}else{
			ret(0.);
		}
	}else{
		ret(0.);
	}
}
	
fclamp() {
	int i;

	/* two args are maxlevel, loc */
	/* three args are level, duration, vc */
	i = chkarg(1, 0., 10000.);
	if (ifarg(3)) {
		int num;
		if (i >= maxlevel) {
			hoc_execerror("level index out of range", (char *)0);
		}
		duration[i] = chkarg(2, 0., 1e21);
		vc[i] = *getarg(3);

		tswitch[0] = -1e-9;
		for (num = 0; num < maxlevel; num++) {
			tswitch[num+1] = tswitch[num] + duration[num];
		}
		oldsw = 0;
		ret(tswitch[maxlevel-1]);
		return;
	} else {
		free_clamp();
		maxlevel = i;
		if (maxlevel) {
			duration = (double *)emalloc((unsigned)(maxlevel * sizeof(double)));
			vc = (double *)emalloc((unsigned)(maxlevel * sizeof(double)));
			tswitch = (double *)emalloc((unsigned)((maxlevel + 1) * sizeof(double)));
			for (i = 0; i<maxlevel; i++) {
				duration[i] = 0.;
				vc[i] = 0.;
				tswitch[i] = -1e-9;
			}
			tswitch[maxlevel] = -1e-9;
			loc = chkarg(2, 0., 1.);
			sec = chk_access();
			section_ref(sec);
			clamp_prepare();
		}
	}
	ret(0.);
}

free_clamp() {
	if (maxlevel) {
		free((char *)duration);
		free((char *)vc);
		free((char *)tswitch);
		maxlevel = 0;
		section_unref(sec);
		sec = 0;
	}
}
	
clamp_prepare()	/*fill in the section info*/
{
	Node *node_ptr();
	double area;

	if (!maxlevel) {
		return;
	}
	if (sec->prop) {
		pnd = node_ptr(sec, loc, &area);
	}else{
		free_clamp();
		return;
	}
	if (clamp_resist <= 0) {
		hoc_execerror("clamp_resist must be > 0 in megohms", (char *)0);
	}
}

activclamp() {
	double v;
	if (!maxlevel) {
		return;
	}
	v = clampval();
	pnd->rhs += gtemp*v;
	pnd->d += gtemp;
}


static double
clampval() {
	gtemp = 1.e2/clamp_resist/pnd->area;
	for (;;) {
		if (t < tswitch[oldsw]) {
			if (oldsw == 0) {
				break;
			}
			oldsw = 0;
		}else{
			if (t < tswitch[oldsw + 1]) {
				break;
			} else {
				if (++oldsw == maxlevel) {
					--oldsw;
					gtemp = 0.;
					break;
				}
			}
		}
	}
	return vc[oldsw];
}
