/* $Header: /timshel2g/src/master/nrn/src/nrnoc/eion.c,v 1.2 1994/11/01 18:50:36 hines Exp $ */
/*
$Log: eion.c,v $
 * Revision 1.2  1994/11/01  18:50:36  hines
 * eca not being initialized when cai was a state
 *
 * Revision 1.1.1.1  1994/10/12  17:22:32  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.80  1994/10/03  20:42:00  hines
 * nernst and ghk functions. A special form for the nernst function
 * is available and looks like nernst("nai"), that returns the nai based
 * on the values of nao, ena, celsius, valence, and the nernst equation.
 *
 * Revision 4.79  1994/10/03  17:51:31  hines
 * more flexibility in relationship between reversal potential and concentrations. May also specify whether CONST, DEP, STATE
 * see ion_style
 *
 * Revision 4.77  1994/09/26  18:26:47  hines
 * ions more nearly do the "right thing"
 * presence of mechanism that WRITE's a concentration computes
 * the reversal potential every fadvance and finitialize, initializes
 * the concentrations to global variables, makes the concentrations have
 * the type STATE and the reversal potential the style DEP
 * presence of mech that READ's a concentration computes reversal
 * every finitialize, and treats concentration as CONST and reversal
 * potential style DEP.
 * If no mechanism reads or writes concentrations the reversal is
 * treated as CONST.
 *
 * Revision 4.38  1993/07/13  11:49:06  hines
 * declare all current functions properly (now Pfri instead of Pfrd)
 *
 * Revision 4.35  93/06/01  09:00:11  hines
 * can see if a mechanism is an ion with nrn_is_ion(type)
 * can't uninsert ions
 * 
 * Revision 3.33  92/10/24  16:30:12  hines
 * freeing vars notifies interviews.
 * sec_and_position(sec, inode) returns string in form section(x)
 * some portability fixes from pittsburgh
 * 
 * Revision 2.86  92/08/13  11:20:31  hines
 * default parameters collected into membdef.h
 * 
 * Revision 2.73  92/08/05  10:28:54  hines
 * got rid of objectcenter messages
 * 
 * Revision 2.71  92/08/05  09:26:58  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.60  92/03/20  09:38:39  hines
 * finitialize()
 * 
 * Revision 2.50  92/02/05  08:48:05  hines
 * Can connect nmodl POINTER variables to external variables using:
 * connect var_suffix.section(x), anyvar for range variables or
 * connect var_suffix[i], anyvar for point process variables
 * This can be dangerous if the user fails to make these variables point to
 * something.
 * 
 * Revision 2.29  91/07/30  10:24:14  hines
 * for eion, e, ci, co are parameters, i is assigned.
 * 
 * Revision 1.91  90/11/29  17:22:16  hines
 * secondorder replaces implicit
 * secondorder=0 means implicit with no adjustment of current
 * secondorder=1 means crank-nicolson with no adjustment of current
 * secondorder=2 means crank-nicolson with adjustment of current to t-dt/2
 * 
 * Revision 1.90  90/11/29  14:47:01  hines
 * 2nd order correctness when interpreting v(t) current(t-dt/2)
 * state(t+dt/2) and concentration(t) following fadvance().
 * fcurrent sets all assigned variables by setting dt <= 0 and
 * calling state functions then setting matrix.
 * 
 * Revision 1.89  90/11/12  07:38:52  hines
 * Big change in way ions are handled.  Each ion species has its own
 * mechanism which allocates current, reversal potential, inside concentration
 * and outside concentration.  Mechanisms which need an ion register their
 * need via ion_reg(name) during initialization.
 * 
 * Revision 1.87  90/10/29  12:31:23  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:36  hines
 * saber warnings gone
 * 
 * Revision 1.79  90/10/10  08:50:25  hines
 * Oops. fix of warnings didn't make it to the files
 * 
 * Revision 1.72  90/07/31  10:47:35  hines
 * detailed info removed from cabvars. Only thing necessary is to
 * declare a registration function.
 * 
 * Revision 1.52  90/01/24  07:04:42  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.42  89/10/13  09:01:24  mlh
 * free the clamp, synapse, and stim when new sections created
 * de-lint some minor things
 * 
 * Revision 1.37  89/09/20  12:02:22  mlh
 * ionic reversal potentials and currents now in Eion
 * other mechanisms, such as HH, use and modify Eion parameters.
 * a mechanism allocation routine can "need" another mechanism and
 * communicate via Datum.pval
 * 
 * Revision 1.1  89/09/20  07:53:13  mlh
 * Initial revision
 * 
*/


#include	"section.h"
#include	"neuron.h"
#include	"membfunc.h"
#include	"ytab.h"
#include	"membdef.h"

extern double chkarg();

static char *mechanism[] = { /*just a template*/
	"na_ion",
	"ena", "nao", "nai", 0,
	"ina", 0,
	0
};
static DoubScal scdoub[] = { /* just a template*/
	"ci0_na_ion", 0,
	"co0_na_ion", 0,
	0, 0
};

static ion_alloc(), ion_cur(), ion_init(), ion_state();
double nrn_nernst(), nrn_ghk();
static int na_ion, k_ion, ca_ion; /* will get type for these special ions */

int
nrn_is_ion(type)
	int type;
{
	return (memb_func[type].alloc == ion_alloc);
}

static int ion_global_map_size;
static double** ion_global_map;
#define global_conci(type) ion_global_map[type][0]
#define global_conco(type) ion_global_map[type][1]
#define global_charge(type) ion_global_map[type][2]

ion_reg(name, valence)
	char *name;
	double valence;
{
	int i;
	Symbol *s;
	char buf[6][50];
	double val;
#define VAL_SENTINAL -10000.

	Sprintf(buf[0], "%s_ion", name);
	Sprintf(buf[1], "e%s", name);
	Sprintf(buf[2], "%si", name);
	Sprintf(buf[3], "%so", name);
	Sprintf(buf[5], "i%s", name);
	for (i=0; i<6; i++) {
		mechanism[i] = buf[i];
	}
	mechanism[4] = (char *)0;
	s = hoc_lookup(buf[0]);
	if (!s || s->type != MECHANISM
	  || memb_func[s->subtype].alloc != ion_alloc) {
#if VECTORIZE
		register_mech(mechanism, ion_alloc, ion_cur, ion_state, ion_init, -1, 1);
#else
		register_mech(mechanism, ion_alloc, ion_cur, ion_state, ion_init,-1);
#endif
		s = hoc_lookup(buf[0]);

		if (ion_global_map_size <= s->subtype) {
			ion_global_map_size = s->subtype + 1;
			ion_global_map = (double**)erealloc(ion_global_map,
				sizeof(double*)*ion_global_map_size);
		}
		ion_global_map[s->subtype] = (double*)emalloc(3*sizeof(double));
		Sprintf(buf[0], "%si0_%s", name, s->name);
		scdoub[0].name = buf[0];
		scdoub[0].pdoub = ion_global_map[s->subtype];		
		Sprintf(buf[1], "%so0_%s", name, s->name);
		scdoub[1].name = buf[1];
		scdoub[1].pdoub = ion_global_map[s->subtype] + 1;
		hoc_register_var(scdoub, (DoubVec*)0, (IntFunc*)0);
		if (strcmp("na", name) == 0) {
			na_ion = s->subtype;
			global_conci(s->subtype) = DEF_nai;
			global_conco(s->subtype) = DEF_nao;
			global_charge(s->subtype) = 1.;
		}else if (strcmp("k", name) == 0) {
			k_ion = s->subtype;
			global_conci(s->subtype) = DEF_ki;
			global_conco(s->subtype) = DEF_ko;
			global_charge(s->subtype) = 1.;
		}else if (strcmp("ca", name) == 0) {
			ca_ion = s->subtype;
			global_conci(s->subtype) = DEF_cai;
			global_conco(s->subtype) = DEF_cao;
			global_charge(s->subtype) = 2.;
		}else{
			global_conci(s->subtype) = DEF_ioni;
			global_conco(s->subtype) = DEF_iono;
			global_charge(s->subtype) = VAL_SENTINAL;
		}			
		for (i=0; i < 3; ++i) { /* used to be CONST */
			s->u.ppsym[i]->subtype = _AMBIGUOUS;
		}
	}
	val = global_charge(s->subtype);
	if (valence != VAL_SENTINAL && val != VAL_SENTINAL && valence != val) {
		fprintf(stderr, "%s ion valence defined differently in\n\
two USEION statements (%g and %g)\n",
			s->name, valence, global_charge(s->subtype));
		exit(1);
	}else if (valence == VAL_SENTINAL && val == VAL_SENTINAL) {
		fprintf(stderr, "%s ion valence must be defined in\n\
the USEION statement of any model using this ion\n", s->name);
		exit(1);
	}else if (valence != VAL_SENTINAL) {
		global_charge(s->subtype) = valence;
	}
}

#define FARADAY 96520.
#define ktf (1000.*8.3134*(celsius + 273.15)/FARADAY)
double nrn_nernst(ci, co, z) double z, ci, co; {
	if (ci <= 0.) {
		return 1e6;
	}else if (co <= 0.) {
		return -1e6;
	}else{
		return ktf/z*log(co/ci);
	}
}

int nernst() {
	double val;
	
	if (hoc_is_str_arg(1)) {
		Symbol* s = hoc_lookup(gargstr(1));
		if (s && ion_global_map[s->u.rng.type]) {
			Section* sec = chk_access();
			double* nrn_rangepointer();
			Symbol* ion = memb_func[s->u.rng.type].sym;
			double z = global_charge(s->u.rng.type);
			double *ci, *co, *e, x;
			if (ifarg(2)) {
				x = chkarg(2, 0., 1.);
			}else{
				x = .5;
			}
			ci = nrn_rangepointer(sec, ion->u.ppsym[1] , x);
			co = nrn_rangepointer(sec, ion->u.ppsym[2], x);
			e = nrn_rangepointer(sec, ion->u.ppsym[0], x);
			switch (s->u.rng.index) {
			case 0:
				val = nrn_nernst(*ci, *co, z);
				ret();
				hoc_pushx(val);
				return;
			case 1:
				val = *co*exp(-z/ktf* *e);
				ret();
				hoc_pushx(val);
				return;
			case 2:
				val = *ci*exp(z/ktf* *e);
				ret();
				hoc_pushx(val);
				return;
			}
		}
		hoc_execerror(gargstr(1), " not a reversal potential or concentration");
	}else{
		val = nrn_nernst(*getarg(1), *getarg(2), *getarg(3));
	}
	ret();
	hoc_pushx(val);
}

static double efun(x) double x; {
	if (fabs(x) < 1e-4) {
		return 1. - x/2.;
	}else{
		return x/(exp(x) - 1);
	}
}

double nrn_ghk(v, ci, co, z) double v, ci, co, z; {
	double eco, eci, temp;
	temp = z*v/ktf;
	eco = co*efun(temp);
	eci = ci*efun(-temp);
	return (.001)*z*FARADAY*(eci - eco);
}
int ghk() {
	double val = nrn_ghk(*getarg(1), *getarg(2), *getarg(3), *getarg(4));
	ret();
	hoc_pushx(val);
}
#if VECTORIZE
#define erev	pd[i][0]	/* From Eion */
#define conci	pd[i][1]
#define conco	pd[i][2]
#define cur	pd[i][3]
#define dcurdv	pd[i][4]

/*
 handle erev, conci, conc0 "in the right way" according to ion_style
 default. See nrn/lib/help/nrnoc.help.
ion_style("name_ion", [c_style, e_style, einit, eadvance, cinit])

 ica is assigned
 eca is parameter but if conc exists then eca is assigned
 if conc is CONST then eca calculated on finitialize
 if conc is STATE then eca calculated on fadvance and conc finitialize
 	with global nai0, nao0

 nernst(ci, co, charge) and ghk(v, ci, co, charge) available to hoc
 and models.
*/

#define iontype ppd[i][0].i	/* how _AMBIGUOUS is to be handled */
/*the bitmap is
03	concentration unused, CONST, DEP, STATE
04	initialize concentrations
030	reversal potential unused, CONST, DEP, STATE
040	initialize reversal potential
0100	calc reversal during fadvance
*/

#define charge global_charge(type)
#define conci0 global_conci(type)
#define conco0 global_conco(type)

ion_style() {
	Symbol* s;
	int istyle, i;

	s = hoc_lookup(gargstr(1));
	if (!s || s->type != MECHANISM || !nrn_is_ion(s->subtype)) {
		hoc_execerror(gargstr(1), " is not an ion");
	}
/*   if (ifarg(2)) */{
	istyle = (int)chkarg(2, 0., 3.); /* c_style */
	istyle += 010*(int)chkarg(3, 0., 3.); /* e_style */
	istyle += 040*(int)chkarg(4, 0., 1.); /* einit */
	istyle += 0100*(int)chkarg(5, 0., 1.); /* eadvance */
	istyle += 04*(int)chkarg(6, 0., 1.); /* cinit*/

#if 0	/* global effect */
	{
		int count;
		Datum** ppd;
		v_setup_vectors();
		count = memb_list[s->subtype].nodecount;
		ppd = memb_list[s->subtype].pdata;
		for (i=0; i < count; ++i) {
			iontype = istyle;
		}
	}
#else	/* currently accessed section */
	{
		Section* sec;
		Prop* p, *nrn_mechanism();
		sec = chk_access();
		for (i=0; i < sec->nnode; ++i) {
			p = nrn_mechanism(s->subtype, sec->node + i);
			if (p) {
				p->dparam[0].i = istyle;
			}
		}
	}
#endif
   }
	hoc_ret();
	hoc_pushx((double)istyle);
}

int nrn_vartype(sym) Symbol* sym; {
	int i;
	i = sym->subtype;
	if (i == _AMBIGUOUS) {
		Section* sec;
		Prop* p, *nrn_mechanism();
		sec = chk_access();
		p = nrn_mechanism(sym->u.rng.type, sec->node);
		if (p) {
			int it = p->dparam[0].i;
			if (sym->u.rng.index == 0) { /* erev */
				i = (it & 030)>>3; /* unused, CONST, DEP, or STATE */
			}else{	/* concentration */
				i = (it & 03);
			}
		}
	}
	return i;
}

/* the ion mechanism it flag  defines how _AMBIGUOUS is to be interpreted */
nrn_promote(p, conc, rev)
	Prop* p;
	int conc, rev;
{
	int oldconc, oldrev;
	int* it = &p->dparam[0].i;
	oldconc = (*it & 03);
	oldrev = (*it & 030)>>3;
	/* precedence */
	if (oldconc < conc) {
		oldconc = conc;
	}
	if (oldrev < rev) {
		oldrev = rev;
	}
	/* promote type */
	if (oldconc > 0 && oldrev < 2) {
		oldrev = 2;
	}
	*it &= ~0177;	/* clear the bitmap */
	*it += oldconc + 010*oldrev;
	if (oldconc == 3) { /* if state then cinit */
		*it += 4;
		if (oldrev == 2) { /* if not state (WRITE) then eadvance */
			*it += 0100;
		}
	}
	if (oldconc > 0 && oldrev == 2) { /*einit*/
		*it += 040;
	}
}

/* Must be called prior to any channels which update the currents */
static ion_cur(count, vnode, pd, ppd, type)
	int count, type;
	Node* vnode[];
	double *pd[];
	Datum *ppd[];
{
	/*ARGSUSED*/
	int i;
#if _CRAY
#pragma _CRI ivdep
#endif
	for (i=0; i < count; ++i) {
		dcurdv = 0.;
		cur = 0.;
	};
}

/* Must be called prior to other models which possibly also initialize
	concentrations based on their own states
*/
static ion_init(count, vnode, pd, ppd, type)
	int count, type;
	Node* vnode[];
	double *pd[];
	Datum *ppd[];
{
	/*ARGSUSED*/
	int i;
#if _CRAY
#pragma _CRI ivdep
#endif
	for (i=0; i < count; ++i) {
		if (iontype & 04) {
			conci = conci0;
			conco = conco0;
		}
	}
#if _CRAY
#pragma _CRI ivdep
#endif
	for (i=0; i < count; ++i) {
		if (iontype & 040) {
			erev = nrn_nernst(conci, conco, charge);
		}
	}
}

static ion_state(count, vnode, pd, ppd, type)
	int count, type;
	Node* vnode[];
	double *pd[];
	Datum *ppd[];
{
	/*ARGSUSED*/
	int i;
#if _CRAY
#pragma _CRI ivdep
#endif
	for (i=0; i < count; ++i) {
		if (iontype & 0100) {
			erev = nrn_nernst(conci, conco, charge);
		}
	}
}

static ion_alloc(p)
	Prop *p;
{
	double *pd[1];
	int i=0;
	
#define	nparm 5
	pd[0] = (double *)emalloc(nparm * sizeof(double));
	p->param_size = nparm;

	cur = 0.;
	dcurdv = 0.;
	if (p->type == na_ion) {
		erev = DEF_ena;
		conci = DEF_nai;
		conco = DEF_nao;
	}else if (p->type == k_ion) {
		erev = DEF_ek;
		conci = DEF_ki;
		conco = DEF_ko;
	}else if (p->type == ca_ion) {
		erev = DEF_eca;
		conci = DEF_cai;
		conco = DEF_cao;
	}else{
		erev = DEF_eion;
		conci = DEF_ioni;
		conco = DEF_iono;
	}
	p->param = pd[0];
	
	p->dparam = (Datum*)emalloc(1*sizeof(Datum));
	p->dparam->i = 0;
}

second_order_cur() {
	extern int secondorder;
	int j, i;
#define c 3
#define dc 4
  if (secondorder == 2) {
	for (j = 0; j < n_memb_func; ++j) if (memb_func[j].alloc == ion_alloc) {
		for (i = 0; i < memb_list[j].nodecount; ++i) {
			memb_list[j].data[i][c] += memb_list[j].data[i][dc]
			   * ( memb_list[j].nodelist[i]->rhs
			         - memb_list[j].nodelist[i]->v
			     )
			;
		}
	}
   }
}

#else	/*!VECTORIZE*/

#define erev	pd[0]	/* From Eion */
#define conci	pd[1]
#define conco	pd[2]
#define cur	pd[3]
#define dcurdv	pd[4]

/* Must be called prior to any channels which update the currents */
static double ion_cur(p, pdiag, v)
	Prop *p;
	double *pdiag;
	double v;
{
	/*ARGSUSED*/
	double *pd;

	pd = p->param;
	dcurdv = 0.;
	cur = 0.;
	return 0.;
}

static ion_alloc(p)
	Prop *p;
{
	double *pd;
	
#define	nparm 5
	pd = (double *)emalloc(nparm * sizeof(double));

	cur = 0.;
	dcurdv = 0.;
	if (p->type == na_ion) {
		erev = 115 - 65.;
		conci = 15.7;
		conco = 116.;
	}else if (p->type == k_ion) {
		erev = -12. - 65.;
		conci = 0.;
		conco = 0.;
	}else if (p->type == ca_ion) {
		erev = 115.12925;
		conci = 1e-3;
		conco = 10.;
	}else{
		erev = 0.;
		conci = 1.;
		conco = 1.;
	}
	p->param = pd;
	p->dparam = (Datum *)0;
}

second_order_cur() {
	short isec, inode;
	Section *sec;
	extern int secondorder;
	Node *nd;
	double dv, *pd;
	Prop *p;

  if (secondorder == 2) {
	for (isec = nsection - 1; isec >= 0; isec--) {
		sec = section + isec;
		for (inode = sec->nnode - 1; inode >= 0; inode--) {
			nd = sec->node + inode;
#if 0
			if (secondorder < 2) {
				dv = (nd->rhs - nd->v)/2.;
			}else
#endif
			{
				dv = (nd->rhs - nd->v);
			}
#if EXTRACELLULAR
			if (sec->extnode) {
				Extnode *nde = sec->extnode + inode;
#if 0
				if (secondorder < 2) {
					dv -= (nde->rhs - nde->v)/2.;
				}else
#endif
				{
					dv -= (nde->rhs - nde->v);
				}
			}
#endif
			for (p = nd->prop; p; p = p->next) {
				if (memb_func[p->type].alloc == ion_alloc) {
					pd = p->param;
					cur += dcurdv*dv;
				}
			}
		}
	}
   }
}
#endif
