/* $Header: /timshel2g/src/master/nrn/src/nrnoc/extcell.c,v 1.1.1.1 1994/10/12 17:22:33 hines Exp $ */
/*
$Log: extcell.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:33  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.44  1993/08/15  12:55:47  hines
 * fix warning of function declared external and later static
 * y
 *
 * Revision 4.38  93/07/13  11:49:07  hines
 * declare all current functions properly (now Pfri instead of Pfrd)
 * 
 * Revision 4.37  93/07/09  16:22:37  hines
 * point processes now work correctly when they are electrodes and an
 * extracellular mechananism is present.
 * vclmp.mod and stim.mod are electrodes
 * positive electrode current causes depolarization (normal mechanism
 * current is positive outward)
 * 
 * Revision 4.36  93/07/08  14:31:45  hines
 * a start on fixing the bug of electode point processes not working
 * correctly in presence of extracellular. At this time the functionality
 * is identical as before but passing node to  models in nrn_cur instead
 * of passing pointer to nd->d and returning rhs.
 * There is an alternative now to NONSPECIFIC_CURRENT which is
 * ELECTRODE_CURRENT
 * 
 * Revision 4.25  93/04/20  08:58:36  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.11  93/01/27  13:50:07  hines
 * extracellular works again
 * 
 * Revision 2.60  92/03/20  09:38:41  hines
 * finitialize()
 * 
 * Revision 2.50  92/02/05  08:48:06  hines
 * Can connect nmodl POINTER variables to external variables using:
 * connect var_suffix.section(x), anyvar for range variables or
 * connect var_suffix[i], anyvar for point process variables
 * This can be dangerous if the user fails to make these variables point to
 * something.
 * 
 * Revision 2.11  91/01/04  09:56:48  hines
 * enclose extcell.c with #if EXTRACELLULAR
 * 
 * Revision 2.4  90/12/07  12:38:42  hines
 * a dparam variable was accessed as a param
 * 
 * Revision 1.91  90/11/29  17:22:18  hines
 * secondorder replaces implicit
 * secondorder=0 means implicit with no adjustment of current
 * secondorder=1 means crank-nicolson with no adjustment of current
 * secondorder=2 means crank-nicolson with adjustment of current to t-dt/2
 * 
 * Revision 1.87  90/10/29  12:31:25  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:37  hines
 * saber warnings gone
 * 
 * Revision 1.79  90/10/10  08:50:29  hines
 * Oops. fix of warnings didn't make it to the files
 * 
 * Revision 1.77  90/10/09  08:01:52  hines
 * extracellular axial resistance now Mohms/cm  axial capacitance
 * not working
 * 
 * Revision 1.72  90/07/31  10:47:36  hines
 * detailed info removed from cabvars. Only thing necessary is to
 * declare a registration function.
 * 
 * Revision 1.57  90/02/12  11:56:01  mlh
 * xg and xc are per unit area
 * 
 * Revision 1.55  90/01/25  10:30:36  mlh
 * 2 errors corrected. bad pointer  and wrong axial conductance of last node
 * 
 * Revision 1.52  90/01/24  07:04:45  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.48  90/01/16  07:05:26  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.1  90/01/12  07:37:22  mlh
 * Initial revision
 * 
*/

#include	<stdio.h>
#include	"section.h"
#include	"membfunc.h"
#include	"hocassrt.h"

#if EXTRACELLULAR

static char *mechanism[] = {
	"extracellular",
	"xraxial", "xcaxial", "xg", "xc", 0,
	0,
	"vext", 0,
};
static extcell_alloc();
static extcell_init();

extracell_reg() {
	register_mech(mechanism, extcell_alloc, (Pfri)0, (Pfri)0,
		extcell_init, -1, 1);
}

/* solving is Based on code in solve.c. */

/*
 The only way to figure this out is to do the matrix by hand
 Triangularization proceeds from bottom up. Note that x terms are in the
 extnode and entries after the diagonal are in the child.

 b  xbin  d  xain  a			inside
     xb xbout xd xaout xa		outside
          b  xbin  d  xain  a		inside
              xb xbout xd xaout xa	outside
                      ...

The vector part is straightforward. Just remember that a section
with or without extracellular may have a parent node with or without
extracellular. We rely on zero area nodes to make this work properly
in the case where a section without extrcellular is connected to the
end of a section with extracellular
*/

nrn_triang_2d()
{
	int i, inode;
	register Node *nd, *pnd;
	register Extnode *nde, *pnde;
	double p;

	inode = rootnodecount - 1;
	for (i = v_node_count - 1; i > inode; --i) {
		nd = v_node[i];
		pnd = v_parent[i];
		nde = nd->extnode;
		pnde = pnd->extnode;

		/* elim effect of vext[i] on inside[i] */
  if (nde) {
		p = nde->xain / nde->d;
		nd->d -= p * nde->xbout;
		nde->xbin -= p * nde->b;
		nd->rhs -= p * nde->rhs;
		
		/* elim effect of vext[i] on outside[i-1] */
    if (pnde) {
		p = nde->a / nde->d;
		nde->xaout -= p * nde->xbout;
		pnde->d -= p * nde->b;
		pnde->rhs -= p * nde->rhs;
	
		/* elim effect of vin[i] on outside[i-1] */
		p = nde->xaout / nd->d;
		pnde->d -= p * nde->xbin;
		pnde->xbout -= p * nd->b;
		pnde->rhs -= p * nd->rhs;
    }
  }
		
		/* elim effect of vin[i] on inside[i-1] */
		p = nd->a / nd->d;
  if (nde && pnde) {
		pnde->xain -= p * nde->xbin;
  }
		pnd->d -= p * nd->b;
		pnd->rhs -= p * nd->rhs;
	}
}

nrn_bksub_2d()
{
	int i, inode;
	register Node *nd, *cnd;
	register Extnode *nde, *cnde;

	inode = rootnodecount;
	for (i = 0; i < inode; ++i) {
		v_node[i]->rhs /= v_node[i]->d;
	}
	for (i = inode; i < v_node_count;++i) {
		cnd = v_node[i];
		nd = v_parent[i];
		cnde = cnd->extnode;
		nde = nd->extnode;

		cnd->rhs -= cnd->b * nd->rhs;
	  if (cnde) {
	    if (nde) {
		cnd->rhs -= cnde->xbin * nde->rhs;
		cnde->rhs -= cnde->b * nde->rhs;
	    }
	  }
		cnd->rhs /= cnd->d;
	  if (cnde) {
		cnde->rhs -= cnde->xbout * cnd->rhs;
		cnde->rhs /= cnde->d;
	  }
	}

}

/* based on update() in fadvance.c */
/* update has already been called so we only need to update extracellular
nodes (and fix the mistake in using rhs which was vin instead of vm)
*/
nrn_update_2d()
{
	int i, cnt;
	extern int secondorder;
	Node *nd, **ndlist;
	Extnode *nde;
	
	cnt = memb_list[EXTRACELL].nodecount;
	ndlist = memb_list[EXTRACELL].nodelist;
	
	if (secondorder) {
		for (i=0; i < cnt; ++i) {
			nd = ndlist[i];
			nde = nd->extnode;
			nde->v = 2.*nde->rhs - nde->v;
			nd->v -= 2.*nde->rhs;
		}
	}else{
		for (i=0; i < cnt; ++i) {
			nd = ndlist[i];
			nde = nd->extnode;
			nde->v = nde->rhs;
			nd->v -= nde->rhs;
		}
	}
}

/* interface between hoc and extcell */
#define xraxial	pd[0]	/* From Eion */
#define xcaxial	pd[1]
#define xg	pd[2]
#define xc	pd[3]

#if 0
double extcell_cur(p, pdiag, v)
	Prop *p;
	double *pdiag;
	double v;
{
	double *pd, **ppd;

	LINTUSE(pdiag)
	LINTUSE(v)
	pd = p->param;
	return 0.;
}
#endif

static extcell_alloc(p)
	Prop *p;
{
	double *pd;
	
#define	nparm 4
	pd = (double *)emalloc(nparm * sizeof(double));

	xraxial= 1.e9;
	xcaxial = 0.;
	xg = 1.e9;
	xc = 0.;
	p->param = pd;
}

/*ARGSUSED*/
static extcell_init(ndcount, ndlist, data, pdata)
	int ndcount;
	Node** ndlist;
	double* data;
	Datum* pdata;
{
	int i;
	for (i=0; i < ndcount; ++i) {
		ndlist[i]->extnode->v = 0.;
	}
}

extcell_2d_alloc(sec)
	Section *sec;
{
	int i;
	Node *nd;
	Extnode *nde;
	
	nde = (Extnode *)emalloc((unsigned)(sec->nnode*sizeof(Extnode)));
	for (i = sec->nnode-1; i>=0; i--) {
		sec->node[i].extnode = nde + i;
		nde[i].v = 0.;
		nde[i].param = (double *)0;
		nd = sec->node + i;
		assert(nd->prop->type == EXTRACELL);
		nde[i].param = nd->prop->param;
	}
}

nrn_init_ext()
{
	int i, cnt;
	Node** ndlist;
	Extnode* nde;
	cnt = memb_list[EXTRACELL].nodecount;
	ndlist = memb_list[EXTRACELL].nodelist;
	for (i = 0; i < cnt; ++i) {
		nde = ndlist[i]->extnode;
		nde->d = 0.;
		nde->rhs = 0.;
	}
}

static set_extnode();

/* from treesetup.c */
nrn_setup_ext()
{
	int i, cnt;
	Node *nd, *pnd, **ndlist;
	Extnode *nde, *pnde;
	
	cnt = memb_list[EXTRACELL].nodecount;
	ndlist = memb_list[EXTRACELL].nodelist;
	
	/* d contains all the membrane conductances */
	for (i=0; i < cnt; ++i) {
		nd = ndlist[i];
		nde = nd->extnode;

		nde->d += nd->d;
		nde->xain = -nde->d;
		nde->xbout = -nde->d;
		nde->rhs -= nd->rhs;
		nde->xbin = 0.;
		nde->xaout = 0.;
	}
	for (i=0; i < cnt; ++i) {
		nd = ndlist[i];
		nde = nd->extnode;
		pnd = v_parent[nd->v_node_index];
		pnde = pnd->extnode;
		set_extnode(pnde, nde);
	}
}

static
set_extnode(pnde,nde)
	Extnode *nde, *pnde;
{
	extern int secondorder;
	extern double dt;
	double *pd;
	double cfac;
	
	/* xcaxial not supported at this time */
	pd = nde->param;
	if (secondorder < 2) {
		cfac = 1.e3/dt; /* since xcaxial/xgaxial = sec
note xcaxial=1 uF and dt of 1000 msec is analogous to xgaxial=1 umho */
	}else{
		cfac = 2.e3/dt;
	}

	/* axial connections */
	if (pnde) { /* parent sec may not be extracellular */
		nde->d -= nde->b;
		pnde->d -= nde->a;
	}
	
	/* series resistance and capacitance to ground */
	if (secondorder < 2) {
		cfac = 1.e-3/dt; /* since xc/xg = usec */
/*note xc=1 uF/cm^2 and dt of .001 msec is analogous to xg=1 mho/cm^2 */
	}else{
		cfac = 2.e-3/dt;
	}

	nde->d += (xg + xc*cfac);
	nde->rhs += (xc*cfac * nde->v);
}

/* based on treeset.c */
ext_con_coef()	/* setup a and b */
{
	int j;
	double dx, area;
	hoc_Item* qsec;
	Node *nd;
	Extnode *nde;
	double *pd;
	
	/* temporarily store half segment resistances in rhs */
	/* temporarily store half segment capacitances in d */
#if 0
	if (section->extnode) for (j=0; j < section->nnode; j++) {
		nde = section->extnode + j;
		nde->rhs = 0.;
		nde->d = 0.;
		nde->a = nde->b = 0.;	/* done with section 0 */
	}
#endif
	ForAllSections(sec) /*{*/
	  if (sec->node->extnode){
		dx = sec->prop->dparam[2].val/((double)(sec->nnode - 1));
		for (j = 0; j < sec->nnode-1; j++) {
			nde = sec->node[j].extnode;
			pd = nde->param;
			nde->rhs = 1e-4*xraxial*(dx/2.); /*Megohms*/
			nde->d = 0.;
			assert (xcaxial==0.);
		}
		/* last segment has 0 length. */
		nde = sec->node[j].extnode;
		nde->rhs = 0.;
		nde->d = 0.;
		pd = nde->param;
		xc = 0.;
		xg = 0.;
	}}
	/* assume that if only one connection at x=1, then they butte
	together, if several connections at x=1
	then last point is at x=1, has 0 area and other points are at
	centers of nnode-1 segments.
	If interior connection then child half
	section connects straight to the point*/
	/* for the near future we always have a last node at x=1 with
	no properties */
	ForAllSections(sec) /*{*/
	  if(sec->node->extnode){
		/* node half resistances in general get added to the
		node and to the node's "child node in the same section".
		child nodes in different sections don't involve parent
		node's resistance */
		nde = sec->node->extnode;
		nde->b = nde->rhs;
		for (j = 1; j < sec->nnode; j++) {
			nde = sec->node[j].extnode;
			nde->b = nde->rhs + (nde-1)->rhs; /*megohms*/
		}
	}}
	ForAllSections(sec) /*{*/
	  if(sec->node->extnode){
		/* convert to mhos/cm^2 for all nodes except last
		and micromhos for last.  This means that a*V = mamps/cm2
		and a*v in last node = nanoamps. Note that last node
		has no membrane properties and no area. It may perhaps recieve
		current stimulus later */
		/* first the effect of node on parent equation. Note That
		last nodes have area = 1.e2 in dimensionless units so that
		last nodes have units of micromho's */
		nd = sec->node;
		nde = nd->extnode;
		area = sec->parentnode->area;
		/* param[4] is rall_branch */
		nde->a = -1.e2*sec->prop->dparam[4].val/(nde->b * area);
		for (j = 1; j < sec->nnode; j++) {
			nd++;
			nde = nd->extnode;
			area = (nd - 1)->area;
			nde->a = -1.e2/(nde->b * area);
		}
	}}
	/* now the effect of parent on node equation. */
	ForAllSections(sec) /*{*/
	  if(sec->node->extnode){
		for (j=0; j < sec->nnode; j++) {
			nd = sec->node + j;
			nde = nd->extnode;
			nde->b = -1.e2/(nde->b * nd->area);
		}
	}}
}

#endif /*EXTRACELLULAR*/
