/* $Header: /timshel2g/src/master/nrn/src/nrnoc/fadvance.c,v 1.1.1.1 1994/10/12 17:22:33 hines Exp $ */
/*
$Log: fadvance.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:33  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.77  1994/09/26  18:26:47  hines
 * ions more nearly do the "right thing"
 * presence of mechanism that WRITE's a concentration computes
 * the reversal potential every fadvance and finitialize, initializes
 * the concentrations to global variables, makes the concentrations have
 * the type STATE and the reversal potential the style DEP
 * presence of mech that READ's a concentration computes reversal
 * every finitialize, and treats concentration as CONST and reversal
 * potential style DEP.
 * If no mechanism reads or writes concentrations the reversal is
 * treated as CONST.
 *
 * Revision 4.48  1993/10/18  13:37:32  hines
 * very minor fix to synapse.c and fadvance.c for portability
 *
 * Revision 4.36  93/07/08  14:34:14  hines
 * a start on fixing the bug of electode point processes not working
 * correctly in presence of extracellular. At this time the functionality
 * is identical as before but passing node to  models in nrn_cur instead
 * of passing pointer to nd->d and returning rhs.
 * There is an alternative now to NONSPECIFIC_CURRENT which is
 * ELECTRODE_CURRENT
 * 
 * Revision 4.29  93/05/07  16:39:31  hines
 * _ninits incremented by finitialize(). This allows models to use
 * the lag function and perhaps schedule.
 * 
 * Revision 4.28  93/04/29  16:36:01  hines
 * sibling pointers now reffed. Fix several problems with disconnecting
 * and reconnecting sections as well as handle point processes moving with
 * their proper section.
 * Still not perfect though.
 * 
 * Revision 4.12  93/02/02  10:24:56  hines
 * declare static functions before use
 * 
 * Revision 4.11  93/01/27  13:50:08  hines
 * extracellular works again
 * 
 * Revision 4.5  93/01/07  08:14:55  hines
 * psection prints Ra
 * warning if Ra set but not in every section
 * fix batch_run so not extra iteration on cray (tstop -=dt/4)
 * 
 * Revision 4.4  93/01/06  09:44:42  hines
 * minor adjustments for cray c90
 * 
 * Revision 3.45  92/12/18  15:17:01  hines
 * for _CRAY allow multiple instances of a point process in same node.
 * 
 * Revision 3.44  92/12/17  08:28:48  hines
 * delete_section() removes currently accessed section from the list of
 * sections to be simulated and releases its property and nodes.
 * 
 * Revision 3.30  92/10/14  10:06:25  hines
 * use right hoc_pgetarg function
 * 
 * Revision 3.18  92/10/02  17:29:34  hines
 * batch_run(tstop, tstep, "filename")
 * batch_save()	starts new list
 * batch_save(&var, &var, ...) adds pointers to the list
 * 
 * Revision 3.12  92/09/25  17:57:14  hines
 * for method3. now tested and working for transient passive cables.
 * but perhaps the equations could be written with better roundoff
 * error when dt is small
 * 
 * Revision 3.11  92/09/24  17:03:35  hines
 * METHOD3 option. use with spatial_method(i) with i=0-3
 * 0 is the default and is the normal neuron method with zero area nodes
 * at boundaries and branch points.
 * 1 is the standard method with half area nodes at boundaries
 * 2 is a modified second order method which weights by 2/12, 8/12, 2/12
 * 3 is a third order correct method which weights by 1/12 10/12 1/12
 * with the last three methods x = i/nnode with i=0 to nnode
 * methods 1 and 2 are at present quite inefficient since they use the
 * data structures of 3.
 * Works in steady state. not tested with dynamic simulations
 * At this time, discontinuities in parameters (except diameter) is
 * not handled well across nodes and all parameters are assumed to
 * be constant in interval between node and parent node.
 * Good deal of difficulty remains in managing values that change discontinuously
 * 
 * Revision 3.5  92/08/20  14:38:17  hines
 * finitialize checks diam_changed. recalc_diam sets diam_changed to 0
 * and call functions that depend on diam.
 * 
 * Revision 2.75  92/08/07  16:11:53  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.73  92/08/05  10:28:55  hines
 * got rid of objectcenter messages
 * 
 * Revision 2.71  92/08/05  09:27:00  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.64  92/04/13  15:40:57  hines
 * fadvance now calls treeset with t=t+dt/2. This means that models wich depend
 * explicitly on time are second order correct when secondorder=2
 * 
 * Revision 2.60  92/03/20  09:38:43  hines
 * finitialize()
 * 
 * Revision 2.35  91/08/13  07:11:11  hines
 * solve() changed to nrn_solve() so that name doesn't conflict
 * with scopmath
 * 
 * Revision 1.91  90/11/29  17:22:20  hines
 * secondorder replaces implicit
 * secondorder=0 means implicit with no adjustment of current
 * secondorder=1 means crank-nicolson with no adjustment of current
 * secondorder=2 means crank-nicolson with adjustment of current to t-dt/2
 * 
 * Revision 1.90  90/11/29  14:49:47  hines
 * 2nd order correctness when interpreting v(t) current(t-dt/2)
 * state(t+dt/2) and concentration(t) following fadvance().
 * fcurrent sets all assigned variables by setting dt <= 0 and
 * calling state functions then setting matrix.
 * 
 * Revision 1.48  90/01/16  07:05:30  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.44  89/10/30  07:44:43  mlh
 * crank-nicolson available with implicit=0
 * 
 * Revision 1.30  89/09/06  10:25:58  mlh
 * newcable to neuron
 * 
 * Revision 1.23  89/08/29  15:10:08  mlh
 * bug in turboc with respect to incrementing section and node pointers.
 * explicitly iterate over indices
 * 
 * Revision 1.18  89/08/12  14:48:47  mlh
 * Lint free
 * 
 * Revision 1.16  89/08/12  09:51:18  mlh
 * now does all channel states
 * 
 * Revision 1.9  89/08/07  10:23:33  mlh
 * getting good v with single section. matrix solving for v (not deltav)
 * 
 * Revision 1.1  89/08/07  07:06:14  mlh
 * Initial revision
 * 
*/

#include <errno.h>
#include "neuron.h"
#include "section.h"
#include "membfunc.h"

/*
 after an fadvance from t-dt to t, v is defined at t
 states that depend on v are defined at t+dt/2
 states, such as concentration that depend on current are defined at t
 and currents are defined at t-dt/2.
 */
/*
 fcurrent is used to set up all assigned variables  without changing any
 states.  It assumes all states have their values at time t which is
 only first order correct. It determines the nonvint assignments first
 so that ena, etc. are correct for the current determinations. We
 demand that nonvint assignments can be done without changing states when
 dt is temporarily set to 0.

 It turned out to be a bad idea to do this. Many methods (KINETIC, PROCEDURE)
 for solveing are not even called when dt =0. Also it plays havoc with tables
 that are recomputed several times when dt is changed then changed back.
 Therefore we are no longer trying to initialize assigned variables within
 SOLVE'd blocks. Instead we are making initialization more convenient,
 at least within the normal situations, by introducing a finitialize()
 with an optional argument vinit. finitialize will call the INITIALIZE block
 for all mechanisms in all segments. (Default initialization sets all states
 to state0.) The user can set things up specially in a models INITIAL
 block. INITIAL blocks can make use of v. With care they can make use of
 ionic concentrations just like breakpoint and solve blocks. When the argument
 is present, v for all segments are set to that value.
 */
 
extern void setup_tree_matrix(), nrn_solve();
extern int tree_changed;
extern int diam_changed;
#if VECTORIZE
extern int v_structure_change;
#endif

static update();

fadvance()
{
	if (tree_changed) {
		setup_topology();
	}
#if VECTORIZE
	if (v_structure_change) {
		v_setup_vectors();
	}
#endif
	t += dt/2.;
	setup_tree_matrix(); /* currents ambiguous between t and t+dt/2
				ie. v at tentry and states at tentry+dt/2 */
	nrn_solve();		/* rhs = v(texit) */
	second_order_cur();	/*ion currents adjusted to t(entry)+dt/2
				but individual mechanism currents not adjusted
				 */
	update();		/* v at texit */
	t += dt/2.;		/* t = texit */
	nonvint();	/* m,h,n go to t+dt/2, concentrations go to t */
#if METHOD3
	if (_method3 == 3) {
		method3_axial_current();
	}
#endif
	ret(1.);
}

/*
  batch_save() initializes list of variables
  batch_save(&varname, ...) adds variable names to list for saving
  batch_run(tstop, tstep, "file") saves variables in file every tstep
*/

static batch_out(), batch_open(), batch_close();
batch_run() /* avoid interpreter overhead */
{
	double chkarg();
	double tstop, tstep, tnext;
	char* filename;

	tstop = chkarg(1,0.,1e20);
	tstep = chkarg(2, 0., 1e20);
	if (ifarg(3)) {
		filename = gargstr(3);
	}else{
		filename = 0;
	}
	
	if (tree_changed) {
		setup_topology();
	}
#if VECTORIZE
	if (v_structure_change) {
		v_setup_vectors();
	}
#endif
	batch_open(filename, tstop, tstep);
	batch_out();
	tstep -= dt/4.;
	tstop -= dt/4.;
	tnext = t + tstep;
	while (t < tstop) {
		t += dt/2.;
		setup_tree_matrix();
		nrn_solve();
		second_order_cur();
		update();
		t += dt/2.;
		nonvint();
#if METHOD3
		if (_method3 == 3) {
			method3_axial_current();
		}
#endif
		if (t > tnext) {
			batch_out();
			tnext = t + tstep;
		}
	}
	batch_close();
	ret(1.);
}

static update()
{
#if VECTORIZE
	int i;
#else
	short isec, inode;
	Section *sec;
	Node *nd;
#endif

#if VECTORIZE
	if (secondorder) {
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < v_node_count; ++i) {
			v_node[i]->v = 2.*v_node[i]->rhs - v_node[i]->v;
		}
	}else{
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < v_node_count; ++i) {
			v_node[i]->v = v_node[i]->rhs;
		}
	}
#if EXTRACELLULAR
	nrn_update_2d();
#endif

#else
	for (isec = nsection - 1; isec >= 0; isec--) {
		sec = section + isec;
#if EXTRACELLULAR
		if (sec->extnode) {
			update_sec_2d(sec);
			continue;
		}
#endif
		for (inode = sec->nnode - 1; inode >= 0; inode--) {
			nd = sec->node + inode;
			if (!secondorder) {
				nd->v = nd->rhs;
			}else{
				nd->v = 2.*nd->rhs - nd->v;
			}
		}
	}
#endif
}

fcurrent()
{

	if (tree_changed) {
		setup_topology();
	}
#if VECTORIZE
	if (v_structure_change) {
		v_setup_vectors();
	}
#endif

#if 0	/* doesn't work for all possible models. */
	savdt = dt;
	dt = -1e-9;
	nonvint();	/* no states computed when dt <= 0 */
	dt = savdt;
#endif
	setup_tree_matrix();
	ret(1.);
}

fmatrix() {
	extern int section_count;
	extern Section** secorder;
	int isec, inode;
	Section* sec;
	Node* nd;
	for (isec = 0; isec < section_count; ++isec) {
		sec = secorder[isec];
		for (inode = 0; inode < sec->nnode; ++inode) {
			nd = sec->node + inode;
printf("%d %d %g %g %g %g\n", isec, inode, nd->b, nd->a, nd->d, nd->rhs);
		}
	}
	ret(1.);
}
nonvint()
{
#if VECTORIZE
	int i;
#else
	int isec, inode;
	Section *sec;
	Node *nd;
	Prop *p;
	double vm;
#endif

#if VECTORIZE
#if _CRAY
	cray_node_init();
#endif
	for (i=0; i < n_memb_func; ++i) if (memb_func[i].state && memb_list[i].nodecount){
		if (memb_list[i].vectorized) {
			memb_func[i].state(
			   memb_list[i].nodecount,
			   memb_list[i].nodelist,
			   memb_list[i].data,
			   memb_list[i].pdata,
			   i
			);
		}else{
			int j, count;
			Pfri s = memb_func[i].state;
			Memb_list* m = memb_list + i;
			count = m->nodecount;
			for (j = 0; j < count; ++j) {
				Node* nd = m->nodelist[j];
				(*s)(nd, m->data[j], m->pdata[j]);
			};
		}
		if (errno) {
			if (nrn_errno_check(i)) {
hoc_warning("errno set during calculation of states", (char*)0);
			}
		}
	}
#else
	for (isec = nsection - 1; isec >= 0; isec--) {
		sec = section + isec;
		for (inode = sec->nnode - 1; inode >= 0; inode--) {
			nd = sec->node + inode;
#if EXTRACELLULAR
			if (sec->extnode) {
				vm = nd->v - sec->extnode[inode].v;
			}else
#endif
			{
				vm = nd->v;
			}
			for (p = nd->prop; p; p = p->next) {
				if (memb_func[p->type].state) {
					(memb_func[p->type].state)(p, vm);
					if (errno) {
						if (nrn_errno_check(p, inode, sec)) {
hoc_warning("errno set during calculation of states", (char*)0);
						}
					}
				}
			}
		}
	}
#endif
}

#if VECTORIZE
nrn_errno_check(i)
	int i;
{
	int ierr;
	ierr = hoc_errno_check();
	if (ierr) {
		fprintf(stderr, 
"errno set during call to mechanism %s\n",
memb_func[i].sym->name);
	}
	return ierr;
}
#else
nrn_errno_check(p, inode, sec)
	Prop* p;
	int inode;
	Section* sec;
{
	int ierr;
	char* secname();
	ierr = hoc_errno_check();
	if (ierr) {
		fprintf(stderr, 
"errno set during call to mechanism %s at node %d in section %s\n",
memb_func[p->type].sym->name, inode, secname(sec));
	}
	return ierr;
}
#endif

finitialize() {
#if VECTORIZE
	int i, setv=0;
	double v;
#else
	int isec, inode, setv=0;
	Section *sec;
	Node *nd;
	Prop *p;
	double vm, v;
#endif
	extern int _ninits;
	++_ninits;

	if (ifarg(1)) {
		v = *getarg(1);
		setv = 1;
	}
	t = 0.;
	if (tree_changed) {
		setup_topology();
	}
#if VECTORIZE
	if (v_structure_change) {
		v_setup_vectors();
	}
	if (diam_changed) {
		recalc_diam();
	}
	if (setv) {
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < v_node_count; ++i) {
			v_node[i]->v = v;
		}
	}
#if METHOD3
	if (_method3 == 3) {
#if _CRAY
#pragma _CRI ivdep
#endif
		for (i=0; i < v_node_count; ++i) {
			v_node[i]->toparent.current = 0.;
			v_node[i]->fromparent.current = 0.;
		}
	}
#endif
		
#if _CRAY
	cray_node_init();
#endif
	for (i=0; i < n_memb_func; ++i) if (memb_func[i].initialize && memb_list[i].nodecount){
		if (memb_list[i].vectorized) {
			memb_func[i].initialize(
			   memb_list[i].nodecount,
			   memb_list[i].nodelist,
			   memb_list[i].data,
			   memb_list[i].pdata,
			   i
			);
		}else{
			int j, count;
			Pfri s = memb_func[i].initialize;
			Memb_list* m = memb_list + i;
			count = m->nodecount;
			for (j = 0; j < count; ++j) {
				Node* nd = m->nodelist[j];
				(*s)(nd, m->data[j], m->pdata[j]);
			};
		}
		if (errno) {
			if (nrn_errno_check(i)) {
hoc_warning("errno set during call to INITIAL block", (char*)0);
			}
		}
	}
#else
	/* set all v first */
	for (isec = nsection - 1; isec >= 0; isec--) {
		sec = section + isec;
		for (inode = sec->nnode - 1; inode >= 0; inode--) {
			nd = sec->node + inode;
#if EXTRACELLULAR
			if (sec->extnode) {
				sec->extnode[inode].v = 0;
			}
#endif
			if (setv) {
				nd->v = v;
			}
		}
	}
	/* now call all INITIAL functions */
	for (isec = nsection - 1; isec >= 0; isec--) {
		sec = section + isec;
		for (inode = sec->nnode - 1; inode >= 0; inode--) {
			nd = sec->node + inode;
#if EXTRACELLULAR
			if (sec->extnode) {
				vm = nd->v - sec->extnode[inode].v;
			}else
#endif
			{
				vm = nd->v;
			}
			for (p = nd->prop; p; p = p->next) {
				if (memb_func[p->type].initialize) {
					(memb_func[p->type].initialize)(p, vm);
					if (errno) {
						if (nrn_errno_check(p, inode, sec)) {
hoc_warning("errno set during call to INITIAL block", (char*)0);
						}
					}
				}
			}
		}
	}
#endif
	ret(1.);
}
	

static FILE* batch_file;
static int batch_size;
static int batch_n;
static double** batch_var;

static batch_open(name, tstop, tstep) char* name; double tstop, tstep; {

	if (batch_file) {
		batch_close();
	}
	if (!name) {
		return;
	}
	batch_file = fopen(name, "w");
	if (!batch_file) {
		hoc_execerror("Couldn't open batch file", name);
	}
	fprintf(batch_file, "batch_run from t = %g to %g in steps of %g with dt = %g\n",
		t, tstop, tstep, dt);
#if 0
	fprintf(batch_file, "variable names --\n");
	if (!batch_var) {
		batch_var = hoc_newlist();
	}
	count = 0;
	ITERATE(q, batch_varname) {
		fprintf(batch_file, "%s\n", STR(q));
		++count;
	}
	if (count != batch_n) {
		if (batch_var) {
			free(batch_var);
		}
		batch_n = count;
		batch_var = (double**)ecalloc(batch_n, sizeof(double*));
	}
	count = 0;
	ITERATE(q, batch_varname) {
		batch_var[count++] = hoc_val_pointer(STR(q));
	}
#endif
}

static batch_close() {
	if (batch_file) {
		fclose(batch_file);
		batch_file = 0;
	}
}

static batch_out() {
	if (batch_file) {
		int i;
		for (i =0; i < batch_n; ++i) {
			fprintf(batch_file, " %g", *batch_var[i]);
		}
		fprintf(batch_file,"\n");
	}
}

batch_save() {
	double* hoc_pgetarg();
	int i;
	if (!ifarg(1)) {
		batch_n = 0;
	}else{
		for (i=1; ifarg(i); ++i) {	
			if ( batch_size <= batch_n){
				batch_size += 20;
				batch_var = (double**)erealloc(batch_var, batch_size*sizeof(double*));
			}
			batch_var[batch_n] = hoc_pgetarg(i);
			++batch_n;
		}
	}
	ret(1.);
}
