/* $Header: /timshel2g/src/master/nrn/src/nrnoc/fstim.c,v 1.1.1.1 1994/10/12 17:22:33 hines Exp $ */
/*
$Log: fstim.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:33  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.44  1993/08/15  12:56:04  hines
 * fix warning of function declared external and later static
 * y
 *
 * Revision 4.25  93/04/20  08:58:38  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.9  93/01/21  12:21:15  hines
 * watch out for freed sections in old style fstim, fsyn, and fclamp
 * correct memory leakage on access section
 * 
 * Revision 3.23  92/10/09  09:13:52  hines
 * clean up some warnings and bugs. sectionname was broken and
 * sometimes fstim and fsyn were used when they hadn't gotten a section
 * pointer.
 * 
 * Revision 2.75  92/08/07  16:11:55  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.42  91/08/22  16:58:21  hines
 * for distribution to work on RS6000
 * 
 * Revision 1.84  90/10/24  10:56:38  hines
 * saber warnings gone
 * 
 * Revision 1.59  90/03/15  16:39:43  mlh
 * error in handling pstim[] for multiple stimuli in return of current
 * and conductance (fsyng) function.
 * 
 * Revision 1.52  90/01/24  07:04:49  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.42  89/10/13  09:01:17  mlh
 * free the clamp, synapse, and stim when new sections created
 * de-lint some minor things
 * 
 * Revision 1.39  89/09/25  16:07:03  mlh
 * print point current info with prstim()
 * 
 * Revision 1.30  89/09/06  10:26:01  mlh
 * newcable to neuron
 * 
 * Revision 1.18  89/08/12  14:48:50  mlh
 * Lint free
 * 
 * Revision 1.10  89/08/09  10:50:23  mlh
 * fstim works, v accessible at ends, for (x) goes from 0<=x<=1
 * 
 * Revision 1.1  89/08/08  13:23:03  mlh
 * Initial revision
 * 
*/


/* copy of synapse.c modified to simulate current stimulus pulses */
/*
fstim(maxnum)
 allocates space for maxnum synapses. Space for
 previously existing synapses is released. All synapses initialized to
 0 maximum conductance.

fstim(i, loc, delay, duration, stim)
 The ith current stimulus is injected at parameter `loc'
 different current stimuli do not concatenate but can ride on top of
 each other. delay refers to onset of stimulus relative to t=0
 delay and duration are in msec.
 stim in namps.
  
fstimi(i)
 returns stimulus current for ith stimulus at the value of the
 global time t.

*/

#include "neuron.h"
#include "section.h"

typedef struct Stimulus {
	double loc;	/* parameter location (0--1) */
	double delay;	/* value of t in msec for onset */
	double duration;/* turns off at t = delay + duration */
	double mag;	/* magnitude in namps */
	double mag_seg;	/* value added to rhs, depends on area of seg*/
	Node *pnd;	/* segment location */
	Section* sec;
} Stimulus;

static maxstim = 0;		/* size of stimulus array */
static Stimulus *pstim;		/* pointer to stimulus array */

extern double *getarg(), chkarg();
extern char *secname();

print_stim() {
	int i;
	
	if (maxstim == 0) return;
	/*SUPPRESS 440*/
	Printf("fstim(%d)\n/* section	fstim( #, loc, delay(ms), duration(ms), magnitude(namp)) */\n", maxstim);
	for (i=0; i<maxstim; i++) {
		Printf("%-15s fstim(%2d,%4g,%10g,%13g,%16g)\n",
		secname(pstim[i].sec), i,
		pstim[i].loc, pstim[i].delay, pstim[i].duration, pstim[i].mag);
	}
}

static double stimulus();

fstimi() {
	int i;
	double cur;
	
	i = chkarg(1, 0., (double)(maxstim-1));
	if ((cur = stimulus(i)) != 0.) {
		cur = pstim[i].mag;
	}
	ret(cur);
}

static stim_record();

fstim() {
	int i;

	i = chkarg(1, 0., 10000.);
	if (ifarg(2)) {
		if (i >= maxstim) {
			hoc_execerror("index out of range", (char *)0);
		}
		pstim[i].loc = chkarg(2, 0., 1.);
		pstim[i].delay = chkarg(3, 0., 1e21);
		pstim[i].duration = chkarg(4, 0., 1e21);
		pstim[i].mag = *getarg(5);
		pstim[i].sec = chk_access();
		section_ref(pstim[i].sec);
		stim_record(i);
	} else {
		free_stim();
		maxstim = i;
		if (maxstim) {
			pstim = (Stimulus *)emalloc((unsigned)(maxstim * sizeof(Stimulus)));
		}
		for (i = 0; i<maxstim; i++) {
			pstim[i].loc = 0;
			pstim[i].mag = 0.;
			pstim[i].delay = 1e20;
			pstim[i].duration = 0.;
			pstim[i].sec = 0;
			stim_record(i);
		}
	}
	ret(0.);
}

free_stim() {
	int i;
	if (maxstim) {
		for (i=0; i < maxstim; ++i) {
			if (pstim[i].sec) {
				section_unref(pstim[i].sec);
			}
		}
		free((char *)pstim);
		maxstim=0;
	}
}

static
stim_record(i)	/*fill in the section info*/
	int i;
{
	Node *node_ptr();
	double area;
	Section* sec;

	sec = pstim[i].sec;
	if (sec) {
		if (sec->prop) {
			pstim[i].pnd = node_ptr(sec, pstim[i].loc, &area);
			pstim[i].mag_seg = 1.e2*pstim[i].mag / area;
		}else{
			section_unref(sec);
			pstim[i].sec = 0;
		}
	}
}

stim_prepare() {
	int i;
	
	for (i=0; i<maxstim; i++) {
		stim_record(i);
	}
}

static double
stimulus(i)
	int i;
{
	if (t < pstim[i].delay-1e-9
	  || t > pstim[i].delay + pstim[i].duration - 1e-9) {
		return 0.0;
	}
	return pstim[i].mag_seg;
}

activstim() {
	
	int i;

	for (i=0; i<maxstim; i++) {
		if (pstim[i].sec) {
			pstim[i].pnd->rhs += stimulus(i);
		}
	}
}
