/* $Header: /timshel2g/src/master/nrn/src/nrnoc/point.c,v 1.2 1994/11/02 18:53:45 hines Exp $ */

/* modl description via modlreg calls point_register_mech() and
saves the pointtype as later argument to create and loc */

#include "section.h"
#include "membfunc.h"
#include "ytab.h"

extern char pnt_map[];
extern Symbol *pointsym[]; /*list of variable symbols in s->u.ppsym[k]
				with number in s->s_varn */

extern double loc_point_process();

static int cppp_semaphore = 0;	/* connect point process pointer semaphore */
static double** cppp_pointer;
static free_one_point();

void* create_point_process(pointtype, ho)
	int pointtype;
	Object* ho;
{
	Point_process* pp;
	pp = (Point_process*)emalloc(sizeof(Point_process));
	pp->node = 0;
	pp->sec = 0;
	pp->prop = 0;
	pp->ob = ho;
	if (ifarg(1)) {
		loc_point_process(pointtype, (void*)pp);
	}
	return (void*)pp;
}

Object*
nrn_new_pointprocess(sym)
	Symbol* sym;
{
	void* v;
	Object* hoc_new_object();
	Object* ob;
	extern Symlist* hoc_built_in_symlist;
	Symbol* hoc_table_lookup();
	int pointtype;
	assert(sym->type == MECHANISM && memb_list[sym->subtype].is_point);
	pointtype = pnt_map[sym->subtype];
	push_frame(sym, 0);
	v = create_point_process(pointtype);
	pop_frame();
	sym = hoc_table_lookup(sym->name, hoc_built_in_symlist);
	ob =  hoc_new_object(sym, v); 
	((Point_process*)v)->ob = ob;
	return ob;
}

destroy_point_process(v)
	void* v;
{
	Point_process* pp = (Point_process*)v;
	free_one_point(pp);
	free((char*)pp);
}

nrn_loc_point_process(pointtype, pnt, sec, node)
	int pointtype;
	Point_process* pnt;
	Section* sec;
	Node* node;
{
	extern Prop *prop_alloc_disallow(), *prop_alloc();
	Prop* p;
	double x, nrn_arc_position();
	
	x = nrn_arc_position(sec, node);
	if (x == 0. || x == 1.) {
		p = prop_alloc_disallow(&(node->prop), pointsym[pointtype]->subtype);
	}else{
		p = prop_alloc(&(node->prop), pointsym[pointtype]->subtype);
	}
	if (pnt->prop) {
		if (p->param) {
			free((char*)p->param);
		}
		p->param = pnt->prop->param;
		pnt->prop->param = (double*)0;
		free_one_point(pnt);
	}
	nrn_sec_ref(&pnt->sec, sec);
	pnt->node = node;
	pnt->prop = p;
	pnt->prop->dparam[0].pval = &(node->area);
	pnt->prop->dparam[1]._pvoid = (void*)pnt;
	if (pnt->ob) {
		if (pnt->ob->observers) {
			hoc_obj_notify(pnt->ob);
		}
		if (pnt->ob->template->observers) {
			hoc_template_notify(pnt->ob, 2);
		}
	}
}


nrn_relocate_old_points(oldsec, oldnode, sec, node)
	Section* oldsec, *sec;
	Node* oldnode, *node;
{
	Point_process* pnt;
	Prop* p, *pn;
	if (oldnode) for (p = oldnode->prop; p; p = pn) {
		pn = p->next;
		if (memb_list[p->type].is_point) {
			pnt = (Point_process*)p->dparam[1]._pvoid;
			if (oldsec == pnt->sec) {
				if (oldnode == node) {
					nrn_sec_ref(&pnt->sec, sec);
				}else{
#if 0
double nrn_arc_position();
char* secname();
printf("relocating a %s to %s(%d)\n",
memb_func[p->type].sym->name,
secname(sec), nrn_arc_position(sec, node)
);
#endif
					nrn_loc_point_process(
					   pnt_map[p->type], pnt, sec, node
					);
				}
			}
		}
	}
}

double loc_point_process(pointtype, v)
	int pointtype;
	void* v;
{
	Point_process* pnt = (Point_process*)v;
	double x, chkarg();
	Section *sec;
	extern Prop *prop_alloc();
	Node *node, *node_exact();
	
	x = chkarg(1, 0., 1.);
	sec = chk_access();
	node = node_exact(sec, x);
	
	nrn_loc_point_process(pointtype, pnt, sec, node);
	return x;
}

double
get_loc_point_process(v)
	void* v;
{
#if METHOD3
	extern int _method3;
#endif
	double x, nrn_arc_position();
	Point_process *pnt = (Point_process*)v;
	Section* sec;
	
	if (pnt->prop == (Prop *)0) {
		hoc_execerror("point process not located in a section", (char*)0);
	}
	sec = pnt->sec;
	x = nrn_arc_position(sec, pnt->node);
	nrn_pushsec(sec);
	return x;
}

double
has_loc_point(v)
	void* v;
{
	Point_process *pnt = (Point_process*)v;
	return (pnt->sec != 0);
}

double* point_process_pointer(pnt, sym, index)
	Point_process* pnt;
	Symbol* sym;
	int index;
{
	static double dummy;
	double* pd;
	if (pnt->prop == (Prop *)0) {
		hoc_execerror("point process not located in a section", (char*)0);
	}
	if (sym->subtype == NRNPOINTER) {
		pd = (pnt->prop->dparam)[sym->u.rng.index + index].pval;
		if (cppp_semaphore) {
			++cppp_semaphore;
			cppp_pointer = &((pnt->prop->dparam)[sym->u.rng.index + index].pval);
			pd = &dummy;
		}else if(!pd) {
		  hoc_execerror(sym->name, "wasn't made to point to anything");
		}
	}else{
		pd = pnt->prop->param + sym->u.rng.index + index;
	}
	return pd;
}

void steer_point_process(v) /* put the right double pointer on the stack */
	void* v;
{
	Symbol* sym, *hoc_spop();
	int index;
	Point_process *pnt = (Point_process*)v;
	sym = hoc_spop();
	if (ISARRAY(sym)) {
		index = hoc_araypt(sym, SYMBOL);
	}else{
		index = 0;
	}
	hoc_pushpx(point_process_pointer(pnt, sym, index));
}

nrn_cppp() {
	cppp_semaphore = 1;
}

connect_point_process_pointer() {
	double* hoc_pxpop();
	if (cppp_semaphore != 2) {
		cppp_semaphore = 0;
		hoc_execerror("not a point process pointer");
	}
	cppp_semaphore = 0;
	*cppp_pointer = hoc_pxpop();
	hoc_nopop();
}

static free_one_point(pnt) /* must unlink from node property list also */
	Point_process *pnt;
{
	Prop *p, *p1;

	p = pnt->prop;
	if (!p) {
		return;
	}
	p1 = pnt->node->prop;
	if (p1 == p) {
		pnt->node->prop = p1->next;
	}else for (; p1; p1 = p1->next) {
		if (p1->next == p) {
			p1->next = p->next;
			break;
		}
	}
#if VECTORIZE
	{ extern int v_structure_change;
	  v_structure_change = 1;
	}
#endif
	if (p->param) {
		hoc_free_val_array(p->param, p->param_size);
	}
	if (p->dparam) {
		free((char*)p->dparam);
	}
	free((char*)p);
	pnt->prop = (Prop *)0;
	pnt->node = (Node *)0;
	section_unref(pnt->sec);
	pnt->sec = (Section *)0;
}

clear_point_process_struct(p) /* called from prop_free */
	Prop *p;
{
	Point_process* pnt;
	pnt = (Point_process*)p->dparam[1]._pvoid;
	if (pnt) {
		free_one_point(pnt);	
		if (pnt->ob) {
			if (pnt->ob->observers) {
				hoc_obj_notify(pnt->ob);
			}
			if (pnt->ob->template->observers) {
				hoc_template_notify(pnt->ob, 2);
			}
		}
	} else {
		if (p->param) {
			hoc_free_val_array(p->param, p->param_size);
		}
		if (p->dparam) {
			free((char*)p->dparam);
		}
		free((char*)p);
	}
}

int is_point_process(ob)
	Object* ob;
{
	if (ob && ob->template->steer == steer_point_process) {
		return 1;
	}else{
		return 0;
	}
}
