/* $Header: /timshel2g/src/master/nrn/src/nrnoc/psection.c,v 1.1.1.1 1994/10/12 17:22:34 hines Exp $ */
/*
$Log: psection.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:34  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.78  1994/09/27  14:11:17  hines
 * get the vartype from the function in psection in order to print CONST
 * ion parameters
 *
 * Revision 4.27  1993/04/21  13:24:15  hines
 * watch out for nil parentnode when haven't done setup_topology yet.
 *
 * Revision 4.25  93/04/20  08:58:49  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.12  93/02/02  10:25:01  hines
 * declare static functions before use
 * 
 * Revision 4.5  93/01/07  08:15:00  hines
 * psection prints Ra
 * warning if Ra set but not in every section
 * fix batch_run so not extra iteration on cray (tstop -=dt/4)
 * 
 * Revision 3.32  92/10/22  11:13:01  hines
 * remove iv stuff from point.c and psection.v
 * 
 * Revision 3.10  92/09/24  07:15:51  hines
 * submenu for new graph that makes voltage,current or state graphs
 * 
 * Revision 3.7  92/09/18  15:14:39  hines
 * better nrnmainmenu
 * 
 * Revision 2.75  92/08/07  16:12:04  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.68  92/07/31  13:02:53  hines
 * plots/ms instead of steps/ms
 * 
 * Revision 2.62  92/04/01  12:59:56  hines
 * use new button value
 * 
 * Revision 2.61  92/03/29  16:11:23  hines
 * labeling of mech and points as parameters, globals, etc.
 * not a bad control menu
 * 
 * Revision 2.59  92/03/19  09:18:13  hines
 * menuFileMenu and erase button in control menu
 * 
 * Revision 2.56  92/03/11  15:42:14  hines
 * controlmenu. requires some hoc code to be present.
 * most things seem to work pretty well
 * 
 * Revision 2.55  92/03/10  09:26:40  hines
 * nrniv: automatic menus - mainmenu(), good start.
 * 
 * Revision 2.42  91/08/22  16:58:24  hines
 * for distribution to work on RS6000
 * 
 * Revision 2.32  91/08/08  16:40:29  hines
 * saber lint free. can make saber_src_all which is all the src pertaining
 * to neuron
 * 
 * Revision 2.28  91/07/29  14:59:06  hines
 * psection no longer checks to see if range variables are constant. Instead
 * it explicitly prints the constants for the first segment. Also
 * all constants are printed, not just the ones that differ from the default
 * values.
 * 
 * Revision 1.87  90/10/29  12:31:33  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:45  hines
 * saber warnings gone
 * 
 * Revision 1.45  89/10/30  09:55:43  mlh
 * range variables have a subtype of CONST, DEPENDENT, or STATE
 * psection prints only non-default CONST
 * 
 * Revision 1.39  89/09/25  16:07:10  mlh
 * print point current info with prstim()
 * 
 * Revision 1.37  89/09/20  12:02:33  mlh
 * ionic reversal potentials and currents now in Eion
 * other mechanisms, such as HH, use and modify Eion parameters.
 * a mechanism allocation routine can "need" another mechanism and
 * communicate via Datum.pval
 * 
 * Revision 1.32  89/09/14  15:19:00  mlh
 * have to say something about sections connected to section[0]
 * call the nodes of section[0] cells.
 * 
 * Revision 1.27  89/09/01  12:04:20  mlh
 * psection() prints non_default section information in hoc executable form
 * to the standard output.
 * 
 * Revision 1.1  89/08/31  15:54:28  mlh
 * Initial revision
 * 
*/

#include <stdio.h>
#include "section.h"
#include "ytab.h"
#include "membfunc.h"

extern char *secname();
extern double nrn_ra();
static pnode();

psection()
{
	Section *sec;
	Prop *p, *p1;
	
	sec = chk_access();
	p = sec->prop;
	Printf("%s {", secname(sec));
	Printf(" nseg=%d  L=%g  Ra=%g", sec->nnode - 1, p->dparam[2].val,
		nrn_ra(sec));
	if (p->dparam[4].val != 1) {
		Printf(" rallbranch=%g", p->dparam[4].val);
	}
	Printf("\n");
	if (sec->parentsec) {
		Printf("	%s ",secname(sec->parentsec));
		Printf("connect %s (%g), %g\n",	secname(sec),
			p->dparam[3].val, p->dparam[1].val);
	} else {
		v_setup_vectors();
		/*SUPPRESS 440*/
		Printf("	/*location %g attached to cell %d*/\n",
			p->dparam[3].val, sec->parentnode->v_node_index);
	}
	if (sec->nnode) {
		/*SUPPRESS 440*/
		Printf("	/* First segment only */\n");
		p1 = sec->node->prop;
		pnode(p1);
	}
	Printf("}\n");
	ret(1.);
}

static pnode(p1)
	Prop *p1;
{
	Symbol *sym;
	int j;
	
	if (!p1) {
		return;
	}
	pnode(p1->next);	/*print in insert order*/
	sym = memb_func[p1->type].sym;
	Printf("	insert %s {", sym->name);
	if (sym->s_varn) {
		for (j=0; j < sym->s_varn; j++ ) {
			if (nrn_vartype(sym->u.ppsym[j]) == CONST) {
				Printf(" %s=%g",
					sym->u.ppsym[j]->name, p1->param[j]);
			}
		}
	}
	Printf("}\n");
}

prstim()
{
	print_stim();
	print_clamp();
	print_syn();
	ret(1.);
}

