/* $Header: /timshel2g/src/master/nrn/src/nrnoc/section.h,v 1.1.1.1 1994/10/12 17:22:32 hines Exp $ */
#ifndef section_h
#define section_h
/*
$Log: section.h,v $
 * Revision 1.1.1.1  1994/10/12  17:22:32  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.58  1994/05/09  19:14:43  hines
 * Point Process structure keeps a pointer to the owning object.
 * This allows notification of object holders when the point
 * process location changes.
 *
 * Also one can determine if a point process is located in a section with
 * pp.has_loc()
 *
 * Revision 4.33  1993/05/31  15:35:03  hines
 * for (ch=sec->child; ch; ch=ch->sibling) is loop over all sections connect
 * directly to sec in order of increasing distance from connection end of sec.
 * there is no longer a Node->child pointer.
 *
 * Revision 4.25  93/04/20  08:58:51  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.12  93/02/02  10:25:02  hines
 * declare static functions before use
 * 
 * Revision 4.11  93/01/27  13:50:11  hines
 * extracellular works again
 * 
 * Revision 3.40  92/11/27  08:22:04  hines
 * boolean nrn_exists(Symbol*, Node*)
 * sections contain a short volatile_mark used for associative searching.
 * 
 * Revision 3.33  92/10/24  16:30:21  hines
 * freeing vars notifies interviews.
 * sec_and_position(sec, inode) returns string in form section(x)
 * some portability fixes from pittsburgh
 * 
 * Revision 3.22  92/10/09  07:46:30  hines
 * replace vector style point process syntax with objectvar style.
 * POINTER not yet implemented and old syntax not yet removed from parser.
 * usage:
 * objectvar s
 * s = PulseStim(.5)
 * s.del = 5
 * relocate with
 * s.loc(x)
 * x = s.getloc() pushes section on section stack and must be removed with
 * popsec()
 * 
 * Revision 3.21  92/10/08  10:26:52  hines
 * third order correct with _method3 = 3 and when more than 1 segment
 * per section and interior segments do not have point processes.
 * when not active then third order correct no matter what.
 * 
 * Revision 3.12  92/09/25  18:03:24  hines
 * for method3. now tested and working for transient passive cables.
 * but perhaps the equations could be written with better roundoff
 * error when dt is small
 * 
 * Revision 3.11  92/09/24  17:03:50  hines
 * METHOD3 option. use with spatial_method(i) with i=0-3
 * 0 is the default and is the normal neuron method with zero area nodes
 * at boundaries and branch points.
 * 1 is the standard method with half area nodes at boundaries
 * 2 is a modified second order method which weights by 2/12, 8/12, 2/12
 * 3 is a third order correct method which weights by 1/12 10/12 1/12
 * with the last three methods x = i/nnode with i=0 to nnode
 * methods 1 and 2 are at present quite inefficient since they use the
 * data structures of 3.
 * Works in steady state. not tested with dynamic simulations
 * At this time, discontinuities in parameters (except diameter) is
 * not handled well across nodes and all parameters are assumed to
 * be constant in interval between node and parent node.
 * Good deal of difficulty remains in managing values that change discontinuously
 * 
 * Revision 2.82  92/08/10  16:02:35  hines
 * section lists with seclistdef ,,,
 * append_seclist(seclist)
 * remove_seclist(seclist)
 * forsec seclist
 * ifsec seclist
 * sections are now reference counted.
 * 
 * Revision 2.75  92/08/07  16:12:05  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.71  92/08/05  09:27:17  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.49  92/01/30  11:08:35  hines
 * verified with respect to all neuron/examples. Under control of imake.
 * ObjectCenter warning free. Some changes to match neuron.
 * 
 * Revision 2.31  91/08/08  14:58:32  hines
 * 3-D information attached to sections for simulation of reconstructed
 * neurons.  This information is not discarded when nseg is changed.
 * 
 * Revision 2.23  91/05/31  15:57:51  hines
 * DIAMLIST
 * 
 * Revision 2.7  90/12/14  09:48:30  hines
 * First pass at handling point processes.
 * 
 * Revision 1.87  90/10/29  12:31:34  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.56  90/01/29  14:13:53  mlh
 * debugsolve undefined again
 * 
 * Revision 1.52  90/01/24  07:05:04  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.48  90/01/16  07:05:41  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.6  89/08/02  16:11:55  mlh
 * range variables implemented in open mechanism context
 * 
 * Revision 1.3  89/07/21  17:38:31  mlh
 * compiles
 * 
 * Revision 1.1  89/07/20  07:52:20  mlh
 * Initial revision
 * 
*/


/* In order to support oc objects containing sections, instead of vector
	of ordered sections, we now have a list (in the nmodl sense)
	of unordered sections. The lesser efficiency is ok because the
	number crunching is vectorized. ie only the user interface deals
	with sections and that needs to be convenient
*/
/* Data structure for solving branching 1-D tree diffusion type equations.
	Vector of ordered sections each of which points to a vector of nodes.
	Each section must have at least one node. There may be 0 sections.
	The order of last node to first node is used in triangularization.
	First node to last is used in back substitution.
	The first node of a section is connected to some node of a section
	  with lesser index.
*/
/* An equation is associated with each node. d and rhs are the diagonal and
   right hand side respectively.  a is the effect of this node on the parent
   node's equation.  b is the effect of the parent node on this node's
   equation.
   d is assumed to be non-zero.
   d and rhs is calculated from the property list.
*/

#include "options.h"
#include "list.h"

/*#define DEBUGSOLVE 1*/
#define xpop	hoc_xpop
#define pc	hoc_pc
#define spop	hoc_spop
#define execerror	hoc_execerror
#include "hocdec.h"

typedef struct Section {
	short refcount;		/* may be in more than one list */
	short 	nnode;		/* Number of nodes for ith section */
	struct Section*	parentsec;	/* parent section of node 0 */
	struct Section* child;		/* root of the list of children
						connected to this parent kept in
						order of increasing x */
	struct Section*	sibling;	/* used as list of sections that have same parent */


	/* the parentnode is only valid when tree_changed = 0 */
	struct Node*    parentnode;     /* parent node */  
	struct Node	*node;	/* Pointer to  vector of node structures */
	int order;		/* index of this in secorder vector */
	short volatile_mark;	/* for searching */
#if DIAMLIST
	short  npt3d;	/* amount of allocated space for x, d pairs */
	struct Pt3d *pt3d; /* list of 3d points with diameter */
#endif
	struct Prop	*prop;	/* eg. length, etc. */
} Section;

#if DIAMLIST
typedef struct Pt3d {
	float x,y,z,d;	/* 3d point, microns */
	double arc;
} Pt3d;
#endif

#if METHOD3
typedef float NodeCoef;
typedef double NodeVal;

typedef struct Info3Coef {
	NodeVal current;	/* for use in next time step */
	NodeVal djdv0;
	NodeCoef coef0;		/* 5dx/12 */
	NodeCoef coefn;		/* 1dx/12 */
	NodeCoef coefjdot;	/* dx^2*ra/12 */
	NodeCoef coefdg;	/* dx/12 */
	NodeCoef coefj;		/* 1/(ra*dx) */
	struct Node* nd2;	/* the node dx away in the opposite direction*/
/* note above implies that nodes next to branches cannot have point processes
and still be third order correct. Also nodes next to branches cannot themselves
be branch points */
} Info3Coef;

typedef struct Info3Val {	/* storage to help build matrix efficiently */
	NodeVal GC;		/* doesn't include point processes */
	NodeVal EC;
	NodeCoef Cdt;
} Info3Val;

/*METHOD3*/
#endif

/* if any double is added after area then think about changing
the notify_free_val parameter in node_free in solve.c
*/
typedef struct Node {
	double	v;			/* main state variable, voltage or calcium */
	double	d;			/* diagonal element in node equation */
	double	rhs;			/* right hand side in node equation */
	double	a;			/* effect of node in parent equation */
	double	b;			/* effect of parent in node equation */
	double area;			/* area in um^2 but see treesetup.c */
	struct Prop	*prop;	/* Points to beginning of property list */
	Section* child;		/* section connected to this node */
						/* 0 means no other section connected */
#if EXTRACELLULAR
	struct Extnode* extnode;
#endif

#if EXTRAEQN
	struct Eqnblock *eqnblock;	/* hook to other equations which
			need to be solved at the same time as the membrane
			potential. eg. fast changeing ionic concentrations */
#endif /*MOREEQN*/

#if DEBUGSOLVE
	double savd;
	double savrhs;
#endif /*DEBUGSOLVE*/
#if VECTORIZE
	int v_node_index;	/* only used to calculate parent_node_indices*/
#endif
#if METHOD3
	Info3Coef toparent;
	Info3Coef fromparent;
	Info3Val thisnode;
#endif
} Node;

#if EXTRACELLULAR
typedef struct Extnode {
	double	*param;	/* points to extracellular parameter vector */
	double	v; /* v external. In corresponding node, v is v membrane. */
	double	d; /* the internal potential is  node->v + extnode->v */
	double	rhs; /* d, rhs, a, and b are analogous to those in node */
	double	a;
	double	b;
double  xbin;           /* 0 effect of vext[parent] on internal current of i */
double  xain;           /* effect of vext[i] on internal current of i */
double  xbout;          /* effect of v[i] on external current of i*/
double  xaout;          /* 0 effect of v[i] on external current of parent */
#if DEBUGSOLVE
	double savd;
	double savrhs;
	double savxain;
	double savxbout;
#endif /*DEBUGSOLVE*/
} Extnode;
#endif

#if !INCLUDEHOCH
#include "hocdec.h"		/* Prop needs Datum and Datum needs Symbol */
#endif

typedef struct Prop {
	struct Prop	*next;	/* linked list of properties */
	short	type;		/* type of membrane, e.g. passive, HH, etc. */
	int	param_size;	/* for notifying hoc_free_val_array */
	double	*param;	/* vector of doubles for this property */
	Datum	*dparam; /* usually vector of pointers to doubles
			    of other properties but maybe other things as well
			    for example one cable section property is a
			    symbol */
} Prop;

/* a point process is computed just like regular mechanisms. Ie it appears
in the property list whose type specifies which allocation, current, and
state functions to call.  This means some nodes have more properties than
other nodes even in the same section.  The Point_process structure allows
the interface to hoc variable names.
Each variable symbol u.rng->type refers to the point process mechanism.
The variable is treated as a vector
variable whose first index specifies "which one" of that mechanisms insertion
points we are talking about.  Finally the variable u.rng->index tells us
where in the p-array to look.  The number of point_process vectors is the
number of different point process types.  This is different from the
mechanism type which enumerates all mechanisms including the point_processes.
It is the responsibility of create_point_process to set up the vectors and
fill in the symbol information.  However only after the process is given
a location can the variables be set or accessed. This is because the
allocation function may have to connect to some ionic parameters and the
process exists primarily as a property of a node.
*/
typedef struct Point_process {
	Section *sec;	/* section and node location for the point mechanism*/
	Node *node;
	Prop *prop;	/* pointer to the actual property linked to the
			   node property list */
	Object* ob;	/* object that owns this process */
} Point_process;

#if EXTRAEQN
/*Blocks of equations can hang off each node of the current conservation
equations. These are equations which must be solved simultaneously
because they depend on the voltage and affect the voltage. An example
are fast changing ionic concentrations (or merely if we want to be
able to calculate steady states using a stable method).
*/
typedef struct Eqnblock {
	struct Eqnblock *eqnblock_next; /* may be several such blocks */
	Pfri	eqnblock_triang;	/* triangularization function */
	Pfri	eqnblock_bksub;	/* back substitution function */
	double	*eqnblock_data;
#if 0
	the solving functions know how to find the following info from
	the eqnblock_data.
	double *eqnblock_row;	/* current conservation depends on states */
	double *eqnblock_col;	/* state equations depend on voltage */
	double *eqnblock_matrix; /* state equations depend on states */
	double *eqnblock_rhs:
	the functions merely take a pointer to the node and this Eqnblock
	in order to update the values of the diagonal, v, and the rhs
	The interface with EXTRACELLULAR makes things a bit more subtle.
	It seems clear that we will have to change the meaning of v to
	be membrane potential so that the internal potential is v + vext.
	This will avoid requiring two rows and two columns since
	the state equations will depend only on v and not vext.
	In fact, if vext did not have longitudinal relationships with
	other vext the extracellular mechanism could be implemented in
	this style.
#endif
} Eqnblock;
#endif /*EXTRAEQN*/

extern int rootnodecount;
extern hoc_List* section_list;	/* Where the Sections live */
extern Section	*sec_alloc();		/* Allocates a single section */
extern void	node_alloc();		/* Allocates node vectors in a section*/
extern Section* chk_access();

/* vectorization no longer optional since change to section list */
#if VECTORIZE
extern int v_node_count;
extern Node** v_node;
extern Node** v_parent;
#endif

/* loop over sections. Must previously declare Item* qsec. Contains the {! */
#define ForAllSections(sec) \
	ITERATE(qsec, section_list) { Section* sec = SEC(qsec);

#endif

#if METHOD3
extern int _method3;
#endif
