/* $Header: /timshel2g/src/master/nrn/src/nrnoc/solve.c,v 1.1.1.1 1994/10/12 17:22:35 hines Exp $ */
/*
$Log: solve.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:35  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.66  1994/06/30  14:42:47  hines
 * #if away triang and bksub when EXTRACELLULAR is compiled in solve.c
 *
 * Revision 4.53  1994/01/21  19:34:19  hines
 * distance(0,x) can be used to set an exact origin for distance(x)
 * or distance(1,x) measurement
 *
 * Revision 4.51  1993/12/20  18:52:14  hines
 * combination of no extracellular and no cray and vectorization
 * causes a bad triang statement to be compiled.
 *
 * Revision 4.39  1993/07/13  11:49:26  hines
 * sibling pointers in sections are not referenced. bug when they were unreffed.
 *
 * Revision 4.33  93/05/31  15:35:04  hines
 * for (ch=sec->child; ch; ch=ch->sibling) is loop over all sections connect
 * directly to sec in order of increasing distance from connection end of sec.
 * there is no longer a Node->child pointer.
 * 
 * Revision 4.28  93/04/29  16:36:04  hines
 * sibling pointers now reffed. Fix several problems with disconnecting
 * and reconnecting sections as well as handle point processes moving with
 * their proper section.
 * Still not perfect though.
 * 
 * Revision 4.27  93/04/21  13:24:17  hines
 * watch out for nil parentnode when haven't done setup_topology yet.
 * 
 * Revision 4.26  93/04/21  10:33:20  hines
 * point processes located at ends of sections stay in those sections when
 * those sections are disconnected from their parent.
 * 
 * Revision 4.25  93/04/20  08:58:52  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.22  93/04/06  08:04:29  hines
 * define EXTRACELLULAR 1 in options.h
 * move nrnmainmenu and nrncontrolmenu out of c++ and into stdrun.hoc
 * 
 * Revision 4.21  93/03/23  17:24:22  hines
 * allow deletion of all sections
 * fix bug having to do with deletion of multiple sections
 * 
 * Revision 4.19  93/03/15  10:06:10  hines
 * when deleteing sections clear all the fields. if a list sees a
 * bad section is removes it during the forsec execution
 * 
 * Revision 4.18  93/03/05  15:58:56  hines
 * switch from hoc_free_val to hoc_freed
 * 
 * Revision 4.11  93/01/27  13:50:13  hines
 * extracellular works again
 * 
 * Revision 3.44  92/12/17  08:28:53  hines
 * delete_section() removes currently accessed section from the list of
 * sections to be simulated and releases its property and nodes.
 * 
 * Revision 3.40  92/11/27  08:22:06  hines
 * boolean nrn_exists(Symbol*, Node*)
 * sections contain a short volatile_mark used for associative searching.
 * 
 * Revision 3.37  92/11/09  17:17:21  hines
 * using topol_distance in nrniv SpacePlot
 * also added
 * double section_length(Section*) and
 * int arc0at0(Section*) which returns true if x=0 is connected to parent
 * 
 * Revision 3.35  92/10/27  12:09:53  hines
 * list.c list.h moved from nrnoc  to oc
 * 
 * Revision 3.33  92/10/24  16:30:23  hines
 * freeing vars notifies interviews.
 * sec_and_position(sec, inode) returns string in form section(x)
 * some portability fixes from pittsburgh
 * 
 * Revision 3.29  92/10/14  09:18:01  hines
 * fix bug in distance when sections not connected
 * when sections not connected distance(x) return 1e20
 * 
 * Revision 3.16  92/09/30  16:52:41  hines
 * fix some problems haveing to do with reallocating segments and
 * strength of stimuli.
 * 
 * Revision 3.4  92/08/18  08:57:22  hines
 * sections in different objects can have different array sizes and
 * be created in procedures within objects
 * 
 * Revision 2.86  92/08/13  11:20:47  hines
 * default parameters collected into membdef.h
 * 
 * Revision 2.84  92/08/12  11:59:18  hines
 * initnrn separated into hoc_last_init and initnrn. this last can be
 * called from hoc to reset built-in globals such as secondorder, celsius, etc.
 * 
 * Revision 2.82  92/08/10  16:02:37  hines
 * section lists with seclistdef ,,,
 * append_seclist(seclist)
 * remove_seclist(seclist)
 * forsec seclist
 * ifsec seclist
 * sections are now reference counted.
 * 
 * Revision 2.76  92/08/08  08:43:26  hines
 * section_unlink removes references to the section by other sections
 * 
 * Revision 2.75  92/08/07  16:12:07  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.73  92/08/05  10:28:58  hines
 * got rid of objectcenter messages
 * 
 * Revision 2.71  92/08/05  09:27:19  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.39  91/08/16  13:47:40  hines
 * distance() did not work correctly. Much better now ubt still not fully
 * tested.
 * 
 * Revision 2.35  91/08/13  07:11:49  hines
 * solve() changed to nrn_solve() so that name doesn't conflict
 * with scopmath
 * 
 * Revision 2.31  91/08/08  14:58:33  hines
 * 3-D information attached to sections for simulation of reconstructed
 * neurons.  This information is not discarded when nseg is changed.
 * 
 * Revision 2.27  91/07/11  17:09:15  hines
 * problem with extracellular assertion in debugsolve fixed
 * 
 * Revision 2.23  91/05/31  15:57:52  hines
 * DIAMLIST
 * 
 * Revision 2.17  91/03/01  13:14:35  hines
 * got all "'" taken care of in topology()
 * added functions neuron2nemo ...
 * 
 * Revision 2.15  91/02/21  11:29:22  hines
 * topology was printing ' instead of `
 * 
 * Revision 2.13  91/02/04  12:04:34  hines
 * distance function was not working. fixed but not fully tested.
 * 
 * Revision 2.10  90/12/15  12:22:15  hines
 * distance() function added
 * 
 * Revision 1.87  90/10/29  12:31:35  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:46  hines
 * saber warnings gone
 * 
 * Revision 1.52  90/01/24  07:05:07  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.51  90/01/19  07:30:37  mlh
 * recover from out of memory when too many nodes and calloc so
 * 16 bit machines do not wrap size
 * 
 * Revision 1.50  90/01/16  07:22:07  mlh
 * screwed up the last change. Don't check things in till they have been
 * tested.
 * 
 * Revision 1.49  90/01/16  07:17:14  mlh
 * 1 assertion done incorrectly and one unneccessary. Lint found them in
 * debugsolve().
 * 
 * Revision 1.48  90/01/16  07:05:46  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.38  89/09/25  14:29:33  mlh
 * when node is allocated, voltage must be initialized to 0 to prevent
 * core dump when inadvertently uninitialized.
 * 
 * Revision 1.24  89/08/31  07:46:56  mlh
 * solve.c is lint free
 * 
 * Revision 1.23  89/08/29  15:10:11  mlh
 * bug in turboc with respect to incrementing section and node pointers.
 * explicitly iterate over indices
 * 
 * Revision 1.18  89/08/12  14:49:07  mlh
 * Lint free
 * 
 * Revision 1.15  89/08/11  15:12:05  mlh
 * rall rall branching, several bugs fixed
 * 
 * Revision 1.9  89/08/07  10:24:07  mlh
 * getting good v with single section. matrix solving for v (not deltav)
 * 
 * Revision 1.8  89/08/05  14:25:41  mlh
 * The root of the tree has a node of length 0.
 * We can define the voltage from  0<x<1 and get it right even
 * at 0 and 1
 * The last node of each section is has no length
 * good start on calculating matrix coefficients
 * 
 * Revision 1.7  89/08/04  14:48:04  mlh
 * 2nd order correct in dx handled pretty well with nnode-1 segments
 * and last node being at the end of the section with 0 length.
 * Problem with connecting to position 0 since first node
 * not at beginning of section.
 * 
 * Revision 1.4  89/07/27  16:14:46  mlh
 * at point where topology() works
 * 
 * Revision 1.3  89/07/21  17:38:34  mlh
 * compiles
 * 
 * Revision 1.1  89/07/20  07:52:49  mlh
 * Initial revision
 * 
*/

/* solve.c 15-Dec-88 */

/* The data structures in section.h were developed primarily for the needs of
     solving and setting up the matrix equations reasonably efficiently in
     space and time. I am hypothesizing that they will also be convenient
     for accessing parameters.
     Important properties of the structures used here are:
	Each section must have at least one node. There may be 0 sections.
	The order for back substitution is given by section[i]->order.
	The order of last node to first node is used in triangularization.
	First node to last is used in back substitution.
*/
/* An equation is associated with each node. d and rhs are the diagonal and
   right hand side respectively.  a is the effect of this node on the parent
   node's equation.  b is the effect of the parent node on this node's
   equation.
   d is assumed to be non-zero.
*/

/* We have seen that it is best to have nodes generally denote the
   centers of segments because most properties are most easily defined
   at those points. The old problem then arises again of 2nd order correct
   demands that a point be exactly at any branches. For this reason we
   always allocate one extra node at the end with 0 length.  Sections
   that connect at x=1 connect to this node.  Sections that connect
   from 0<x<1 connect to nodes 0 to nnode-2 directly to the point
   (no parent resistance, only a half segment resistance). It was an error
   to try to connect a section to position 0.  Now we are going to allow
   that by the following artifice.
   
   Section 0 is always special. Its nodes act as roots to the independent
   trees.  They do not connect to each other. It has no properties.
*/

#if EXTRACELLULAR
/* Two users (Padjen and Shrager) require that the extracellular potential
be simulated.  To do this we introduce a new node structure into the
section called extnode.  This points to a vector of extnodes containing
the extracellular potential, the Node vector
is interpreted as the internal potential. Thus the membrane potential is
node[i].v - extnode[i].v
*/
/*
With vectorization, node.v is the membrane potential and node.rhs
after solving is the internal potential. Thus internal potential is
node.v + extnode.v
*/

#endif

#include	<stdio.h>
#include	"section.h"
#include	"membdef.h"

extern char	*secname();
extern void	prop_free();
extern int	tree_changed;
extern double nrn_connection_position();
static void node_free();

/* used for vectorization and distance calculations */
int section_count;
Section** secorder;

/* nrn_solve() solves the matrix equations represented in "section"
   rhs is replaced by the solution (delta v's).
   d is destroyed.
*/
/* Section *sec_alloc(nsec) allocates a vector of nsec sections and returns a
   pointer to the first one in the list. No nodes are allocated.
   The usage is normally "section = sec_alloc(nsection)"
   After allocation one must allocate nodes for each section with
   node_alloc(sec, n).
   After all this connect sections together using section indices.
   Finally order the sections with section_order(section, nsec).
   When the section vector is no longer needed (and before creating another
   one) free the space with sec_free(section, nsection);
*/

#if DEBUGSOLVE
double
debugsolve()	/* returns solution error */
{
#if !EXTRACELLULAR
	void triang(), bksub();
#endif
	short inode;
	Section *sec, *psec, *ch;
	Node *nd, *pnd;
	double err, sum, fabs();

	/* save parts of matrix that will be destroyed */
assert(0)
	/* need to save the rootnodes too */
	ForAllSections(sec)
		assert(sec->node && sec->nnode);
		for (inode = sec->nnode-1; inode >= 0; inode--) {
			nd = sec->node + inode;
			nd->savd = nd->d;
			nd->savrhs = nd->rhs;
		}
#if EXTRACELLULAR
		if (sec->node->extnode) for (inode = sec->nnode-1; inode >= 0; inode--) {
			Extnode *nd;
			nd = sec->node[inode].extnode;
			nd->savd = nd->d;
			nd->savrhs = nd->rhs;
			nd->savxain = nd->xain;
			nd->savxbout = nd->xbout;
		}
#endif
	}

#if EXTRACELLULAR
	nrn_triang_2d();
	nrn_bksub_2d();
#else
	triang();
	bksub();
#endif

	err = 0.;
/* need to check the rootnodes too */
	ForAllSections(sec)
		for (inode = sec->nnode-1; inode >= 0; inode--) {
			nd = sec->node + inode;
			/* a single internal current equation */
			sum = nd->savd * nd->rhs;
			if (inode > 0) {
				sum += nd->b * (nd - 1)->rhs;
			}else{
				pnd = sec->parentnode;
				sum += nd->b * pnd->rhs;
			}
			if (inode < sec->nnode-1) {
				sum += (nd + 1)->a * (nd +1)->rhs;
			}
			for (ch = nd->child; ch; ch = ch->sibling){
				psec = ch;
				pnd = psec->node;
				assert(pnd && psec->nnode);
				sum += pnd->a * pnd->rhs;
			}
			sum -= nd->savrhs;
#if EXTRACELLULAR
if (sec->extnode) {
			Node *ndin = sec->node + inode;
			Extnode *pnd, *nd = sec->node[inode].extnode;
			sum += nd->savxain * nd->rhs;
			err += fabs(sum);
if(fabs(sum) > 1.e-10) {
	fprintf(stderr, "internal current in node=%d of section %s is %g\n",inode, secname(sec), sum);
}
			/* end of internal current equation */

			/* begin external current equation */
			sum = nd->savd * nd->rhs  +  nd->savxbout * ndin->rhs;
			if (inode > 0) {
				sum += nd->b * (nd - 1)->rhs;
			} else {
				if (sec->parentnode->extnode) {
					pnd = sec->parentnode->extnode;
					sum += nd->b * pnd->rhs;
				}
			}
			if (inode < sec->nnode-1) {
				sum += (nd + 1)->a * (nd +1)->rhs;
			}
			for (ch = ndin->child; ch; ch = ch->sibling){
				psec = ch;
				pnd = psec->node->extnode;
				if (pnd) {
					sum += pnd->a * pnd->rhs;
				}
			}
			sum -= nd->savrhs;
if(fabs(sum) > 1.e-10) {
	fprintf(stderr, "external current in node=%d of section %s is %g\n",inode, secname(sec), sum);
}
}
#endif /*EXTRACELLULAR*/
			err += fabs(sum);
		}
	}

	return err;
}	
#endif /*DEBUGSOLVE*/


double
node_dist(sec, node)
	Section *sec;
	Node* node;
{
	int inode;
	double ratio;
	
	if (!sec || sec->parentnode == node) {
		return 0.;
	}else if ((inode = node - sec->node) == sec->nnode-1) {
		ratio = 1.;
	}else{
		ratio = ((double)inode+.5)/((double)sec->nnode - 1.);
	}
	return  sec->prop->dparam[2].val*ratio;
}

double
topol_distance(sec1, node1, sec2, node2, prootsec, prootnode)
	Section *sec1, *sec2, **prootsec;
	Node* node1, *node2, **prootnode;
{	/* returns the distance between the two nodes
		Ie the sum of the appropriate length portions
		of those sections connecting these two
		nodes.
	*/
	double d, x1, x2;

	d = 0;
	v_setup_vectors();
	/* keep moving toward a common node */
	while (sec1 != sec2) {
		if (!sec1) {
			d += node_dist(sec2, node2);
			node2 = sec2->parentnode;
			sec2 = sec2->parentsec;
		}else if (!sec2){
			d += node_dist(sec1, node1);
			node1 = sec1->parentnode;
			sec1 = sec1->parentsec;
		}else if (sec1->order > sec2->order) {
			d += node_dist(sec1, node1);
			node1 = sec1->parentnode;
			sec1 = sec1->parentsec;
		}else{
			d += node_dist(sec2, node2);
			node2 = sec2->parentnode;
			sec2 = sec2->parentsec;
		}
	}
	if (!sec1) {
		if (node1 != node2) {
			sec1 = 0;
			d = 1e20;
			node1 = (Node*)0;
		}
	} else if (node1 != node2) {
		x1 = node_dist(sec1, node1);
		x2 = node_dist(sec2, node2);
		if (x1 < x2) {
			d += x2 - x1;
		}else{
			node1 = node2;
			d += x1 - x2;
		}
	}
	*prootsec = sec1;
	*prootnode = node1;
	return d;
}

static Section *origin_sec;

int distance() {
	double d, chkarg();
	int mode;
	Node* node, *node_exact();
	Section *sec;	
	static Node* origin_node;
	
	v_setup_vectors();
			
	sec = chk_access();
	if (ifarg(2)) {
		d = chkarg(2, 0., 1.);
		mode = (int) chkarg(1, 0., 1.);
	}else if (ifarg(1)) {
		d = chkarg(1, 0., 1.);
		mode = 1;
	}else{
		d = 0.;
		mode = 0;
	}
	node = node_exact(sec, d);
	if (mode == 0) {
		origin_node = node;
		origin_sec = sec;
	}else{
		if (!origin_sec) {
hoc_execerror("tree has changed.","Need to initialize origin with distance()");
		}
		d = topol_distance(origin_sec, origin_node, sec, node,
			&sec, &node );
	}
	ret(d);
}
	
int
topology() /* print the topology of the branched cable */
{
	hoc_Item* q;
	extern hoc_List* rootsection_list;

	v_setup_vectors();
	ITERATE(q, rootsection_list) {
		Printf("|");
		dashes((Section*)VOIDITM(q), 0, '-');
	}
	Printf("\n");
	ret(1.);
}

dashes(sec, offset,first)
	Section *sec;
	int offset, first;
{
	int i, scnt;
	Section* ch;
	
	for (i=0; i<offset; i++) Printf(" ");
	Printf("%c", first);
	for (i=2; i<sec->nnode; i++) Printf("-");
	if (sec->prop->dparam[4].val == 1) {
		Printf("|       %s\n", secname(sec));
	}else{
		Printf("|       %s with %g rall branches\n",
			secname(sec), sec->prop->dparam[4].val);
	}
	/* navigate the sibling list backwards */
	/* note that the sibling list is organized monotonically by
	  increasing distance from parent */
	for (scnt=0, ch=sec->child; ch; ++scnt, ch = ch->sibling) {
		hoc_pushobj((Object**)ch);
	}
	while(scnt--) {		
		Object** hoc_objpop();
		ch = (Section*)hoc_objpop();
		i = node_index_exact(sec, nrn_connection_position(ch));
		Printf(" ");
		dashes(ch, i+offset+1, 0140); /* the ` char*/
	}
}

/* solve the matrix equation */
void
nrn_solve() {
	void triang(), bksub();

#if DEBUGSOLVE
	double err;
	err = debugsolve();
	if (err > 1.e-10) {
		Fprintf(stderr, "solve error = %g\n", err);
	}
#else
#if EXTRACELLULAR
	nrn_triang_2d();
	nrn_bksub_2d();
#else
	triang();
	bksub();
#endif
#endif
}

#if VECTORIZE && _CRAY
extern Node*** v_node_depth_lists;
extern Node*** v_parent_depth_lists; /* parents must be unique in each list */
extern int* v_node_depth_count;
extern int v_node_depth; /* so depth may be more than twice what you'd expect */
#endif

#if !EXTRACELLULAR
/* triangularization of the matrix equations */
void
triang()
{
	register Node *nd, *pnd;
	double p;
#if VECTORIZE
#if _CRAY
	int i, j;
#else
	int i;
#endif
#else
	short isec, inode;
	Section *sec, *psec;
#endif

#if VECTORIZE
#if _CRAY
	for (j = v_node_depth - 1; j > 0; --j) {
		int count = v_node_depth_count[j];
		Node** nodes = v_node_depth_lists[j];
		Node** parents = v_parent_depth_lists[j];
		/* next loop can be done in parallel */
#pragma _CRI ivdep
		for ( i = 0; i < count; ++i) {
			nd = nodes[i];
			pnd = parents[i];
			p = nd->a / nd->d;
			pnd->d -= p * nd->b;
			pnd->rhs -= p * nd->rhs;
		}
	}
#else
	for (i = v_node_count - 1; i >= rootnodecount; --i) {
		nd = v_node[i];
		pnd = v_parent[i];
		p = nd->a / nd->d;
		pnd->d -= p * nd->b;
		pnd->rhs -= p * nd->rhs;
	}
#endif
#else /* ! VECTORIZE */
	for (isec = nsection - 1; isec > 0; isec--) {
		sec = section + section[isec].order;
#if EXTRACELLULAR
		if (sec->extnode) {
			triang_sec_2d(sec);
			continue;
		}
#endif
		for (inode = sec->nnode-1; inode > 0; inode--) {
			nd = sec->node + inode;
#if EXTRAEQN
{
			Eqnblock *e;
			for (e = nd->eqnblock; e; e = e->eqnblock_next) {
				(e->eqnblock_triang)(nd, e);
			}
}
#endif /*EXTRAEQN*/
			p = nd->a / nd->d;
			pnd = nd - 1;
			pnd->d -= p * nd->b;
			pnd->rhs -= p * nd->rhs;
		}
		/* if a and b == 0 then this section is not connected to its parent
		   and parentsec=parentnode=0 is adequate */
		/* here we assume each section has at least one node */
		nd = sec->node;
		assert(nd);
#if EXTRAEQN
{
			Eqnblock *e;
			for (e = nd->eqnblock; e; e = e->eqnblock_next) {
				(e->eqnblock_triang)(nd, e);
			}
}
#endif /*EXTRAEQN*/
		p = nd->a / nd->d;
		psec = section + sec->parentsec;
		pnd = psec->node + sec->parentnode;
		pnd->d -= p * nd->b;
		pnd->rhs -= p * nd->rhs;
	}
#if 0	/* section 0 nodes do not connect to each other */
	sec = section + section[0].order;
	for (inode = sec->nnode-1; inode > 0; inode--) {
		nd = sec->node + inode;
		p = nd->a / nd->d;
		pnd = nd - 1;
		pnd->d -= p * nd->b;
		pnd->rhs -= p * nd->rhs;
	}
#endif
#endif /* VECTORIZE */
}

/* back substitution to finish solving the matrix equations */
void
bksub()
{
	register Node *nd, *cnd;
#if VECTORIZE
#if _CRAY
	Node** nodes;
	Node** parents;
	int i, j, count;
#else
	int i;
#endif
#else
	short isec, inode, ch;
	Section *sec, *csec;
#endif

#if VECTORIZE
#if _CRAY
	count = v_node_depth_count[0];
	nodes = v_node_depth_lists[0];
	/* following loop can be done in parallel */
#pragma _CRI ivdep
	for (i=0; i < count; ++i) { /* roots of unconnected cells */
		nodes[i]->rhs /= nodes[i]->d;
	}
	for (j = 1; j <  v_node_depth; ++j) {
		count = v_node_depth_count[j];
		nodes = v_node_depth_lists[j];
		parents = v_parent_depth_lists[j];
		/* note that since parent is read only don't require the depth
			needed for triangularization */
		/* next loop can be done in parallel */
#pragma _CRI ivdep
		for (i=0; i < count; ++i) {
			cnd = nodes[i];
			nd = parents[i];
			cnd->rhs -= cnd->b * nd->rhs;
			cnd->rhs /= cnd->d;
		}
	}
#else
	for (i = 0; i < rootnodecount; ++i) {
		v_node[i]->rhs /= v_node[i]->d;
	}
	for (i = rootnodecount; i < v_node_count; ++i) {
		cnd = v_node[i];
		nd = v_parent[i];
		cnd->rhs -= cnd->b * nd->rhs;
		cnd->rhs /= cnd->d;
	}	

#endif
#endif /*Vectorize*/
}

#endif /* ! EXTRACELLULAR */

nrn_clear_mark() {
	hoc_Item* qsec;
	ForAllSections(sec)
		sec->volatile_mark = 0;
	}
}
short nrn_increment_mark(sec) Section* sec; { return sec->volatile_mark++;}
short nrn_value_mark(sec) Section* sec; { return sec->volatile_mark;}

/* allocate space for sections (but no nodes) */
Section *		/* returns pointer to Section */
sec_alloc()
{
	Section  *sec;

	sec = (Section *)emalloc(sizeof(Section));
	sec->refcount = 0;
	sec->nnode = 0;
	sec->parentsec = sec->sibling = sec->child = (Section*)0;
	sec->parentnode = (Node*)0;
	sec->node = (Node *)0;
#if DIAMLIST
	sec->npt3d = 0;
	sec->pt3d = (Pt3d *)0;
#endif
	sec->prop = (Prop *)0;

	return sec;
}

/* free everything about sections */
void
sec_free(secitem)
	hoc_Item* secitem;
{
	Section *sec;
	
	if (!secitem) {
		return;
	}
	sec = SEC(secitem);
	assert(sec);
/*printf("sec_free %s\n", secname(sec));*/
	section_unlink(sec);
	hoc_l_delete(secitem);
	prop_free(&(sec->prop));
	node_free(sec);
	if (!sec->parentsec && sec->parentnode) {
		nrn_rootnode_free(sec->parentnode, sec);
	}
#if DIAMLIST
	if (sec->pt3d) {
		free((char *)sec->pt3d);
		sec->pt3d = (Pt3d*)0;
		sec->npt3d = 0;
	}
#endif
	section_unref(sec);
}


/* can't actually release the space till the refcount goes to 0 */
section_unref(sec)
	Section* sec;
{
/*printf("section_unref %lx %d\n", (long)sec, sec->refcount-1);*/
	if (--sec->refcount <= 0) {
#if 0
printf("section_unref: freed\n");
#endif
		assert (!sec->parentsec);
		free((char*)sec);
	}
}
section_ref(sec)
	Section* sec;
{
/*printf("section_ref %lx %d\n", (long)sec,sec->refcount+1);*/
	++sec->refcount;
}

nrn_sec_ref(psec, sec)
	Section** psec;
	Section* sec;
{
	Section* s = *psec;
	if (sec) {
		section_ref(sec);
	}
	*psec = sec;
	if (s) {
		section_unref(s);
	}
}

section_unlink(sec) /* other sections no longer reference this one */
	Section* sec;
{
	/* only sections that are explicitly connected to this are disconnected */
	Section* child;
	tree_changed = 1;
	/* disconnect the sections connected to this at the parent end */
	for (child = sec->child; child; child = child->sibling) {
		nrn_disconnect(child);
	}
	nrn_disconnect(sec);
}

Node*
node_construct(n)
int n;
{
	Node* nd, *node;
	int i;
	
	node = (Node *)ecalloc((unsigned)n, sizeof(Node));
	for (i = n - 1; i >= 0; i--) {
		nd = node + i;
		nd->prop = (Prop *)0;
		nd->v = DEF_vrest;
#if EXTRACELLULAR
		nd->extnode = (Extnode*)0;
#endif
#if EXTRAEQN
		nd->eqnblock = (Eqnblock *)0;
#endif
	}
	return node;
}

node_destruct(node, n)
	Node* node;
	int n;
{
	int i;
	Node* nd;
	
	for (i = n - 1; i >= 0; i--) {
		nd = node + i;
		prop_free(&(nd->prop));
		notify_freed((void*)&nd->v, 6);
#if EXTRACELLULAR
		if (nd->extnode) {
			notify_freed((void*)&nd->extnode->v, 9);
		}
#endif
#if EXTRAEQN
{
		Eqnblock *e, *e1;
		for (e = nd->eqnblock; e; e = e1) {
			e1 = e->eqnblock_next;
			free((char *)e);
		}
}
#endif
	}
#if EXTRACELLULAR
	if (node->extnode) {
		/* was allocated as a vector */
		free((char *)node->extnode);
	}
#endif
	free((char *)node);
}

/* allocate node vectors for a section list */
void
node_alloc(sec, nseg)
	Section *sec;
	short nseg;
{
	node_free(sec);
	if (nseg == 0) {
		return;
	}
	sec->node = node_construct(nseg);
	sec->nnode = nseg;
}

/* free a node vector for one section */
static void
node_free(sec)
	Section *sec;
{
	Node *nd;

	nd = sec->node;
	if (!nd) {
		sec->nnode = 0;
	}
	if (sec->nnode == 0) return;
	node_destruct(sec->node, sec->nnode);
	sec->node = (Node *)0;
	sec->nnode = 0;
}

void
section_order() /* create a section order consistent */
		/* with connection info */
{
	int order, isec;
	Section* ch;
	Section *sec;
	hoc_Item* qsec;

	/* count the sections */
	section_count = 0;
	/*SUPPRESS 765*/
	ForAllSections(sec)
		sec->order = -1;
		++section_count;
	}
	
	origin_sec = (Section *)0;
	if (secorder) {
		free((char *)secorder);
	}
	secorder = (Section**)emalloc(section_count*sizeof(Section*));
	order = 0;
	ForAllSections(sec) /* all the roots first */
		if (!sec->parentsec) {
			secorder[order] = sec;
			sec->order = order;
			++order;
		}
	}

	for (isec=0; isec<section_count; isec++) {
		if (isec >= order) { /* there is a loop */
			ForAllSections(sec)
				Section* psec, *s = sec;
				for (psec = sec->parentsec; psec; s = psec, psec = psec->parentsec) {
					if (!psec || s->order >= 0) {
						break;
					}else if (psec == sec) {
fprintf(stderr, "A loop exists consisting of:\n %s", secname(sec));
for (s = sec->parentsec; s != sec; s = s->parentsec) {
	fprintf(stderr, " %s", secname(s));
}
fprintf(stderr, " %s\nUse <section> disconnect() to break the loop\n ", secname(sec));
hoc_execerror("A loop exists involving section", secname(sec));
					}
				}
			}
		}
		sec = secorder[isec];
		for (ch = sec->child; ch; ch = ch->sibling) {
			secorder[order] = ch;
			ch->order = order;
			++order;
		}
	}
	assert(order == section_count);
}
