/* $Header: /timshel2g/src/master/nrn/src/nrnoc/synapse.c,v 1.1.1.1 1994/10/12 17:22:35 hines Exp $ */
/*
$Log: synapse.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:35  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.48  1993/10/18  13:37:32  hines
 * very minor fix to synapse.c and fadvance.c for portability
 *
 * Revision 4.44  93/08/15  12:56:06  hines
 * fix warning of function declared external and later static
 * y
 * 
 * Revision 4.9  93/01/21  12:21:16  hines
 * watch out for freed sections in old style fstim, fsyn, and fclamp
 * correct memory leakage on access section
 * 
 * Revision 3.23  92/10/09  09:13:54  hines
 * clean up some warnings and bugs. sectionname was broken and
 * sometimes fstim and fsyn were used when they hadn't gotten a section
 * pointer.
 * 
 * Revision 2.75  92/08/07  16:12:08  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.64  92/04/13  15:41:37  hines
 * fadvance now calls treeset with t=t+dt/2. This means that models wich depend
 * explicitly on time are second order correct when secondorder=2
 * 
 * Revision 2.42  91/08/22  16:58:25  hines
 * for distribution to work on RS6000
 * 
 * Revision 1.84  90/10/24  10:56:48  hines
 * saber warnings gone
 * 
 * Revision 1.76  90/08/09  16:44:06  hines
 * make synapse second order correct by calculating alpha at midpoint of
 * time step.
 * 
 * Revision 1.59  90/03/15  16:41:07  mlh
 * error in handling pstim[] for multiple stimuli in return of current
 * and conductance (fsyng) function.
 * 
 * Revision 1.52  90/01/24  07:05:12  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.42  89/10/13  09:01:21  mlh
 * free the clamp, synapse, and stim when new sections created
 * de-lint some minor things
 * 
 * Revision 1.39  89/09/25  16:07:13  mlh
 * print point current info with prstim()
 * 
 * Revision 1.30  89/09/06  10:26:10  mlh
 * newcable to neuron
 * 
 * Revision 1.18  89/08/12  14:49:12  mlh
 * Lint free
 * 
 * Revision 1.14  89/08/10  16:23:36  mlh
 * not really tested
 * 
 * Revision 1.1  89/08/10  15:36:46  mlh
 * Initial revision
 * 
*/


/* modified from fstim.c */
/*
fsyn(maxnum)
 allocates space for maxnum synapses. Space for
 previously existing synapses is released. All synapses initialized to
 0 maximum conductance.

fsyn(i, loc, delay, tau, gmax, erev)
 The ith synapse is injected at parameter `loc'
 different synapses do not concatenate but can ride on top of
 each other. delay refers to onset of synapse relative to t=0
 delay and duration are in msec.
 stim in namps.

 a synaptic current defined by
 	i = g * (v - erev) 	i(nanoamps), g(micromhos);
 	where
 	 g = 0 for t < delay and
 	 g = gmax * (t - delay)/tau * exp(-(t - delay - tau)/tau)
 	  for t > onset
 this has the property that the maximum value is gmax and occurs at
  t = delay + tau.
  
fsyni(i)
 returns synaptic current for ith synapse at the value of the
 global time t in units of nanoamps.

fsyng(i)
 returns synaptic conductance for ith synapse at the value of the
 global time t.

*/

#include "neuron.h"
#include "section.h"

/* impress the stimulus code to do synapses */
typedef struct Stimulus {
	double loc;	/* parameter location (0--1) */
	double delay;	/* value of t in msec for onset */
	double duration;/* turns off at t = delay + duration */
	double mag;	/* conductance in micromhos */
	double erev;
	double mag_seg;	/* value added to rhs, depends on area of seg*/
	Node *pnd;	/* segment location */
	Section* sec;
} Stimulus;

static maxstim = 0;		/* size of stimulus array */
static Stimulus *pstim;		/* pointer to stimulus array */

extern double *getarg(), chkarg();
extern char *secname();

print_syn() {
	int i;
	
	if (maxstim == 0) return;
	/*SUPPRESS 440*/
	Printf("fsyn(%d)\n/* section	fsyn( #, loc, delay(ms), tau(ms), conduct(umho), erev(mV)) */\n", maxstim);
	for (i=0; i<maxstim; i++) {
		Printf("%-15s fsyn(%2d,%4g,%10g,%8g,%14g,%9g)\n",
		secname(pstim[i].sec), i,
		pstim[i].loc, pstim[i].delay, pstim[i].duration, pstim[i].mag,
		pstim[i].erev);
	}
}

static double stimulus();

fsyni() {
	int i;
	double cur, g;
	
	i = chkarg(1, 0., (double)(maxstim-1));
	if ((cur = stimulus(i, &g)) != 0.) {
		cur *= pstim[i].mag / pstim[i].mag_seg;
	}
	ret(cur);
}

fsyng() {
	int i;
	double g;
	
	i = chkarg(1, 0., (double)(maxstim-1));
	IGNORE(stimulus(i, &g));
	if (g != 0.) {
		g *= pstim[i].mag / pstim[i].mag_seg;
	}
	ret(g);
}

static stim_record();

fsyn() {
	int i;

	i = chkarg(1, 0., 10000.);
	if (ifarg(2)) {
		if (i >= maxstim) {
			hoc_execerror("index out of range", (char *)0);
		}
		pstim[i].loc = chkarg(2, 0., 1.);
		pstim[i].delay = chkarg(3, 0., 1e21);
		pstim[i].duration = chkarg(4, 0., 1e21);
		pstim[i].mag = *getarg(5);
		pstim[i].erev = *getarg(6);
		pstim[i].sec = chk_access();
		section_ref(pstim[i].sec);
		stim_record(i);
	} else {
		free_syn();
		maxstim = i;
		if (maxstim) {
			pstim = (Stimulus *)emalloc((unsigned)(maxstim * sizeof(Stimulus)));
		}
		for (i = 0; i<maxstim; i++) {
			pstim[i].loc = 0;
			pstim[i].mag = 0.;
			pstim[i].delay = 1e20;
			pstim[i].duration = 0.;
			pstim[i].erev = 0.;
			pstim[i].sec = 0;
			stim_record(i);
		}
	}
	ret(0.);
}

free_syn() {
	int i;
	if (maxstim) {
		for (i=0; i < maxstim; ++i) {
			if (pstim[i].sec) {
				section_unref(pstim[i].sec);
			}
		}
		free((char *)pstim);
		maxstim = 0;
	}
}

static
stim_record(i)	/*fill in the section info*/
	int i;
{
	Node *node_ptr();
	double area;
	Section* sec;

	sec = pstim[i].sec;
	if (sec) {
		if (sec->prop) {
			pstim[i].pnd = node_ptr(sec, pstim[i].loc, &area);
			pstim[i].mag_seg = 1.e2*pstim[i].mag / area;
		}else{
			section_unref(sec);
			pstim[i].sec = 0;
		}
	}
}

synapse_prepare() {
	int i;
	
	for (i=0; i<maxstim; i++) {
		stim_record(i);
	}
}

static double alpha();

static double
stimulus(i, pg)
	int i;
	double *pg;
{
	double x, g;
	
	if ((g = pstim[i].mag_seg) == 0.0) {
		*pg = 0.0;
		return 0.0;
	}
	x = (t - pstim[i].delay)/pstim[i].duration;
	*pg = g * alpha(x);
	return *pg * (pstim[i].pnd->v - pstim[i].erev);
}

static double
alpha(x)
	double x;
{
	double exp();
	
	if (x > 0.0 && x < 10.0) {
		return x * exp(-x + 1.0);
	}
	return 0.0;
}

activsynapse() {
	
	int i;
	double g, cur;

	for (i=0; i<maxstim; i++) {
		if (pstim[i].sec) {
			cur = stimulus(i, &g);
			pstim[i].pnd->rhs += g * pstim[i].pnd->v  -  cur;
			pstim[i].pnd->d += g;
		}
	}
}
