/* $Header: /timshel2g/src/master/nrn/src/nrnoc/treeset.c,v 1.1.1.1 1994/10/12 17:22:35 hines Exp $ */
/*
$Log: treeset.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:35  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.77  1994/09/26  18:26:47  hines
 * ions more nearly do the "right thing"
 * presence of mechanism that WRITE's a concentration computes
 * the reversal potential every fadvance and finitialize, initializes
 * the concentrations to global variables, makes the concentrations have
 * the type STATE and the reversal potential the style DEP
 * presence of mech that READ's a concentration computes reversal
 * every finitialize, and treats concentration as CONST and reversal
 * potential style DEP.
 * If no mechanism reads or writes concentrations the reversal is
 * treated as CONST.
 *
 * Revision 4.70  1994/07/08  20:42:47  hines
 * properly handle freeing of point process property. solves several
 * kinds of segmentation violation, reading freed memory, and memory leakage.
 *
 * Revision 4.65  1994/06/29  20:46:44  hines
 * allow easier coverage of treeset.c and benign umr of capac.c
 *
 * Revision 4.64  1994/06/29  13:24:34  hines
 * under unusual circumstances sections could be unreffed and never freed. fixed
 * benign uninitialized memory read of param_size when prop is freed before
 * memory is allocated for param. fixed
 *
 * Revision 4.60  1994/06/07  18:43:45  hines
 * trivial objectcenter warning
 *
 * Revision 4.56  1994/05/02  12:56:19  hines
 * don't print error message for mechanism not inserted if hoe_execerror_messages is false
 * nrn_complain(double* _p) pirnts location for this param on the standard error
 * device
 *
 * Revision 4.46  1993/08/20  13:01:42  hines
 * give message about a.a - a.a which is parsed as a.a {-a.a}
 * fix some problems regarding define_shape
 * allow change of L and diam when 3dpoints exist (pt3dconst(0) is default)
 *
 * Revision 4.36  93/07/08  14:34:21  hines
 * a start on fixing the bug of electode point processes not working
 * correctly in presence of extracellular. At this time the functionality
 * is identical as before but passing node to  models in nrn_cur instead
 * of passing pointer to nd->d and returning rhs.
 * There is an alternative now to NONSPECIFIC_CURRENT which is
 * ELECTRODE_CURRENT
 * 
 * Revision 4.33  93/05/31  15:35:06  hines
 * for (ch=sec->child; ch; ch=ch->sibling) is loop over all sections connect
 * directly to sec in order of increasing distance from connection end of sec.
 * there is no longer a Node->child pointer.
 * 
 * Revision 4.26  93/04/21  10:33:21  hines
 * point processes located at ends of sections stay in those sections when
 * those sections are disconnected from their parent.
 * 
 * Revision 4.25  93/04/20  08:58:54  hines
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * There is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 * 
 * Revision 4.20  93/03/21  17:51:20  hines
 * new APCount() point process which is useful for determining occurence and
 * time of single action potential
 * 
 * node_data("filenam") prints minimal complete state (but not global variables)
 * for use with a minimal neuron system without the interpreter
 * 
 * Revision 4.18  93/03/05  15:58:57  hines
 * switch from hoc_free_val to hoc_freed
 * 
 * Revision 4.17  93/02/25  13:26:59  hines
 * need_memb takes symbol arg now instead of name. This avoid hoc_lookup.
 * 
 * Revision 4.13  93/02/02  10:30:42  hines
 * static functions declared before used
 * 
 * Revision 4.12  93/01/27  13:54:39  hines
 * forgot to checkin treeset.c after making extracellular work again
 * 
 * Revision 4.5  93/01/07  08:15:01  hines
 * psection prints Ra
 * warning if Ra set but not in every section
 * fix batch_run so not extra iteration on cray (tstop -=dt/4)
 * 
 * Revision 4.4  93/01/06  09:44:46  hines
 * minor adjustments for cray c90
 * 
 * Revision 4.2  93/01/05  11:53:54  hines
 * Fix bug in _CRAY portion where writing to same place at same time
 * 
 * Revision 3.45  92/12/18  15:17:20  hines
 * for _CRAY allow multiple instances of a point process in same node.
 * 
 * Revision 3.44  92/12/17  08:28:55  hines
 * delete_section() removes currently accessed section from the list of
 * sections to be simulated and releases its property and nodes.
 * 
 * Revision 3.43  92/12/16  14:24:37  hines
 * Ra switched from global variable to section variable in which it can
 * take on different values in different sections. Similar to L.
 * No need to set diam_changed when Ra changed.
 * 
 * Revision 3.39  92/11/16  11:23:28  hines
 * double nrn_diameter(Node*)
 * nrn_define_shape uses diam and L and parent position to define
 * two pt3d points so we can see shape in nrniv
 * 
 * Revision 3.35  92/10/27  12:09:55  hines
 * list.c list.h moved from nrnoc  to oc
 * 
 * Revision 3.33  92/10/24  16:30:25  hines
 * freeing vars notifies interviews.
 * sec_and_position(sec, inode) returns string in form section(x)
 * some portability fixes from pittsburgh
 * 
 * Revision 3.22  92/10/09  07:46:34  hines
 * replace vector style point process syntax with objectvar style.
 * POINTER not yet implemented and old syntax not yet removed from parser.
 * usage:
 * objectvar s
 * s = PulseStim(.5)
 * s.del = 5
 * relocate with
 * s.loc(x)
 * x = s.getloc() pushes section on section stack and must be removed with
 * popsec()
 * 
 * Revision 3.11  92/09/24  17:03:52  hines
 * METHOD3 option. use with spatial_method(i) with i=0-3
 * 0 is the default and is the normal neuron method with zero area nodes
 * at boundaries and branch points.
 * 1 is the standard method with half area nodes at boundaries
 * 2 is a modified second order method which weights by 2/12, 8/12, 2/12
 * 3 is a third order correct method which weights by 1/12 10/12 1/12
 * with the last three methods x = i/nnode with i=0 to nnode
 * methods 1 and 2 are at present quite inefficient since they use the
 * data structures of 3.
 * Works in steady state. not tested with dynamic simulations
 * At this time, discontinuities in parameters (except diameter) is
 * not handled well across nodes and all parameters are assumed to
 * be constant in interval between node and parent node.
 * Good deal of difficulty remains in managing values that change discontinuously
 * 
 * Revision 3.5  92/08/20  14:39:12  hines
 * finitialize checks diam_changed. recalc_diam sets diam_changed to 0
 * and call functions that depend on diam.
 * 
 * Revision 3.1  92/08/13  16:31:18  hines
 * merge sejnowski contributions of synapse and parameter dump. All files
 * present with sej_ prefix. invoke by defineing Sejnowski in Imakefile
 * Unfortunately much of the implementation needs to be changed to conform
 * to new section list. Problem is that much use was made of section indices.
 * 
 * Revision 2.85  92/08/12  16:19:36  hines
 * uninsert mechanism_name
 * do not uninsert an ion unless no other mechanism in the section uses the
 * ion. Invalid memory references will result.
 * 
 * Revision 2.75  92/08/07  16:12:10  hines
 * sections as objects. nsection and section gone. now list of sections
 * in nmodl list style in its place. vectorization no longer optional
 * probably many bugs at this point but all examples run correctly.
 * Need tests with complete code coverage.
 * 
 * Revision 2.73  92/08/05  10:29:00  hines
 * got rid of objectcenter messages
 * 
 * Revision 2.71  92/08/05  09:27:21  hines
 * VECTORIZE, NTS_SPINE, EXTRAEQN merged from neuron
 * 
 * Revision 2.46  91/09/23  09:19:26  hines
 * allow a diam3d value of 0. Will increment axial resistance by 1e10 Mohm.
 * There will still be an error if area of segment turns out to be 0.
 * 
 * Revision 2.32  91/08/08  16:41:11  hines
 * saber lint free. can make saber_src_all which is all the src pertaining
 * to neuron
 * 
 * Revision 2.31  91/08/08  14:58:34  hines
 * 3-D information attached to sections for simulation of reconstructed
 * neurons.  This information is not discarded when nseg is changed.
 * 
 * Revision 2.24  91/06/20  17:27:40  hines
 * diamlst() --- trapezoidal integration of diam, area, ri.
 * second order effect of taper on area computed correctly.
 * 
 * Revision 2.23  91/05/31  15:57:54  hines
 * DIAMLIST
 * 
 * Revision 2.7  90/12/14  09:48:32  hines
 * First pass at handling point processes.
 * 
 * Revision 1.87  90/10/29  12:31:37  hines
 * main parameter list for a property is now a double vector. This makes
 * it easier to interface model descriptions using nmodl since only a pointer
 * change is needed to switch between segments.  cases such as ena, ina,
 * and section names are handled as vector of Datum as before but in
 * the dparam record.
 * 
 * Revision 1.84  90/10/24  10:56:49  hines
 * saber warnings gone
 * 
 * Revision 1.77  90/10/09  08:01:54  hines
 * extracellular axial resistance now Mohms/cm  axial capacitance
 * not working
 * 
 * Revision 1.54  90/01/24  12:21:41  mlh
 * make prop_alloc() work correctly when we run out of memory
 * 
 * Revision 1.52  90/01/24  07:05:16  mlh
 * emalloc() and ecalloc() are macros which first return a null and then
 * do execerror if out of space.
 * 
 * Revision 1.48  90/01/16  07:05:51  mlh
 * extracellular membrane type: tested up to single section
 * 
 * Revision 1.37  89/09/20  12:02:37  mlh
 * ionic reversal potentials and currents now in Eion
 * other mechanisms, such as HH, use and modify Eion parameters.
 * a mechanism allocation routine can "need" another mechanism and
 * communicate via Datum.pval
 * 
 * Revision 1.30  89/09/06  10:26:13  mlh
 * newcable to neuron
 * 
 * Revision 1.23  89/08/29  15:10:15  mlh
 * bug in turboc with respect to incrementing section and node pointers.
 * explicitly iterate over indices
 * 
 * Revision 1.21  89/08/24  09:21:47  mlh
 * New channel additions localized to cabvars.h which in turn is
 * used only by init.c
 * 
 * Revision 1.18  89/08/12  14:49:15  mlh
 * Lint free
 * 
 * Revision 1.15  89/08/11  15:12:09  mlh
 * rall rall branching, several bugs fixed
 * 
 * Revision 1.14  89/08/10  16:23:59  mlh
 * clamp and synapses
 * 
 * Revision 1.10  89/08/09  10:50:38  mlh
 * fstim works, v accessible at ends, for (x) goes from 0<=x<=1
 * 
 * Revision 1.9  89/08/07  10:24:12  mlh
 * getting good v with single section. matrix solving for v (not deltav)
 * 
 * Revision 1.8  89/08/05  14:25:47  mlh
 * The root of the tree has a node of length 0.
 * We can define the voltage from  0<x<1 and get it right even
 * at 0 and 1
 * The last node of each section is has no length
 * good start on calculating matrix coefficients
 * 
 * Revision 1.7  89/08/04  14:48:09  mlh
 * 2nd order correct in dx handled pretty well with nnode-1 segments
 * and last node being at the end of the section with 0 length.
 * Problem with connecting to position 0 since first node
 * not at beginning of section.
 * 
 * Revision 1.3  89/07/21  17:38:38  mlh
 * compiles
 * 
 * Revision 1.1  89/07/20  07:53:32  mlh
 * Initial revision
 * 
*/

#include	<stdio.h>
#include	<errno.h>
#include	<math.h>
#include	"section.h"
#include	"membfunc.h"
#include	"neuron.h"
#include	"ytab.h"

extern int	diam_changed;
extern int	tree_changed;
extern double	chkarg();
extern double	nrn_ra();

#if VECTORIZE
/*
When properties are allocated to nodes or freed, v_structure_change is
set to 1. This means that the mechanism vectors need to be re-determined.
*/
int v_structure_change;
int v_node_count;
Node** v_node;
Node** v_parent;

#if _CRAY
/* in case several instance of point process at same node */
static struct CrayNodes {
	int cnt;
	Node** pnd1;
	Node* nd2;
} cray_nodes;

Node*** v_node_depth_lists;
Node*** v_parent_depth_lists;
int* v_node_depth_count;
int v_node_depth;
#endif

#endif
extern int section_count;
extern Section** secorder;

void
setup_tree_matrix() /* construct diagonal elements */
{
#if VECTORIZE
	int i;
#else
	int i, isec;
	Section *sec;
	Node *nd, *pnd;
	Prop *p;
#endif

#if METHOD3
	if (_method3) {
		method3_setup_tree_matrix();
		return;
	}
#endif
	if (diam_changed) {
		recalc_diam();
	}
#if VECTORIZE
#if _CRAY
#pragma _CRI ivdep
#endif
	for (i = 0; i < v_node_count; ++i) {
		v_node[i]->rhs = 0.;
		v_node[i]->d = 0.;
	}

#if _CRAY
/*
	there is a chance that several instances of a point process appear
	at the same node. If this ocurred then, to avoid writing to the
	node at the same time, they were given virtual nodes to point to
	and now we need to initialize those virtual nodes.
*/
	if (cray_nodes.cnt){
		int i;
		Node* nd1, *nd2;
#pragma _CRI ivdep
		for (i=0; i < cray_nodes.cnt; ++i) {
			nd1 = cray_nodes.pnd1[i];
			nd2 = cray_nodes.nd2 + i;
			nd2->rhs = 0.;
			nd2->d = 0.;
			nd2->v = nd1->v;
		}
	}
		
#endif

#if EXTRACELLULAR
	nrn_init_ext();
#endif

	for (i=0; i < n_memb_func; ++i) if (memb_func[i].current && memb_list[i].nodecount) {
		if (memb_list[i].vectorized) {
			memb_func[i].current(
			   memb_list[i].nodecount,
			   memb_list[i].nodelist,
			   memb_list[i].data,
			   memb_list[i].pdata,
			   i
			);
		}else{
			int j, count;
			Pfri s = memb_func[i].current;
			Memb_list* m = memb_list + i;
			count = m->nodecount;
			for (j = 0; j < count; ++j) {
				Node* nd = m->nodelist[j];
#if 0
				nd->rhs -= (*s)(m->data[j], m->pdata[j],
				  		&nd->d,nd->v);
#else
				(*s)(nd, m->data[j], m->pdata[j]);
#endif
			};
		}
		if (errno) {
			if (nrn_errno_check(i)) {
hoc_warning("errno set during calculation of currents", (char*)0);
			}
		}
	}
#if _CRAY
/*
	there is a chance that several instances of a point process appear
	at the same node. If this ocurred then, to avoid writing to the
	node at the same time, they were given virtual nodes to point to
	and now nd.d and nd.rhs must be added to the correct node.
*/
	if (cray_nodes.cnt){
		int i;
		Node* nd1, *nd2;
		for (i=0; i < cray_nodes.cnt; ++i) {
			nd1 = cray_nodes.pnd1[i];
			nd2 = cray_nodes.nd2 + i;
			nd1->rhs += nd2->rhs;
			nd1->d += nd2->d;
		}
	}
		
#endif

	/* at this point d contains all the membrane conductances */

#if EXTRACELLULAR
	nrn_setup_ext();
#endif

	/* now add the axial currents */
#if _CRAY
#pragma _CRI ivdep
#endif
	for (i = 0; i < v_node_count; ++i) {
		v_node[i]->d -= v_node[i]->b;
	}
#if _CRAY 
        for (i = v_node_depth - 1; i > 0; --i) {   
		int j;
                int count = v_node_depth_count[i];
                Node** nodes = v_node_depth_lists[i];
                Node** parents = v_parent_depth_lists[i];
                /* next loop can be done in parallel */
#pragma _CRI ivdep
                for ( j = 0; j < count; ++j) {
                        parents[j]->d -= nodes[j]->a;
                }       
        } 
#else 
        for (i=rootnodecount; i < v_node_count; ++i) {
                v_parent[i]->d -= v_node[i]->a;
        }
#endif

/* so ends the vectorization */
#else
	for (i = section[0].nnode - 1; i >= 0; i--) {
		nd = section[0].node + i;
	  	nd->rhs = 0;
	  	nd->d = 0;
	}
	for (isec = 1; isec < nsection; isec++) {
		sec = section + section[isec].order;
#if EXTRACELLULAR
		if (sec->extnode) {
			setup_ext_sec(sec);
			continue;
		}
#endif
		/* only section 0 doesn't have a parent */
		pnd = (section + sec->parentsec)->node + sec->parentnode;
		for (i = 0; i < sec->nnode; i++) {
			nd = sec->node + i;
			nd->rhs = 0.;
			nd->d = -nd->b;
			pnd->d -= nd->a;	/* this is elegant */
			pnd = nd;		/* but tricky */

			for (p = nd->prop; p; p = p->next) {
				if (memb_func[p->type].current) {
	nd->rhs -= (memb_func[p->type].current)(p, &nd->d, nd->v);
					if (errno) {
						if (nrn_errno_check(p, i, sec)) {
hoc_warning("errno set during calculation of currents", (char*)0);
						}
					}
				}
			}
		}
	}
#endif
	activstim();
	activsynapse();
#if SEJNOWSKI
	activconnect();
#endif
	activclamp();
}

/* membrane mechanisms needed by other mechanisms (such as Eion by HH)
may require that the needed mechanism appear before it in the list.
(because ina must be initialized before it is incremented by HH)
With current implementation, a chain of "needs" may not be in list order.
*/
static Prop **current_prop_list; /* the one prop_alloc is working on
					when need_memb is called */
static int disallow_needmemb = 0; /* point processes cannot use need_memb
	when inserted at locations 0 or 1 */

Prop *
need_memb(sym)
	Symbol* sym;
{
	int type;
	Prop *mprev, *m, *prop_alloc();

	if (disallow_needmemb) {
		fprintf(stderr, "You can not locate a point process at\n\
 position 0 or 1 if it needs an ion\n");
		hoc_execerror(sym->name, "can't be inserted in this node");
	}
	type = sym->subtype;
	mprev = (Prop *)0; /* may need to relink m */
	for (m = *current_prop_list; m; mprev = m, m = m->next) {
		if (m->type == type) {
			break;
		}
	}
	if (m) {
		/* a chain of "needs" will not be in list order
		so it is important that order only important for Eion */
		assert(mprev);
		mprev->next = m->next;
		m->next = *current_prop_list;
		*current_prop_list = m;
	} else {
		m = prop_alloc(current_prop_list, type);
	}
	return m;
}
	
Prop *
prop_alloc(pp, type)	/* link in new property at head of list */
	Prop **pp;	/* returning *Prop because allocation may */
	short type;	/* cause other properties to be linked ahead */
{
	Prop *p;

#if VECTORIZE
	v_structure_change = 1;
#endif
	current_prop_list = pp;
	p = (Prop *)emalloc(sizeof(Prop));
	p->type = type;
	p->next = *pp;
	*pp = p;
	assert(memb_func[type].alloc);
	p->dparam = (Datum *)0;
	p->param = (double *)0;
	p->param_size = 0;
	(memb_func[type].alloc)(p);
	return p;
}

Prop*
prop_alloc_disallow(pp, type)
	Prop** pp;
	short type;
{
	Prop *p;
	disallow_needmemb = 1;
	p = prop_alloc(pp, type);
	disallow_needmemb = 0;
	return p;
}

void
prop_free(pp)	/* free an entire property list */
	Prop **pp;
{
	Prop *p, *pn;
	p = *pp;
	*pp = (Prop *)0;
	while (p) {
		pn = p->next;
		single_prop_free(p);
		p = pn;
	}
}

single_prop_free(p)
	Prop* p;
{
	extern char pnt_map[];
#if VECTORIZE
	v_structure_change = 1;
#endif
	if (pnt_map[p->type]) {
		clear_point_process_struct(p);
		return;
	}
	if (p->param) {
		hoc_free_val_array(p->param, p->param_size);
	}
	if (p->dparam) {
		if (p->type == CABLESECTION) {
			notify_freed((void*)&p->dparam[2].val, 3);
		}
		free((char *)p->dparam);
	}
	free((char *)p);
}

/* For now there is always one more node in a section than there are
segments */
/* except in section 0 in which all nodes serve as x=0 to connecting
sections */

#undef PI
#define PI	3.14159

connection_coef()	/* setup a and b */
{
	int j;
	double dx, diam, area, ra;
	hoc_Item* qsec;
	Node *nd;
	Prop *p;
#if RA_WARNING
	extern int nrn_ra_set;
#endif
	
#if 1
	/* now only called from recalc_diam */
	assert(!tree_changed);
#else
	if (tree_changed) {
		setup_topology();
	}
#endif

#if RA_WARNING
	if (nrn_ra_set > 0 && nrn_ra_set < section_count - 1) {
		hoc_warning("Don't forget to set Ra in every section",
			"eg. forall Ra=34.5");
	}
#endif
	/* temporarily store left and right half segment resistances in d,rhs */
	for (j=0; j < rootnodecount; j++) {
		nd = v_node[j];
		nd->rhs = 0;
		nd->area = 1.e2;
		nd->a = nd->b = 0.;	/* done with section 0 */
	}
	ForAllSections(sec)
#if DIAMLIST
		if (sec->npt3d) {
			sec->prop->dparam[2].val = sec->pt3d[sec->npt3d - 1].arc;
		}
#endif
		ra = nrn_ra(sec);
		dx = sec->prop->dparam[2].val/((double)(sec->nnode - 1));
		for (j = 0; j < sec->nnode-1; j++) {
			nd = sec->node + j;
			for (p = nd->prop; p; p = p->next) {
				if (p->type == MORPHOLOGY) {
					break;
				}
			}
			assert(p);
#if DIAMLIST
			if (sec->npt3d) {
  /* trapezoidal integration of diam, area, left half segment resistance, and
     right half segment resistance.  The resistances are in node[j].d and
     node[j].rhs respectively
  */
				diam_from_list(sec, j,  p);
			}else
#endif
			{
				diam = p->param[0];
				nd->area = PI*diam*dx;	/* um^2 */
				nd->rhs = 1e-2*ra*(dx/2)/(PI*diam*diam/4.); /*Megohms*/
				nd->d = nd->rhs;
			}
		}
		/* last segment has 0 length. area is 1e2 
		in dimensionless units */
		sec->node[j].rhs = 0.;
		sec->node[j].d = 0.;
		sec->node[j].area = 1.e2;
	}
	/* assume that if only one connection at x=1, then they butte
	together, if several connections at x=1
	then last point is at x=1, has 0 area and other points are at
	centers of nnode-1 segments.
	If interior connection then child half
	section connects straight to the point*/
	/* for the near future we always have a last node at x=1 with
	no properties */
	ForAllSections(sec)
		/* node half resistances in general get added to the
		node and to the node's "child node in the same section".
		child nodes in different sections don't involve parent
		node's resistance */
		nd = sec->node;
		nd->b = nd->d; /* left half segment resistance */
		for (j = 1; j < sec->nnode; j++) {
			nd++;
			nd->b = nd->d + (nd-1)->rhs; /*megohms*/
			/* left half segment + right half of parent */
		}
	}
	ForAllSections(sec)
		/* convert to mhos/cm^2 for all nodes except last
		and micromhos for last.  This means that a*V = mamps/cm2
		and a*v in last node = nanoamps. Note that last node
		has no membrane properties and no area. It may perhaps receive
		current stimulus later */
		/* first the effect of node on parent equation. Note That
		last nodes have area = 1.e2 in dimensionless units so that
		last nodes have units of micromho's */
		nd = sec->node;
		area = sec->parentnode->area;
		/* dparam[4] is rall_branch */
		nd->a = -1.e2*sec->prop->dparam[4].val/(nd->b * area);
		for (j = 1; j < sec->nnode; j++) {
			nd++;
			area = (nd - 1)->area;
			nd->a = -1.e2/(nd->b * area);
		}
	}
	/* now the effect of parent on node equation. */
	ForAllSections(sec)
		for (j=0; j < sec->nnode; j++) {
			nd = sec->node + j;
			nd->b = -1.e2/(nd->b * nd->area);
		}
	}
#if EXTRACELLULAR
	ext_con_coef();
#endif
}

int recalc_diam() {
#if METHOD3
	if (_method3) {
		method3_connection_coef();
	}else
#endif
	v_setup_vectors();
	connection_coef();
	diam_changed = 0;
	stim_prepare();
	synapse_prepare();
#if SEJNOWSKI
	connect_prepare();
#endif
	clamp_prepare();
}

int
area() { /* returns area (um^2) of segment containing x */
	double x;
	Section *sec;
	Node *node_ptr();
	if (tree_changed) {
		setup_topology();
	}
	if (diam_changed) {
		recalc_diam();
	}
	x = *getarg(1);
	if (x == 0. || x == 1.) {
		ret(0.);
	}else{
		sec = chk_access();
		ret(sec->node[node_index(sec, x)].area);
	}
}

int
ri() { /* returns resistance (Mohm) between center of segment containing x
		and the center of the parent segment */
	double area;
	Node *np, *node_ptr();
	if (tree_changed) {
		setup_topology();
	}
	if (diam_changed) {
		recalc_diam();
	}
	np = node_ptr(chk_access(), *getarg(1), &area);
	if (np->b) {
		ret(-1.e2/(area *np->b)); /* Megohms */
	}else{
		ret(1.e30);
	}
}


#if DIAMLIST

static int pt3dconst_;

pt3dconst() {
	int i = pt3dconst_;
	if (ifarg(1)) {
		pt3dconst_ = (int)chkarg(1, 0., 1.);	
	}
	ret((double)i);
}

pt3dclear() { /*destroys space in current section for 3d points.*/
	nrn_pt3dclear(chk_access());
	ret(0.);
}

nrn_pt3dclear(sec)
	Section* sec;
{
	if (sec->pt3d) {
		free((char *)(sec->pt3d));
	}
	sec->pt3d = (Pt3d *)0;
	sec->npt3d = 0;
}

nrn_diam_change(sec, d)
	Section* sec;
	double d;
{
	if (!pt3dconst_) {
		int i;
		for (i=0; i < sec->npt3d; ++i) {
			sec->pt3d[i].d = d;
		}
	}
}

nrn_length_change(sec, d)
	Section* sec;
	double d;
{
	if (!pt3dconst_ && sec->npt3d) {
		int i;
		double x0,y0,z0;
		double fac;
		double l;
		x0 = sec->pt3d[0].x;
		y0 = sec->pt3d[0].y;
		z0 = sec->pt3d[0].z;
		l = sec->pt3d[sec->npt3d-1].arc;
		fac = d/l;
		for (i=0; i < sec->npt3d; ++i) {
			sec->pt3d[i].arc =     sec->pt3d[i].arc    *fac;
			sec->pt3d[i].x = x0 + (sec->pt3d[i].x - x0)*fac;
			sec->pt3d[i].y = y0 + (sec->pt3d[i].y - y0)*fac;
			sec->pt3d[i].z = z0 + (sec->pt3d[i].z - z0)*fac;
			
		}
	}
}

int
can_change_morph(sec)
	Section* sec;
{
#if 1
	if (!pt3dconst_) {
		nrn_pt3dclear(sec);
	}
#endif
	return pt3dconst_ == 0;
}
	
int
pt3dadd() {
	/*pt3add(x,y,z, d) stores 3d point at end of current pt3d list.
	  first point assumed to be at arc length position 0. Last point
	  at 1. arc length increases monotonically.
	*/
	stor_pt3d(chk_access(), *getarg(1), *getarg(2),
		*getarg(3), *getarg(4));
	ret(1.);
}

int
n3d() { /* returns number of 3d points in section */
	Section* sec;
	sec = chk_access();
	ret ((double)sec->npt3d);
}

int
x3d() { /* returns x value at index of 3d list  */
	Section* sec;
	int n, i;
	sec = chk_access();
	n = sec->npt3d - 1;
	i = chkarg(1, 0., (double)n);
	ret((double)sec->pt3d[i].x);
}

int
y3d() { /* returns x value at index of 3d list  */
	Section* sec;
	int n, i;
	sec = chk_access();
	n = sec->npt3d - 1;
	i = chkarg(1, 0., (double)n);
	ret((double)sec->pt3d[i].y);
}
int
z3d() { /* returns x value at index of 3d list  */
	Section* sec;
	int n, i;
	sec = chk_access();
	n = sec->npt3d - 1;
	i = chkarg(1, 0., (double)n);
	ret((double)sec->pt3d[i].z);
}

int
arc3d() { /* returns x value at index of 3d list  */
	Section* sec;
	int n, i;
	sec = chk_access();
	n = sec->npt3d - 1;
	i = chkarg(1, 0., (double)n);
	ret((double)sec->pt3d[i].arc);
}

int
diam3d() { /* returns x value at index of 3d list  */
	Section* sec;
	double d, fabs();
	int n, i;
	sec = chk_access();
	n = sec->npt3d - 1;
	i = chkarg(1, 0., (double)n);
	d = (double)sec->pt3d[i].d;
	ret(fabs(d));
}

int
spine3d() { /* returns x value at index of 3d list  */
	Section* sec;
	int n, i;
	double d;
	sec = chk_access();
	n = sec->npt3d - 1;
	i = chkarg(1, 0., (double)n);
	d = (double)sec->pt3d[i].d;
	if (d < 0) {
		ret(1.);
	}else{
		ret(0.);
	}
}

stor_pt3d(sec, x,y,z, d)
	Section *sec;
	double x,y,z, d;
{
	int n;
	double sqrt();
	
	diam_changed = 1;
#if NTS_SPINE
#else
	if (d < 0.) {
		hoc_execerror("Diameter less than 0", (char *)0);
	}
#endif
	n = sec->npt3d;
	if ((sec->pt3d =
		(Pt3d *)hoc_Erealloc((char *)sec->pt3d, (n + 1)*sizeof(Pt3d)))
		== (Pt3d *)0) {
			sec->npt3d = 0;
			hoc_malchk();
	}

	sec->npt3d++;			
	if (n == 0) {
		sec->prop->dparam[2].val=0.;
	}else{
		double t1,t2,t3;
		t1 = x - sec->pt3d[n-1].x;
		t2 = y - sec->pt3d[n-1].y;
		t3 = z - sec->pt3d[n-1].z;
		sec->prop->dparam[2].val += sqrt(t1*t1 + t2*t2 + t3*t3);
	}
	sec->pt3d[n].x = x;
	sec->pt3d[n].y = y;
	sec->pt3d[n].z = z;
	sec->pt3d[n].d = d;
	sec->pt3d[n].arc = sec->prop->dparam[2].val;
}

static double spinearea=0.;

setSpineArea() {
	spinearea = *getarg(1);
	diam_changed = 1;
	ret(spinearea);
}

getSpineArea() {
	ret(spinearea);
}

define_shape() {
	nrn_define_shape();
	ret(1.);
}

static nrn_translate_shape(sec, x, y, z)
	Section* sec;
	float x, y, z;
{
	int i;
	float dx, dy, dz;
	dx = x - sec->pt3d[0].x;
	dy = y - sec->pt3d[0].y;
	dz = z - sec->pt3d[0].z;
/*	if (dx*dx + dy*dy + dz*dz < 10.)*/
	for (i=0; i < sec->npt3d; ++i) {
		sec->pt3d[i].x += dx;
		sec->pt3d[i].y += dy;
		sec->pt3d[i].z += dz;
	}
}

nrn_define_shape() {
	int i;
	Section* sec, *psec, *ch, *nrn_trueparent();
	float x, y, z, dz, x1, y1;
	float nch, ich, angle;
	double arc;
	double nrn_diameter(), section_length(), nrn_connection_position();
	recalc_diam();
	dz = 100.;
	sec = secorder[0];
	for (i=0; i < section_count; ++i) {
		sec = secorder[i];
		arc = nrn_connection_position(sec);
		if ((psec = sec->parentsec) == (Section*)0) {
			/* section has no parent */
			x = 0;
			y = 0;
			z = i * dz;
			x1 = -1;
			y1 = 0;
		}else{
			double arc1 = arc;
			if (!arc0at0(psec)) {
				arc1 = 1. - arc;
			}
x = psec->pt3d[psec->npt3d-1].x*arc1 + psec->pt3d[0].x*(1-arc1);
y = psec->pt3d[psec->npt3d-1].y*arc1 + psec->pt3d[0].y*(1-arc1);
z = psec->pt3d[psec->npt3d-1].z*arc1 + psec->pt3d[0].z*(1-arc1);
x1 = psec->pt3d[psec->npt3d-1].x*(1-arc1) + psec->pt3d[0].x*(arc1);
y1 = psec->pt3d[psec->npt3d-1].y*(1-arc1) + psec->pt3d[0].y*(arc1);
		}
		if (sec->npt3d) {
			nrn_translate_shape(sec, x, y, z);
			continue;
		}
		angle = atan2(y - y1, x - x1);
		nch = 0.;
		if (psec) for (ch=psec->child; ch; ch = ch->sibling) {
			if (ch == sec) {
				ich = nch;
			}
			if (arc == nrn_connection_position(ch)) {
				++nch;
			}
		}		
		if (!nrn_trueparent(sec)) {
			--nch;
		}
		if (nch > 1) {
			angle += ich/(nch - 1.) * .8 - .4;
		}
		x1 = x + section_length(sec)*cos(angle);
		y1 = y + section_length(sec)*sin(angle);
		stor_pt3d(sec, x, y, z, nrn_diameter(sec->node));
		stor_pt3d(sec, x1, y1, z, nrn_diameter(sec->node + sec->nnode-2));
	}
}

diam_from_list(sec, inode, p) /* p->param[0] is diam of inode in sec.*/
	Section *sec;
	int inode;
	Prop *p;
{
	/* Basic algorithm assumes a set of monotonic points on which a
	   function is defined. The extension is the piecewise continuous
	   function. y(x) for 0<=x<=x[n] (extrapolate for x>x[n]).
	   The problem is to form the integral of f(y(s)) over intervals
	   s[i] <= s <= s[i+1]. Note the intervals don't have to be equal.
	   This implementation is specific to a call to this function for
	   each interval in order.
	*/
	/* computes diam as average, area, and ri. Slightly weirder since
	   interval spit in two to compute right and left half values of ri
	*/
	static int j;
	static double x1, y1, ds;
	double fabs(), sqrt();
	int ihalf;
	double si, sip;
	double diam, delta, temp, ri, area, ra;
	int npt, nspine;

	if (inode == 0) {
		j = 0;
		si = 0.;
		x1 = sec->pt3d[j].arc;
		y1 = fabs(sec->pt3d[j].d);
		ds = sec->pt3d[sec->npt3d - 1].arc / ((double)(sec->nnode - 1));
	}
	si = (double)inode*ds;
	npt = sec->npt3d;
	diam = 0.;
	area = 0.;
	nspine = 0;
	ra = nrn_ra(sec);
    for (ihalf = 0; ihalf < 2; ihalf++) {
	ri = 0.;
    	sip = si + ds/2.;
	for (;;) {
		int jp, jnext;
		double x2, y2, xj, xjp;
		jp = j + 1;
		xj = sec->pt3d[j].arc;
#if NTS_SPINE
		if (sec->pt3d[j].d < 0 && xj >= si && xj < sip) {
			nspine++;
		}
#endif
		xjp = sec->pt3d[jp].arc;
		if (xjp >= sip || jp == npt - 1) {
			double frac;
			if (fabs(xjp - xj) < 1e-10) {
				frac = 1;
			}else{
				frac = (sip - xj)/(xjp - xj);
			}
			x2 = sip;
			y2 = (1. - frac)*fabs(sec->pt3d[j].d) + frac*fabs(sec->pt3d[jp].d);
			jnext = j;
		}else{
			x2 = xjp;
			y2 = fabs(sec->pt3d[jp].d);
			jnext = jp;
		}

		/* f += integrate(x1,y1, x2, y2) */
		delta = (x2 - x1);
		if (delta < 1e-10) {
			delta = 1e-10;
		}
		diam += (y2 + y1)*delta;
		if ((temp = y2*y1/delta) == 0) {
			temp = 1e-10;
		}
		ri += 1/temp;
#if 1			/*taper is very seldom taken into account*/
			temp = .5*(y2 - y1);
			temp = sqrt(delta*delta + temp*temp);
#else
			temp = delta;
#endif

		area += (y2 + y1)*temp;
		/* end of integration */

		x1 = x2;
		y1 = y2;
		if (j == jnext) {
			break;
		}
		j = jnext;
	}
	if (ihalf == 0) {
		sec->node[inode].d = ri*ra/PI*(4.*.01); /*left seg resistance*/
	}else{
		sec->node[inode].rhs = ri*ra/PI*(4.*.01);	/* MegOhms */
		/* above is right half segment resistance */
	}
	si = sip;
    }
	/* answer for inode is here */
	p->param[0] = diam*.5/(ds);	/* microns */
	sec->node[inode].area = area*.5*PI;	/* microns^2 */
#if NTS_SPINE
	sec->node[inode].area += nspine*spinearea;
#endif
}

#endif /*DIAMLIST*/

#if VECTORIZE
v_setup_vectors() {
	int inode, i;
	int isec;
	Section* sec;
	Node* nd;
	Prop* p;
	
	if (tree_changed) {
		setup_topology();
		v_structure_change = 1;
	}
	/* get rid of the old */
	if (!v_structure_change) {
		return;
	}
	if (v_node) {
		free((char*) v_node);
		free((char*) v_parent);
		v_node = 0;
		v_parent = 0;
		v_node_count = 0;
	}
	for (i=0; i < n_memb_func; ++i)
	  if (memb_func[i].current || memb_func[i].state || memb_func[i].initialize) {
		if (memb_list[i].nodecount) {
			memb_list[i].nodecount = 0;
			free((char*)memb_list[i].nodelist);
			free((char*)memb_list[i].data);
			free((char*)memb_list[i].pdata);
		}
	}
		
	/* count up what we need */

	for (isec = 0; isec < section_count; ++isec) {
		sec = secorder[isec];
		for (inode = -1; inode < sec->nnode; ++inode) {
			if (inode == -1) {
				if (sec->parentsec) {
					continue;
				}else{
					nd = sec->parentnode;
				}
			}else {
				nd = sec->node + inode;
			}
			++v_node_count;
			for (p = nd->prop; p; p = p->next) {
				if (memb_func[p->type].current
				   || memb_func[p->type].state
				   || memb_func[p->type].initialize
				   ) {
					++memb_list[p->type].nodecount;
				}
			}
		}
	}
	
	/* allocate it*/

	if (v_node_count) {
		v_node = (Node**) emalloc(v_node_count*sizeof(Node*));
		v_parent = (Node**) emalloc(v_node_count*sizeof(Node*));
		v_node_count = 0; /* counted again below */
	}
	for (i=0; i < n_memb_func; ++i)
	  if (memb_func[i].current || memb_func[i].state || memb_func[i].initialize) {
		if (memb_list[i].nodecount) {
memb_list[i].nodelist = (Node**) emalloc(memb_list[i].nodecount * sizeof(Node*));
memb_list[i].data = (double**) emalloc(memb_list[i].nodecount * sizeof(double*));
memb_list[i].pdata = (Datum**) emalloc(memb_list[i].nodecount * sizeof(Datum*));
			memb_list[i].nodecount = 0; /* counted again below */
		}
	}

	/* setup the pointers */
	/* put the nodes in an order suitable for vectorizing the elimination */
	for (isec = 0; isec < rootnodecount; ++isec) {
		sec = secorder[isec];
		assert(!sec->parentsec);
		nd = sec->parentnode;
		v_node[isec] = nd;
		v_parent[isec] = nd;
		v_node[v_node_count]->v_node_index = v_node_count;
		++v_node_count;
			for (p = nd->prop; p; p = p->next) {
				i = p->type;
				if (memb_func[i].current
				   || memb_func[i].state
				   || memb_func[i].initialize
				   ) {
memb_list[i].nodelist[memb_list[i].nodecount] = nd;
memb_list[i].data[memb_list[i].nodecount] = p->param;
memb_list[i].pdata[memb_list[i].nodecount] = p->dparam;
					++memb_list[i].nodecount;
				}
			}
	}
	for (isec = 0; isec < section_count; ++isec) {
		sec = secorder[isec];
		for (inode = 0; inode < sec->nnode; ++inode) {
			nd = sec->node + inode;
			v_node[v_node_count] = nd;
			if (inode) {
				v_parent[v_node_count] = nd - 1;
			}else{
				v_parent[v_node_count] = sec->parentnode;
			}
			v_node[v_node_count]->v_node_index = v_node_count;
			++v_node_count;
			for (p = nd->prop; p; p = p->next) {
				i = p->type;
				if (memb_func[i].current
				   || memb_func[i].state
				   || memb_func[i].initialize
				   ) {
memb_list[i].nodelist[memb_list[i].nodecount] = nd;
memb_list[i].data[memb_list[i].nodecount] = p->param;
memb_list[i].pdata[memb_list[i].nodecount] = p->dparam;
					++memb_list[i].nodecount;
				}
			}
		}
	}
	v_structure_change = 0;
#if _CRAY
	cray_solve();
	cray_node_setup();
#endif
}

#if _CRAY

cray_solve() {
	int i, id, max_depth;
	int* nchild, *depth;
	/* free the old stuff */
	if (v_node_depth) {
		free((char*)v_node_depth_count);
		for (i = 0; i < v_node_depth; i++) {
			free((char*)v_node_depth_lists[i]);
			free((char*)v_parent_depth_lists[i]);
		}
		free((char*)v_node_depth_lists);
		free((char*)v_parent_depth_lists);
		v_node_depth = 0;
		v_node_depth_lists = 0;
		v_parent_depth_lists = 0;
		v_node_depth_count = 0;
	}
	
	/*
	  The bad thing is when we have one long
	  cable. Then the depth increases and there aren't many other nodes
	  that can share the depth.
	*/
	/*
	  We require that
	  no nodes at the same depth can share the same parent. Otherwise
	  during triangularizaton several nodes may try to change the same
	  parent at the same time. The curious fact is that a node may
	  increase its depth as long as it does not go to a depth that
	  has a node with the same parent and remains at a depth which is
	  less than any of its children.
	*/
  	depth = (int*)ecalloc(v_node_count, sizeof(int));
	nchild = (int*)ecalloc(v_node_count, sizeof(int));
	max_depth = 0;
	for (i=rootnodecount; i < v_node_count; ++i){
		int pindex = v_parent[i]->v_node_index;
		++nchild[pindex];
		depth[i] = depth[pindex] + nchild[pindex];
		if (depth[i] > max_depth) {
			max_depth = depth[i];
		}
	}
		
	/*allocate memory*/
	v_node_depth = max_depth + 1;
	v_node_depth_count = (int*)ecalloc(v_node_depth, sizeof(int));
	/* count the list sizes */
	for (i=0; i < v_node_count; ++i) {
		++v_node_depth_count[depth[i]];
	}
	v_node_depth_lists = (Node***)ecalloc(v_node_depth, sizeof(Node**));
	v_parent_depth_lists = (Node***)ecalloc(v_node_depth, sizeof(Node**));
	for (i=0; i < v_node_depth; ++i) {
v_node_depth_lists[i] = (Node**)ecalloc(v_node_depth_count[i], sizeof(Node*));
v_parent_depth_lists[i] = (Node**)ecalloc(v_node_depth_count[i], sizeof(Node*));
	}
	/* set up the lists */
	for (i=0; i < v_node_depth; ++i) {
		v_node_depth_count[i] = 0;
	}
	for (i = 0; i < v_node_count; ++i) {
		id = depth[i];
		v_node_depth_lists[id][v_node_depth_count[id]] = v_node[i];
		v_parent_depth_lists[id][v_node_depth_count[id]] = v_parent[i];
		++v_node_depth_count[id];
	}

#if 0
	/* for debugging so we can know which node is which place the
		section index in nd->d */
	for (i=0; i < section_count; ++i) {
		sec = secorder[i];
		for (j=0; j < sec->nnode; ++j) {
			sec->node[j].d = (double)i;
		}
	}
	printf("v_node_depth = %d\n", v_node_depth);
	for (i=0; i < v_node_depth; ++i) {
		extern char* secname();
		count = v_node_depth_count[i];
		printf("  depth = %d  count = %d\n", i, count);
		for (j = 0; j < count; ++j) {
			nd = v_node_depth_lists[i][j];
			pnd = v_parent_depth_lists[i][j];
			sec = secorder[(int)(nd->d)];
			psec = secorder[(int)(pnd->d)];
printf("    %d node %s[%d]", j, secname(sec), nd - sec->node);
printf("  parent %s[%d]\n", secname(psec), pnd - psec->node);
  		}
	}
#endif
	free((char*)depth);
	free((char*)nchild);
}

static cray_node_setup() {
	int i, j, k;
	cray_nodes.cnt = 0;
	if (cray_nodes.pnd1) {
		free((char*) cray_nodes.pnd1);
		free((char*) cray_nodes.nd2);
	}

	for (i=0; i < n_memb_func; ++i) if (
	   memb_list[i].is_point
	   && memb_list[i].vectorized
	   && memb_func[i].current
	   && memb_list[i].nodecount > 1)
	{
		for (j =  memb_list[i].nodecount - 1; j; --j) {
			if (memb_list[i].nodelist[j] == memb_list[i].nodelist[j-1]) {
				++cray_nodes.cnt;
			}
		}
	}

	if (cray_nodes.cnt) {
		cray_nodes.pnd1 = (Node**)ecalloc(cray_nodes.cnt, sizeof(Node*));
		cray_nodes.nd2 = (Node*)ecalloc(cray_nodes.cnt, sizeof(Node));
		k = 0;
		for (i=0; i < n_memb_func; ++i) if (
		   memb_list[i].is_point
		   && memb_list[i].vectorized
		   && memb_func[i].current
		   && memb_list[i].nodecount > 1)
		{
			for (j =  memb_list[i].nodecount - 1; j; --j) {
				if (memb_list[i].nodelist[j] == memb_list[i].nodelist[j-1]) {
cray_nodes.pnd1[k] = memb_list[i].nodelist[j];
cray_nodes.nd2[k].area = memb_list[i].nodelist[j]->area;
#if 1
/* the following should not be necessary */
cray_nodes.nd2[k].prop = memb_list[i].nodelist[j]->prop;
cray_nodes.nd2[k].child = memb_list[i].nodelist[j]->child;
#endif
memb_list[i].nodelist[j] = cray_nodes.nd2 + k;
					++k;
				}
			}
		}
	}
}

cray_node_init() {
	if (cray_nodes.cnt){
		int i;
		Node* nd1, *nd2;
#pragma _CRI ivdep
		for (i=0; i < cray_nodes.cnt; ++i) {
			nd1 = cray_nodes.pnd1[i];
			nd2 = cray_nodes.nd2 + i;
			nd2->v = nd1->v;
		}
	}
		
}

#endif

#endif

#define NODE_DATA 1
#if NODE_DATA
static FILE* fnd;

#undef P
#undef Pn
#undef Pd
#undef Pg

#define P fprintf(fnd,
#define Pn P "\n")
#define Pd(arg) P "%d\n", arg)
#define Pg(arg) P "%g\n", arg)

node_data_scaffolding() {
	int i;
	Pd(n_memb_func);
/*	P "Mechanism names (first two are nil) beginning with memb_func[2]\n");*/
	for (i=2; i < n_memb_func; ++ i){
		P "%s", memb_func[i].sym->name); Pn;
	}
}

node_data_structure() {
	int i, j;
	Pd(v_node_count);
	
	Pd(rootnodecount);
/*	P "Indices of node parents\n");*/
	for (i=0; i < v_node_count; ++i) {
		Pd(v_parent[i]->v_node_index);
	}
/*	P "node lists for the membrane mechanisms\n");*/
	for (i=2; i < n_memb_func; ++ i){
/*		P "count, node list for mechanism %s\n", memb_func[i].sym->name);*/
		Pd(memb_list[i].nodecount);
		for (j=0; j < memb_list[i].nodecount; ++j) {
			Pd(memb_list[i].nodelist[j]->v_node_index);
		}
	}
}

node_data_values() {
	int i, j, k;
/*	P "data for nodes then for all mechanisms in order of the above structure\n");	*/
	for (i=0; i < v_node_count; ++ i) {
		Pg(v_node[i]->v);
		Pg(v_node[i]->a);
		Pg(v_node[i]->b);
		Pg(v_node[i]->area);
	}
	for (i=2; i < n_memb_func; ++ i){
		Prop* prop, *nrn_mechanism();
		int cnt;
		double* pd;
		if (memb_list[i].nodecount) {
			prop = nrn_mechanism(i, memb_list[i].nodelist[0]);
			cnt = prop->param_size;
			Pd(cnt);
		}
		for (j=0; j < memb_list[i].nodecount; ++j) {
			pd = memb_list[i].data[j];
			for (k = 0; k < cnt; ++k) {
				Pg(pd[k]);
			}
		}
	}
}

node_data() {
	fnd = fopen(gargstr(1), "w");
	if (!fnd) {
		hoc_execerror("node_data: can't open", gargstr(1));
	}
	if (tree_changed) {
		setup_topology();
	}
	if (v_structure_change) {
		v_setup_vectors();
	}
	if (diam_changed) {
		recalc_diam();
	}
	node_data_scaffolding();
	node_data_structure();
	node_data_values();
	fclose(fnd);
	ret(1.);
}

#else
node_data() {
	ret(0.);
}

#endif

nrn_complain(pp) double* pp; {
	/* print location for this param on the standard error */
	extern char* secname();
	extern double nrn_arc_position();
	Node* nd;
	hoc_Item* qsec;
	int j;
	Prop* p;
	ForAllSections(sec)
		for (j = 0; j < sec->nnode; ++j) {
			nd = sec->node + j;
			for (p = nd->prop; p; p = p->next) {
				if (p->param == pp) {
					
fprintf(stderr, "Error at section location %s(%g)\n", secname(sec),
  nrn_arc_position(sec, nd));
					return;
				}
			}
		}
	}
	fprintf(stderr, "Don't know the location of params at %lx\n", (long)pp);
}
