
/* $Header: /timshel2g/src/master/nrn/src/nrnoc/version.c,v 1.1.1.1 1994/10/12 17:22:36 hines Exp $ */
/* $Log: version.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:36  hines
 * NEURON 3.0 distribution
 *
 * Revision 4.80  1994/10/03  20:42:00  hines
 * nernst and ghk functions. A special form for the nernst function
 * is available and looks like nernst("nai"), that returns the nai based
 * on the values of nao, ena, celsius, valence, and the nernst equation.
 *
 * Revision 4.79  1994/10/03  17:51:31  hines
 * more flexibility in relationship between reversal potential and concentrations. May also specify whether CONST, DEP, STATE
 * see ion_style
 *
 * Revision 4.78  1994/09/27  14:11:17  hines
 * get the vartype from the function in psection in order to print CONST
 * ion parameters
 *
 * Revision 4.77  1994/09/26  18:26:47  hines
 * ions more nearly do the "right thing"
 * presence of mechanism that WRITE's a concentration computes
 * the reversal potential every fadvance and finitialize, initializes
 * the concentrations to global variables, makes the concentrations have
 * the type STATE and the reversal potential the style DEP
 * presence of mech that READ's a concentration computes reversal
 * every finitialize, and treats concentration as CONST and reversal
 * potential style DEP.
 * If no mechanism reads or writes concentrations the reversal is
 * treated as CONST.
 *
 * Revision 4.76  1994/08/17  19:50:55  hines
 * cray does not allow the extracellular option.
 * get ocmain etc and nrnnoiv.o from lib
 *
 * Revision 4.75  1994/08/05  14:19:08  hines
 * default is to optimize
 *
 * Revision 4.74  1994/07/21  14:22:21  hines
 * fixed an error in forall inside object resulting from recent change
 * to object names.
 *
 * Revision 4.73  1994/07/21  11:54:06  hines
 * nrn_get_mechtype(mechname) and nrn_instance_count(mechtype)
 * introduced to provide info required for vectorizing kinetic blocks
 *
 * Revision 4.72  1994/07/12  13:49:42  hines
 * use hoc_object_name instead of building own format for it
 *
 * Revision 4.71  1994/07/12  12:56:00  hines
 * SEClamp single electrode clamp is built-in
 *
 * Revision 4.70  1994/07/08  20:42:47  hines
 * properly handle freeing of point process property. solves several
 * kinds of segmentation violation, reading freed memory, and memory leakage.
 *
 * Revision 4.69  1994/07/06  20:44:57  hines
 * popsec is not used. pop_section is the name
 *
 * Revision 4.68  1994/07/04  15:12:14  hines
 * new banner for a distributed version; Version 3.0. This avoids confusion
 * with the many rcs versions of the several modules that make up NEURON.
 * It is intended that the version will be incremented when a new
 * distribution is made
 *
 * Revision 4.67  1994/07/01  17:53:56  hines
 * error in ifsec SectionList when currently accessed section was not
 * in the list. fixed
 *
 * Revision 4.66  1994/06/30  14:42:47  hines
 * #if away triang and bksub when EXTRACELLULAR is compiled in solve.c
 *
 * Revision 4.65  1994/06/29  20:46:44  hines
 * allow easier coverage of treeset.c and benign umr of capac.c
 *
 * Revision 4.64  1994/06/29  13:24:34  hines
 * under unusual circumstances sections could be unreffed and never freed. fixed
 * benign uninitialized memory read of param_size when prop is freed before
 * memory is allocated for param. fixed
 *
 * Revision 4.63  1994/06/27  11:36:40  hines
 * small changes to allow code coverage tests for cabcode.c
 * dimensioned sections were not freed properly.
 * fix v(0:1) = 1:2 for both orientations
 *
 * Revision 4.62  1994/06/21  12:39:28  hines
 * allow recursive calls to interpreter even in case of hoc_execerror
 *
 * Revision 4.61  1994/06/09  14:25:10  hines
 * uninitialized memory read in hocprax.c fixed
 *
 * Revision 4.60  1994/06/07  18:43:45  hines
 * trivial objectcenter warning
 *
 * Revision 4.59  1994/06/07  16:02:24  hines
 * optimize the mod files
 *
 * Revision 4.58  1994/05/09  19:14:43  hines
 * Point Process structure keeps a pointer to the owning object.
 * This allows notification of object holders when the point
 * process location changes.
 *
 * Also one can determine if a point process is located in a section with
 * pp.has_loc()
 *
 * Revision 4.57  1994/05/03  15:16:48  hines
 * temporarily use nil for checkpoint functions
 *
 * Revision 4.56  1994/05/02  12:56:19  hines
 * don't print error message for mechanism not inserted if hoe_execerror_messages is false
 * nrn_complain(double* _p) pirnts location for this param on the standard error
 * device
 *
 * Revision 4.55  1994/03/08  17:49:02  hines
 * change of clamp_resist takes immediate effect
 *
 * Revision 4.54  1994/02/22  13:48:20  hines
 * vclmp.mod: on switch to ELECTRODE_CURRENT failed to make inward current
 * positive. This caused the clamp to be unstable instead of slow when it
 * was less than perfect.
 *
 * Revision 4.53  1994/01/21  19:34:19  hines
 * distance(0,x) can be used to set an exact origin for distance(x)
 * or distance(1,x) measurement
 *
 * Revision 4.52  1994/01/21  18:17:55  hines
 * make tau values accessible as well as gna, gk
 *
 * Revision 4.51  1993/12/20  18:52:14  hines
 * combination of no extracellular and no cray and vectorization
 * causes a bad triang statement to be compiled.
 *
 * Revision 4.50  1993/12/15  15:04:19  hines
 * SectionRef class holds pointer to section.
 * special .sec method handles section stack properly
 *
 * Revision 4.49  1993/11/04  15:56:56  hines
 * port to solaris2 (no more warnings)
 *
 * Revision 4.48  1993/10/18  13:37:32  hines
 * very minor fix to synapse.c and fadvance.c for portability
 *
 * Revision 4.47  1993/08/20  13:02:20  hines
 * botched last checkin for neuron.h
 *
 * Revision 4.46  93/08/20  13:01:49  hines
 * give message about a.a - a.a which is parsed as a.a {-a.a}
 * fix some problems regarding define_shape
 * allow change of L and diam when 3dpoints exist (pt3dconst(0) is default)
 * 
 * Revision 4.45  93/08/20  10:29:42  hines
 * writing to a constant string. gcc doesn't like that unless we use
 * -fwritable-strings. repaired in init.c
 * 
 * Revision 4.44  93/08/15  12:56:07  hines
 * fix warning of function declared external and later static
 * y
 * 
 * Revision 4.43  93/08/07  10:24:54  hines
 * added praxis functions. perhaps should have been added to oc
 * 
 * Revision 4.42  93/07/27  15:52:09  hines
 * Xmech dummy mechananism for installation of unknown mechanism for
 * teaching and examination purposes.
 * 
 * Revision 4.41  93/07/19  09:37:37  hines
 * forgot to set v_structure_change when  point process freed.
 * 
 * Revision 4.40  93/07/16  08:37:01  hines
 * push_section("sectionname")
 * 
 * Revision 4.39  93/07/13  11:50:07  hines
 * sibling pointers in sections are not referenced. bug when they were unreffed.
 * 
 * Revision 4.38  93/07/13  11:49:11  hines
 * declare all current functions properly (now Pfri instead of Pfrd)
 * 
 * Revision 4.37  93/07/09  16:25:04  hines
 * point processes now work correctly when they are electrodes and an
 * extracellular mechananism is present.
 * vclmp.mod and stim.mod are electrodes
 * positive electrode current causes depolarization (normal mechanism
 * current is positive outward)
 * 
 * Revision 4.36  93/07/08  14:34:26  hines
 * a start on fixing the bug of electode point processes not working
 * correctly in presence of extracellular. At this time the functionality
 * is identical as before but passing node to  models in nrn_cur instead
 * of passing pointer to nd->d and returning rhs.
 * There is an alternative now to NONSPECIFIC_CURRENT which is
 * ELECTRODE_CURRENT
 * 
 * Revision 4.35  93/06/01  09:00:15  hines
 * can see if a mechanism is an ion with nrn_is_ion(type)
 * can't uninsert ions
 * 
 * Revision 4.34  93/06/01  08:13:36  hines
 * Object* nrn_new_pointprocess(sym) where sym is the mechanism symbol
 * creates a new point process.
 * 
 * Revision 4.33  93/05/31  15:35:07  hines
 * for (ch=sec->child; ch; ch=ch->sibling) is loop over all sections connect
 * directly to sec in order of increasing distance from connection end of sec.
 * there is no longer a Node->child pointer.
 * 
 * Revision 4.31  93/05/25  14:04:06  hines
 * insert and uninsert can be done in c.
 * 
 * Revision 4.29  93/05/07  16:40:18  hines
 * _ninits incremented by finitialize(). This allows models to use
 * the lag function and perhaps schedule.
 * 
 * Revision 4.28  93/04/29  16:36:05  hines
 * sibling pointers now reffed. Fix several problems with disconnecting
 * and reconnecting sections as well as handle point processes moving with
 * their proper section.
 * Still not perfect though.
 * 
 * Revision 4.27  93/04/21  13:24:18  hines
 * watch out for nil parentnode when haven't done setup_topology yet.
 * 
 * Revision 4.26  93/04/21  10:33:23  hines
 * point processes located at ends of sections stay in those sections when
 * those sections are disconnected from their parent.
 * 
 * Revision 4.25  93/04/20  09:10:34  hines
 * the former version gave a hash table overflow on checkout
 * 
 *
 * rootsection no longer exists. section structure changed so
 * sec->parentnode is a Node*. sec->parentsec no longer changed but
 * kept the way user specified it. The function Section* nrn_trueparent(sec)
 * returns the section that used to be in parentsec after setup_topology()
 * was executed. no more inode_exact(Section** psec, double x)---has been
 * replaced by Node* node_exact(Section* sec, double x)
 * nThere is now a rootnode list. parentsec of a section which was not
 * connect'ed to anything has a parentsec of nil. One can now disconnect
 * and delete sections without otherwise changing user spec (except
 * point processes at ends may be moved or changed -- will be fixed later)
 *
 * Revision 4.24  93/04/14  08:05:51  hines
 * trivial typo
 * 
 * Revision 4.23  93/04/12  14:15:05  hines
 * first pass at merging Fisher's addition of LINDA
 * 
 * Revision 4.22  93/04/06  08:06:21  hines
 * define EXTRACELLULAR 1 in options.h
 * move nrnmainmenu and nrncontrolmenu out of c++ and into stdrun.hoc
 * 
 * Revision 4.21  93/03/23  17:24:24  hines
 * allow deletion of all sections
 * fix bug having to do with deletion of multiple sections
 * 
 * Revision 4.20  93/03/21  17:51:22  hines
 * new APCount() point process which is useful for determining occurence and
 * time of single action potential
 * 
 * node_data("filenam") prints minimal complete state (but not global variables)
 * for use with a minimal neuron system without the interpreter
 * 
 * Revision 4.19  93/03/15  10:06:12  hines
 * when deleteing sections clear all the fields. if a list sees a
 * bad section is removes it during the forsec execution
 * 
 * Revision 4.18  93/03/05  15:58:59  hines
 * switch from hoc_free_val to hoc_freed
 * 
 * Revision 4.17  93/02/25  13:27:27  hines
 * need_memb takes symbol arg now instead of name. This avoid hoc_lookup.
 * 
 * Revision 4.16  93/02/23  14:44:45  hines
 * compiles under DOS
 * 
 * 
 * Revision 4.15  93/02/20  09:03:24  hines
 * aix
 * 
 * Revision 4.14  93/02/15  08:59:54  hines
 * linux
 * 
 * Revision 4.13  93/02/02  10:31:00  hines
 * static functions declared before used
 * 
 * Revision 4.12  93/01/27  13:55:10  hines
 * forgot to checkin treeset.c after making extracellular work again
 * 
 * Revision 4.11  93/01/27  13:50:15  hines
 * extracellular works again
 * 
 * Revision 4.10  93/01/23  13:27:09  hines
 * NeXT won't include files twice. so split nmodl_redef.h into
 * nmodl1_redef.h and nmodl2_redef.h
 * 
 * Revision 4.9  93/01/21  12:21:18  hines
 * watch out for freed sections in old style fstim, fsyn, and fclamp
 * correct memory leakage on access section
 * 
 * Revision 4.8  93/01/14  13:05:07  hines
 * changed meaning of OPSTR
 * 
 * Revision 4.7  93/01/14  10:38:26  hines
 * Steve Fisher's contribution to RCS parameter dumps.
 * 
 * Revision 4.6  93/01/12  09:03:19  hines
 * setting hoc strings now done with hoc_assign_str(char**, char*)
 * 
 * Revision 4.5  93/01/07  08:15:03  hines
 * psection prints Ra
 * warning if Ra set but not in every section
 * fix batch_run so not extra iteration on cray (tstop -=dt/4)
 * 
 * Revision 4.4  93/01/06  09:44:48  hines
 * minor adjustments for cray c90
 * 
 * Revision 4.3  93/01/05  17:22:39  hines
 * can push and pop null sections.
 * void sec_access_pop()   Section* nrn_pop_sec()
 * 
 * Revision 4.2  93/01/05  11:54:20  hines
 * Fix bug in _CRAY portion where writing to same place at same time
 * 
 * Revision 4.1  92/12/31  09:44:14  hines
 * ready for beta release
 * 
 * Revision 3.47  92/12/30  12:38:58  hines
 * buf needed to be static in sec_and_position
 * 
 * Revision 1.2  89/07/20  10:17:47  mlh
 * first version
 *  */

/* newcable  */
char *RCS_cableversion = "$Revision: 1.1.1.1 $";
char *RCS_cabledate = "$Date: 1994/10/12 17:22:36 $";


