// VecView.m, draws text and vectors.
// Copyright Costa M. Colbert 1992,1993.

#define BORDER 36.0

#import <libc.h>                // For random(), etc...
#import <dpsclient/wraps.h>	// For PS and DPS function prototypes
#import <appkit/appkit.h>	
#import "YapWrap.h"

#import "VecView.h"
#import <stdio.h>

extern char	**Gargv, **Genvp;
extern int	Gargc;
extern FILE     *hoc_fin;

extern int	fromExec;

@implementation VecView

- initFrame:(const NXRect *)rect
{
    [super initFrame:rect];
    PSsetgray(0.0);
    NXEraseRect(&bounds);
    [self notifyToInitGState:YES];
    [self allocateGState];
    userPath = newUserPath();
    return self;
}

- initGState
{
	PSsetgray(0.0);
	NXEraseRect(&bounds);
	PSsetlinewidth(0.15);
	[self setscale: 12.0];
	PStranslate(BORDER,BORDER);
	return self;
}

- (ctxtInfo*)thectxtInfo
{
    [self lockFocus];
    GetFocus (&theInfo.c1x, &theInfo.c1y, &theInfo.c2x,
              &theInfo.c2y, theInfo.winCTM, &theInfo.realWinNum);
    [self unlockFocus];
    return(&theInfo);
}

- (float)width
{
	return bounds.size.width;
}

- (float)height
{
	return bounds.size.height;
}

- (NXRect*)bounds
{
	return &bounds;
}

- printPSCode:sender
{
  return [super printPSCode:sender];
}

- free
{
    freeUserPath(userPath);
    return [super free];
}

- drawSelf:(const NXRect *)rects :(int)rectCount
{
	if (!fromExec) return self;

	return self;
}

- printtext: (float)xpos: (float)ypos: (char*)s
{
	[self lockFocus];
		PSmoveto(xpos,ypos);
		PSshow(s); 
		PSstroke();
	[self unlockFocus];
	[[self window] flushWindow];
	NXPing();
	return self;
}

- (UserPath *) startVec
{
	beginUserPath(userPath, NO);
	return userPath;
}

- drawVec
{
	[self lockFocus];
	  endUserPath (userPath, dps_ustroke);
          sendUserPath(userPath);
	[self unlockFocus];
	return self;
}

- setscale:(float)scale
{
	PSselectfont("Times-Roman",scale);
	return self;
}

- drawLine: (float)x1: (float)y1: (float)x2: (float)y2
{
	[self lockFocus];
		PSmoveto(x1,y1);
		PSlineto(x2,y2);
		PSstroke();
		if (manual){
			 [[self window] flushWindow];
			 NXPing();
		}
	[self unlockFocus];
	return self;
}

- drawPoint:(float)x1:(float)y1
{
	[self lockFocus];
		PSmoveto(x1,y1);
		PSlineto(x1,y1);
		PSstroke();
		if (manual){
			 [[self window] flushWindow];
			 NXPing();
		}
	[self unlockFocus];
	return self;

}

- clearArea: (NXRect*)aRect
{
	[self lockFocus];
		NXEraseRect(aRect);
		[[self window] flushWindow];
		NXPing();
	[self unlockFocus];
	return self;
}

- clear
{
	[self lockFocus];
		// PStranslate(-BORDER,-BORDER);
		NXEraseRect(&bounds);
		[[self window] flushWindow];
		NXPing();
	[self unlockFocus];
	return self;
}

- doClear:sender
{
	[self clear];
	return self;
}

- setManual:(BOOL)bool
{
	if (bool==YES) manual = 1;
	else manual = 0;
	return self;
}

@end
