/* $Header: /timshel2g/src/master/nrn/src/oc/hoc.c,v 1.6 1994/11/08 19:37:10 hines Exp $ */
/*
$Log: hoc.c,v $
 * Revision 1.6  1994/11/08  19:37:10  hines
 * command line editing works while in graphics mode with GRX and GO32
 *
 * Revision 1.5  1994/11/02  18:55:23  hines
 * can find hoc object name from a scene. constructor of built in classes
 * now gets the Object* argument of the hoc object. Also some porting mods
 * for GO32
 *
 * Revision 1.4  1994/10/28  12:11:04  hines
 * port to __GO32__
 *
 * Revision 1.3  1994/10/25  17:24:40  hines
 * having the ytab files make for too large patches
 *
 * Revision 1.2  1994/10/18  13:34:54  hines
 * faster load_proc,load_func,load_template
 * cache of all names constructed by lib/hocload.sh on first call
 * the cache is unchanged throughout that neuron session.
 * On exit, all oc"pid".* files removed from /tmp
 *
 * Revision 1.1.1.1  1994/10/12  17:22:08  hines
 * NEURON 3.0 distribution
 *
 * Revision 2.116  1994/09/27  17:03:05  hines
 * when oc_jump_target failed then the next error could try to longjump to
 * a function that had already returned. fixed.
 *
 * Revision 2.108  1994/08/05  14:16:14  hines
 * show_errmess_always()
 * errno not defined from include file in hoc.c
 * NullArgument replaces null arguments in Imakefile
 *
 * Revision 2.105  1994/07/12  17:52:26  hines
 * more portable way of dealing with ocjump
 *
 * Revision 2.102  1994/07/04  15:10:54  hines
 * only print Oc banner if not NEURON
 *
 * Revision 2.100  1994/06/21  12:36:04  hines
 * allow recursive calls to interpreter even in case of hoc_execerror.
 *
 * Revision 2.99  1994/06/17  13:00:35  hines
 * a number of things fixed while looking at the test coverage of hoc.c
 * mostly just to allow coverage.
 * CBUFSIZE is larger than 256
 * array index computed with double arithmetic.
 * corrected handling of end of file while in comments
 *
 * Revision 2.90  1994/05/10  20:19:14  hines
 * global lineno no longer initialized when defined.
 * Note: there is a potential problem when calling oc_run() from
 * a descendent of an object since when there is an error, the highest
 * caller gets the execerror and not the lowest. it would be better ot
 * nicely unwrap all the calls instead of doing a longjmp
 *
 * Revision 2.84  1994/05/02  12:53:54  hines
 * quit() function
 *
 * Revision 2.82  1994/04/27  12:17:56  hines
 * minor warnings from objectcenter fixed (and one serious problem from
 * last checkin)
 * PI, E, etc are now userdoubles and are available within objects.
 *
 * Revision 2.81  1994/04/27  11:21:31  hines
 * support for ivoc/SRC/checkpoint.c
 * usage is checkpoint("filename")
 * then ivoc filename ...
 * so far only for oc
 *
 * Revision 2.77  1994/03/10  15:26:07  hines
 * capture SIGPIPE print message and ignore
 *
 * Revision 2.76  1994/03/10  14:33:13  hines
 * error on last checkin
 *
 * Revision 2.75  1994/03/10  14:28:33  hines
 * invoke coredump_on_error() if want it on next segmentation violation or
 * bus error.
 *
 * Revision 2.73  1994/03/04  21:37:01  hines
 * help for files invoked on command line
 *
 * Revision 2.71  1994/02/23  15:51:16  hines
 * RETREAT_SIGNAL (SIGHUP) handled and user written routine "linda_retreat()"
 * called on a doEvent(). Just exit if no such routine.
 *
 * Revision 2.70  1994/02/23  12:53:31  hines
 * help word word ... in ivoc tickles the help system
 *
 * Revision 2.67  1994/01/11  20:22:24  hines
 * set errno to 0 every input line so don't get spurious interrupted
 * system call messages from ivoc select system call
 *
 * Revision 2.65  1993/11/09  14:30:04  hines
 * partial port to djg dos extender go32
 *
 * Revision 2.63  1993/11/04  15:55:48  hines
 * port to solaris2 (no more warnings)
 *
 * Revision 2.62  1993/10/18  13:24:19  hines
 * A start on an audit system. see audit.c
 *
 * Revision 2.59  93/08/19  16:38:19  hines
 * if (!0) print "hello"
 * now works. before the ! was not transmitted from yylex as NOT
 * and print "hello" was given the wrong $$ code index
 * 
 * Revision 2.56  93/08/14  15:03:54  hines
 * when in multiline statement typed directly to interpreter from console
 * we shut off the event driven interface so that there can be no 
 * extraneous statements mixed with the current statement.
 * 
 * Revision 2.55  93/08/13  17:11:23  hines
 * fix bug where we did not allow multiple line statements entered
 * from console as direct command
 * 
 * Revision 2.40  93/04/13  12:11:33  hines
 * SIG_RETURN already defined in HPUX : so defined as SIG_RETURN_TYEP
 * 
 * Revision 2.39  93/04/12  14:08:36  hines
 * first pass at merging Fisher's LINDA addition
 * 
 * Revision 2.27  93/02/22  13:12:26  hines
 * works with turboc on DOS (but no readline)
 * 
 * Revision 2.22  93/02/12  08:51:37  hines
 * beginning of port to PC-Dos
 * 
 * Revision 2.20  93/02/03  11:27:44  hines
 * a bit more generic for portability. will possibly work on NeXT
 * 
 * Revision 2.19  93/02/02  10:34:27  hines
 * static functions declared before used
 * 
 * Revision 2.14  93/01/21  13:19:21  hines
 * // commenst out the rest of the line
 * 
 * Revision 2.8  93/01/12  16:53:15  hines
 * "str" = strdef was being allowed
 * 
 * Revision 1.38  92/11/27  08:19:30  hines
 * can turn off error messages temporarily
 * 
 * Revision 1.37  92/10/24  12:12:30  hines
 * botched last checkin. now its right.
 * 
 * Revision 1.36  92/10/24  12:01:10  hines
 * portability fixes found at Pittsburgh for CRAY and HP
 * 
 * Revision 1.35  92/10/24  10:16:25  hines
 * noiv.c is place to put function stubs that only make sense with interviews
 * whenever any VAR is freed it should be done with hoc_free_val... which
 * will notify the interviews objects that hold pointers to variables.
 * 
 * Revision 1.34  92/10/23  14:07:26  hines
 * only complete statements can be parsed in hoc_oc for now
 * also resetting parser yystart on hoc-execerror
 * object_name(Object*) returns object name
 * 
 * Revision 1.33  92/10/23  08:20:56  hines
 * fix some memory leaks with purify having to do with failing to free
 * arayinfo
 * 
 * Revision 1.32  92/10/07  15:42:45  hines
 * arg syntax changed to $s1 and $o1 because O looked too much like 0
 * 
 * Revision 1.31  92/10/07  13:51:09  hines
 * strings can be passed as args and read and set with $S1 = $S2
 * 
 * Revision 1.30  92/08/18  07:31:39  hines
 * arrays in different objects can have different sizes.
 * Now one uses araypt(symbol, SYMBOL) or araypt(symbol, OBJECTVAR) to
 * return index of an array variable.
 * 
 * Revision 1.29  92/08/17  12:49:32  hines
 * arrays in different objects can have different sizes. Now arayinfo
 * gets carried along in objectdata. arayinfo is reference counted.
 * 
 * Revision 1.28  92/08/15  12:41:27  hines
 * can pass object variable as arg to oc functions and procedures. Within
 * the procedures the syntax is
 * $$1.ob.ob.var etc.
 * see test7.hoc for usage with array of strings
 * 
 * Revision 1.27  92/08/12  10:45:33  hines
 * Changes of sejnowski lab. also gets sred from hoc. Major addition is
 * a new x.c which is modified for screen updating following an expose
 * event. This is in x_sejnowski.c and will compile to x.o when
 * Sejnowski is defined as 1 in Imakefile (don't forget imknrn -a when
 * changed and delete old x.o)
 * Does not contain get_default on this checkin
 * 
 * Revision 1.26  92/07/31  16:16:54  hines
 * float_epsilon used for logical comparisons, int(), indexing and short for
 * default value is 1e-11
 * Hopfully no one has to worry about roundoff for the loops to come out
 * right.
 * 
 * 
 * Revision 1.25  92/07/31  14:16:56  hines
 * better handling of errno as in hoc
 * 
 * Revision 1.24  92/03/27  14:05:19  hines
 * turn off readline event hook if hoc_interviews becomes 0
 * 
 * Revision 1.23  92/03/11  13:55:52  hines
 * hoc.c inadvertently checked in last time with sigsegvec removed
 * 
 * Revision 1.22  92/03/11  13:51:15  hines
 * all parse errors now hoc_execerror
 * 
 * Revision 1.21  92/03/06  16:01:16  hines
 * hoc_oc can be used while executing. uses rinitcode(). But a problem
 * exists when single tokens are parsed.
 * 
 * Revision 1.20  92/03/05  14:45:31  hines
 * number that terminates string demonstrated a bug in ungetc. now
 * we test to make sure we don't unget a '\0'
 * 
 * Revision 1.19  92/03/05  13:24:46  hines
 * fix problem where not initcode after a execution by hoc_oc()
 * xvalue added
 * 
 * Revision 1.18  92/02/26  10:27:45  hines
 * xview removed. nrnoc verified (needed fixes) for reversed section syntax.
 * 
 * Revision 1.17  92/02/21  17:01:07  hines
 * new signal handling. hoc_oc() gets a longjump and returns. on return
 * the normal longjump is to hoc_run1()
 * 
 * Revision 1.16  92/02/21  14:38:53  hines
 * Completely event driven with event loop from InterViews.
 * 
 * Revision 1.15  92/02/20  10:54:42  hines
 * after em call from hoc_oc, execute the buffer
 * 
 * Revision 1.14  92/02/06  12:10:40  hines
 * Can have comments on same line with no tokens in between.
 * error fixed in translation of range vector with explicit section as first
 * item in a statement.
 * 
 * Revision 1.13  92/02/05  08:42:13  hines
 * Can connect a NEURON {POINTER var,...} in models to external variables using
 * connect var_suffix, anyvar
 * 
 * Revision 1.12  92/02/03  11:22:42  hines
 * rudimentary event driven version of oc.
 * 
 * Revision 1.11  92/02/03  11:21:32  hines
 * rudimentary event driven version of oc. tested with the
 * interviews based simpleeditor.
 * 
 * Revision 1.10  92/01/30  08:17:12  hines
 * bug fixes found in hoc incorporated. if()return, no else, objectcenter
 * warnings.
 * 
 * Revision 1.9  91/11/18  10:52:46  hines
 * xview menus. Doesn't work too well. ^C exits altogether.
 * 
 * Revision 1.8  91/10/30  14:40:16  hines
 * stack.hoc works and can be reloaded
 * 
 * Revision 1.7  91/10/29  15:04:58  hines
 * emacs command line editor with history
 * 
 * Revision 1.6  91/10/25  09:32:44  hines
 * object arrays. stack.hoc works. need to work on stop command and
 * necessity for braces around single statements in for loops.
 * 
 * Revision 1.5  91/10/22  15:47:09  hines
 * objects and components. can assign, evaluate, and print.
 * No object arrays or function components yet.
 * At level of test3.hoc
 * 
 * Revision 1.4  91/10/18  14:40:37  hines
 * symbol tables now are type Symlist containing pointers to first and last
 * symbols.  New symbols get added onto the end.
 * 
 * Revision 1.3  91/10/17  15:01:29  hines
 * VAR, STRING now handled with pointer to value located in object data space
 * to allow future multiple objects. Ie symbol for var, string, objectvar
 * has offset into a pointer data space.
 * 
 * Revision 1.2  91/10/14  17:36:09  hines
 * scaffolding for oop in place. Syntax about right. No action yet.
 * 
 * Revision 1.1  91/10/11  11:12:04  hines
 * Initial revision
 * 
 * Revision 4.43  91/10/01  11:34:21  hines
 * gather console input at one place in preparation for adding
 * emacs like command line editing. Hoc input now reads entire line.
 * 
 * Revision 4.41  91/08/23  08:28:48  hines
 * minor mods to get distribution on RS6000 with no warnings.
 * 
 * Revision 4.36  91/08/13  10:57:52  hines
 * should work on rs6000 when SYSV is set and -lcurses instead of -ltermlib
 * 
 * Revision 4.35  91/08/12  11:28:09  hines
 * makefile a bit more generic. hoc_e.c can work if Char is
 * defined as an signed char on any machine.
 * 
 * Revision 4.22  91/04/30  16:19:33  hines
 * in hoc_run need to check if pipe_flag==1 to see if we rerun with stdin.
 * 
 * Revision 4.21  91/04/30  11:02:21  hines
 * allow hoc to be part of another program. Use hoc_main1(argc, argv, env)
 * on return hoc can be restarted by hoc_run() (but be sure to reopen the
 * stdin with clearerr(stdin). on final exit be sure to call hoc_final_exit()
 * to save edit buffers.
 * 
 * Revision 4.20  91/04/03  16:01:25  hines
 * mistyped ||
 * 
 * Revision 3.108  90/10/24  09:44:11  hines
 * saber warnings gone
 * 
 * Revision 3.107  90/10/24  09:06:28  hines
 * device dependent close plot moved to plot.c
 * 
 * Revision 3.86  90/07/31  14:06:17  hines
 * call init before calling init_plot
 * 
 * Revision 3.62  90/05/20  16:09:20  mlh
 * opens separate plot window and closes it on exit.
 * uses retained segment but facility remains for temporary segment
 * 
 * Revision 3.50  90/02/17  10:12:28  mlh
 * lint free on sparc and makfile good for sparc
 * 
 * Revision 3.46  90/01/24  06:37:44  mlh
 * emalloc() and ecalloc() are macros which return null if out of space
 * and then call execerror.  This ensures that pointers are set to null.
 * If more cleanup necessary then use hoc_Emalloc() followed by hoc_malchk()
 * 
 * Revision 3.33  89/10/21  10:50:48  mlh
 * mistake on cray checkin
 * 
 * Revision 3.32  89/10/21  10:31:41  mlh
 * works on CRAY
 * 
 * Revision 3.27  89/09/29  16:01:03  mlh
 * ecalloc needed in some places to initialize allocated space
 * 
 * Revision 3.23  89/09/14  14:33:22  mlh
 * for turboc 2.0 void the signal and malloc, and remove calloc
 * 
 * Revision 3.20  89/08/15  08:29:47  mlh
 * compiles under turbo-c 1.5 -- some significant bugs found
 * 
 * Revision 3.16  89/08/10  11:14:38  mlh
 * return a single . when not part of number
 * 
 * Revision 3.7  89/07/13  08:21:34  mlh
 * stack functions involve specific types instead of Datum
 * 
 * Revision 3.4  89/07/12  10:27:04  mlh
 * Lint free
 * 
 * Revision 3.3  89/07/10  15:46:04  mlh
 * Lint pass1 is silent. Inst structure changed to union.
 * 
 * Revision 2.0  89/07/07  11:31:03  mlh
 * Preparation for newcable
 * 
 * Revision 1.1  89/07/07  11:16:05  mlh
 * Initial revision
 * 
*/

/* 5-5-89 call hoc_menu_cleanup() in execerror */
/*version 7.2.1 2-jan-89 */
#include "hoc.h"
#include "equation.h"
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include "ytab.h"
#if defined(__GO32__)
#include <dos.h>
#include <go32.h>
#endif

int pipeflag;
#if -1 == '\377'
#define CHAR char
#else
#define CHAR signed char
#endif
#define CBUFSIZE 300
static CHAR	cbuf[CBUFSIZE];
static CHAR	*ctp = cbuf;
int hoc_ictp;
extern Symlist *symlist;	/* This list is permanent */
extern Symlist *p_symlist; /* Constants, strings, auto variables */
extern char *RCS_hoc_version;
extern char *RCS_hoc_date;

#define EPS hoc_epsilon
extern double EPS;
extern arayinstal();
FILE	*fin;				/* input file pointer */

#include <ctype.h>
char	*progname;	/* for error messages */
int	lineno;

#include <signal.h>
#include <setjmp.h>
static int control_jmpbuf = 0; /* don't change jmp_buf if being controlled */
jmp_buf begin;
static int hoc_oc_jmpbuf;
static jmp_buf hoc_oc_begin;
int	intset;		/* safer interrupt handling */
int	indef;
char	*infile;	/* input file name */
char	**gargv;	/* global argument list */
int	gargc;
static int c = '\n';	/* global for use by warning() */



#if LINDA

int hoc_retreat_flag;
#define RETREAT_SIGNAL SIGHUP

static
SIG_RETURN_TYPE
retreat_handler(sig) int sig;	/* catch interrupt */
{
	/*ARGSUSED*/
	if (hoc_retreat_flag++) {
		/* never managed the first one properly */
		exit(1);
	}
	if (!lookup("linda_retreat")) {
		hoc_retreat_flag = 0;
		/* user did not give us a safe retreat so it would be nice */
		/* to take up later exactly at this point */
		exit(1);
	}
	IGNORE(signal(RETREAT_SIGNAL, retreat_handler));
}

hoc_retreat() {
	hoc_obj_run("linda_retreat()\n", (Object*)0);
	exit(0);
}

#endif

#define HAS_SIGPIPE 1
#if HAS_SIGPIPE
/*ARGSUSED*/
static
SIG_RETURN_TYPE
sigpipe_handler(sig) int sig; {
	fprintf(stderr, "writing to a broken pipe\n");
	signal(SIGPIPE, sigpipe_handler);
}
#endif

int
getnb()	/* get next non-white character */
{
	int c;

	/*EMPTY*/
	while ((c=Getc(fin)) == ' ' || c == '\t') {
		;
	}
	return c;
}

/* yylex modified to return -3 when at end of cbuf. The parser can
   return and take up where it left off later. Supported by modification
   of bison.simple to allow event driven programming.
*/
#define YYNEEDMORE	-3
/* for now we say comments or strings that span lines on stdin are in error */
#if 0
#define INCOMMENT	1;
#define INSTRING	2;
static int lexstate = 0;
#endif
/* sometimes is... doesn't work with -3. Hence Getc returns null and look
   at eos to see if true.*/
static int eos;

yylex()			/* hoc6 */
{
   restart:	/* when no token in between comments */
	eos = 0;
#if 0 /* do we really want to have several states? */
	switch (lexstate) {

	case INCOMMENT:
		goto incomment;
	case INSTRING:
		goto instring;
	}
#endif

	if ((c=getnb()) == EOF) {
		return 0;
	}
	if (c == '/' && follow('*', 1, 0))	/* comment */
	{
		while ((c=Getc(fin)) != '*' || follow('/', 0, 1)) {
			if (c == EOF)
				return(0);
/*			if (c == YYNEEDMORE) {*/
			if (eos) {
				acterror("comment not complete", " in cbuf");
			}
		}
		goto restart;
	}
	if (c == '.' || isdigit(c))	/* number */
	{
		char *npt;
		double d;
		IGNORE(unGetc(c, fin));
		npt = (char *)ctp;
		/*EMPTY*/
		while (isdigit(c = Getc(fin))) {
			;
		}
		if (c == '.') {
			/*EMPTY*/
			while(isdigit(c = Getc(fin))) {
				;
			}
		}
		if (*npt == '.' && !isdigit(npt[1])) {
			IGNORE(unGetc(c, fin));
			return  (int)(*npt);
		}
		if (c == 'E' || c == 'e') {
			if (isdigit(c = Getc(fin)) || c == '+' || c == '-') {
				/*EMPTY*/
				while (isdigit(c = Getc(fin))) {
					;
				}
			}
		}
		IGNORE(unGetc(c, fin));
		IGNORE(sscanf(npt, "%lf", &d));
		if (d == 0.)
			return NUMZERO;
		yylval.sym = install("", NUMBER, d, &p_symlist);
		return NUMBER;
	}
	if (isalpha(c))
	{
		Symbol *s;
		char sbuf[100], *p = sbuf;
		do
		{
			*p++ = c;
		} while ((c=Getc(fin)) != EOF && (isalnum(c) || c == '_'));
		IGNORE(unGetc(c, fin));
		*p = '\0';
		if ((s=lookup(sbuf)) == 0)
			s = install(sbuf, UNDEF, 0.0, &symlist);
		yylval.sym = s;
		return s->type == UNDEF ? VAR : s->type;
	}
	if (c == '$') {	/* argument? */
		int n = 0;
		int retval = follow('o', OBJECTARG, ARG);
		if (retval == ARG) retval = follow('s', STRINGARG, ARG);
		while (isdigit(c=Getc(fin)))
			n = 10 * n + c - '0';
		IGNORE(unGetc(c, fin));
		if (n == 0)
			acterror("strange $...", (char *)0);
		yylval.narg = n;
		return retval;
	}
	if (c == '"')	/* quoted string */
	{
		char sbuf[256], *p;
		for (p = sbuf; (c=Getc(fin)) != '"'; p++)
		{
			if (c == '\n' || c == EOF || c == YYNEEDMORE)
				acterror("missing quote", "");
			if (p >= sbuf + sizeof(sbuf) -1)
			{
				*p = '\0';
				acterror("string too long", sbuf);
			}
			*p = backslash(c);
		}
		*p = 0;
		yylval.sym = install("", CSTRING, 0.0, &p_symlist);

		(yylval.sym)->u.cstr = (char *)emalloc((unsigned)(strlen(sbuf)+1));
		Strcpy((yylval.sym)->u.cstr, sbuf);
		return CSTRING;
	}
	switch (c)
	{
	case 0:		{if (eos) return YYNEEDMORE; else return 0;}
	case '>':	return follow('=', GE, GT);
	case '<':	return follow('=', LE, LT);
	case '!':	return follow('=', NE, NOT);
	case '=':
		{	int i;
			i = follow('=', EQ, '=');
			if (do_equation && i == '=')
				return EQNEQ;
			else
			return i;
		}
	case '|':	return follow('|', OR, '|');
	case '&':	return follow('&', AND, '&');
	case '\\':
		{	int i;	/* continuation line if last char in line is \ */
			i = follow('\n', 1000, '\\');
			if (i == 1000) {
				return yylex();
			}
			return i;
	}
	case '\n':	return '\n';
	case '/':	if (follow('/', 1, 0)) {
				while(*ctp) {
					++ctp;
				}
				return '\n';
			}else{
				return '/';
			}
	default:	return c;
	}
}

backslash(c)	/* get next char with \'s interpreted */
	int c;
{
	static char transtab[] = "b\bf\fn\nr\rt\t";
	if (c != '\\')
		return c;
	c = Getc(fin);
	if (islower(c) && strchr(transtab, c))
		return strchr(transtab, c)[1];
	return c;
}

follow(expect, ifyes, ifno)	/* look ahead for >=, etc. */
{
	int c = Getc(fin);

	if (c == expect)
		return ifyes;
	IGNORE(unGetc(c, fin));
	return ifno;
}

arayinstal()	/* allocate storage for arrays */
{
	int i, nsub;
	Symbol * sp;
#if	defined(__TURBOC__)
	Inst *pcc;	/* sometimes pop messes up pc */
#endif

	nsub = (pc++)->i;
#if	defined(__TURBOC__)
	pcc = pc;
#endif
	sp = spop();

	hoc_freearay(sp);
	sp->type = VAR;
	sp->s_varn = 0;
	i = hoc_arayinfo_install(sp, nsub);
	if ((OPVAL(sp) = (double *) hoc_Ecalloc((unsigned)i, sizeof(double)))
	== (double *) 0) {
		hoc_freearay(sp);
		Fprintf(stderr, "Not enough space for array %s\n", sp->name);
		hoc_malchk();
		hoc_execerror("", (char *)0);
	}
#if	defined(__TURBOC__)
	pc = pcc;
#endif
}

int
hoc_arayinfo_install(sp, nsub)
	Symbol *sp;
{
	double total, subscpt;
	int i;
	free_arrayinfo(sp->arayinfo);
	sp->arayinfo = (Arrayinfo *) emalloc((unsigned)(
			sizeof(Arrayinfo)+nsub*sizeof(int)));
/*printf("emalloc arrayinfo at %lx\n", sp->arayinfo);*/
	sp->arayinfo->a_varn = (unsigned *) 0;
	sp->arayinfo->nsub = nsub;
	sp->arayinfo->refcount = 1;
	total = 1.;
	while (nsub)
	{
		subscpt = floor(xpop() + EPS);
		if (subscpt <= 0.)
			execerror("subscript < 1", sp->name);
		total = total*subscpt;
		sp->arayinfo->sub[--nsub] = (int)subscpt;
	}
	if ( (double)(i = (int)total) != total) {
		free((char *)sp->arayinfo);
		sp->arayinfo = (Arrayinfo *)0;
		execerror(sp->name, ":total subscript too large");
	}
	if (OPARINFO(sp)) {
/* probably never get here */
		free_arrayinfo(OPARINFO(sp));
	}
	OPARINFO(sp) = sp->arayinfo;
	++sp->arayinfo->refcount;
	return i;
}

hoc_freearay(sp)
	Symbol *sp;
{
	Arrayinfo** pa = &(OPARINFO(sp));
	if (sp->type == VAR)
	{
		hoc_free_val_array(OPVAL(sp), hoc_total_array(sp));
		sp->type = UNDEF;
	}
	free_arrayinfo(*pa);
	free_arrayinfo(sp->arayinfo);
	sp->arayinfo = (Arrayinfo *)0;
	*pa = (Arrayinfo *)0;
}

free_arrayinfo(a)
	Arrayinfo* a;
{
	if (a != (Arrayinfo *) 0)
	{
	    if ((--a->refcount) <= 0) {
		if (a->a_varn != (unsigned *) 0)
			free((char *)(a->a_varn));
		free((char *)a);
/*		printf("free arrayinfo at %lx\n", a);*/
	    }
	}
}

defnonly(s)	/* warn if illegal definition */
	char *s;
{
	if (!indef)
		acterror(s, "used outside definition");
}

/* messages can turned off but the user had better check the return
value of oc_run()
*/
static int debug_message_;
hoc_show_errmess_always() {
	double x;
	x = chkarg(1, 0., 1.);
	debug_message_ = (int)x;
	ret();
	pushx(x);
}

int hoc_execerror_messages;

/* this is possibly non-portable since it is based on the declaration in
	setjmp.h of
	typedef int jmp_buf[_JBLEN];
*/
void (*oc_jump_target_)();	/* see ivoc/SRC/ocjump.c */


int yystart;

execerror(s, t)	/* recover from run-time error */
	char *s, *t;
{
	extern int hoc_in_yyparse;
	hoc_in_yyparse = 0;
	yystart = 1;
	hoc_menu_cleanup();
	hoc_errno_check();
#if 0
	hoc_xmenu_cleanup();
#endif
	if (debug_message_ || hoc_execerror_messages) {
		warning(s, t);
		frame_debug();
#if defined(__GO32__)
		{extern int egagrph;
		if (egagrph) {
			hoc_outtext("Error:");
			hoc_outtext(s);
			if (t) {
				hoc_outtext(" ");
				hoc_outtext(t);
			}
			hoc_outtext("\n");
		}}
#endif
	}
	if (oc_jump_target_) {
		(*oc_jump_target_)();
	}
	hoc_execerror_messages = 1;
	if (pipeflag == 0)
		IGNORE(fseek(fin, 0L, 2));	/* flush rest of file */
	hoc_oop_initaftererror();
	if (hoc_oc_jmpbuf) {
		longjmp(hoc_oc_begin, 1);
	}
	longjmp(begin, 1);
}

SIG_RETURN_TYPE
onintr(sig) int sig;	/* catch interrupt */
{
	/*ARGSUSED*/
	if (intset++)
		execerror("interrupted", (char *) 0);
	IGNORE(signal(SIGINT, onintr));
}

#if DOS
#include <float.h>
#endif

SIG_RETURN_TYPE
fpecatch(sig) int sig;	/* catch floating point exceptions */
{
	/*ARGSUSED*/
#if DOS
_fpreset();
#endif
	execerror("floating point exception", (char *) 0);
}

static int coredump;

hoc_coredump_on_error() {
	coredump = 1;
	ret();
	pushx(1.);
}

#if HAS_SIGSEGV
SIG_RETURN_TYPE
sigsegvcatch(sig) int sig;	/* segmentation violation probably due to arg type error */
{
	/*ARGSUSED*/
	if (coredump) {
		abort();
	}
	execerror("Segmentation violation", "See $NEURONHOME/lib/help/oc.help");
}
#endif

#if HAS_SIGBUS
SIG_RETURN_TYPE
sigbuscatch(sig) int sig;
{
	/*ARGSUSED*/
	if (coredump) {
		abort();
	}
	execerror("Bus error", "See $NEURONHOME/lib/help/oc.help");
}
#endif

int hoc_pid() { return (int)getpid();} /* useful for making unique temporary file names */

/* has got to be called first. oc can only be event driven after this returns */
hoc_main1_init(pname, envp)
	char *pname, **envp;
{
	static int inited = 0;
	
	yystart = 1;
	lineno = 1;
	if (inited) {
		return;
	}
	inited = 1;

	if (!parallel_sub) {
	    if (!nrn_is_cable()) {
	      Fprintf(stderr, "OC INTERPRETER   %s   %s\n",
		    RCS_hoc_version, RCS_hoc_date);
	      Fprintf(stderr,
"Copyright 1992 -  Michael Hines, Neurobiology Dept., DUMC, Durham, NC.  27710\n");
	    }
	}
	progname = pname;
	if (setjmp(begin)) {
		exit(1);
	}

	save_parallel_envp(envp);

	init();
	initplot(envp);
#if defined(__GO32__)
	setcbrk(0);
#endif
}

static hoc_run1();

hoc_main1(argc, argv, envp)	/* hoc6 */
	char *argv[], *envp[];
{
#if 0
	int controlled;
#endif
	save_parallel_argv(argc, argv);

	hoc_audit_from_hoc_main1(argc, argv, envp);
	hoc_main1_init(argv[0], envp);
#if LINDA
	signal(RETREAT_SIGNAL, retreat_handler);
#endif
#if HAS_SIGPIPE
	signal(SIGPIPE, sigpipe_handler);
#endif	
#if 0
	controlled = control_jmpbuf;
	if (!controlled) {
		control_jmpbuf = 1;
		if (setjmp(begin)) {
			control_jmpbuf = 0;
			return 1;
		}
	}
	if (!controlled) {
		control_jmpbuf = 0;
	}
#endif
	gargv = argv;
	gargc = argc;
	if (argc > 1 && argv[1][0] != '-') {
		/* first file may be a checkpoint file */
		extern int hoc_readcheckpoint();
		switch (hoc_readcheckpoint(argv[1])) {
		case 1:
			++gargv;
			--gargc;
			break;
		case 2:
			exit(1);
			break;
		default:
			break;
		}			

	}
	if (gargc == 1)	/* fake an argument list */
	{
		static char *stdinonly[] = { "-" };

		gargv = stdinonly;
		gargc = 1;
	}
	else
	{
		++gargv;
		--gargc;
	}

	while (moreinput())
		hoc_run1();
	return 0;
}
	
hoc_final_exit() {
	char buf[256];
	hoc_audit_from_final_exit();
	hoc_edit_quit();

	/* Don't close the plots for the sub-processes when they finish,
	   by default they are then closed when the master process ends */
	NOT_PARALLEL_SUB(hoc_close_plot();)
	
#if defined(__GO32__)
	sprintf(buf, "sh $NEURONHOME/lib/cleanup %d", hoc_pid());
#else
	sprintf(buf, "$NEURONHOME/lib/cleanup %d", hoc_pid());
#endif
	system(buf);
}
	
hoc_quit() {
	hoc_final_exit();
	exit(0);
}

moreinput()
{
	extern char hoc_xopen_file_[];
	if (pipeflag)
	{
		pipeflag = 0;
		return 1;
	}
	if (gargc-- <= 0)
		return 0;
	if (fin && fin != stdin)
		IGNORE(fclose(fin));
	infile = *gargv++;
	lineno = 1;
	if (strcmp(infile, "-") == 0)
	{
		fin = stdin;
		infile = 0;
	}
	else if (strcmp(infile, "-parallel") == 0) 
	{
	        /* ignore "val" as next argument */
	        infile = *gargv++;
		gargc--; 
	        return moreinput();
	}
	else if ((fin=fopen(infile, "r")) == NULL)
	{
		Fprintf(stderr, "%s: can't open %s\n", progname, infile);
		return moreinput();
	}
	if (infile) {
		strcpy(hoc_xopen_file_, infile);
	}
	return 1;
}

#if 0
hoc_run() {
	hoc_run1();
	while (pipeflag == 1) {
		pipeflag = 0;
		hoc_run1();
	}
}
#endif

typedef SIG_RETURN_TYPE (*SignalType)();

static SignalType signals[4];

static
set_signals() {
	signals[0] = signal(SIGINT, onintr);
	signals[1] = signal(SIGFPE, fpecatch);
#if HAS_SIGSEGV
	signals[2] = signal(SIGSEGV, sigsegvcatch);
#endif
#if HAS_SIGBUS
	signals[3] = signal(SIGBUS, sigbuscatch);
#endif
}

static
restore_signals() {
	signals[0] = signal(SIGINT, signals[0]);
	signals[1] = signal(SIGFPE, signals[1]);
#if HAS_SIGSEGV
	signals[2] = signal(SIGSEGV, signals[2]);
#endif
#if HAS_SIGBUS
	signals[3] = signal(SIGBUS, signals[3]);
#endif
}
	
static
hoc_run1()	/* execute until EOF */
{
	int controlled = control_jmpbuf;
	if (!controlled) {
		set_signals();
		control_jmpbuf = 1;
		IGNORE(setjmp(begin));
		intset = 0;
	}
	hoc_execerror_messages = 1;
	if (pipeflag == 1) {	/*at this location multiple emacs errors */
		hoc_pipeflush(); /* don't eat up stack space */
	}else{
		pipeflag=0;
	}
	for (initcode(); hoc_yyparse(); initcode())
		execute(progbase);
	if (intset)
		execerror("interrupted", (char *)0);
	if (!controlled) {
		restore_signals();
		control_jmpbuf = 0;
	}
}

/* event driven interface to oc. This routine always returns after processing
   its argument. The normal case is where the argument consists of a complete
   statement which can be parsed and executed. However this routine can
   be called with single tokens as well and the parsing will proceed
   incrementally until the last token that completes a statement is seen
   at which time the statement is executed.
   The routine can also be called with a string containing multiple statements
   separated by '\n's.
   During execution a ^C will halt
   processing and this routine will return to the calling statement. (if it
   is the controlling call for the jmp_buf; see below).
   Parse errors and execution errors will also result in a normal return to the
   calling statement.

   To avoid blocking other input events, this routine should not be called
   with statements which take too much time to execute or block
   waiting for input. With regard to long executing statements, at least in
   xview it is possible to run the event loop from the execute loop at
   intervals in code.c or else to run the event loop once via a hoc command.

   Where do we go in case of an hoc_execerror. We have to go to the beginning
   of hoc. But just maybe that is here. However hoc_oc may be called
   recursively. Or it may be called from the original hoc_run. Or it may be
   There is therefore a notion of the controlling routine for the jmp_buf begin.
   We only do a setjmp and set the signals
   when there is no other controlling routine.
*/

int
hoc_oc(buf) char *buf; {
	char *cp;
	int controlled;
#if 0
	int yret;
#endif
	
	controlled = hoc_oc_jmpbuf || oc_jump_target_;
	if (!controlled) {
		hoc_oc_jmpbuf = 1;
		if (setjmp(hoc_oc_begin)) {
			hoc_oc_jmpbuf = 0;
			restore_signals();
			initcode();
			intset = 0;
			return 1;
		}
		set_signals();
	}
	intset = 0;

	cp = buf;
	while (*cp) {
		for (ctp = cbuf; *cp;) {
			*ctp++ = *cp++;
			if (ctp[-1] == '\n') {
				break;
			}
		}
		*ctp = '\0';
		ctp = cbuf;	

#if 1
		hoc_ParseExec(yystart);
#else
		if (yystart) {
			initcode();
		}
		yret = yyparse();
		switch (yret) {
		case 1:
			execute(progbase);
			initcode();
			break;
		case 'e':
			hoc_edit();
			for (initcode(); hoc_yyparse(); initcode())
				execute(progbase);
			break;
		default:
			break;
		}
#endif
		if (intset) {
			execerror("interrupted", (char *)0);
		}
	}
	
	if (!controlled) {
		hoc_oc_jmpbuf = 0;
		restore_signals();
	}
	hoc_execerror_messages = 1;
	return 0;
}

warning(s, t)	/* print warning message */
	char *s, *t;
{
	CHAR *cp;
	Fprintf(stderr, "%s: %s", progname, s);
	if (t)
		Fprintf(stderr, " %s", t);
	if (infile)
		Fprintf(stderr, " in %s", infile);
	Fprintf(stderr, " near line %d", lineno);
	Fprintf(stderr, "\n%s", cbuf);
	for (cp = cbuf; cp != ctp; cp++) {
		IGNORE(putc(' ', stderr));
	}
	Fprintf(stderr,"^\n");
	ctp = cbuf;
	*ctp = '\0';
}


int
Getc(fp)
	FILE *fp;
{
	/*ARGSUSED*/
	if (*ctp) {
		++hoc_ictp;
		return *ctp++;
	}

#if 0
/* don't allow parser to block. Actually partial statements were never
allowed anyway */
	if (!pipeflag && fp == stdin) {
		eos = 1;
		return 0;
	}
#endif

	if (hoc_get_line() == EOF) {
		return EOF;
	}
	return *ctp++;
}

unGetc(c, fp)
	int c;
	FILE *fp;
{
	/*ARGSUSED*/
	if (c != EOF && c && ctp != cbuf) {
		*(--ctp) = c;
	}
}

int hoc_in_yyparse=0;

int hoc_yyparse() {
	/* read line before calling yyparse() and set flag that we
	are inside yyparse() since re-entry is not allowed.
	This allows xview menus to work since most of the time
	we are reading a line from this point instead of in
	Getc which was called from yyparse().
	*/
	/*
	the above may be obsolete.
	I switched to a modified bison generator from yacc which is generated
	a parser exactly like it used to be but may return YYNEEDMORE=-3
	instead of blocking in yylex. If so another call to yyparse will
	take up exactly where it left off.  This makes it easier to
	support event driven processing.
	Maybe it's time to redo the get_line process which fills cbuf with
	the next line to read. A real event driven program would fill cbuf
	and then call yyparse() directly. yyparse() returns
	0 : end of file
	'\n' : ready to execute a command
	'e' : user gave it em command, you should go to an editor
	-3: need more input, not at a point where it accepts or rejects the
		input.
	*/

	int i;
	
	if (hoc_in_yyparse) {
		hoc_execerror("Cannot re-enter parser", (char *)0);
	}
	do {
		if (hoc_get_line() == EOF) {
			return 0;
		}
		hoc_in_yyparse = 1;
		i = yyparse();
		hoc_in_yyparse = 0;
		switch (i) {
		case 'e':
			i = '\n';
			hoc_edit();
			break;
		case -3 : /* need more input */
			hoc_in_yyparse = 1;
			i = '\n';
			break;
		}
	}while (i == '\n');
	return i;
}

#if defined(__TURBOC__) || defined(__GO32__)
#define READLINE 1
#else
#define READLINE 1
#endif

#if defined(__GO32__)
#define INTERVIEWS 1
#endif

#if INTERVIEWS
int hoc_interviews = 0;
extern int run_til_stdin(); /* runs the interviews event loop. Returns 1
				if there is input to be read. Returns 0
				if somebody said quit but there is no input
				*/

extern hoc_notify_value();
#if READLINE
static int event_hook() {
	return run_til_stdin();
}
extern Pfri rl_event_hook;
#endif
#endif

int hoc_get_line(){ /* supports re-entry. fill cbuf with next line */
	char *hoc_pipegets();
	if (*ctp) {
hoc_execerror("Internal error:", "Not finished with previous input line");
	}
	ctp = cbuf;
	*ctp = '\0';
	if (pipeflag) {
		if (hoc_pipegets(cbuf, CBUFSIZE) == (char *)0) {
			return EOF;
		}
	}else{
#if READLINE
		if (fin == stdin) { char *line, *readline(); int n;
#if INTERVIEWS
			if (hoc_interviews && !hoc_in_yyparse) {
				rl_event_hook = event_hook;
				hoc_notify_value();
			}else{
				rl_event_hook = (Pfri)0;
			}
#endif
			if ((line = readline("oc>")) == (char *)0) {
				extern int hoc_notify_stop;
				return EOF;
			}
#if defined(__GO32__)
			hoc_check_intupt(0);
#endif
			n = strlen(line);
			if (n >= CBUFSIZE) {
				free(line);
				hoc_execerror("input line too long", (char *)0);
			}
			strcpy((char*)cbuf, line);
			cbuf[n] = '\n';
			cbuf[n+1] = '\0';
			if (line && *line) {
				add_history(line);
			}
			free(line);
			hoc_audit_command(cbuf);
		}else{
			if (fgets((char*)cbuf, CBUFSIZE, fin) == (char *)0) {
				return EOF;
			}
		}
#else
#if INTERVIEWS
		if (fin == stdin && hoc_interviews && !hoc_in_yyparse) {
			run_til_stdin();
		}
#endif
		if (fgets(cbuf, CBUFSIZE, fin) == (char *)0) {
			return EOF;
		}
#endif
	}
	errno = 0;
	lineno++;
	ctp = cbuf;
	hoc_ictp=0;
	return 1;
}

hoc_help() {
#if INTERVIEWS
	if (hoc_interviews) {
		ivoc_help(cbuf);
	}else
#endif
	{
hoc_warning("Help only available from version with ivoc library", 0);
	}
ctp = cbuf + strlen(cbuf) - 1;
}

#if defined(__GO32__)
hoc_check_intupt(intupt) int intupt; {
	if (_go32_was_ctrl_break_hit()) {
		if (intupt) {
			execerror("interrupted", (char*)0);
		}
	}
}
#endif

