#include "hoc.h"
#include "ytab.h"
#include "hocassrt.h"
#include "list.h"

#define PDEBUG 0
extern Symlist  *hoc_symlist;
extern Symlist* hoc_built_in_symlist;
extern Object	**hoc_objpop();
extern double	*hoc_pxpop();
extern char	**hoc_strpop();
extern Symbol	*hoc_table_lookup();
extern Symbol	*hoc_install();
extern Object** hoc_objgetarg();

#if CABLE
#include "section.h"
int section_object_seen;
struct Section* nrn_sec_pop();
double* nrn_rangepointer();
#endif

#define PUBLIC_TYPE 1
#define EXTERNAL_TYPE 2
static call_constructor();
static free_objectdata();
extern Symbol* hoc_decl();

char* hoc_object_name(ob)
	Object* ob;
{
	static char s[100];
	if (ob) {
		Sprintf(s, "%s[%d]", ob->template->sym->name, ob->index);
	}else{
		Sprintf(s, "NULLobject");
	}
	return s;
}
	
int hoc_total_array(s) /* total number of elements in array pointer */
	Symbol *s;
{
	int total = 1, i;
	Arrayinfo* a = OPARINFO(s);
	if (a) {
		for (i = a->nsub-1; i>=0; --i) {
			total *= a->sub[i];
		}
	}
	return total;
}
	
int hoc_total_array_data(s, obd) /* total number of elements in array pointer */
	Symbol *s;
	Objectdata* obd;
{
	Arrayinfo* a;
	int total = 1, i;

	if (!obd) {
		a = s->arayinfo;
	}else switch (s->type) {
#if CABLE
	case RANGEVAR:
		a = s->arayinfo;
		break;
#endif
	default:
		a = obd[s->u.oboff + 1].arayinfo;
		break;
	}
	if (a) {
		for (i = a->nsub-1; i>=0; --i) {
			total *= a->sub[i];
		}
	}
	return total;
}
	
static int icntobjectdata=0;
Object* hoc_thisobject;
Objectdata *hoc_objectdata = (Objectdata *)0;
Objectdata *hoc_top_level_data;
extern Symlist *hoc_top_level_symlist;
static int icnttoplevel;

Objectdata* hoc_objectdata_save() {
	if (hoc_objectdata == hoc_top_level_data) {
		return 0;
	}else{
		return hoc_objectdata;
	}
}

Objectdata* hoc_objectdata_restore(obdsav) Objectdata* obdsav; {
	if (obdsav) {
		return obdsav;
	}else{
		return hoc_top_level_data;
	}
}

hoc_obvar_declare(sym, type, pmes)
	Symbol *sym;
	int type, pmes;
{
	
	if (sym->type != UNDEF) {
		return;
	}
	if (pmes && hoc_symlist == hoc_top_level_symlist) {
	        NOT_PARALLEL_SUB(printf("first instance of %s\n", sym->name);)
		sym->defined_on_the_fly = 1;
	}
	hoc_install_object_data_index(sym);
	sym->type = type;
	switch (type) {
	case VAR:
/*printf("hoc_obvar_declare %s\n", sym->name);*/
		OPVAL(sym) = (double *)emalloc(sizeof(double));
		break;
	case STRING:
		OPSTR(sym) = (char **)0;
		break;
	case OBJECTVAR:
		break;
#if CABLE
	case SECTION:
		OPSECITM(sym) = (struct Item**)0;
		break;
#endif
	default:
		hoc_execerror(sym->name, "can't declare this in obvar_declare");
		break;
	}
}

/*-----------------------------------------------*/
/* there exists a type stack which parallels the objects on the
regular stack. This give dynamic type checking */

#define STKDEBUG 0
#define NTYPESTACK  30
static int typestack[NTYPESTACK];
static int *typestackp = typestack;

static int poptypestack() {
	if (typestackp == typestack) {
		hoc_execerror("typestack underflow", (char *)0);
	}
#if STKDEBUG
printf("poptypestack(%d)\n", typestackp[-1]);
#endif
	return *(--typestackp);
}

static pushtypestack(type) int type; {
	if (typestackp == typestack+NTYPESTACK) {
		typestackp = typestack;
		hoc_execerror("typestack overflow", (char *)0);
	}
#if STKDEBUG
printf("pushtypestack(%d)\n", type);
#endif
	*(typestackp++) = type;
}

/* template stack so nested templates are ok */
typedef union {
	Symbol *sym;
	Symlist *symlist;
	Objectdata *odata;
	Object* o;
	int	i;
}Templatedatum;
#define NTEMPLATESTACK  20
static Templatedatum templatestack[NTEMPLATESTACK];
static Templatedatum *templatestackp = templatestack;

static Templatedatum *poptemplate() {
	if (templatestackp == templatestack) {
		hoc_execerror("templatestack underflow", (char *)0);
	}
	return (--templatestackp);
}

#define pushtemplatesym(arg) chktemplate(); (templatestackp++)->sym = arg
#define pushtemplatesymlist(arg) chktemplate(); (templatestackp++)->symlist = arg
#define pushtemplatei(arg) chktemplate(); (templatestackp++)->i = arg
#define pushtemplateodata(arg) chktemplate(); (templatestackp++)->odata = arg
#define pushtemplateo(arg) chktemplate(); (templatestackp++)->o = arg

static chktemplate(){
	if (templatestackp == templatestack+NTEMPLATESTACK) {
		templatestackp = templatestack;
		hoc_execerror("templatestack overflow", (char *)0);
	}
}
/*------------------------------------------------*/

/* mostly to allow saving of objects */

#define OBJ_STACK_SIZE 10
static Object* obj_stack_[OBJ_STACK_SIZE];
static obj_stack_loc;

hoc_object_push() {
	Object* ob = *hoc_objgetarg(1);
	if (ob->template->constructor) {
		hoc_execerror("Can't do object_push for built-in class", 0);
	}
	if (obj_stack_loc >= OBJ_STACK_SIZE) {
		hoc_execerror("too many object context stack depth", 0);
	}
	obj_stack_[obj_stack_loc++] = hoc_thisobject;
	hoc_thisobject = ob;
	if (ob) {
		hoc_symlist = ob->template->symtable;
		hoc_objectdata = ob->u.dataspace;
	}else{
		hoc_symlist = hoc_top_level_symlist;
		hoc_objectdata = hoc_top_level_data;
	}
	hoc_ret();
	pushx(0.);
}
	
hoc_object_pop() {
	Object* ob;
	if (obj_stack_loc < 1) {
		hoc_execerror("object context stack underflow", 0);
	}
	ob = obj_stack_[--obj_stack_loc];
	hoc_thisobject = ob;
	if (ob) {
		hoc_symlist = ob->template->symtable;
		hoc_objectdata = ob->u.dataspace;
	}else{
		hoc_symlist = hoc_top_level_symlist;
		hoc_objectdata = hoc_top_level_data;
	}
	hoc_ret();
	pushx(0.);
}
/*-----------------------------------------------*/
int hoc_resize_toplevel(more) int more; {
	if (more > 0) {
		icnttoplevel += more;
hoc_top_level_data = (Objectdata *)erealloc((char *)hoc_top_level_data,
				icnttoplevel*sizeof(Objectdata));
		if (templatestackp == templatestack) {
			hoc_objectdata = hoc_top_level_data;
		}
	}
	return icnttoplevel;
}

hoc_install_object_data_index(sp)
	Symbol *sp;
{
	
	if (!hoc_objectdata) {
		icntobjectdata = 0;
	}
	sp->u.oboff = icntobjectdata;
	icntobjectdata += 2;	/* data pointer and Arrayinfo */
	hoc_objectdata = (Objectdata *)erealloc((char *)hoc_objectdata,
				icntobjectdata*sizeof(Objectdata));
	hoc_objectdata[icntobjectdata-1].arayinfo = sp->arayinfo;
	if (sp->arayinfo) {
		++sp->arayinfo->refcount;
	}
	if (templatestackp == templatestack) {
		hoc_top_level_data = hoc_objectdata;
		icnttoplevel = icntobjectdata;
	}
}

int hoc_obj_run(cmd, ob)
	char* cmd;
	Object* ob;
{
	int err;
	Object* objsave;
	Objectdata* obdsave;
	Symlist* slsave;
	objsave = hoc_thisobject;
	obdsave = hoc_objectdata_save();
	slsave = hoc_symlist;

  	if (ob) {
		hoc_thisobject = ob;
		hoc_objectdata = ob->u.dataspace;
		hoc_symlist = ob->template->symtable;
	}else{
		hoc_thisobject = 0;
		hoc_objectdata = hoc_top_level_data;
		hoc_symlist = hoc_top_level_symlist;
	}
		
	err = hoc_oc(cmd);

	hoc_thisobject = objsave;
	hoc_objectdata = hoc_objectdata_restore(obdsave);
	hoc_symlist = slsave;

	return err;
}

hoc_exec_cmd() { /* execute string from top level or within an object context */
	int err;
	char* cmd;
	char buf[256];
	Object* ob = 0;
	cmd = gargstr(1);
	if (cmd[0] == '~') {
		sprintf(buf, "%s\n", cmd+1);
	}else{
		sprintf(buf, "{%s}\n", cmd);
	}
	if (ifarg(2)) {
		ob = *hoc_objgetarg(2);
	}
	err = hoc_obj_run(buf, ob);
	if (err) {
		hoc_execerror("execute error:", cmd);
	}
	hoc_ret();
	pushx((double)(err));
}

/* call a function within the context of an object. Args must be on stack */
double hoc_call_objfunc(s, narg, ob)
	Symbol* s;
	int narg;
	Object* ob;
{
	double d, hoc_call_func();
	Object* objsave;
	Objectdata* obdsave;
	Symlist* slsave;
	objsave = hoc_thisobject;
	obdsave = hoc_objectdata_save();
	slsave = hoc_symlist;

  	if (ob) {
		hoc_thisobject = ob;
		hoc_objectdata = ob->u.dataspace;
		hoc_symlist = ob->template->symtable;
	}else{
		hoc_thisobject = 0;
		hoc_objectdata = hoc_top_level_data;
		hoc_symlist = hoc_top_level_symlist;
	}
		
	d = hoc_call_func(s, narg);

	hoc_thisobject = objsave;
	hoc_objectdata = hoc_objectdata_restore(obdsave);
	hoc_symlist = slsave;

	return d;
}
	
hoc_oop_initaftererror() {
	hoc_symlist = hoc_top_level_symlist;
	typestackp = typestack;
	templatestackp = templatestack;
	icntobjectdata = icnttoplevel;
	hoc_objectdata = hoc_top_level_data;
	hoc_thisobject = (Object*)0;
	obj_stack_loc = 0;
}

oc_save_hoc_oop(a1, a2, a3, a4, a5)
	Object*		*a1;
	Objectdata*	*a2;
	int*		*a3;
	int		*a4;
	Symlist*	*a5;
{
	*a1 = hoc_thisobject;
	*a2 = hoc_objectdata;
	*a3 = typestackp;
	*a4 = obj_stack_loc;
	*a5 = hoc_symlist;
}

oc_restore_hoc_oop(a1, a2, a3, a4, a5)
	Object*		*a1;
	Objectdata*	*a2;
	int*		*a3;
	int		*a4;
	Symlist*	*a5;
{
	hoc_thisobject = *a1;
	hoc_objectdata = *a2;
	typestackp = *a3;
	obj_stack_loc = *a4;
	hoc_symlist = *a5;
}

Object* hoc_new_object(symtemp, v)
	Symbol* symtemp;
	void* v;
{
	Object* ob;
#if PDEBUG
	printf("new object from template %s created.\n", symtemp->name);
#endif
	ob = (Object *)emalloc(sizeof(Object));
	ob->refcount = 0;
	ob->observers = (void*)0;
	ob->template = symtemp->u.template;
	ob->itm_me = hoc_l_lappendobj(ob->template->olist, ob);
	ob->template->count++;
	ob->index = ob->template->index++;
	if (symtemp->subtype == CPLUSOBJECT) {
		ob->u.this_pointer = v;
		if (v) {
			hoc_template_notify(ob, 1);
		}
	}else{
		ob->u.dataspace = 0;
	}
	return ob;
}

hoc_new_object_asgn(obp, st, v)
	Object** obp;
	Symbol* st;
	void* v;
{
	hoc_dec_refcount(obp);
	*obp = hoc_new_object(st, v);
	hoc_obj_ref(*obp);
}

/*
At this time we are dealing uniformly with object variables and cannot
deal cleanly with objects.  Eventually it may be possible to put an
object pointer on the stack but not now.  Try to avoid using "functions
which return new objects" as arguments to other functions.  If this is
done then it may happen that when the stack pointer is finally used it
may point to a different object than when it was put on the stack. 
Things are safe when a temp_objvar is finally removed from the stack. 
Memory leakage will occur if a temp_objvar is passed as an arg but never
assigned to a full fledged object variable.  ie its reference count is 0
but unref will never be called on it. 
The danger is illustrated with
	proc p(obj.func_returning_object()) { // $o1 is on the stack
		print $o1	// correct object
		for i=0,100 {
			o = obj.func_returning_different_object()
			print i, $o1  //when i=50 $o1 will be different
		}
	}
In this case one should first assign $o1 to another object variable
and then use that object variable exclusively instead of $o1.
This also prevent leakage of the object pointed to by $o1.

If this ever becomes a problem then it is not too difficult to
implement objects on the stack with garbage collection.
*/

Object** hoc_temp_objptr(obj) Object* obj; {
#define MAX_TEMP_OBJ 50
	static Object* pobj[MAX_TEMP_OBJ];
	static int i = 0;
	i = (i+1)%MAX_TEMP_OBJ;
	pobj[i] = obj;
	return pobj + i;
}

Object** hoc_temp_objvar(symtemp, v)
	Symbol* symtemp;
	void* v;
{
	return hoc_temp_objptr(hoc_new_object(symtemp, v));
}

hoc_newobj() { /* template at pc+1 */
	Object *ob, **obp;
	Objectdata *obd;
	Symbol *sym, *s;
	int i, total;
	int narg;
	Datum* hoc_look_inside_stack();
	
	sym = (pc++)->sym;
	narg = (pc++)->i;
	if (poptypestack() != OBJECTVAR) {
		hoc_execerror(sym->name, "not being assigned to an objectvar");
	}
	ob = hoc_new_object(sym, (void*)0);
	obp = hoc_look_inside_stack(narg, OBJECTVAR)->pobj;
	ob->refcount = 1;
   if (sym->subtype == CPLUSOBJECT) {
	call_constructor(ob, sym, narg);
	hoc_nopop();
   }else{
	ob->u.dataspace = obd = (Objectdata *)ecalloc(ob->template->dataspace_size, sizeof(Objectdata));
	for (s = ob->template->symtable->first; s; s = s->next) {
		switch (s->type) {
		case VAR:
			if ((obd[s->u.oboff + 1].arayinfo = s->arayinfo) != (Arrayinfo*)0) {
				++s->arayinfo->refcount;
			}
			total = hoc_total_array_data(s, obd);
			obd[s->u.oboff].pval = (double *)emalloc(total*sizeof(double));
			for(i=0; i<total; i++) {
				(obd[s->u.oboff].pval)[i] = 0.;
			}
			break;
		case STRING:
			obd[s->u.oboff + 1].arayinfo = (Arrayinfo*)0;
			obd[s->u.oboff].ppstr = (char**)emalloc(sizeof(char*));
			*obd[s->u.oboff].ppstr = (char *)emalloc(sizeof(char));
			**(obd[s->u.oboff].ppstr) = '\0';
			break;
		case OBJECTVAR:
			if ((obd[s->u.oboff + 1].arayinfo = s->arayinfo) != (Arrayinfo*)0) {
				++s->arayinfo->refcount;
			}
			total = hoc_total_array_data(s, obd);
			obd[s->u.oboff].pobj = (Object **)emalloc(total*sizeof(Object *));
			for (i=0; i<total; i++) {
				(obd[s->u.oboff].pobj)[i] = (Object *)0;
			}
			if (strcmp(s->name, "this") == 0) {
				obd[s->u.oboff].pobj[0] = ob;
			}
			break;
#if CABLE
		case SECTION:
			if ((obd[s->u.oboff + 1].arayinfo = s->arayinfo) != (Arrayinfo*)0) {
				++s->arayinfo->refcount;
			}
			total = hoc_total_array_data(s, obd);
			obd[s->u.oboff].psecitm = (struct Item **)emalloc(total*sizeof(struct Item*));
			new_sections(ob, s, obd[s->u.oboff].psecitm, total);
			break;
#endif
		}
	}
	if (ob->template->init) {
		call_ob_proc(ob, ob->template->init, narg);
		hoc_nopop();
	}else{
		++narg; /* got to pop object pointer also */
		for (i=0; i < narg; ++i) {
			hoc_nopop();
		}
	}
   }
	hoc_dec_refcount(obp);
	*(obp) = ob;
	hoc_template_notify(ob, 1);
	hoc_pushobj(obp);
	pushtypestack(OBJECTVAR);
}

static call_constructor(ob, sym, narg)
	Object *ob;
	Symbol *sym;
	int narg;
{
	Inst *pcsav;
	Symlist *slsav;
	Objectdata *obdsav;
	Object* obsav;
	int *typestacksav;

	typestacksav = typestackp;
	slsav = hoc_symlist;
	obdsav = hoc_objectdata_save();
	obsav = hoc_thisobject;
	pcsav = pc;

	push_frame(sym, narg);
	ob->u.this_pointer = (ob->template->constructor)(ob);
	pop_frame();
	
	if (typestacksav != typestackp) {
		hoc_execerror("typestack wrong on return from constructor", (char *)0);
	}
	pc = pcsav;
	hoc_symlist = slsav;
	hoc_objectdata = hoc_objectdata_restore(obdsav);
	hoc_thisobject = obsav;
}

call_ob_proc(ob, sym, narg)
	Object *ob;
	Symbol *sym;
	int narg;
{
	Inst *pcsav, callcode[4];
	Symlist *slsav;
	Objectdata *obdsav;
	Object* obsav;
	int *typestacksav;
	extern int call(); 

	typestacksav = typestackp;
	slsav = hoc_symlist;
	obdsav = hoc_objectdata_save();
	obsav = hoc_thisobject;
	pcsav = pc;

   if (ob->template->sym->subtype == CPLUSOBJECT) {
	push_frame(sym, narg);
	if (sym->type == OBFUNCTION) {
		Object** o;
		o = (*(sym->u.u_proc->defn.pfo))(ob->u.this_pointer);
		pop_frame();
		hoc_pushobj(o);
	}else{
		double x;
		x = (*(sym->u.u_proc->defn.pfd))(ob->u.this_pointer);
		pop_frame();
		hoc_pushx(x);
	}
   }else{
	callcode[0].pf = call;
	callcode[1].sym = sym;
	callcode[2].i = narg;
	callcode[3].in = STOP;

	hoc_objectdata = ob->u.dataspace;
	hoc_thisobject = ob;
	hoc_symlist = ob->template->symtable;
	execute(callcode);
	if (sym->type == PROCEDURE) {
		hoc_nopop();
	}
   }
	if (typestacksav != typestackp) {
		hoc_execerror("typestack wrong on return from call", (char *)0);
	}
	pc = pcsav;
	hoc_symlist = slsav;
	hoc_objectdata = hoc_objectdata_restore(obdsav);
	hoc_thisobject = obsav;
}

hoc_objvardecl() {/* symbol at pc+1, number of indices at pc+2 */
	Symbol *sym;
	int nsub, size, i;
	Object **pobj;

	sym = (pc++)->sym;
#if PDEBUG
	printf("declareing %s as objectvar\n", sym->name);
#endif
	if (sym->type == OBJECTVAR) { int total, i;
		total = hoc_total_array(sym);
		for (i=0; i<total; i++) {
			hoc_dec_refcount((OPOBJ(sym))+i);
		}
		free((char*)hoc_objectdata[sym->u.oboff].pobj);
		hoc_freearay(sym);
	}else{
		sym->type = OBJECTVAR;
		hoc_install_object_data_index(sym);
	}
	nsub = (pc++)->i;
	if (nsub) {
		size = hoc_arayinfo_install(sym, nsub);
	}else{
		size = 1;
	}
hoc_objectdata[sym->u.oboff].pobj = pobj = (Object **)emalloc(size*sizeof(Object *));
	for (i=0; i<size; i++) {
		pobj[i] = (Object *)0;
	}
}

hoc_cmp_otype() {/* NUMBER, OBJECTVAR, or STRING must be the type */
	int type;
	type = (pc++)->i;

	if (type == 0) { /* don't check */
		poptypestack();
		return;
	}
	if (type != poptypestack()) {
		hoc_execerror("Object type mismatch", (char *)0);
	}
}

hoc_known_type() {
	pushtypestack((pc++)->i);
}

hoc_objectvar() { /* object variable symbol at pc+1. */
		  /* pointer to correct object left on stack */
	Symbol *obs;
	Object **obp;
#if PDEBUG
	printf("code for hoc_objectvar()\n");
#endif
	obs = (pc++)->sym;
	obp = OPOBJ(obs);
	if (ISARRAY(obs)) {
		hoc_pushobj(obp + araypt(obs, OBJECTVAR));
	}else{
		hoc_pushobj(obp);
	}
	pushtypestack(OBJECTVAR);
}

hoc_objectarg() { /* object arg index at pc+1. */
		  /* pointer to correct object left on stack */
	Object **obp, **hoc_objgetarg();
#if PDEBUG
	printf("code for hoc_objectarg()\n");
#endif
	obp = hoc_objgetarg((pc++)->i);
	hoc_pushobj(obp);
	pushtypestack(OBJECTVAR);
}

hoc_constobject() { /* template at pc, index at pc+1, objpointer left on stack*/
	char buf[200];
	Object *obj;
	Item* q;
	Template* t = (pc++)->sym->u.template;
	int index = (int)hoc_xpop();
	ITERATE (q, t->olist) {
		obj = OBJ(q);
		if (obj->index == index) {
			hoc_pushobj(hoc_temp_objptr(obj));
			pushtypestack(OBJECTVAR);
			return;
		}else if (obj->index > index) {
			break;
		}
	}
	sprintf(buf, "%s[%d]\n", t->sym->name, index);
	hoc_execerror("Object ID doesn't exist:", buf);
}

hoc_object_id() {
	Object *ob, **hoc_objgetarg();
	ob = *(hoc_objgetarg(1));
	hoc_ret();
	pushx((double)((long)ob));
}

#if CABLE
static range_suffix(sym, nindex, narg)
	Symbol* sym;
	int nindex, narg;
{
	int bdim = 0;
	poptypestack();
	if (ISARRAY(sym)) {
		if (nindex != sym->arayinfo->nsub) {
			bdim = 1;
		}
	}else{
		if (nindex != 0) {
			bdim = 1;
		}
	}
	if (bdim) {
		hoc_execerror(sym->name, "wrong number of array dimensions");
	}

	if (sym->type == RANGEVAR) {
		hoc_pushi(narg);
		hoc_pushs(sym);
		pushtypestack(RANGEVAR);
	}else if (sym->subtype == USERPROPERTY) {
		if (narg) {
			hoc_execerror(sym->name, "section property can't have argument");
		}
		hoc_pushs(sym);
		pushtypestack(USERPROPERTY);
	}else{
		hoc_execerror(sym->name, "suffix not a range variable or section property");
	}	
}
#endif

hoc_object_component() { /* number of indices at pc+2, number of args at pc+3,
				 symbol at pc+1 */
			/* object pointer on stack after indices */
	/* if component turns out to be an object then make sure pointer
	to correct object, symbol, etc is left on stack for evaluation,
	assignment, etc. */
	Symbol *sym0, *sym;
	extern int hoc_returning;
	int nindex, narg, cplus;
	Object *obp, *obsav;
	Objectdata *psav;
	Datum *hoc_look_inside_stack();
	int* ptid;
	Symbol** psym;

#if PDEBUG
	printf("code for hoc_object_component()\n");
#endif
	sym0 = (pc++)->sym;
	nindex = (pc++)->i;
	narg = (pc++)->i;
	ptid = &(pc++)->i;
	psym = &(pc++)->sym;

#if CABLE
	if (section_object_seen) {
		section_object_seen = 0;
		range_suffix(sym0, nindex, narg);
		return;
	}
#endif
	if (nindex) {
		if (narg) {
hoc_execerror("[...](...) syntax only allowed for array range variables:", sym0->name);
		}
	}else{
		nindex = narg;
	}
	if (poptypestack() != OBJECTVAR) {
		hoc_execerror(sym0->name, ": the prefix of this name is not an object");
	}
	
	obp = *(hoc_look_inside_stack(nindex, OBJECTVAR)->pobj);
	if (obp) {
		if (obp->template->id == *ptid) {
			sym = *psym;
		}else{
			sym = hoc_table_lookup(sym0->name, obp->template->symtable);
			if (!sym || sym->public != PUBLIC_TYPE) {
				fprintf(stderr, "%s ", sym0->name);
				hoc_execerror("not a public member of", obp->template->sym->name);
			}
			*ptid = obp->template->id;
			*psym = sym;
		}
	}else{
		hoc_execerror(sym0->name, ": object prefix is NULL");
	}

	psav = hoc_objectdata_save();
	obsav = hoc_thisobject;
	cplus = (obp->template->sym->subtype == CPLUSOBJECT);
	if (!cplus) { /* c++ classes don't have a hoc dataspace */
		hoc_objectdata = obp->u.dataspace;
		hoc_thisobject = obp;
	}
	switch (sym->type) {
	case OBJECTVAR:
		if (nindex) {
			if (!ISARRAY(sym) || OPARINFO(sym)->nsub != nindex) {
				hoc_execerror(sym->name, ":not right number of subscripts");
			}
			nindex = araypt(sym, OBJECTVAR);
		}
		hoc_nopop(); /*finally get rid of symbol */
		hoc_pushobj(OPOBJ(sym) + nindex);
		pushtypestack(sym->type);
		break;
	case VAR:
		if (cplus) {
			double* pd;
			if (nindex) {
				if (!ISARRAY(sym) || sym->arayinfo->nsub != nindex) {
					hoc_execerror(sym->name, ":not right number of subscripts");
				}
			}
			hoc_pushs(sym);
			(*obp->template->steer)(obp->u.this_pointer);
			pd = hoc_pxpop();
			hoc_nopop();
			hoc_pushpx(pd);
		}else{
			if (nindex) {
				if (!ISARRAY(sym) || OPARINFO(sym)->nsub != nindex) {
					hoc_execerror(sym->name, ":not right number of subscripts");
				}
				nindex = araypt(sym, OBJECTVAR);
			}
			hoc_nopop(); /*finally get rid of symbol */
			hoc_pushpx(OPVAL(sym) + nindex);
		}
		pushtypestack(NUMBER);
		break;
	case STRING:
		if (nindex) {
			hoc_execerror(sym->name, ": string can't have function arguments or array indices");
		}
		hoc_nopop(); /*finally get rid of symbol */
		hoc_pushstr(OPSTR(sym));
		pushtypestack(sym->type);
		break;
	case PROCEDURE:
	case FUNCTION: {
		double d=0.;
		call_ob_proc(obp, sym, nindex);
		if (hoc_returning) {
			break;
		}
		if (sym->type == FUNCTION) {
			d = hoc_xpop();
		}
		hoc_nopop(); /*finally get rid of symbol */
		hoc_pushx(d);
		pushtypestack(sym->type);
		break;}
	case OBFUNCTION: {
		Object** d;
		call_ob_proc(obp, sym, nindex);
		if (hoc_returning) {
			break;
		}
		d = hoc_objpop();
		hoc_nopop(); /*finally get rid of symbol */
		hoc_pushobj(d);
		pushtypestack(OBJECTVAR);
		break;}
#if CABLE
	case SECTIONREF: {
		Section* sec;
		section_object_seen = 1;
		if (nindex) {
			hoc_execerror(sym->name, ":no subscript allowed");
		}
		hoc_nopop(); /*finally get rid of symbol */
		sec = (Section*)obp->u.this_pointer;
		if (!sec->prop) {
			hoc_execerror("Section was deleted", (char*)0);
		}
		nrn_pushsec(sec);
		pushtypestack(SECTION);
		break;
		}
	case SECTION: {
		section_object_seen = 1;
		if (nindex) {
			if (!ISARRAY(sym) || OPARINFO(sym)->nsub != nindex) {
				hoc_execerror(sym->name, ":not right number of subscripts");
			}
			nindex = araypt(sym, OBJECTVAR);
		}
		hoc_nopop(); /*finally get rid of symbol */
		ob_sec_access_push(*(OPSECITM(sym) + nindex));
		pushtypestack(SECTION);
		break;
		}
#endif /*CABLE*/
	default:
		if (cplus) {
			double* pd;
			hoc_pushs(sym);
			(*obp->template->steer)(obp->u.this_pointer);
			pd = hoc_pxpop();
			hoc_nopop();
			hoc_pushpx(pd);
		}else{
			hoc_execerror(sym->name, ": can't push that type onto stack");
		}
		pushtypestack(NUMBER);
	}
	hoc_objectdata = hoc_objectdata_restore(psav);
	hoc_thisobject = obsav;
}

hoc_object_eval() {
	int type;
#if PDEBUG
	printf("code for hoc_object_eval\n");
#endif
	type = poptypestack();
	switch (type) {
	case NUMBER:
		hoc_pushx(*(hoc_pxpop()));
		pushtypestack(type);
		break;
	case PROCEDURE:
	case FUNCTION:
		pushtypestack(NUMBER);
		break;
#if CABLE
	case RANGEVAR: {
			double* nrn_rangepointer();
			Symbol* sym = hoc_spop();
			int narg = hoc_ipop();
			struct Section* sec = nrn_sec_pop();
			double x;
			if (narg) {
				x = hoc_xpop();
			}else{
				x = .5;
			}
			hoc_pushx(*(nrn_rangepointer(sec, sym, x)));
		}
		pushtypestack(NUMBER);
		break;
	case USERPROPERTY: {
			double cable_prop_eval();
			hoc_pushx(cable_prop_eval(hoc_spop()));
		}
		pushtypestack(NUMBER);
		break;
#endif
	default:
		pushtypestack(type);
	}
}

hoc_ob_pointer() {
	int type;
#if PDEBUG
	printf("code for hoc_ob_pointer\n");
#endif
	type = poptypestack();
	switch (type) {
	case NUMBER:
		break;
#if CABLE
	case RANGEVAR: {
			double* nrn_rangepointer();
			Symbol* sym = hoc_spop();
			int nindex = hoc_ipop();
			struct Section* sec = nrn_sec_pop();
			double x;
			if (nindex) {
				x = hoc_xpop();
			}else{
				x = .5;
			}
			hoc_pushpx(nrn_rangepointer(sec, sym, x));
		}
		break;
	case USERPROPERTY: {
			double* cable_prop_eval_pointer();
			hoc_pushpx(cable_prop_eval_pointer(hoc_spop()));
		}
		break;
#endif
	default:
		hoc_execerror("Not a double pointer", 0);
	}
}

hoc_asgn_obj_to_str() { /* string on stack */
	char *d, **pstr;
	if (poptypestack() != STRING) {
		hoc_execerror("Object must be of type string", (char *)0);
	}
	d = *(hoc_strpop());
	pstr = hoc_strpop();
	hoc_assign_str(pstr, d);
}

int hoc_oop_type() { return poptypestack();}

hoc_object_asgn() {
	int type1, type2;
	type1 = poptypestack();
	type2 = poptypestack();
#if CABLE
	if (type2 == RANGEVAR && type1 == NUMBER) {
		double d = hoc_xpop();
		struct Section* sec;
		Symbol* sym = hoc_spop();
		int nindex = hoc_ipop();
		sec = nrn_sec_pop();
		if (nindex) {
			double* pd;
			pd = nrn_rangepointer(sec, sym, hoc_xpop());
			*pd = d;
		}else{
			nrn_rangeconst(sec, sym, d);
		}
		hoc_pushx(d);
		pushtypestack(NUMBER);
		return;
	}else if (type2 == USERPROPERTY && type1 == NUMBER) {
		double d = hoc_xpop();
		cable_prop_assign(hoc_spop(), d);
		hoc_pushx(d);
		pushtypestack(NUMBER);
		return;
	}
#endif
	if (type1 != type2) {
		hoc_execerror("Object type mismatch in assignment", (char *)0);
	}
	switch (type2) {
	case NUMBER: { double d, *pd;
		d = hoc_xpop();
		pd = hoc_pxpop();
		*pd = d;
		hoc_pushx(d);
		}
		pushtypestack(type2);
		break;
	case OBJECTVAR: { Object *d, **pd;
		d = *(hoc_objpop());
		pd = hoc_objpop();
		if (d) {
			d->refcount++;
		}
		hoc_dec_refcount(pd);
		*pd = d;		
		hoc_pushobj(pd);
		}
		pushtypestack(type2);
		break;
	case STRING: { char *d, **pd;
		d = *(hoc_strpop());
		pd = hoc_strpop();		
		hoc_assign_str(pd, d);
		hoc_pushstr(pd);
		}
		pushtypestack(type2);
		break;
	}
}

/* if the name isn't a template then look in the top level symbol table.
This allows objects to create objects of any class defined at the top level
*/
Symbol* hoc_which_template(s) Symbol* s; {
	if (s->type != TEMPLATE) {
		Symbol* s1;
		s1 = hoc_table_lookup(s->name, hoc_top_level_symlist);
		if (!s1 || s1->type != TEMPLATE) {
			hoc_execerror(s->name, "is not a template");
		}
		s = s1;
	}
	return s;
}

/* pushes old symtable and template name on template stack.
And creates new symtable. The new symtable
is used for all non-builtin names until an endtemplate is reached
*/
static int template_id;

hoc_begintemplate(t1) Symbol *t1; {
	Symbol* t;
	int type;
	t = hoc_decl(t1);
	
#if PDEBUG
	printf("begin template %s\n", t->name);
#endif
	type = t->type;
	if (type == TEMPLATE) {
		hoc_free_symspace(t);
	}else if (type != UNDEF) {
		hoc_execerror(t->name, "already used as something besides template");
	}
	t->u.template = (Template *)emalloc(sizeof(Template));
	t->type = TEMPLATE;
	t->u.template->sym = t;
	t->u.template->symtable = (Symlist *)0;
	t->u.template->dataspace_size = 0;
	t->u.template->constructor = 0;
	t->u.template->destructor = 0;
	t->u.template->steer = 0;
	t->u.template->checkpoint = 0;
	t->u.template->id = ++template_id;
	pushtemplatei(icntobjectdata);
	pushtemplateodata(hoc_objectdata);
	pushtemplateo(hoc_thisobject);
	pushtemplatesymlist(hoc_symlist);
	pushtemplatesym(t);
	hoc_objectdata = (Objectdata *)0;
	hoc_thisobject = (Object*)0;
	hoc_symlist = t->u.template->symtable;
}

hoc_endtemplate(t) Symbol *t; {
	Symbol *ts, *s;
#if PDEBUG
	printf("end template %s\n", t->name);
#endif
	ts = (poptemplate())->sym;
	if (strcmp(ts->name, t->name) != 0) {
		hoc_execerror(t->name, "- end template mismatched with begin");
	}
	ts->u.template->dataspace_size = icntobjectdata;
	ts->u.template->symtable = hoc_symlist;
	ts->u.template->count = 0;
	ts->u.template->index = 0;
	ts->u.template->olist = hoc_l_newlist();
	ts->u.template->observers = (void*)0;
	hoc_symlist = (poptemplate())->symlist;
	free_objectdata(hoc_objectdata, ts->u.template);
	hoc_thisobject = (poptemplate())->o;
	hoc_objectdata = (poptemplate())->odata;
	icntobjectdata =  (poptemplate())->i;
ts->u.template->init = s = hoc_table_lookup("init", ts->u.template->symtable);
	if (s && s->type != PROCEDURE) {
hoc_execerror("'init' can only be used as the initialization procedure for new objects",
				(char *)0);
	}
}

typedef struct Member_func {
	char* name;
	double (*member)();
}Member_func;

typedef struct Member_ret_obj_func {
	char* name;
	Object** (*omember)();
}Member_ret_obj_func;

void class2oc(name, cons, destruct, m, checkpoint,  mobjret)
	char* name;
	void* (*cons)();
	void (*destruct)();
	Member_func* m;
	int (*checkpoint)();
	Member_ret_obj_func* mobjret;
{
	Symbol* tsym, *s;
	Template* t;
	int i;
	
	if (hoc_lookup(name)) {
		hoc_execerror(name, "already being used as a name");
	}
	tsym = hoc_install(name, UNDEF, 0.0, &hoc_symlist);
	tsym->subtype = CPLUSOBJECT;
	hoc_begintemplate(tsym);
	t = tsym->u.template;
	t->constructor = cons;
	t->destructor = destruct;
	t->steer = 0;
	t->checkpoint = checkpoint;
	if (m) for (i=0; m[i].name; ++i) {
		s = hoc_install(m[i].name, FUNCTION, 0.0, &hoc_symlist);
		s->u.u_proc->defn.pfd = m[i].member;
		hoc_add_publiclist(s);
	}
	if (mobjret) for (i=0; mobjret[i].name; ++i) {
		s = hoc_install(mobjret[i].name, OBFUNCTION, 0.0, &hoc_symlist);
		s->u.u_proc->defn.pfo = mobjret[i].omember;
		hoc_add_publiclist(s);
	}
	hoc_endtemplate(tsym);		
}

Symbol*
hoc_decl(s) Symbol* s; {
	Symbol* ss;	
	if (templatestackp == templatestack) {
		if (s == hoc_table_lookup(s->name, hoc_built_in_symlist)) {
			hoc_execerror(s->name, ": Redeclaring at top level");
		}
		return s;
	}
	ss = hoc_table_lookup(s->name, hoc_symlist);
	if (!ss) {
		ss = hoc_install(s->name, UNDEF, 0.0, &hoc_symlist);
	}
	return ss;
}

hoc_add_publiclist(s) Symbol *s; {
	Symbol* ss;
#if PDEBUG
	printf("public name %s with type %d\n", s->name, s->type);
#endif
	if (templatestackp == templatestack) {
		hoc_execerror("Not in a template\n", 0);
	}
	ss = hoc_decl(s);
	ss->public = PUBLIC_TYPE;
}

hoc_external_var(s) Symbol *s; {
	Symbol* s0;
	if (templatestackp == templatestack) {
		hoc_execerror("Not in a template\n", 0);
	}
	if (s->public == PUBLIC_TYPE) {
		hoc_execerror(s->name, "can't be public and external");
	}
	s->public = EXTERNAL_TYPE;
	s0 = hoc_table_lookup(s->name, hoc_top_level_symlist);
	if (!s0) {
		hoc_execerror(s->name, "not declared at the top level");
	}
	s->type = s0->type;
	s->subtype = s0->subtype;
	switch(s->type) {
	case FUNCTION:
	case PROCEDURE:
		s->u.u_proc = s0->u.u_proc;
		break;
	default:
		hoc_execerror(s->name, "type is not allowed external");
		break;
	}
}

hoc_ob_check(type) int type; {
	int t;
	t = ipop();
	if (type == -1) {
		if (t == OBJECTVAR) {/* don't bother to check */
			Code(hoc_cmp_otype); codei(0);
		}				
	}else if (type) {
		if (t == OBJECTVAR) {/* must check dynamically */
#if PDEBUG
			printf("dymnamic checking of type=%d\n", type);
#endif
			Code(hoc_cmp_otype); codei(type);
		}else if (type != t) { /* static check */
			hoc_execerror("Type mismatch", (char *)0);
		}
	}else{
		if (t != OBJECTVAR) {
			Code(hoc_known_type); codei(t);
		}
	}
}

hoc_free_allobjects(template, sl, data)
Template *template; Symlist *sl; Objectdata *data;
{
	/* look in all object variables that point to
		objects with this template and null them */
	Symbol *s;
	int total, i;
	Object **obp;

	if (sl) for (s = sl->first; s; s = s->next) {
		if (s->type == OBJECTVAR) {
			total = hoc_total_array_data(s, data);
			for (i=0; i<total; i++) {
				obp = data[s->u.oboff].pobj + i;
				if (*obp) {
#if 1
					if ((*obp)->template == template) {
						hoc_dec_refcount(obp);
					}else if (s->subtype != CPLUSOBJECT) {
						/* descend to look for more */
hoc_free_allobjects(template, (*obp)->template->symtable, (*obp)->u.dataspace);
					}
#else
					if (s->subtype != CPLUSOBJECT) {
						/* descend to look for more */
hoc_free_allobjects(template, (*obp)->template->symtable, (*obp)->u.dataspace);
					}
					if ((*obp)->template == template) {
						hoc_dec_refcount(obp);
					}
#endif
				}
			}
		}
	}
}

#define objectpath hoc_objectpath_impl
#define pathprepend hoc_path_prepend

pathprepend(path, name, indx)
	char* path, *name, *indx;
{
	char buf[200];
	if (path[0]) {
		strcpy(buf, path);
		sprintf(path, "%s%s.%s",name, indx, buf);
	}else{
		sprintf(path, "%s%s", name, indx);
	}
}

int objectpath(ob, oblook, path, depth)
	Object* ob;
	Object* oblook;
	char* path;
{
	/* recursively build the pathname to the object */
	Symbol *s;
	Symlist* sl;
	int total, i;
	Objectdata* od;
	Object **obp;
	extern char* hoc_araystr();

	if (ob == oblook) {
		return 1;
	}
	if (oblook) {
		if (depth++ > 5) {
			hoc_warning("objectpath depth > 4 for", oblook->template->sym->name);
			return 0;
		}
		if (oblook->template->constructor) {
			return ivoc_list_look(ob, oblook, path, depth);
		}else{
			od = oblook->u.dataspace;
			sl = oblook->template->symtable;
		}
	}else{
		od = hoc_top_level_data;
		sl = hoc_top_level_symlist;
	}
	if (sl) for (s = sl->first; s; s = s->next) {
		if (s->type == OBJECTVAR) {
			total = hoc_total_array_data(s, od);
			for (i=0; i<total; i++) {
				obp = od[s->u.oboff].pobj + i;
				if (*obp && *obp != oblook && objectpath(ob, *obp, path, depth)) {
					pathprepend(path, s->name, hoc_araystr(s, i, od));
					return 1;
				}
			}
		}
	}
	return 0;
}

char* hoc_object_pathname(ob) Object* ob; {
	static char path[200];
	path[0] = '\0';
	if (objectpath(ob, (Object*)0, path, 0)) {
		return path;
	}else{
		hoc_warning("Couldn't find a pathname to the object pointer",
			ob->template->sym->name);
		return (char*)0;
	}
}

hoc_obj_ref(obj)
	Object* obj;
{
	if (obj) {
		++obj->refcount;
	}
}

hoc_dec_refcount(pobj)
	Object **pobj;
{
	Object* obj;

	obj = *pobj;
	if (obj == (Object *)0) {
		return;
	}
	*pobj = (Object *)0;
	assert(obj->refcount > 0);
	hoc_obj_unref(obj);	
}

hoc_obj_unref(obj)
	Object* obj;
{
	Object *obsav;

	if (!obj) {
		return;
	}
#if 0
printf("unreffing %s with refcount %d\n", hoc_object_name(obj), obj->refcount);
#endif
	--obj->refcount;
	if (obj->refcount <= 0) {
		if (obj->observers) {
			hoc_obj_disconnect(obj);
		}
		hoc_l_delete(obj->itm_me);
		if (obj->template->observers) {
			hoc_template_notify(obj, 0);
		}
		if (obj->template->sym->subtype == CPLUSOBJECT) {
			(obj->template->destructor)(obj->u.this_pointer);
		}else{
			obsav = hoc_thisobject;
			hoc_thisobject = obj;
			free_objectdata(obj->u.dataspace, obj->template);
			obj->u.dataspace = (Objectdata *)0;
			hoc_thisobject = obsav;
		}
		if (--obj->template->count <= 0) {
			obj->template->index = 0;
		}
		obj->template = (Template *)0;
		/* for testing purposes we don't free the object in order
		to make sure no object variable ever uses a freed object */
#if 1
		hoc_free_object(obj);
#endif
	}
}
		
static free_objectdata(od, template)
	Objectdata *od;
	Template *template;
{
	Symbol *s;
	int i, total;
	Objectdata *psav;
	Object **objp;
	
	psav = hoc_objectdata;
	hoc_objectdata = od;
	if (template->symtable) for (s=template->symtable->first; s; s=s->next) {
		switch (s->type) {
		case VAR:
/*printf("free_objectdata %s\n", s->name);*/
			hoc_free_val_array(OPVAL(s), hoc_total_array(s));
			free_arrayinfo(OPARINFO(s));
			break;
		case STRING:
			hoc_free_pstring(OPSTR(s));
			free_arrayinfo(OPARINFO(s));
			break;
		case OBJECTVAR:
			objp = OPOBJ(s);
			if (strcmp("this", s->name) != 0) {
				total = hoc_total_array(s);
				for (i = 0; i < total; i++) {
					hoc_dec_refcount(objp + i);
				}
			}
			free_arrayinfo(OPARINFO(s));
			free((char*)objp);
			break;
#if CABLE
		case SECTION:
			total = hoc_total_array(s);
			for (i = 0; i < total; ++i) {
				extern void sec_free();
				sec_free(*(OPSECITM(s) + i));
			}
			free((char*)OPSECITM(s));
			free_arrayinfo(OPARINFO(s));
			break;
#endif
		}
	}
	hoc_objectdata = psav;
	if (od) {
		free((char *)od);
	}
}


hoc_allobjects() {
	hoc_allobjects1(hoc_built_in_symlist, 0);	
	hoc_allobjects1(hoc_top_level_symlist, 0);
	hoc_ret();
	pushx(0.);
}

hoc_allobjects1(sl, nspace)
	Symlist* sl;
	int nspace;
{
	Symbol* s;
	Template* t;
	Object* o;
	Item* q;
	int i;
	if (sl) for (s = sl->first; s; s = s->next) {
		if (s->type == TEMPLATE) {
			t = s->u.template;
			ITERATE(q, t->olist) {
				o = OBJ(q);
				for (i=0; i < nspace; ++i) {
					printf("   ");
				}
printf("%s with %d refs\n", hoc_object_name(o), o->refcount);
			}
			hoc_allobjects1(t->symtable, nspace+1);
		}
	}
}

hoc_allobjectvars() {
	hoc_list_allobjref(hoc_top_level_symlist, hoc_top_level_data, 0);
	hoc_ret();
	pushx(0.);
}

hoc_list_allobjref(sl, data, depth)
	Symlist *sl; Objectdata *data; int depth;
{
	/* look in all object variables that point to
		objects and print them */
	Symbol *s;
	int total, i, id;
	Object **obp;

	if (sl) for (s = sl->first; s; s = s->next) {
		if (s->type == OBJECTVAR) {
			total = hoc_total_array_data(s, data);
			for (i=0; i<total; i++) {
				obp = data[s->u.oboff].pobj + i;
				for (id=0; id<depth; id++) {
					printf("   ");
				}
				if (*obp) {
printf("obp %s[%d] -> template:%s ob:%lx with %d refs.\n",
	s->name, i, (*obp)->template->sym->name, (unsigned long)(*obp),
	(*obp)->refcount);
				}else{
					printf("obp %s[%d] -> NULL\n", s->name, i);
				}
				if (*obp && s->subtype != CPLUSOBJECT
				   && (*obp)->u.dataspace != data /* not this */) {
					/* descend to look for more */
hoc_list_allobjref((*obp)->template->symtable, (*obp)->u.dataspace, depth+1);
				}
			}
		}
	}
}

check_obj_type(obj, typename)
	Object* obj;
	char* typename;
{
	char buf[100];
	if (!obj || strcmp(obj->template->sym->name, typename) != 0) {
		if (obj) {
			sprintf(buf, "object type is %s instead of",
			obj->template->sym->name);
		}else{
			sprintf(buf, "object type is nil instead of");
		}
		hoc_execerror(buf, typename);
	}
}

int is_obj_type(obj, typename)
	Object* obj;
	char* typename;
{
	if (obj && strcmp(obj->template->sym->name, typename) == 0) {
		return 1;
	}else{
		return 0;
	}
}


