/* $Header: /timshel2g/src/master/nrn/src/oc/math.c,v 1.1.1.1 1994/10/12 17:22:11 hines Exp $ */
/*
$Log: math.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:11  hines
 * NEURON 3.0 distribution
 *
 * Revision 2.48  93/05/25  14:02:38  hines
 * some functions that check on the arg type. eg hoc_is_object_arg(narg)
 * 
 * Revision 2.39  93/04/12  14:08:42  hines
 * first pass at merging Fisher's LINDA addition
 * 
 * Revision 2.5  93/01/06  09:26:18  hines
 * cray c90 needs include <errno.h>   remove some codecenter warnings
 * 
 * Revision 2.4  93/01/05  17:20:16  hines
 * print perror() instead of errno=
 * void sec_access_pop  Section* nrn_pop_sec
 * 
 * Revision 1.4  92/10/05  10:15:48  hines
 * when _CRAY math functions get l suffix
 * 
 * Revision 1.3  92/07/31  16:16:57  hines
 * float_epsilon used for logical comparisons, int(), indexing and short for
 * default value is 1e-11
 * Hopfully no one has to worry about roundoff for the loops to come out
 * right.
 * 
 * 
 * Revision 1.2  92/07/31  14:17:02  hines
 * better handling of errno as in hoc
 * 
 * Revision 1.1  91/10/11  11:12:11  hines
 * Initial revision
 * 
 * Revision 3.96  90/09/04  08:31:26  hines
 * try to get dos and unix together again in one RCS directory
 * 
 * Revision 3.83  90/07/25  10:40:28  hines
 * almost lint free on sparc 1+ under sunos 4.1
 * 
 * Revision 2.0  89/07/07  11:32:40  mlh
 * Preparation for newcable
 * 
 * Revision 1.1  89/07/07  11:17:15  mlh
 * Initial revision
 * 
*/

#include <math.h>
#include <errno.h>
#include <stdio.h>
# include	"redef.h"

extern int	errno;
#define EPS hoc_epsilon
extern double EPS;

#if LINT
int	errno;
#endif

#if _CRAY
#define log logl
#define log10 log10l
#define exp expl
#define sqrt sqrtl
#define pow powl
#endif

double	errcheck();

double
Log(x)
	double x;
{
	return	errcheck(log(x), "log");
}

double
Log10(x)
	double x;
{
	return errcheck(log10(x), "log10");
}

double
Exp(x)
	double x;
{
	return errcheck(exp(x), "exp");
}

double
Sqrt(x)
	double x;
{
	return errcheck(sqrt(x), "sqrt");
}

double
Pow(x, y)
	double x, y;
{
	return errcheck(pow(x, y), "exponentiation");
}

double
integer(x)
	double x;
{
	return (double)(long)(x + EPS);
}

double
errcheck(d, s)	/* check result of library call */
	double d;
	char *s;
{
	if (errno == EDOM)
	{
		errno = 0;
		execerror(s, "argument out of domain");
	}
	else if (errno == ERANGE)
	{
		errno = 0;
		execerror(s, "result out of range");
	}
	return d;
}

#define MAXERRCOUNT 5
int hoc_errno_count;
int hoc_errno_check() {
	int ierr;
#if LINDA
	static parallel_eagain = 0;
#endif

	if (errno) {
		if (++hoc_errno_count > MAXERRCOUNT) {
			errno = 0;
			return 0;
		}
		switch (errno) {
		case EDOM:
fprintf(stderr, "A math function was called with argument out of domain\n");
			break;
		case ERANGE:
fprintf(stderr, "A math function was called that returned an out of range value\n");
			break;
#if 1
		case ENOENT:
			errno = 0;
			break;
#endif
#if LINDA
/* regularly set by eval() and perhaps other linda commands */
                case EAGAIN:
		        if (parallel_eagain++ == 0) {
			    perror("oc");
fprintf(stderr, "oc: This error occurs often from LINDA and thus will not be further reported.\n"); 
		        }
			break;
#endif
		default:
			perror("oc");
			break;
		}
		if (hoc_errno_count == MAXERRCOUNT) {
fprintf(stderr, "No more errno warnings during this execution\n");
		}
	}
	ierr = errno;
	errno = 0;
	return ierr;
}
