/* $Header: /timshel2g/src/master/nrn/src/oc/symbol.c,v 1.1.1.1 1994/10/12 17:22:14 hines Exp $ */
/*
$Log: symbol.c,v $
 * Revision 1.1.1.1  1994/10/12  17:22:14  hines
 * NEURON 3.0 distribution
 *
 * Revision 2.115  1994/09/26  18:25:40  hines
 * fix warning on sgi
 *
 * Revision 2.80  1994/04/06  17:14:24  hines
 * minor changes to structures to allow checkpointing in ivoc
 * basically no more codepfd
 *
 * Revision 2.52  1993/07/19  09:36:13  hines
 * freeing double arrays has different notify call
 *
 * Revision 2.46  93/04/29  15:30:12  hines
 * notify_pointer_freed called whenever an object is freed.
 * 
 * Revision 2.31  93/03/05  08:42:19  hines
 * some slight performance improvements by avoidin hoc_lookup for objects
 * and use stack macros in code.c
 * 
 * Revision 2.26  93/02/22  10:57:16  hines
 * checking of malloc out of order
 * 
 * Revision 2.20  93/02/03  11:27:55  hines
 * a bit more generic for portability. will possibly work on NeXT
 * 
 * Revision 2.11  93/01/14  13:04:04  hines
 * switch from Objectdata.pstr to Objectdata.ppstr so we can eventually have
 * arrays of strings and so ivoc's varlabel will work properly.
 * 
 * Revision 2.10  93/01/13  08:52:15  hines
 * stubs for xvarlabel, xradiobutton, and xfixedvalue
 * mechanism in place for notifying if strdef is freed.
 * 
 * Revision 1.17  92/12/22  15:29:28  hines
 * a template can have an external statement followin the public
 * statement which is a comma separated list of functions or procedures
 * defined at the top level which can be executed within the object.
 * (the function is executed in a top level context).
 * public and external statements are optional
 * 
 * Revision 1.16  92/10/29  09:20:14  hines
 * some errors in freeing objects fixed and replace usage of getarg for
 * non numbers.
 * 
 * Revision 1.15  92/10/27  12:08:17  hines
 * list.c list.h moved from nrnoc to oc
 * all templates maintain a list of their objects
 * 
 * Revision 1.14  92/10/24  10:17:03  hines
 * botched last checkin for symbol.c. notifies interviews when hoc_free_val..
 * called
 * 
 * Revision 1.13  92/09/18  13:39:00  hines
 * c++ class members interfaced to oc can return an Object**
 * used first for built in List class
 * 
 * Revision 1.12  92/08/18  10:24:42  hines
 * templates can not make new objects using templates declared at the top level.
 * first looks for local templates though.
 * 
 * Revision 1.11  92/08/18  07:31:43  hines
 * arrays in different objects can have different sizes.
 * Now one uses araypt(symbol, SYMBOL) or araypt(symbol, OBJECTVAR) to
 * return index of an array variable.
 * 
 * Revision 1.10  92/08/17  12:49:36  hines
 * arrays in different objects can have different sizes. Now arayinfo
 * gets carried along in objectdata. arayinfo is reference counted.
 * 
 * Revision 1.9  92/07/31  08:40:51  hines
 * ecalloc returns 0 when requesting 0 space. (now matches hoc)
 * 
 * Revision 1.8  91/10/30  14:40:48  hines
 * stack.hoc works and can be reloaded
 * 
 * Revision 1.7  91/10/24  16:22:10  hines
 * can call object functions and procedures. Object arrays not working yet.
 * object_id(obj) is used for tests.
 * STKDEBUG turned on in code.c in which every element also carries a type.
 * 
 * Revision 1.6  91/10/22  15:47:16  hines
 * objects and components. can assign, evaluate, and print.
 * No object arrays or function components yet.
 * At level of test3.hoc
 * 
 * Revision 1.5  91/10/18  14:40:49  hines
 * symbol tables now are type Symlist containing pointers to first and last
 * symbols.  New symbols get added onto the end.
 * 
 * Revision 1.4  91/10/17  15:01:38  hines
 * VAR, STRING now handled with pointer to value located in object data space
 * to allow future multiple objects. Ie symbol for var, string, objectvar
 * has offset into a pointer data space.
 * 
 * Revision 1.3  91/10/16  13:14:13  hines
 * fix up mistakes in last checkin
 * 
 * Revision 1.2  91/10/16  12:47:59  hines
 * symbol tables for the templates. Prototype not even executed.
 * need to come to grips with how to find data in the objects.
 * 
 * Revision 1.1  91/10/11  11:12:19  hines
 * Initial revision
 * 
 * Revision 4.31  91/08/08  10:07:51  hines
 * hoc_realloc()  reallocation with error checking.
*/

/* version 7.2.1 2-jan-89 */
#include "stdio.h"
#include "hoc.h"
#include "ytab.h"
#include "list.h"
#if INCLUDE_MALLOC_H
#include <malloc.h>
#endif
#if INCLUDE_ALLOC_H
#include <alloc.h>	/* at least for turbo C 2.0 */
#endif

#if OOP
Symlist	*hoc_built_in_symlist = (Symlist *)0; /* keywords, built-in functions,
	all name linked into hoc. Look in this list last */
Symlist	*hoc_top_level_symlist = (Symlist *)0; /* all user names seen at top-level
		(non-public names inside templates do not appear here) */
extern Objectdata *hoc_top_level_data;
#endif /*OOP*/
						
Symlist	*symlist = (Symlist *)0;	/* the current user symbol table: linked list */
Symlist	*p_symlist = (Symlist *)0; /* current proc, func, or temp table */
			/* containing constants, strings, and auto */
			/* variables. Discarding these lists at */
			/* appropriate times prevents storage leakage. */

Symbol *hoc_table_lookup(s, tab) /* find s in specific table */
	char *s;
	Symlist *tab;
{
	Symbol *sp;
	if (tab) for (sp=tab->first ; sp != (Symbol *) 0; sp = sp->next) {
		if (strcmp(sp->name, s) == 0) {
			return sp;
		}
	}
	return (Symbol *)0;
}

Symbol *
lookup(s)	/* find s in symbol table */
	char *s; /* look in p_symlist then built_in_symlist then symlist */
{
	Symbol *sp;

	if ((sp = hoc_table_lookup(s, p_symlist)) != (Symbol *)0) {
		return sp;
	}
	if ((sp = hoc_table_lookup(s, symlist)) != (Symbol *)0) {
		return sp;
	}
#if OOP
	if ((sp = hoc_table_lookup(s, hoc_built_in_symlist)) != (Symbol *)0) {
		return sp;
	}
#endif

	return 0;	/* 0 ==> not found */
}

Symbol *
install(s, t, d, list)	/* install s in the list symbol table */
	char *s;
	int t;
	double d;
	Symlist **list;
{
	Symbol *sp;

	sp = (Symbol *) emalloc(sizeof(Symbol));
	sp->name = (char *)emalloc((unsigned)(strlen(s)+1));	/* +1 for '\0' */
	Strcpy(sp->name, s);
	sp->type = t;
	sp->subtype = NOTUSER;
	sp->defined_on_the_fly = 0;
	sp->public = 0;
	sp->s_varn = 0;
	sp->arayinfo = (Arrayinfo *)0;
	if (!(*list)) {
		*list = (Symlist *)emalloc(sizeof(Symlist));
		(*list)->first = (*list)->last = (Symbol *)0;
	}
	hoc_link_symbol(sp, *list);
	switch (t) {
	case NUMBER:
		sp->u.pnum = (double *)emalloc(sizeof(double));
		*sp->u.pnum = d;
		break;
	case VAR:
		hoc_install_object_data_index(sp);
		OPVAL(sp) = (double *)emalloc(sizeof(double));
		*(OPVAL(sp)) = d;
		break;
	case PROCEDURE:
	case FUNCTION:
	case FUN_BLTIN:
	case OBFUNCTION:
		sp->u.u_proc = (Proc *)emalloc(sizeof(Proc));
		sp->u.u_proc->list = (Symlist*)0;
		sp->u.u_proc->size = 0;
		break;
	default:
		sp->u.pnum = (double *)0;
		break;
	}
	return sp;
}

hoc_unlink_symbol(s, list)
	Symbol* s;
	Symlist* list;
{
	Symbol *sp;
	assert(list);
	
	if (list->first == s) {
		list->first = s->next;
		if (list->last == s) {
			list->last = (Symbol*)0;
		}
	}else {
		for (sp = list->first ; sp != (Symbol *) 0; sp = sp->next) {
			if (sp->next == s) {
				break;
			}
		}
		assert(sp);
		sp->next = s->next;
		if (list->last == s) {
			list->last = sp;
		}
	}
	s->next = (Symbol*)0;
}

hoc_link_symbol(sp, list)
	Symbol* sp;
	Symlist* list;
{
	/* put at end of list */
	if (list->last) {
		list->last->next = sp;
	}else{
		list->first= sp;
	}
	list->last = sp;
	sp->next = (Symbol *)0;
}

static int malloc_error=0;

hoc_malchk() {
	if (malloc_error) {
		malloc_error = 0;
		execerror("out of memory", (char *) 0);
	}
}
	
#if LINT
double *
#else
char *
#endif
hoc_Emalloc(n)	/* check return from malloc */
	unsigned n;
{
	char *p;

	p = (char *)malloc(n);
	if (p == 0)
		malloc_error = 1;
#if LINT
	{static double *p; return p;}
#else
	return p;
#endif
}

#if LINT
double *
#else
char *
#endif
hoc_Ecalloc(n, size)	/* check return from calloc */
	unsigned n, size;
{
	char *p;

	if (n == 0) {
		return (char*)0;
	}
	p = (char *)calloc(n, size);
	if (p == 0)
		malloc_error = 1;
#if LINT
	{static double *p; return p;}
#else
	return p;
#endif
}

#if LINT
double *
#else
char *
#endif
hoc_Erealloc(ptr, size)	/* check return from realloc */
	char *ptr;
	unsigned size;
{
	char *p;

	if (!ptr) {
		return hoc_Emalloc(size);
	}
	p = (char *)realloc(ptr, size);
	if (p == 0) {
		free(ptr);
		malloc_error = 1;
	}
#if LINT
	{static double *p; return p;}
#else
	return p;
#endif
}

hoc_free_symspace(s1)	/* frees symbol space. Marks it UNDEF */
	Symbol *s1;
{
	if (s1) {
		switch (s1->type)
		{
		case UNDEF:
			break;
		case STRING:
			break;
		case VAR:
			break;
		case NUMBER:
			free((char *)(s1->u.pnum));
			break;
		case CSTRING:
			free(s1->u.cstr);
			break;
		case PROCEDURE:
		case FUNCTION:
			if (s1->public != 2 && s1->u.u_proc != (Proc *)0) {
			        if (s1->u.u_proc->defn.in != STOP)
			                free((char *) s1->u.u_proc->defn.in);
			        free_list(&(s1->u.u_proc->list));
				free((char *) s1->u.u_proc);
			}
			break;
		case AUTO:
			break;
		case TEMPLATE:
hoc_free_allobjects(s1->u.template, hoc_top_level_symlist, hoc_top_level_data);
			free_list(&(s1->u.template->symtable));
			{hoc_List* l = s1->u.template->olist;
				if (l->next == l) {
					hoc_l_freelist(&s1->u.template->olist);
					free(s1->u.template);
				}else{
hoc_warning("didn't free all objects created with the old template:", s1->name);
				}
			}
			break;
		case OBJECTVAR:
#if 0 /* should have been freed above, otherwise I don't know the exact objects*/
			if (s1->arayinfo) {int i, j, k=0;
			   for (i = 0; i < s1->arayinfo->nsub; i++) {
			      for (j=0; j < s1->arayinfo->sub[i]; j++) {
			         hoc_dec_refcount(OPOBJ(s1) + k);
				 ++k;
			      }
			   }
			}else{
				hoc_dec_refcount(OPOBJ(s1));
			}
			free((char *)OPOBJ(s1));
#endif
			break;
		default:
Fprintf(stderr, "In free_symspace may not free all of %s of type=%d\n", s1->name, s1->type);
		}
		if (s1->arayinfo != (Arrayinfo *)0) {
			free_arrayinfo(s1->arayinfo);
			s1->arayinfo = (Arrayinfo *)0;
		}
	}
	s1->type = UNDEF;
}

free_list(list)	/* free the space in a symbol table */
	Symlist **list;
{
	Symbol *s1, *s2;

	if (*list) {
		for (s1 = (*list)->first; s1; s1 = s2){
			s2 = s1->next;
			hoc_free_symspace(s1);
			if (s1->name) {
				free(s1->name);
			}
			free((char *) s1);
		}
		free((char *)(*list));
	}
	*list = (Symlist *)0;
}

hoc_free_val(p) double* p;
{
	notify_freed((void*)p);
	free((char*)p);
}

hoc_free_val_array(p, size) double* p; int size;
{
	notify_freed_val_array(p, size);
	free((char*)p);
}

hoc_free_object(p) Object* p;
{
	if (p) {
		notify_pointer_freed((void*)p);
		free((char*)p);
	}
}

hoc_free_string(p) char* p;
{
	free(p);
}

hoc_free_pstring(p) char** p;
{
	notify_freed((void*)p);
	if (*p) {
		free(*p);
		free(p);
	}
}

