/* $Header: /timshel2g/src/master/nrn/src/oc/x_sejnowski.c,v 1.1.1.1 1994/10/12 17:22:15 hines Exp $ */

#if X11
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>

/* initial position of window */
#define WX	0
#define WY	0

/* size of window */
#define WIDTH 500
#define HEIGHT 390
#define MARGIN 0

/* size of icon */
#define IWIDTH 64
#define IHEIGHT 20

#define x11_Plot(x,y) XDrawPoint(display,bitmap,gc,(x),(y)); XCopyArea (display, bitmap, win, gc, 0, 0, width, height, 0, 0);

#define Line(x1,y1,x2,y2) XDrawLine(display,bitmap,gc,(x1),(y1),(x2),(y2)); XCopyArea (display, bitmap, win, gc, 0, 0, width, height, 0, 0);

#define Lines()	XDrawLines(display, bitmap, gc, polyline, nlinept, CoordModeOrigin); XCopyArea (display, bitmap, win, gc, 0, 0, width, height, 0, 0);

#define String(s) XDrawString(display, bitmap, gc, xold, yold, (s), strlen((s))); XCopyArea (display, bitmap, win, gc, 0, 0, width, height, 0, 0);

#define LAST	          xold=xnew; yold=ynew

static int fast;	/* initialized to 0; don't flush until plt(-1), 
			   use drawlines */
static int nlinept;
static XPoint polyline[200];
static int maxnlinept=200;
static Display *display;
static Window win; 
static GC gc;
static GC gcreverse;              /* Used for "blanking" out the bitmap */
static int screen;
static XEvent report;
static Pixmap bitmap;             /* Bitmap is a X's pixmap with depth = 1;
				     this bitmap is used to allow refreshing
				     of window due to expose event by saving
				     the output to the bitmap */

/*static XFontStruct *font;*/

/* used with sigio tests */
static int socket_flags;
static int socket_count;

static int D;
#define Color (D > 1)
#define Ncolors 11

static unsigned long colors[Ncolors];

extern void x11_open_window();
extern void x11_draw_vec();
void adjustBitmap();
int x11_init_done;
static int xnew, ynew;
static int xold, yold;
static double xscale, yscale;
static unsigned int width, height;
#define TEKX 1000.
#define TEKY 780.

void x11_fast(mode)
	int mode;
{
	fast = mode;
}

void x11flush() {
        disable_sigio();
	if (fast && nlinept) {
	    x11_draw_vec();
	}
	XFlush(display);
	enable_sigio();
}

static void getscale() {
	int x,y;
	unsigned int border_width, depth;
	Window root;

	disable_sigio();
	XGetGeometry(display, win, &root, &x, &y, &width,
		&height, &border_width, &depth);
	xscale = ((double)width)/TEKX;
	yscale = ((double)height)/TEKY;
	enable_sigio();
}

void x11_coord(x, y)
	double x, y;
{
	xnew = (int)(xscale*x);
	ynew = (int)(yscale*(TEKY - y));
}

void x11_draw_vec() {
	disable_sigio();
	if (nlinept > 1) {
	    Lines();
	}
	nlinept = 0;
	enable_sigio();
}

void x11_vector()
{
	disable_sigio();
	if (fast) {
		if (nlinept == 0) {
			polyline[0].x = xold;
			polyline[0].y = yold;
			++nlinept;
		}
		if (nlinept >= maxnlinept) {
			x11_draw_vec();
		}
		polyline[nlinept].x = xnew;
		polyline[nlinept].y = ynew;
		++nlinept;
	}
	else{
		Line(xold, yold, xnew, ynew);
		XFlush(display);
	}
	LAST;
	enable_sigio();
}

void x11_point()
{
	disable_sigio();
	x11_Plot(xnew, ynew);
	LAST;
	if (!fast)
	    XFlush(display);
	enable_sigio();
}

void x11_move()
{
	disable_sigio();
	if (fast) {
		if (nlinept && (xnew != xold || ynew != yold)) {
			x11_draw_vec();
		}
	}
	LAST;
	enable_sigio();
}

void x11_clear(){
	disable_sigio();
	XClearWindow(display, win);
	XFillRectangle (display, bitmap, gcreverse, 0, 0, width, height);
	XFlush(display);
	getscale();
	adjustBitmap();
	enable_sigio();
}

void x11_cleararea(){
	int w, h, x, y;

	disable_sigio();
	w = xnew-xold;
	h = ynew-yold;
	if (w < 0) {
		w = -w;
		x = xnew;
	}else{
		x = xold;
	}
	if (h < 0) {
		h = -h;
		y = ynew;
	}else{
		y=yold;
	}
	XClearArea(display, win, x, y, w, h, False);
	XFillRectangle (display, bitmap, gcreverse, 0, 0, width, height);
	if (!fast)
	    XFlush(display);
	enable_sigio();
}

void x11_put_text(s)
	char *s;
{
	disable_sigio();
	if(fast && nlinept) {
		x11_draw_vec();
	}
	String(s);
	if (!fast)
	    XFlush(display);
	enable_sigio();
}

void x11_setcolor(c)
     int c;
{
    if (!x11_init_done) {
	x11_open_window();
    }
    disable_sigio();
    x11_draw_vec();

    if (c>=100) {
	XSetWindowBackground(display, win, colors[(c-100)%Ncolors]);
	XSetForeground(display, gcreverse, colors[(c-100)%Ncolors]);
    } else {	  
	if (c == 0) {
	    XSetForeground(display, gc, BlackPixel(display, screen));
	} else if (!Color) {
	    XSetForeground(display, gc, WhitePixel(display, screen));
	} else {
	    XSetForeground(display, gc, colors[c%Ncolors]);
	}		
    }

    if (!fast) {
	XFlush(display);
    }
    enable_sigio();
}

void x11_open_window()
{
	char *window_name = "Xhocplot";
	char *display_name = NULL;
	XSizeHints size_hints;
	XWindowAttributes attr;

	if (x11_init_done) {
		return;
	}
	if ((display = XOpenDisplay(display_name)) == NULL)
	{
		(void)fprintf(stderr, "%s: cannot connect to X server %s\n", 
					XDisplayName(display_name));
	}
	screen = DefaultScreen(display);
	
	win = XCreateSimpleWindow(display, RootWindow(display, screen), 
		WX, WY, WIDTH, HEIGHT, 0, 
		BlackPixel(display, screen),
		WhitePixel(display, screen));

	XGetWindowAttributes(display, win, &attr);
	D = attr.depth;
	if (Color) set_colors();

	size_hints.flags = USPosition|USSize;
	size_hints.x = WX; size_hints.y = WY;
	size_hints.width = WIDTH; size_hints.height = HEIGHT;
	XSetStandardProperties(display, win, window_name, NULL, 
		NULL, NULL, NULL, &size_hints);

	XSetWindowBackground(display, win, WhitePixel(display, screen));

	gcreverse = XCreateGC(display, win, 0, NULL);
	gc = XCreateGC(display, win, 0, NULL);

	XSetForeground(display, gcreverse, WhitePixel(display, screen));
	XSetBackground(display, gcreverse, BlackPixel(display, screen));

	XSetForeground(display, gc, BlackPixel(display, screen));
	XSetBackground(display, gc, WhitePixel(display, screen));

	XMapWindow(display, win);

/*
	font = XLoadQueryFont(display, "9x15");
	XSetFont(display, gc, font->fid);
*/
	/*------------------------------- 
	Commented-out 11-OCT-91 by JC Wathey.
	Causes problems on MIPS machines.
	XFlush(display);
	---------------------------------*/
	
	XSelectInput(display, win, ExposureMask);
	XNextEvent(display, &report);

	/*------------------------------------------------------------- 
	  Commented-out 3/27/92 by M. Wong.  We are now doing bitmap 
	  savings and will need to be able to accept expose events:
	  XSelectInput(display, win, 0L);
	---------------------------------------------------------------*/

	/* Now, setup the bitmap */
	bitmap = XCreatePixmap (display, win, WIDTH, HEIGHT,
				DefaultDepth (display, 
					      DefaultScreen (display)));

	/* Make sure that the bitmap memory is cleared */
 	XCopyArea (display, win, bitmap, gc, 0, 0, WIDTH, HEIGHT, 0, 0);
	if (bitmap == BadAlloc || bitmap == BadDrawable || 
	    bitmap == BadValue)
	    hoc_execerror("Cannot allocate bitmap needed for X", (char *) 0);
	getscale();

	/* for sigio handling of expose events */
	setup_sigio();
	enable_sigio();

	x11_init_done = 1;
}

void x11_close_window()
{
	if (x11_init_done) {
	        disable_sigio();
		XFreeGC(display, gc);
		XFreePixmap(display, bitmap);
		XCloseDisplay(display);
	}
	x11_init_done = 0;
}
/*-----------------------------------------------------------------------------
 *   set_colors - set colors from user resources or defaults
 *---------------------------------------------------------------------------*/
char *color_names[Ncolors] = { "black", "white", "yellow", 
   "red",  "green", "blue",  "magenta", "cyan", "sienna", "orange", "coral" };

static set_colors() {
    int n;
    XColor used, exact;
    
    disable_sigio();
    for(n=0; n<Ncolors; n++) {
	if (XAllocNamedColor(display, DefaultColormap(display,0),
			     color_names[n], &used, &exact)){
	    colors[n] = used.pixel; 
	}
	else {
	    fprintf(stderr, "xhocplot: assuming %s:white\n", 
		    color_names[Ncolors]);
	    colors[n] = WhitePixel(display,0);
	}
    }
    enable_sigio();
}

void adjustBitmap() {

    /* Variable declaration */
    static unsigned int oldwidth = WIDTH;
    static unsigned int oldheight = HEIGHT;

    /* If the width or height has been changed, then we need to reallocate
       a new pixmap */
    if ((width != oldwidth) || (height != oldheight)) {
	XFreePixmap (display, bitmap);
	bitmap = XCreatePixmap (display, win, width, height,
                                DefaultDepth (display, 
					      DefaultScreen (display)));
 	XCopyArea (display, win, bitmap, gc, 0, 0, width, height, 0, 0);
	oldwidth = width;
	oldheight = height;
    }
} /* end: adjustBitmap() */

static handle_sigio()
    /* this will redraw the window following an expose event */
{
    disable_sigio();
    while (XCheckMaskEvent(display, ExposureMask|StructureNotifyMask,&report)) {
	if ((report.xexpose.count == 0) && (report.type == Expose)) {
	    XCopyArea (display, bitmap, win, gc, 0, 0, width, height, 0, 0);
	    XFlush (display);
	}
    }
    XFlush(display);
    enable_sigio();
}

/* handle_sigio_4_view MUST NOT be a static function because view.mod needs 
   it */
int handle_sigio_4_view()
    /* this will redraw the window following an expose event */
{
    while (XCheckMaskEvent(display, ExposureMask|StructureNotifyMask,
			   &report)) {
	if ((report.xexpose.count == 0) && (report.type == Expose)) {
	    XCopyArea (display, bitmap, win, gc, 0, 0, width, height, 0, 0);
	    XFlush (display);
	}
    }
    XFlush(display);
}

/* setup_sigio MUST NOT be a static function because view.mod needs it */
setup_sigio()
    /* setup unix signal handler */
{
    int socket = display->fd;
    socket_flags = fcntl( socket, F_GETFL, 0);
    socket_count = 0;
    signal(SIGIO,handle_sigio);
    fcntl(socket, F_SETOWN, getpid() );
}

/* enable_sigio MUST NOT be a static function because view.mod needs it */
enable_sigio()
    /* turns on the testing of socket io (ie graphics communications to display) */
{
    int socket = display->fd;
    if ( ++socket_count == 1)
	fcntl(socket, F_SETFL, socket_flags | FASYNC);
}

static disable_sigio()
    /* turns off the testing of socket io */
{
    int socket = display->fd;
    if ( --socket_count == 0)
	fcntl(socket, F_SETFL, socket_flags & ~FASYNC);
}

#endif
